unit T_loader;
{
***********************************************************************
**       [STREAM]                                                    **
**                                                                   **
**             ORBIS ... Stream index                                **
**                                                                   **
**                       Topol Pro 1996                             **
**                                                                   **
***********************************************************************
}

interface

uses
   Classes,
   e_Class,
   Elements,
   Tools;

type

TMagic = Array[0..3] of char;

TOrbisLoader = class(TTool)
private
   FFileName: string;
   FIndex: longInt;
   FActualPropertyIndex: integer;
   FContainer: TQuickEntity;
   FActualElement: TElement;
   FActualClass: TOrbisClass;
   FInstance: TOrbisInstance;
   FClasses: TOrbisClasses;
   FMagic: TMagic;
   //FGauge: TF_gaugeForm;
   procedure SetClasses(aValue: TOrbisClasses);
protected
   property  Index: longInt read FIndex write FIndex default 0;
   property  Container: TQuickEntity read FContainer write FContainer;
public
   constructor CreateAll(const aFileName: string);
   procedure   DoIt; override;
   procedure   StartGauge; virtual;
   procedure   EndGauge; virtual;
   procedure   UpdateGauge; virtual;
   function    GetGaugeMax: longInt; virtual; abstract;
   function    NumberToPropertyName(aNumber: longInt): string; override;
   procedure   Prepare; virtual;
   procedure   Finish; virtual;
   procedure   PrepareClasses; virtual;
   procedure   LoadInfo; virtual; abstract;
   procedure   LoadClasses; virtual;
   procedure   LoadClass; virtual; abstract;
   procedure   AddClass(aName: string); virtual;
   procedure   AddClassProperty(const aName: string; aType: integer); virtual;
   procedure   LoadAll; virtual; abstract;
   procedure   NewElement(aClass: longInt); virtual;
   procedure   LoadProperties; virtual; abstract;
   procedure   SetProperty(aIndex: integer; aType: integer; var aValue); virtual;
   procedure   LoadElements; virtual; abstract;
   procedure   AddNextElement(aElement: longInt); virtual;
   property    OrbisClasses: TOrbisCLasses read FClasses write SetClasses;
   property    ActualClass: TOrbisClass read FActualClass write FActualClass;
   property    ActualElement: TElement read FActualElement;
   property    ActualPropertyIndex: integer read FActualPropertyIndex write FActualPropertyIndex;
   property    Instance: TOrbisInstance read FInstance;
published
   property    FileName: string read FFileName write FFileName;
end;



TStreamLoader = class(TOrbisLoader)
private
   FReader: TReader;
   FStream: TStream;
protected
   property  Reader: TReader read FReader write FReader;
   property  Stream: TStream read FStream write FStream;
public
   procedure   DoIt; override;
   function    GetGaugeMax: longInt; override;
   procedure   UpdateGauge; override;
   procedure   UseOldLoader; virtual;
   procedure   LoadInfo; override;
   procedure   LoadClass; override;
   procedure   LoadAll; override;
   procedure   LoadProperties; override;
   procedure   LoadElements; override;
end;

TLoad = class(TStreamLoader);


TLoader = class(TTool)
 private
   FIndex: longInt;
   FReader: TReader;
   FStream: TStream;
   FContainer: TQuickEntity;
 protected
   procedure CreateProperties;
   procedure FreeProperties;
   property  Index: longInt read FIndex write FIndex default 0;
   property  Reader: TReader read FReader write FReader;
   property  Stream: TStream read FStream write FStream;
   property  Container: TQuickEntity read FContainer write FContainer;
 public
   constructor Create; override;
   constructor CreateAll(aStreamName: string);
   procedure   DoIt; override;
end;


implementation

uses

  {!t_Main,}
  {!}PubUnits,

  Controls,
  Dialogs,
  Forms,
  SysUtils,
  TypInfo,
  Worlds;


type

EBadStreamName = class(Exception);
EBadStream = class(Exception);


{ TOrbisLoader }


constructor TOrbisLoader.CreateAll(const aFileName: string);
begin
   inherited Create;
   FileName := aFileName;
end;

procedure   TOrbisLoader.DoIt;
begin
   StartGauge;
   Prepare;
   LoadInfo;
   LoadClasses;
   LoadAll;
   Output := Container.elements[Container.Count - 1];
   Finish;
   EndGauge;
end;

procedure TOrbisLoader.StartGauge;
var
   i: longInt;
begin
   Screen.Cursor:= crHourGlass;
   //FGauge:= TF_GaugeForm.Create(Application);
   //FGauge.Caption:= 'Natn modelu ze souboru ' + FileName;
   i := GetGaugeMax;
   //FGauge.MaxValue:= i; {!!!!!!!!!!!!!!!!!!!!!!!!}
   //FGauge.Show
   StartProgress('Natn modelu ze souboru ' + FileName,0,i);
   Application.ProcessMessages;
end;



procedure TOrbisLoader.EndGauge;
begin
   Screen.Cursor:= crDefault;
   (*
   if FGauge <> nil then
   begin
      FGauge.Free;
      FGauge:= nil
   end;
   *)
   EndProgress;
end;

procedure   TOrbisLoader.UpdateGauge;
begin
end;



function    TOrbisLoader.NumberToPropertyName(aNumber: longInt): string;
begin
   result := 'FileName';
end;



procedure TOrbisLoader.SetClasses(aValue: TOrbisClasses);
begin
   if FClasses <> nil then
   begin
      FreeElement(FCLasses);
      FClasses := nil;
   end;
   FClasses := aValue;
   AddElement(FClasses);
end;



procedure   TOrbisLoader.Prepare;
begin
   Container := TQuickEntity.Create;
   Finstance := TOrbisInstance.Create;
   AddElement(FInstance);
   PrepareClasses;
end;


procedure   TOrbisLoader.Finish;
begin
   Container.Clear;
   Container.Free;
end;

procedure   TOrbisLoader.PrepareClasses;
begin
   OrbisClasses := TOrbisClasses.Create;
end;


procedure   TOrbisLoader.LoadClasses;
begin
end;

procedure   TOrbisLoader.AddClass(aName: string);
var
   OC: TOrbisClass;
begin
   ActualClass := TOrbisClass.Create;
   OrbisClasses.AddElement(ActualClass);
   ActualPropertyIndex := 0;
   OC := TOrbisClasses(World.ObjectClasses).GetOrbisClassByName(aName);
   if OC = nil then
   begin
      { neregistrovan tda !!!!!!!!!!};
      exit;
   end;
   ActualClass.ElementClass := OC.ElementClass;
end;


procedure   TOrbisLoader.AddClassProperty(const aName: string; aType: integer);
var
   OCP: TOrbisClassProperty;
   i: integer;
   S: string;

function CompareTypeKinds(T1,T2: TTypeKind): boolean;
begin
   result := true;
   if T1 = T2 then exit;
   if (T1 in [tkString,tkLString]) and (T1 in [tkString,tkLString]) then exit;
   result := false;
end;

begin
   S := UpperCase(aName);
   if ActualClass = nil then exit;
   OCP := TOrbisClassProperty.Create;
   OCP.Name := aName;
   OCP.TrueIndex := -1;
   ActualClass.AddElement(OCP);
   { Kontrola propert .. nesed-li typ nebo co, TrueIndex zstv -1 }
   { Nen to nhodou aktuln properta ? }
   if AnsiCompareText(aName,ActualClass.PropertyName[ActualPropertyIndex])= 0 then
   begin
      { Jo ... kontrola typu }
      if CompareTypeKinds(ActualClass.PropertyTypeKind[ActualPropertyIndex],TTypeKind(aType)) then
      { Typ Souhlas }
      begin
         OCP.TrueIndex := ActualPropertyIndex;
         OCP.PropInfo  := ActualClass.PropInfo[ActualPropertyIndex];
         inc(FActualPropertyIndex);
      end
      else
      begin
         { Chyba typu !!!!!!}
      end;
      exit;
   end
   else
   for i := 0 to ActualClass.PropCount - 1 do
   begin
      if AnsiCompareText(aName,ActualClass.PropertyName[i]) = 0 then
      { properta existuje }
      begin
         { kontrola typu }
         if CompareTypeKinds(ActualCLass.PropertyTypeKind[i],TTypeKind(aType)) then
         { Typ Souhlas }
         begin
            OCP.TrueIndex := i;
            OCP.PropInfo  := ActualClass.PropInfo[i];
            ActualPropertyIndex := i + 1;
         end
         else
         begin
            { Chyba typu !!!!!!}
         end;
         exit;
      end;
   end;
   { properta neexistuje !!!!!}
end;

procedure   TOrbisLoader.NewElement(aClass: longInt);
var
   OC: TOrbisClass;
begin
   OC := TOrbisClass(OrbisClasses.Elements[aClass]);
   if OC = nil then
   begin
      Instance.Instance := nil;
      exit;
   end;
   ActualClass := OC;
   if ActualClass.ElementClass = nil then exit;
   Instance.Instance := ActualClass.ElementClass.Create;
   FActualElement := Instance.Instance;
   Container.AddElement(ActualElement);
   FActualPropertyIndex := 0;
end;

procedure   TOrbisLoader.SetProperty(aIndex: integer; aType: integer; var aValue);
var
   Idx: integer;
begin
   if (ActualClass = nil) or (ActualClass.ElementClass = nil) then exit;
   Idx := TOrbisClassProperty(ActualClass.Elements[aIndex]).TrueIndex;
   if Idx < 0 then exit;
   if TTypeKind(aType) <> Instance.PropertyTypeKind[Idx] then exit; {  chyba !!!!!!!!!!!!!}
   Instance.SetPropertyValue(Idx, aValue);
end;

procedure   TOrbisLoader.AddNextElement(aElement: longInt);
var
   E: TElement;
begin
   try
      E := Container.Elements[aElement];
      TEntity(ActualElement).AddElement(E);
   except
   end;
end;

{ TStreamLoader }

procedure   TStreamLoader.DoIt;
var
   Magic: array[0..3] of Char;
begin
   if FileName = '' then raise EBadStreamName.Create('Chybn jmno souboru : '+FileName+' !');
   if not FileExists(FileName) then raise EBadStream.Create('Soubor '+FileName+' neexistuje!');
   Stream := TMemoryStream.Create;
   try
      TMemoryStream(Stream).LoadFromFile(FileName);
   except
      Stream.Free;
      Stream := TFileStream.Create(FileName, fmOpenRead{Write !!!!!!!} );
   end;
   Reader := TReader.Create(Stream, 4096);
   Reader.Read(Magic, SizeOf(Magic));
   if Magic = 'TPF0'then
   begin
      Reader.Free;
      Stream.Free;
      UseOldLoader;
      exit;
   end
   else if Magic <> 'ORB0'then
   begin
      Reader.Free;
      Stream.Free;
      raise EBadStream.Create(FileName+' ... neznm typ souboru');
   end;
   inherited DoIt;
   Reader.Free;
   Stream.Free;
end;

function    TStreamLoader.GetGaugeMax: longInt;
begin
   result := Stream.Size;
end;

procedure   TStreamLoader.UpdateGauge;
begin
   if Stream.Position mod 100 = 0 then
   begin
      Progress := FStream.Position;
      ProgressText := IntToStr(Stream.Position)+' byt z celkovch '+IntToStr(Stream.Size);
      //FGauge.Progress:= FStream.Position;
      Application.ProcessMessages
   end;
end;

procedure   TStreamLoader.UseOldLoader;
var
   L: TLoader;
begin
   L := TLoader.Create;
   L.Name := FileName;
   L.DoIt;
   Output := L.Output;
   L.Free;
end;


procedure   TStreamLoader.LoadInfo;
var
   S: string;
begin
   S := Reader.ReadString;
   S := Reader.ReadString;
   S := Reader.ReadString;
end;

procedure   TStreamLoader.LoadClass;
var
   aClassName: string;
   aClassID: longInt;
   aPropertyName: string;
   aPropertyType: integer;
begin
   aClassName := Reader.ReadString;
   aClassID   := Reader.ReadInteger;
   AddClass(aClassName);
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      begin
         aPropertyName := Reader.ReadString;
         aPropertyType := Reader.ReadInteger;
         AddClassProperty(aPropertyName, aPropertyType);
      end;
   end;
   Reader.ReadListEnd;
end;

procedure   TStreamLoader.LoadAll;
var
   aClass: longInt;
   aId: longInt;
   Tmp: longInt;
begin
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aClass := Reader.ReadInteger;
      if aClass = -1 then LoadClass
      else
      begin
         NewElement(aClass);
         aId := Reader.ReadInteger;
         LoadProperties;
         LoadElements;
      end;
      UpdateGauge;
   end;
   Reader.ReadListEnd;
end;

procedure   TStreamLoader.LoadProperties;
var
   aIndex: integer;
   tmpInteger: integer;
   tmpInt64: Int64;
   tmpChar: char;
   tmpString: string;
   tmpFloat: extended;
   tmpElement: TElement;
   TypeKind: TTypeKind;
begin
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aIndex := Reader.ReadInteger;
      TypeKind := TOrbisClassProperty(ActualClass.Elements[aIndex]).TypeKind;
      case TypeKind of
         tkInt64:
         begin
            tmpInt64 := Reader.ReadInt64;
            SetProperty(aIndex,integer(TypeKind), tmpInt64);
         end;
         tkInteger,tkEnumeration:
         begin
            tmpInteger := Reader.ReadInteger;
            SetProperty(aIndex, integer(TypeKind), tmpInteger);
         end;
         tkChar:
         begin
            tmpChar := Reader.ReadChar;
            SetProperty(aIndex, integer(TypeKind), tmpChar);
         end;
         tkFloat:
         begin
            tmpFloat := Reader.ReadFloat;
            SetProperty(aIndex, integer(TypeKind), tmpFloat);
         end;
         tkString
         {$IFDEF Win32}
         ,tkLString
         {$ENDIF}
                   :
         begin
            tmpString := Reader.ReadString;
            SetProperty(aIndex, integer(TypeKind), tmpString);
         end;

         tkClass:
         begin
            tmpInteger := Reader.ReadInteger;
            if tmpInteger = -1 then tmpElement := nil
                               else tmpElement := Container.Elements[tmpInteger];
            SetProperty(aIndex, integer(TypeKind), tmpElement);
         end;
      end;
   end;
   Reader.ReadListEnd;
end;

procedure   TStreamLoader.LoadElements;
var
   Realy: integer;
   aIndex: longInt;
begin
   Realy := Reader.ReadInteger;
   if not boolean(Realy) then exit;
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aIndex := Reader.ReadInteger;
      AddNextElement(aIndex);
   end;
   Reader.ReadListEnd;
end;


{ TLoader }

constructor TLoader.Create;
begin
   inherited Create;
   Name := 'ORBIS.ORB';
end;



constructor TLoader.CreateAll(aStreamName: string);
begin
   inherited Create;
   Name := aStreamName;
end;


procedure   TLoader.CreateProperties;
begin
   if Name = '' then raise EBadStreamName.Create('Chybn jmno streamu : '+Name+' !');
   Stream := TFileStream.Create(Name, fmOpenRead);
   Reader := TReader.Create(Stream, 64000);
   Container := TQuickEntity.Create;
end;

procedure   TLoader.FreeProperties;
begin
   Container.Clear;
   Container.Free;
   Reader.Free;
   Stream.Free;
end;


procedure   TLoader.DoIt;
var
   aClassName: string;
   aClass: TClass;
   Element: tElement;
begin
   Screen.Cursor := crHourGlass;
   CreateProperties;
   Reader.ReadSignature;
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
     try
      aClassName := Reader.ReadString;
      aClass := GetOrbisClass(aClassName);
      if aClass = nil then
      begin
         MessageDlg('Neregistrovan element: '+ aClassName, mtError, [mbOK], 0);
         FreeProperties;
         Screen.Cursor := crDefault;
         exit;
      end;
     except
       MessageDlg('Chyba pi ten elementu: '+ aClassName, mtError, [mbOK], 0);
     end;

      try
         Element := TElement(aClass.Create);
         Element.Load(Reader, Container);
         Container.AddElement(Element);
         {
         Application.ProcessMessages;
         }
      except
         MessageDlg('Chyba pi ten elementu: '+ aClassName, mtError, [mbOK], 0);
         FreeProperties;
         Screen.Cursor := crDefault;
         exit;
      end;
   end;
   Reader.ReadListEnd;
   Output := Container.elements[Container.Count - 1];
   FreeProperties;
   Screen.Cursor := crDefault;
end;


BEGIN
   RegisterOrbisClass(TLoader);
   RegisterOrbisClass(TLoad);
END.

