unit e_Gis_Georaster;

interface
uses
   Elements,
   ImgSize,
   e_GFile;

type

TGis_GeoRaster = class(TGraphicFileElement)
  private
    FError: integer;
    FUpperLeftX: double;
    FUpperLeftY: double;
    FPixelWidth: integer;
    FHorizRes: double;
    FGeoRaster: boolean;
    FVertRes: double;
    FPixelHeight: integer;
    procedure SetError(const Value: integer);
    procedure SetGeoRaster(const Value: boolean);
    procedure SetHorizRes(const Value: double);
    procedure SetPixelHeight(const Value: integer);
    procedure SetPixelWidth(const Value: integer);
    procedure SetUpperLeftX(const Value: double);
    procedure SetUpperLeftY(const Value: double);
    procedure SetVertRes(const Value: double);
protected
    procedure SetFileName(aValue: string); override;
published
   property PixelWidth: integer read FPixelWidth write SetPixelWidth;
   property PixelHeight: integer read FPixelHeight write SetPixelHeight;
   property VertRes: double  read FVertRes write SetVertRes;
   property HorizRes: double read FHorizRes write SetHorizRes;
   property UpperLeftX: double read FUpperLeftX write SetUpperLeftX;
   property UpperLeftY: double read FUpperLeftY write SetUpperLeftY;
   property GeoRaster: boolean read FGeoRaster write SetGeoRaster;
   property Error: integer read FError write SetError stored false;
end;


implementation
uses
   Classes,
   SysUtils,
   e_Class,
   u_String;

{ TGis_GeoRaster }

procedure TGis_GeoRaster.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TGis_GeoRaster.SetFileName(aValue: string);
var
   anExt, aGeoF: string;
   aW,aH: word;
   aSL: TStringList;
   i: integer;
   dW,dH: double;
begin
   inherited;
   Error := 0;
   anExt := UpperCase(ExtractFileExt(aValue));
   if not FileExists(aValue) then
   begin
      Error := 1;
      exit;
   end;
   if not SameStr(anExt,'.PNG')  then
   begin
      Error := 2;
      exit;
   end;
   GeoRaster := false;
   // velikost souboru
   GetPNGSize(aValue, aW, aH);
   PixelWidth := aW;
   PixelHeight := aH;
   // nacteni Geoinformace
   aGeoF := ChangeFileExt(aValue,'.pgw');
   if not FileExists(aGeof) then
   begin
      Error := 3;
      exit;
   end;
   aSL := TStringList.Create;
   aSL.LoadFromFile(aGeoF);
   if aSL.Count < 6 then
   begin
      Error := 4;
      exit;
   end;
   HorizRes := StrToExtended(aSL.Strings[0]);
   VertRes  := -StrToExtended(aSL.Strings[3]);
   UpperLeftX := StrToExtended(aSL.Strings[4]);
   UpperLeftY := StrToExtended(aSL.Strings[5]);
   aSL.Free;
   dW := PixelWidth * HorizRes;
   dH := PixelHeight * VertRes;
   MinX := UpperLeftX - (HorizRes / 2);
   MinY := UpperLeftY + (VertRes / 2) - dH;
   Width  := dW;
   Height := dH;
   GeoRaster := true;
end;

procedure TGis_GeoRaster.SetGeoRaster(const Value: boolean);
begin
  FGeoRaster := Value;
end;

procedure TGis_GeoRaster.SetHorizRes(const Value: double);
begin
  FHorizRes := Value;
end;

procedure TGis_GeoRaster.SetPixelHeight(const Value: integer);
begin
  FPixelHeight := Value;
end;

procedure TGis_GeoRaster.SetPixelWidth(const Value: integer);
begin
  FPixelWidth := Value;
end;

procedure TGis_GeoRaster.SetUpperLeftX(const Value: double);
begin
  FUpperLeftX := Value;
end;

procedure TGis_GeoRaster.SetUpperLeftY(const Value: double);
begin
  FUpperLeftY := Value;
end;

procedure TGis_GeoRaster.SetVertRes(const Value: double);
begin
  FVertRes := Value;
end;

initialization
   RegisterOrbisClass(TGis_GeoRaster);
end.
