unit i_AreaElement;
interface

uses
   Graphics,
// KERNEL
   Elements,
   Tools,
   T_Recurs,
// GLIB
   g_Elems,
// SPACE
   e_Geometry,
   //E_Space,
// DEVICE
   e_Device,
   i_Basic;
// SYMBOL
   //e_PlZn,
// TOPOL
   //I_Top;



type

TAreaElementInterpret = class(TRectInterpret)
private
    FColor: integer;
    FMultiColor: integer;
    procedure SetColor(const Value: integer);
    procedure SetMultiColor(const Value: integer);
 protected
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
 public
   constructor Create; override;
   function   CanUniGoInside(Element: TElement): boolean; override;
   procedure  Reset(aDevice: TDevice); override;
   procedure  Close; override;
published
   property Color: integer read FColor write SetColor default $FFFF00;
   property MultiColor: integer read FMultiColor write SetMultiColor;
end;

TAreaAsLineInterpret = class(TAreaElementInterpret)
 protected
   procedure Process(Element: TElement); override;
end;

implementation

uses
   WinTypes,
   WinProcs,
   Dialogs,
   SysUtils,
// KERNEL
   e_Class;
// GLIB
   //ClipArea,
   //E_Color,
// SYMBOL
   //u_Symbol,
// SPACE
   //e_GeometryElement,
// TOPOL
   //e_TopoLElements;
   //E_TplPrp;



{ TAreaElementInterpret }



procedure TAreaElementInterpret.Process(Element: TElement);
var
  PlochaBoundary: TAreaElement;
  NPlocha: TAreaElement;
  i: integer;
  MultiArea: TMultiAreaElement;

procedure ProcessArea(Elem: TElement);
var
   aX,aY: double;
begin
  try
    PlochaBoundary := TAreaElement(Elem);
    aX := PlochaBoundary.X;
    aY := PlochaBoundary.Y;

    NPlocha:= nil;
    if PlochaBoundary is TPolyLineAreaElement then
    begin
      NPlocha:= TPolyLineAreaElement(PlochaBoundary).CreateAreaElement;
      PlochaBoundary:= NPlocha;
    end;

    Device.Brush.Style:= bsSolid;
    Device.Pen.Style:= psSolid;
    Device.Pen.Color:= 0;
    Device.Pen.Width:= 1;

    Device.PolyPolygon(PlochaBoundary);

    //!!! TEST
    {
    Device.Font.Size := 8;
    Device.Font.Color := 255;
    Device.TextOut(aX,aY,Elem.Name);
    }
    //END TEST

    // popisov bod
    Device.Pen.Color:= 255;
    Device.MoveTo(aX - 5, aY);
    Device.LineTo(aX + 5, aY);
    Device.MoveTo(aX, aY - 5);
    Device.LineTo(aX, aY + 5);

    if (NPlocha <> nil) then
    begin
    { POZOR ! nerusit souradnice !}
{      NPlocha.Clear; }
      NPlocha.Free
    end

  except
      //MessageBeep(Word(-1));
  end;
end;
begin
   if Element.ClassType = TMultiAreaElement then
   begin
      MultiArea := TMultiAreaElement(Element);
      Device.Brush.Color:= MultiColor;
      for i := 0 to MultiArea.LastIndex do
      begin
         ProcessArea(MultiArea.Elements[i]);
      end;
   end
   else
   begin
      Device.Brush.Color:= Color;
      ProcessArea(Element);
   end;
end;



function  TAreaElementInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := AObject is TAreaElement;
end;



function  TAreaElementInterpret.QGoInside(Element: TElement): boolean;
begin
  Result:= false;
end;



function  TAreaElementInterpret.QProcessBefore(Element: TElement): boolean;
var
  ModelLength: double;
begin
  result:= false;
  if not (Element is TAreaElement) then exit;
  try
    ModelLength:= Device.Projection.GetModelLength(3);

    if TAreaElement(Element).GetMaxLength < ModelLength then exit;

    if not TAreaElement(Element).IsOverRegion
            (Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY) then exit;
    Result:= true

  except
  end;
end;


procedure   TAreaElementInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
end;

procedure   TAreaElementInterpret.Close;
begin
   inherited Close;
end;


function  TAreaElementInterpret.CanUniGoInside(Element: TElement): boolean;
begin
   result := false;
end;



procedure TAreaElementInterpret.SetColor(const Value: integer);
begin
   FColor := Value;
end;

procedure TAreaElementInterpret.SetMultiColor(const Value: integer);
begin
   FMultiColor := Value;
end;

constructor TAreaElementInterpret.Create;
begin
  inherited;
  Color := $FFFF00;
  MultiColor := 255;
end;

{ TAreaAsLineInterpret }

procedure TAreaAsLineInterpret.Process(Element: TElement);
var
  PlochaBoundary: TAreaElement;
  NPlocha: TAreaElement;
  i: integer;
  MultiArea: TMultiAreaElement;

procedure ProcessArea(Elem: TElement);
begin
  try
    PlochaBoundary := TAreaElement(Elem);

    NPlocha:= nil;
    if PlochaBoundary is TPolyLineAreaElement then
    begin
      NPlocha:= TPolyLineAreaElement(PlochaBoundary).CreateAreaElement;
      PlochaBoundary:= NPlocha;
    end;

    Device.Brush.Style:= bsClear;
    Device.Pen.Style:= psSolid;
    Device.Pen.Color:= Color;
    Device.Pen.Width:= 1;

    Device.PolyPolygon(PlochaBoundary);

    if (NPlocha <> nil) then
    begin
    { POZOR ! nerusit souradnice !}
{      NPlocha.Clear; }
      NPlocha.Free
    end

  except
//      MessageBeep(Word(-1));
  end;
end;
begin
   if Element.ClassType = TMultiAreaElement then
   begin
      MultiArea := TMultiAreaElement(Element);
      //Device.Brush.Color:= MultiColor;
      for i := 0 to MultiArea.LastIndex do
      begin
         ProcessArea(MultiArea.Elements[i]);
      end;
   end
   else
   begin
      //Device.Brush.Color:= Color;
      ProcessArea(Element);
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TAreaElementInterpret);
   RegisterOrbisClass(TAreaAsLineInterpret);
END.
