unit i_AreaGeometryElement;
interface

uses
   Graphics,
// KERNEL
   Elements,
   Tools,
   T_Recurs,
// GLIB
   g_Elems,
// SPACE
   e_Geometry,
   //E_Space,
// DEVICE
   e_Device,
// SYMBOL
   e_PlZn,
   i_Basic;
// TOPOL
   //I_Top;



type

TGeometryAreaElementInterpret = class(TRectInterpret)
private
    PlosneZnacky: TPlosneZnacky;
    FUseColor: boolean;
    FUseSymbol: boolean;
    FColor: integer;
    FSymbol: integer;
    procedure SetColor(const Value: integer);
    procedure SetSymbol(const Value: integer);
    procedure SetUseColor(const Value: boolean);
    procedure SetUseSymbol(const Value: boolean);
   //Rect: TDoubleRect;
 protected
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure ProcessAsUserDefinedPattern(aPlocha: TAreaElement; aZnacka,aColor: integer);
 public
   function   CanUniGoInside(Element: TElement): boolean; override;
   procedure  Reset(aDevice: TDevice); override;
   procedure  Close; override;
   function   GetElementClass: TElementClass; virtual;
   function   Get_Color(const Element: TElement): integer; virtual;
   function   Get_Symbol(const Element: TElement): integer; virtual;
   procedure  ReadAllSymbols;
 published
   property   Color: integer read FColor write SetColor;
   property   UseColor: boolean read FUseColor write SetUseColor;
   property   Symbol: integer read FSymbol write SetSymbol;
   property   UseSymbol: boolean read FUseSymbol write SetUseSymbol;
end;



implementation

uses
   WinTypes,
   WinProcs,
   Dialogs,
   SysUtils,
// KERNEL
   e_Class,
// GLIB
   ClipArea,
   E_Color,
// SYMBOL
   u_Symbol,
// SPACE
   e_GeometryElement;
// TOPOL
   //e_TopoLElements;
   //E_TplPrp;



{ TGeometryAreaElementInterpret }

procedure TGeometryAreaElementInterpret.ProcessAsUserDefinedPattern(aPlocha: TAreaElement; aZnacka,aColor: integer);
var
  Srafa: TPlosnaZnacka;
  Coords, Lengths: pointer;
  aSize, LNumber: integer;
begin
  if aPlocha.IsHole then exit;
  //if not (aProps is TTopolAreaProps) then exit;
  Srafa := PlosneZnacky.VratZnackuDleKodu(aZnacka);
  if not assigned(Srafa) then exit;

  Device.PrepareForPolyPolygon(aPlocha,Coords,Lengths,aSize,LNumber);
  Srafa.ExternalColor := aColor;
  Srafa.VyplnPlochu(Device, TByteArray(Coords^)[0], TLengthArray(Lengths^)[0], LNumber,aPlocha.MinX,aPlocha.MaxY);
  freeMem(Coords, aSize);
  freeMem(Lengths, LNumber*sizeOf(TLengthItem));
end;


procedure TGeometryAreaElementInterpret.Process(Element: TElement);
var
  PlochaBoundary: TAreaElement;
  NPlocha: TAreaElement;
  G: TGeometryElement;
  A: TAreaElement;
  MultiArea: TMultiAreaElement;
  //P: Wintypes.TPoint;
  Znacka, Barva: longInt;
  SimpleArea: boolean;
  i: integer;

procedure ProcessArea(aElem: TElement);
begin
  try
    PlochaBoundary := TAreaElement(aElem);
    if not PlochaBoundary.IsOverRegion(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY) then exit;
    SimpleArea := true;
    NPlocha:= nil;
    if PlochaBoundary is TPolyLineAreaElement then
    begin
      NPlocha:= TPolyLineAreaElement(PlochaBoundary).CreateAreaElement;
      PlochaBoundary:= NPlocha;
      SimpleArea := false;
    end;

    Device.Brush.Style:= bsSolid;

    if UseSymbol then
    begin
       Znacka := Symbol;
    end;

    if UseColor then
    begin
      Device.Brush.Color:= Color;
      Device.Pen.Style:= psClear;
      Device.PolyPolygon(PlochaBoundary)
    end
    else
    if Znacka > 999 then
    begin
      ProcessAsUserDefinedPattern(PlochaBoundary,Znacka,DejBarvicku(Barva));
    end
    else
    if Znacka > 0 then
    begin
       Device.Brush.Color:= DejBarvicku(Barva);
       if SimpleArea then
       begin
          Device.Pen.Style:= psSolid;
          Device.Pen.Color:= 0;
          Device.Pen.Width:= 1;
       end
       else Device.Pen.Style:= psClear;

       Device.PolyPolygon(PlochaBoundary)

    end;
{ Kek ? }

    if (NPlocha <> nil) then
    begin
    { POZOR ! nerusit souradnice !}
{      NPlocha.Clear; }
      NPlocha.Free
    end

  except
//      MessageBeep(Word(-1));
  end;
end;

begin
   G := TGeometryElement(Element);
   Barva := Get_Color(Element);
   Znacka := Get_Symbol(Element);
   A := TAreaElement(G.Geometry);
   if A.ClassType = TMultiAreaElement then
   begin
      MultiArea := TMultiAreaElement(A);
      for i := 0 to MultiArea.LastIndex do
      begin
         ProcessArea(MultiArea.Elements[i]);
      end;
   end
   else
   begin
      ProcessArea(A);
   end;
end;



function  TGeometryAreaElementInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := (AObject.ClassType = TAreaCollection) or (AObject.ClassType = GetElementClass);
end;



function  TGeometryAreaElementInterpret.QGoInside(Element: TElement): boolean;
begin
   //Result:= ((Element is TTopoLAreaElement) and ScaleOK(Element))
   result := (Element is TEntity) and not (Element.ClassType = GetElementClass);
end;



function  TGeometryAreaElementInterpret.QProcessBefore(Element: TElement): boolean;
var
  ModelLength: double;
begin
  result:= false;
  if (Element.ClassType = TAreaCollection) then
  begin
    //OwnerProps:= TAreaString(Element).Properties;
     UseColor := TAreaCollection(Element).UseColor;
     Color :=    TAreaCollection(Element).Color;
     UseSymbol:= TAreaCollection(Element).UseSymbol;
     Symbol := TAreaCollection(Element).Symbol;
     exit;
  end;

  if not (Element.ClassType = GetElementClass) then exit;
  if TGeometryElement(Element).NoGeometry then exit;
  try
    ModelLength:= Device.Projection.GetModelLength(3);

    if TAreaElement(TGeometryElement(Element).Geometry).GetMaxLength < ModelLength then exit;

    if not TGeometryElement(Element).IsOverRegion
            (Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY) then exit;
    Result:= true

  except
  end;
end;


procedure   TGeometryAreaElementInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
   //Rect := Device.Projection.ClipModelView;
   PlosneZnacky := TPlosneZnacky.CreateAll(PathOfLibraries+'\Srafy.txt');
   PlosneZnacky.Device := Device;
end;

procedure   TGeometryAreaElementInterpret.Close;
begin
   if PlosneZnacky <> nil then
   begin
      PlosneZnacky.Free;
      PlosneZnacky := nil;
   end;
   inherited Close;
end;


function  TGeometryAreaElementInterpret.CanUniGoInside(Element: TElement): boolean;
begin
   result := (Element.ClassType = TAreaCollection) and
      TAreaCollection(Element).IsOverRegion
               (Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY);
end;


procedure TGeometryAreaElementInterpret.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TGeometryAreaElementInterpret.SetSymbol(const Value: integer);
begin
  FSymbol := Value;
end;

procedure TGeometryAreaElementInterpret.SetUseColor(const Value: boolean);
begin
  FUseColor := Value;
end;

procedure TGeometryAreaElementInterpret.SetUseSymbol(const Value: boolean);
begin
  FUseSymbol := Value;
end;

function TGeometryAreaElementInterpret.GetElementClass: TElementClass;
begin
   result := TGeometryElement;
end;

function TGeometryAreaElementInterpret.Get_Color(const Element: TElement): integer;
begin
   result := 10;
end;

function TGeometryAreaElementInterpret.Get_Symbol(const Element: TElement): integer;
begin
   result := 1;
end;

procedure TGeometryAreaElementInterpret.ReadAllSymbols;
begin
   if PlosneZnacky = nil then exit;
   PlosneZnacky.PridejVsechnyZnacky;
end;

INITIALIZATION
   RegisterOrbisClass(TGeometryAreaElementInterpret);
END.
