unit t_CreateOverlay;

interface
uses
   Elements,
   e_Geometry,
   e_QTree,
   t_QTreeTools,
   Tools;

type

TCreateOverlayAreas = class(TTool)
  private
    FA1_Areas: TEntity;
    FA2_Areas: TEntity;
    FLines: TPolyLineString;
    FComment: string;
    FA1QT: TRectEntityQTree;
    FA2QT: TRectEntityQTree;
    FAreas: TAreaString;
    FA1_Lines: TEntity;
    FA2_Lines: TEntity;
    FCreateEmpty: boolean;
    FOneLayer: boolean;
    function GetLines: TPolyLineString;
    procedure SetA1_Areas(const Value: TEntity);
    procedure SetA2_Areas(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetA1QT(const Value: TRectEntityQTree);
    procedure SetA2QT(const Value: TRectEntityQTree);
    procedure MakeAreas;
    procedure SetAreas(const Value: TAreaString);
    procedure SetA1_Lines(const Value: TEntity);
    procedure SetA2_Lines(const Value: TEntity);
    procedure SetCreateEmpty(const Value: boolean);
    procedure SetOneLayer(const Value: boolean);
public
   procedure DoIt; override;
   procedure PrepareQTree;
   procedure MakeNewAreas;
   procedure MakeFinalAreas;
   property Lines: TPolyLineString read GetLines;
   property Areas: TAreaString read FAreas write SetAreas;
   property A1QT: TRectEntityQTree read FA1QT write SetA1QT;
   property A2QT: TRectEntityQTree read FA2QT write SetA2QT;
published
   property A1_Areas: TEntity read FA1_Areas write SetA1_Areas;
   property A2_Areas: TEntity read FA2_Areas write SetA2_Areas;
   property A1_Lines: TEntity read FA1_Lines write SetA1_Lines;
   property A2_Lines: TEntity read FA2_Lines write SetA2_Lines;
   property CreateEmpty: boolean read FCreateEmpty write SetCreateEmpty;
   property Comment: string read FComment write SetComment;
   property OneLayer: boolean read FOneLayer write SetOneLayer;
end;


implementation
uses
   Dialogs,
   Forms,
   e_Class,
   e_GeometryElement,
   e_OverlayGeometryElement,
   g_Elems,
   t_AreMak,
   t_GeometryChecks,
   t_LineExtractor;

{ TCreateOverlayAreas }

procedure TCreateOverlayAreas.DoIt;
begin
   if Comment = '' then
      Comment := Name;
   if (A1_Areas = nil) or A1_Areas.NoElements then
   begin
      ShowMessage(Comment+': A1_Areas chyb nebo je przdn!');
      exit;
   end;
   if (A1_Lines = nil) or A1_Lines.NoElements then
   begin
      ShowMessage(Comment+': A1_Lines chyb nebo je przdn!');
      exit;
   end;
   if not OneLayer then   // pokud nejedeme jen na jednu vrstvu
   begin
      if (A2_Areas = nil) or A2_Areas.NoElements then
      begin
         ShowMessage(Comment+':A2_Areas chyb nebo je przdn!');
         exit;
      end;
      if (A2_Lines = nil) or A2_Lines.NoElements then
      begin
         ShowMessage(Comment+':A2_Lines chyb nebo je przdn!');
         exit;
      end;
   end;
   StartProgress(Comment,0,0);
   Application.ProcessMessages;
   PrepareQTree;
   MakeNewAreas;
   MakeFinalAreas;
end;

function TCreateOverlayAreas.GetLines: TPolyLineString;
begin
   if FLines = nil then
   begin
      FLines := TPolyLineString.Create;
      AddElement(FLines);
   end;
   result := FLines;
end;

procedure TCreateOverlayAreas.MakeAreas;
begin

end;

procedure TCreateOverlayAreas.MakeFinalAreas;
var
   QTF: TFindRectEntityInQTreeByXY;
   W: TQTreeWalker;
   i: integer;
   A: TAreaElement;
   OE: TOverlayGeometryElement;
   RE : TRectEntity;
   e: TElement;
   A1,A2: TElement;
   FinalAreas: TGeometryElementCollection;
begin
   W := nil;
   QTF := nil;
   if OneLayer then
   begin
      ProgressText := 'Finalizace - hledn v pekryv ve vrstv...';
      W := TQTreeWalker.Create;
   end
   else
   begin
      ProgressText := 'Finalizace - hledn v pekrvajcch se vrstvch...';
      QTF := TFindRectEntityInQTreeByXY.Create;
   end;
   MaxProgress := Areas.LastIndex;
   Application.ProcessMessages;
   FinalAreas := TGeometryElementCollection.Create;
   FinalAreas.Name := 'OVERLAY';
   Output := FinalAreas;
   AddElement(FinalAreas);
   for i := 0 to Areas.LastIndex do
   begin
      if (i mod 50 = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Areas.Elements[i];
      if not (e is TAreaElement) then continue;
      A := TAreaElement(e);
      A1 := nil;
      A2 := nil;
      if OneLayer then
      begin
         OE := nil;
         E := W.GetFirstElementForRectangle(A1QT,A.Minx,A.MinY,A.MaxX,A.MaxY,1);
         while E <> nil do
         begin
            if E is TRectEntity then
            begin
               RE := TRectEntity(E);
               if RE.IsHitByXY(A.X,A.Y,0) then
               begin
                  if A1 = nil then
                  begin
                     A1 := RE;
                  end
                  else
                  if (A2 = nil) then
                  begin
                     if RE <> A1 then
                     begin
                        A2 := RE;
                        OE := TOverlayGeometryElement.Create;
                        OE.Geometry := A;
                        OE.A1 := A1;
                        OE.A2 := A2;
                        FinalAreas.AddAndUpdateRect(OE);
                     end;
                  end
                  else
                  begin
                     if not RE.IsMyOwner(OE) then
                        OE.AddElement(RE);
                  end;
               end;
            end;
            E := W.GetNextElement;
         end;
      end
      else
      begin
         // hledani A1
         QTF.Input := A1QT;
         QTF.Tolerance := 0;
         QTF.X := A.X;
         QTF.Y := A.Y;
         QTF.DoIt;
         A1 := QTF.Output;
         // hledani A2
         QTF.Input := A2QT;
         QTF.DoIt;
         A2 := QTF.Output;
         if (A1 <> nil) or (A2 <> nil) or CreateEmpty then // je prekryv nebo povoleny prazdne elementy
         begin
            OE := TOverlayGeometryElement.Create;
            OE.Geometry := A;
            OE.A1 := A1;
            OE.A2 := A2;
            FinalAreas.AddAndUpdateRect(OE);
         end;
      end;
   end;
   if OneLayer then
      W.Free
   else
      QTF.Free;
end;

procedure TCreateOverlayAreas.MakeNewAreas;
var
   MA: TAreasFromClearLines;
   CL: TClearNoValidLines;
begin
   Lines.Update;
   ProgressText := '... itn lini';
   Application.ProcessMessages;
   CL := TClearNoValidLines.Create;
   CL.Input := Lines;
   CL.WithoutPrompt := true;
   CL.DoIt;
   CL.Free;
   ProgressText := '... Tvorba pekryvnch ploch';
   Application.ProcessMessages;
   MA := TAreasFromClearLines.Create;
   MA.Input := Lines;
   MA.DoIt;
   Areas := TAreaString(MA.Output); // nov vytvoen Plochy
   MA.Free;
end;

procedure TCreateOverlayAreas.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   LEX: TLineExtractor;
   A1L: TPolyLineString;
   A2L: TPolyLineString;
   CL: TClearLines;
   En: TRectEntity;
begin
   ProgressText := 'Koprovn lini';
   Application.ProcessMessages;
   A2L := nil;
   A1L := TPolyLineString.Create;
   A1L.Name := 'A1L';
   AddElement(A1L);
   if not OneLayer then
   begin
      A2L := TPolyLineString.Create;
      A2L.Name := 'A2L';
      AddElement(A2L);
   end;
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := A1_Lines;
   LEX.Output := A1L;
   LEX.DoIt;
   A1L.Update;
   if not OneLayer then
   begin
      LEX.Input := A2_Lines;
      LEX.Output := A2L;
      LEX.DoIt;
      A2L.Update;
   end;
   ProgressText := '... a spojen lini';
   Application.ProcessMessages;
   CL := TClearLines.Create;
   En := TRectEntity.Create;
   AddElement(En);
   En.AddElement(A1L);
   if not OneLayer then
      En.AddElement(A2L);
   En.Update;
   CL.Input := En;
   {
   if OneLayer then
   begin
      CL.Input := A1L;
   end
   else
   begin
      CL.FixedLines := A1L;
      CL.Input := A2L;
   end;
   }
   CL.SelfTest := true;
   //CL.SnapDistance := 0.5;//!!!!!!!!!!!!!!
   CL.DoIt;
   CL.Free;
   LEX.CopyLines := false;
   LEX.NoDuplicates := True;
   LEX.Output := Lines;
   LEX.Input := En;
   LEX.DoIt;
   En.Free;
   A1L.Free;
   if not OneLayer then
      A2L.Free;
   LEX.Free;
   ProgressText := '... pprava pracovnch QTree ploch A1';
   Application.ProcessMessages;
   A1QT :=TRectEntityQTree.Create;
   A1QT.Name := 'A1 QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := A1_Areas;
   TQ.Output := A1QT;
   TQ.DoIt;
   TQ.Free;
   if not OneLayer then
   begin
      ProgressText := '... pprava pracovnch QTree ploch A2';
      Application.ProcessMessages;
      A2QT :=TRectEntityQTree.Create;
      A2QT.Name := 'A2 QT';
      TQ := TAddEntityToQTree.Create;
      TQ.Input := A2_Areas;
      TQ.Output := A2QT;
      TQ.DoIt;
      TQ.Free;
   end;
end;

procedure TCreateOverlayAreas.SetA1QT(const Value: TRectEntityQTree);
begin
   FA1QT := Value;
   AddElement(FA1QT);
end;

procedure TCreateOverlayAreas.SetA1_Areas(const Value: TEntity);
begin
  FA1_Areas := Value;
end;

procedure TCreateOverlayAreas.SetA1_Lines(const Value: TEntity);
begin
  FA1_Lines := Value;
end;

procedure TCreateOverlayAreas.SetA2QT(const Value: TRectEntityQTree);
begin
   FA2QT := Value;
   AddElement(FA2QT);
end;

procedure TCreateOverlayAreas.SetA2_Areas(const Value: TEntity);
begin
  FA2_Areas := Value;
end;

procedure TCreateOverlayAreas.SetA2_Lines(const Value: TEntity);
begin
  FA2_Lines := Value;
end;

procedure TCreateOverlayAreas.SetAreas(const Value: TAreaString);
begin
   ExchangeFieldElement(TElement(FAreas), Value);
end;

procedure TCreateOverlayAreas.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreateOverlayAreas.SetCreateEmpty(const Value: boolean);
begin
  FCreateEmpty := Value;
end;

procedure TCreateOverlayAreas.SetOneLayer(const Value: boolean);
begin
  FOneLayer := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateOverlayAreas);
END.
