unit t_PickUpGeometry;

interface
uses
  Elements,
  e_Geometry,
  G_Elems,
  T_Recurs;

type

TPickUpGeometry = class(TRecursor)
private
   FFound: boolean;
   FTolerance: double;
   FMulti: boolean;
   FMultiElement: TGeometry;
   FY: double;
   FX: double;
   procedure SetMulti(const Value: boolean);
   procedure SetMultiElement(const Value: TGeometry);
   procedure SetFound(const Value: boolean);
   procedure SetX(const Value: double);
   procedure SetY(const Value: double);
public
   procedure DoIt; override;
   procedure Process(Element: TElement); override;
published
   property Found: boolean read FFound write SetFound;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property Tolerance: double read FTolerance write FTolerance;
   property Multi: boolean read FMulti write SetMulti;
   property MultiElement: TGeometry read FMultiElement write SetMultiElement;
end;


TPickUpPointElement = class(TPickUpGeometry)
public
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
end;



TPickUpPolyLineElement = class(TPickUpGeometry)
public
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
end;


TPickUpAreaElement = class(TPickUpGeometry)
public
   procedure Process(Element: TElement); override;
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
end;

TPickUpGeometryElement = class(TPickUpGeometry)
private
   FElementClass: TElementClass;
   function GetElementClassName: string;
   procedure SetElementClass(const Value: TElementClass);
   procedure SetElementClassName(const Value: string);
public
   procedure Process(Element: TElement); override;
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   property ElementClass: TElementClass read FElementClass write SetElementClass;
published
   property ElementClassName: string read GetElementClassName write SetElementClassName;
end;

implementation

uses
   E_GeometryElement,
   E_Class;


procedure TPickUpGeometry.DoIt;
begin
  Found:= false;
  MultiElement := nil;
  Multi := false;
  inherited DoIt;
end;


procedure TPickUpGeometry.Process(Element: TElement);
begin
  if Found then exit;
  if not (Element is TGeometry) then exit;
  Found:= TGeometry(Element).IsHitByXY(X,Y, Tolerance);
  if Found then
     Output:= Element
end;

procedure TPickUpGeometry.SetFound(const Value: boolean);
begin
  FFound := Value;
end;

procedure TPickUpGeometry.SetMulti(const Value: boolean);
begin
   FMulti := Value;
end;

procedure TPickUpGeometry.SetMultiElement(const Value: TGeometry);
begin
  FMultiElement := Value;
end;

procedure TPickUpGeometry.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TPickUpGeometry.SetY(const Value: double);
begin
  FY := Value;
end;


 { TPickUpPointElement }

function  TPickUpPointElement.QGoInside(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TRectEntity)
           and not (Element is TAreaString)
           and not (Element is TPolyLineString)
           and not (Element is TPolyLineElement)
           and not (Element is TAreaElement)
end;


function  TPickUpPointElement.QProcessBefore(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TPointElement)
end;


 { TPickUpPolyLineElement }

function  TPickUpPolyLineElement.QGoInside(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TRectEntity)
           and not (Element is TPolyLineElement)
           and not (Element is TAreaElement)
           and not (Element is TAreaString)
           and not (Element is TPointString)
end;


function  TPickUpPolyLineElement.QProcessBefore(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TPolyLineElement)
end;





 { TPickUpAreaElement }

procedure TPickUpAreaElement.Process(Element: TElement);
var
   MA: TMultiAreaElement;
   A: TAreaElement;
   i: integer;
begin
  if Found then exit;
  if Element is TMultiAreaElement then
  begin
     MA := TMultiAreaElement(Element);
     for i := 0 to MA.LastIndex do
     begin
        A := TAreaElement(MA.Elements[i]);
        Found:= TGeometry(A).IsHitByXY(X,Y, Tolerance);
        if Found then
        begin
           Multi := true;
           MultiElement := MA;
           Output := A;
           break;
        end;
     end;
  end
  else
  begin
     Found:= TGeometry(Element).IsHitByXY(X,Y, Tolerance);
     if Found then
        Output:= Element
  end;
end;


function  TPickUpAreaElement.QGoInside(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TRectEntity)
           and not (Element is TAreaElement)
           and not (Element is TPolyLineElement)
           and not (Element is TPolyLineString)
           and not (Element is TPointString)
end;


function  TPickUpAreaElement.QProcessBefore(Element: TElement): boolean;
begin
  Result:= not Found and (Element is TAreaElement)
end;


{ TPickUpGeometryElement }

function TPickUpGeometryElement.GetElementClassName: string;
begin
   result := '';
   if ElementClass <> nil then
      result := ElementClass.ClassName;
end;

procedure TPickUpGeometryElement.Process(Element: TElement);
begin
   if Found then exit;
   if TGeometryElement(Element).IshitByXY(X,Y,Tolerance) then
   begin
      Found := true;
      Output := Element;
   end;
end;

function TPickUpGeometryElement.QGoInside(Element: TElement): boolean;
begin
   result := not Found and (Element is TEntity) and not(Element is TGeometry);
end;

function TPickUpGeometryElement.QProcessBefore(Element: TElement): boolean;
begin
   result := false;
   if Found then exit;
   if ElementClass <> nil then
   begin
      result := Element.ClassType = ElementClass;
   end
   else
   begin
      result := Element is TGeometryElement;
   end;
end;

procedure TPickUpGeometryElement.SetElementClass(const Value: TElementClass);
begin
   FElementClass := Value;
end;

procedure TPickUpGeometryElement.SetElementClassName(const Value: string);
begin
   ElementClass := GetOrbisClass(Value);
end;

INITIALIZATION
   RegisterOrbisClass(TPickUpGeometry);
   RegisterOrbisClass(TPickUpGeometryElement);
   RegisterOrbisClass(TPickUpPointElement);
   RegisterOrbisClass(TPickUpPolyLineElement);
   RegisterOrbisClass(TPickUpAreaElement);
END.

