unit t_ExtendedFormTool;

interface
uses
   ActnList,
   Comctrls,
   Controls,
   // KERNEL
   e_ToolModel,
   Elements,
   Tools,
   // APP
   f_ExtendedOrbisForm,
   t_FormTool;

type

TExtendedToolGroup = class(TTechnology)
private
   FActualAction: TAction;
   FActionList: TActionList;
   FexForm: TExtendedOrbis_Form;
   FImageList: TImagelist;
   FToolBar: TToolbar;
   FAlign: TAlign;
   procedure SetActionList(const Value: TActionList);
   procedure SetexForm(const Value: TExtendedOrbis_Form);
   procedure SetImageList(const Value: TImagelist);
    function GetActualCaption: string;
    function GetActualHint: string;
    function GetActualImageIndex: integer;
    function GetActualScript: string;
    function GetActualTool: TTool;
    function GetActualVisible: boolean;
    function GetButtonHeight: integer;
    function GetButtonWidth: integer;
    function GetFlat: boolean;
    function GetImageHeight: integer;
    function GetImageWidth: integer;
    function GetShowCaptions: boolean;
    procedure SetActualCaption(const Value: string);
    procedure setActualHint(const Value: string);
    procedure SetActualImageIndex(const Value: integer);
    procedure SetActualScript(const Value: string);
    procedure SetActualTool(const Value: TTool);
    procedure SetActualVisible(const Value: boolean);
    procedure SetButtonHeight(const Value: integer);
    procedure SetButtonWidth(const Value: integer);
    procedure SetFindActionName(const Value: string);
    procedure SetFlat(const Value: boolean);
    procedure SetImageHeight(const Value: integer);
    procedure SetImageWidth(const Value: integer);
    procedure SetNewImage(const Value: string);
    procedure SetNewScriptAction(const Value: string);
    procedure SetNewToolAction(const Value: string);
    procedure SetShowCaptions(const Value: boolean);
    procedure SetToolBar(const Value: TToolbar);
    procedure SetAlign(const Value: TAlign);
    procedure SetEmpty(const Value: boolean);
    procedure SetNewSeparator(const Value: string);
public
   procedure CreateSelfByModel; override;
   function  CreateToolByModel(aModel: TToolModel): TTool; override;
   procedure AddToolsByModel(aTool: TTool); virtual;
   procedure AddToolByModel(aTool: TTool); virtual;
   procedure ActionExecute(Sender: TObject);
   function  FindActionByTool(aTool: TTool): TAction;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property  ToolBar: TToolbar read FToolBar write SetToolBar;
   property  ImageList: TImagelist read FImageList write SetImageList;
   property  ActionList: TActionList read FActionList write SetActionList;
   property  exForm: TExtendedOrbis_Form read FexForm write SetexForm;
   property  Empty: boolean write SetEmpty;
published
   property   Align: TAlign read FAlign write SetAlign;
   property   Flat: boolean read GetFlat write SetFlat;
   property   ButtonHeight: integer read GetButtonHeight write SetButtonHeight;
   property   ButtonWidth: integer read GetButtonWidth write SetButtonWidth;
   property   ImageHeight: integer read GetImageHeight write SetImageHeight;
   property   ImageWidth: integer read GetImageWidth write SetImageWidth;
   // For manual setting only:
   property   NewImage: string write SetNewImage stored false;
   property   NewScriptAction: string write SetNewScriptAction stored false;
   property   NewToolAction: string write SetNewToolAction stored false;
   property   NewSeparator: string write SetNewSeparator stored false;
   property   FindActionName: string write SetFindActionName stored false;
   property   ActualVisible: boolean read GetActualVisible write SetActualVisible stored false;
   property   ActualScript: string read GetActualScript write SetActualScript stored false;
   property   ActualTool: TTool read GetActualTool write SetActualTool stored false;
   property   ActualImageIndex: integer read GetActualImageIndex write SetActualImageIndex stored false;
   property   ActualCaption: string read GetActualCaption write SetActualCaption stored false;
   property   ActualHint: string read GetActualHint write setActualHint stored false;
   property   ShowCaptions: boolean read GetShowCaptions write SetShowCaptions;
end;

TExtendedFormTool = class(TFormTool)
private
   FTasks: TExtendedToolGroup;
   FTaskTools: TExtendedToolGroup;
   FMonitors: TExtendedToolGroup;
   FFrameSelectors: TTechnology;
   procedure  SetNewGroupName(const Value: string);
   function   GetGroups: TTools;
   procedure  SetGroups(const Value: TTools);
   function   GetMonitors: TExtendedToolGroup;
   function   GetTasks: TExtendedToolGroup;
   function   GetTaskTools: TExtendedToolGroup;
   function   GetActiveGroup: TExtendedToolGroup;
   procedure  SetActiveGroup(const Value: TExtendedToolGroup);
   function   GetActiveTask: TTool;
   procedure  SetActiveTask(const Value: TTool);
   function   GetFrameSelectors: TTechnology;
   function   GetActiveFrameSelector: TTool;
   procedure  SetActiveFrameSelector(const Value: TTool);
protected
   function    GetActiveTool: TTool; override;
   procedure   SetActiveTool(aTool: TTool); override;
public
   procedure   CreateForm; override;
   function    AddTool(aTool: TTool): TTool; override;
   procedure   RemoveTaskTools; virtual;
   procedure   RemoveTasks; virtual;
   procedure   AddTasks(const Task: TTool); virtual;
   procedure   AddTaskTools(const Task: TTool); virtual;
   procedure   on_tm_ExternalTaskSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_TaskSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property    ActiveGroup: TExtendedToolGroup read GetActiveGroup write SetActiveGroup;
   property    ActiveTask: TTool read GetActiveTask write SetActiveTask;
   property    ActiveFrameSelector: TTool read GetActiveFrameSelector write SetActiveFrameSelector;
published
   property    Groups: TTools read GetGroups write SetGroups stored false;
   property    NewGroupName: string write SetNewGroupName stored false;
   property    Tasks: TExtendedToolGroup read GetTasks stored false;
   property    TaskTools: TExtendedToolGroup read GetTaskTools stored false;
   property    Monitors: TExtendedToolGroup read GetMonitors stored false;
   property    FrameSelectors: TTechnology read GetFrameSelectors stored false;
end;

implementation
uses
   Graphics,
   SysUtils,
   ToolWin,
   // KERNEL
   e_Class,
   e_Type,
   m_Orbis,
   u_String,
   // APP
   c_Actions,
   t_FrameSelect,
   t_InterfaceTool;

{ TExtendedFormTool }

procedure TExtendedFormTool.AddTasks(const Task: TTool);
begin
   if Tasks = nil then exit;
   Tasks.AddToolsByModel(Task);
end;

procedure TExtendedFormTool.AddTaskTools(const Task: TTool);
begin
   if TaskTools = nil then exit;
   TaskTools.AddToolsByModel(Task);
end;

function TExtendedFormTool.AddTool(aTool: TTool): TTool;
begin
   result := inherited AddTool(aTool);
   if result is TExtendedToolGroup then
   begin
      TExtendedToolGroup(result).exForm := TExtendedOrbis_Form(aForm);
      if UpperCase(result.Name) = 'TASKS' then FTasks := TExtendedToolGroup(result);
      if UpperCase(result.Name) = 'TASKTOOLS' then FTaskTools := TExtendedToolGroup(result);
      if UpperCase(result.Name) = 'MONITORS' then FMonitors := TExtendedToolGroup(result);
   end
   else
   if result is TTechnology then
   begin
      if UpperCase(result.Name) = 'FRAMESELECTORS' then
         FFrameSelectors := TTechnology(result);
   end;
end;

procedure TExtendedFormTool.CreateForm;
begin
end;



function TExtendedFormTool.GetActiveFrameSelector: TTool;
begin
   result := nil;
   if FrameSelectors = nil then exit;
   result := FrameSelectors.ActiveTool;
end;

function TExtendedFormTool.GetActiveGroup: TExtendedToolGroup;
var
   T: TTool;
begin
   result := nil;
   T := inherited GetActiveTool;
   if not (T is TExtendedToolGroup) then exit;
   result := TExtendedToolGroup(T);
end;

function TExtendedFormTool.GetActiveTask: TTool;
begin
   result := nil;
   if Tasks = nil then exit;
   result := Tasks.ActiveTool;
end;

function TExtendedFormTool.GetActiveTool: TTool;
var
   G: TExtendedToolGroup;
begin
   result := nil;
   G := ActiveGroup;
   if G = nil then exit;
   result := G.ActiveTool;
end;

function TExtendedFormTool.GetFrameSelectors: TTechnology;
begin
   result := FFrameSelectors;
end;

function TExtendedFormTool.GetGroups: TTools;
begin
   result := Tools;
end;

function TExtendedFormTool.GetMonitors: TExtendedToolGroup;
begin
   result := FMonitors;
end;

function TExtendedFormTool.GetTasks: TExtendedToolGroup;
begin
   result := FTasks;
end;

function TExtendedFormTool.GetTaskTools: TExtendedToolGroup;
begin
   result := FTaskTools;
end;

procedure TExtendedFormTool.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   //if aMessage.Message = tm_Resize then
   //   PostTextMsg(tm_AddToProtocol,Name + '.on_Resize',nil);
   Handled := false;
   try
     if (ActiveTool <> nil) then
     begin
        case aMessage.Message  of
           tm_Review, tm_Cursor: ;// udlosti, kter se nepedvaj aktivnmu nstroji
        else
           ActiveTool.HandleMessage(aMessage, Handled);
        end;
     end;
   except
   end;
   if Handled then exit;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_ExternalTaskSelected: on_tm_ExternalTaskSelected(aMessage, Handled);
      tm_TaskSelected:         on_tm_TaskSelected(aMessage, Handled);
      tm_ToolSelected:         on_tm_ToolSelected(aMessage, Handled);
   end;
end;

procedure TExtendedFormTool.on_tm_ExternalTaskSelected(var aMessage: TOrbisMessage; var Handled: boolean);
var
   T: TTool;
   S: string;
begin
   T := TTool(aMessage.Data);
   S := 'No Tool';
   if T <> nil then S := T.Name;
   PostTextMsg(tm_AddToProtocol, Name+'.on_ExternalTask: '+S,nil);
   RemoveTaskTools;
   RemoveTasks;
   AddTasks(T);
   ActiveTask := nil;
end;

procedure TExtendedFormTool.on_tm_TaskSelected(var aMessage: TOrbisMessage;var Handled: boolean);
var
   T: TTool;
begin
   RemoveTaskTools;
   T := TTool(aMessage.Data);
   if T = nil then exit;
   if ActiveTask <> nil then
   begin
      if ActiveTask.DynamicTool then ActiveTask.DestroyDynamicPart;
      ActiveTask := nil;
   end;
   if T.DynamicTool then T.CreateDynamicPart;
   AddTaskTools(TTool(aMessage.Data));
   ActiveTask := T;
   Tasks.FActualAction.Checked := true;
end;

procedure TExtendedFormTool.on_tm_ToolSelected(var aMessage: TOrbisMessage;var Handled: boolean);
var
   G: TExtendedToolGroup;
   T: TTool;
   IT: TInterfaceTool;
   A: TAction;
begin
   T := TTool(aMessage.Data);
   if T = nil then exit;
   G := TExtendedToolGroup(T.FindOwnerClass(TExtendedToolGroup));
   if G = nil then
   begin
      PostTextMsg(tm_AddToprotocol,'Tool: '+ T.Name + ' is not component of '+ Name,nil);
   end;
   A := G.FindActionByTool(T);
   if G = Tasks then
   begin
      Tasks.FActualAction.Checked := false;
      if T = ActiveTask then
      begin
         Tasks.FActualAction.Checked := true;
         exit;
      end;
      Tasks.FActualAction := A;
      PostMsg(tm_TaskSelected,Self,T,false);
      exit;
   end;
   if ActiveTool <> nil then ActiveGroup.FActualAction.Checked := false;
   if T = ActiveTool then
   begin
      ActiveGroup.FActualAction.Checked := true;
      exit;
   end;

   if not (T is TInterfaceTool) then
   begin
      A.Checked := true;
      T.DoIt;
      A.Checked := false;
      if ActiveTool <> nil then ActiveGroup.FActualAction.Checked := true;
      exit;
   end;
   A.Checked := true;
   G.FActualAction := A;
   IT := TInterfaceTool(T);

   if ActiveTool is TInterfaceTool then
      TInterfaceTool(ActiveTool).On_DeActivate;

   Cursor := crDefault;
   ActiveTool := IT;
   //Cursor := IT.Cursor;
   IT.Device := Device;
   IT.Input  := Self.Input;
   if IT is TFrameTool then
      TFrameTool(IT).FrameSelector := TFrameSelector(ActiveFrameSelector);
   IT.On_Activate;
   Handled := true;
   //CheckMenuItem(ActiveTool.ClassName);
   //FormCursor := TInterActiveTool(ActiveTool).Cursor;
end;

procedure TExtendedFormTool.RemoveTasks;
begin
   if Tasks = nil then exit;
   Tasks.Empty := true;
end;

procedure TExtendedFormTool.RemoveTaskTools;
begin
   if TaskTools = nil then exit;
   TaskTools.Empty := true;
end;

procedure TExtendedFormTool.SetActiveFrameSelector(const Value: TTool);
begin
   if FrameSelectors = nil then exit;
   FrameSelectors.ActiveTool := Value;
end;

procedure TExtendedFormTool.SetActiveGroup(const Value: TExtendedToolGroup);
begin
   inherited SetActiveTool(Value);
end;

procedure TExtendedFormTool.SetActiveTask(const Value: TTool);
begin
   if Tasks = nil then exit;
   Tasks.ActiveTool := Value;
end;

procedure TExtendedFormTool.SetActiveTool(aTool: TTool);
var
   G: TExtendedToolGroup;
begin
   if aTool = nil then exit;
   G := TExtendedToolGroup(aTool.FindOwnerClass(TExtendedToolGroup));
   if G = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Tool '+aTool.Name+' is not component of '+ Name,nil);
      exit;
   end;
   ActiveGroup := G;
   G.ActiveTool := aTool;
end;

procedure TExtendedFormTool.SetGroups(const Value: TTools);
begin
   Tools := Value;
end;

procedure TExtendedFormTool.SetNewGroupName(const Value: string);
var
   G: TExtendedToolGroup;
begin
   if GetToolByName(Value) <> nil then
   begin
      PostTextMsg(tm_AddToProtocol, 'Group named '+ Value +' already exists !',nil);
      exit;
   end;
   G := TExtendedToolGroup.Create;
   G.Name := Value;
   AddTool(G);
end;


{ TExtendedToolGroup }

procedure TExtendedToolGroup.ActionExecute(Sender: TObject);
begin
   PostMsgToOwners(tm_ToolSelected,TToolAction(Sender).Tool,false);
   PostTextMsg(tm_AddToProtocol,'SelectedTool: '+TToolAction(Sender).Name,nil);
end;


procedure TExtendedToolGroup.AddToolByModel(aTool: TTool);
var
   aModel: TToolModel;
begin
   if aTool = nil then exit;
   AddTool(aTool);
   aModel := aTool.SelfModel;
   if aModel = nil then exit;
   NewToolAction := aTool.Name;
   ActualTool := aTool;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
end;

procedure TExtendedToolGroup.AddToolsByModel(aTool: TTool);
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
   T: TTechnology;
begin
   if aTool = nil then exit;
   if not (aTool is TTechnology) then exit;
   T := TTechnology(aTool);
   if T.Tools.NoElements then exit;
   M := T.SelfModel;
   if M = nil then exit;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
   for i := 0 to T.Tools.LastIndex do
   begin
      AddToolByModel(T.Tool[i]);
   end;
end;


procedure TExtendedToolGroup.CreateSelfByModel;
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
begin
   M := SelfModel;
   if M = nil then exit;
   inherited;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
end;

function TExtendedToolGroup.CreateToolByModel(aModel: TToolModel): TTool;
begin
   result := nil;
   if aModel = nil then exit;
   if (aModel.Name = '.') or (aModel.Name = '|') then
   begin
      NewSeparator := aModel.Name;
      exit;
   end;
   result := inherited CreateToolByModel(aModel);
   if result = nil then exit;
   NewToolAction := result.Name;
   ActualTool := result;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
end;

function TExtendedToolGroup.FindActionByTool(aTool: TTool): TAction;
var
   i: integer;
   A: TAction;
begin
   result := nil;
   for i := 0 to ActionList.ActionCount - 1 do
   begin
      A := TAction(ActionList.Actions[i]);
      if not (A is TToolAction) then continue;
      if TToolAction(A).Tool = aTool then
      begin
         result := A;
         exit;
      end;
   end;
end;

function TExtendedToolGroup.GetActualCaption: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Name;
end;

function TExtendedToolGroup.GetActualHint: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Hint;
end;

function TExtendedToolGroup.GetActualImageIndex: integer;
begin
   result := -1;
   if FactualAction <> nil then result := FActualAction.ImageIndex;
end;

function TExtendedToolGroup.GetActualScript: string;
begin
   result := '';
   if FactualAction = nil then exit;
   if not (FActualAction is TScriptAction) then exit;
   result := TScriptAction(FActualAction).Script;
end;

function TExtendedToolGroup.GetActualTool: TTool;
begin
   result := nil;
   if FactualAction = nil then exit;
   if not (FActualAction is TToolAction) then exit;
   result := TToolAction(FActualAction).Tool;
end;

function TExtendedToolGroup.GetActualVisible: boolean;
begin
   result := false;
   if FactualAction <> nil then result := FActualAction.Visible;
end;

function TExtendedToolGroup.GetButtonHeight: integer;
begin
   result := 0;
   if ToolBar <> nil then result := ToolBar.ButtonHeight;
end;

function TExtendedToolGroup.GetButtonWidth: integer;
begin
   result := 0;
   if ToolBar <> nil then result := ToolBar.ButtonWidth;
end;

function TExtendedToolGroup.GetFlat: boolean;
begin
   result := false;
   if ToolBar <> nil then result := ToolBar.Flat;
end;

function TExtendedToolGroup.GetImageHeight: integer;
begin
   result := 0;
   if ImageList <> nil then result := ImageList.Height;
end;

function TExtendedToolGroup.GetImageWidth: integer;
begin
   result := 0;
   if ImageList <> nil then result := ImageList.Width;
end;

function TExtendedToolGroup.GetShowCaptions: boolean;
begin
   result := false;
   if ToolBar <> nil then result := ToolBar.ShowCaptions;
end;

procedure TExtendedToolGroup.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   OwnersHandleMessage(aMessage,Handled);
end;

procedure TExtendedToolGroup.SetActionList(const Value: TActionList);
begin
   FActionList := Value;
end;

procedure TExtendedToolGroup.SetActualCaption(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Caption := Value;
end;

procedure TExtendedToolGroup.setActualHint(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Hint := Value;
end;

procedure TExtendedToolGroup.SetActualImageIndex(const Value: integer);
begin
   if FactualAction <> nil then FActualAction.ImageIndex := Value;
end;

procedure TExtendedToolGroup.SetActualScript(const Value: string);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TScriptAction) then exit;
   TScriptAction(FActualAction).Script := Value;
end;

procedure TExtendedToolGroup.SetActualTool(const Value: TTool);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TToolAction) then exit;
   TToolAction(FActualAction).Tool := Value;
end;

procedure TExtendedToolGroup.SetActualVisible(const Value: boolean);
begin
   if FactualAction <> nil then FActualAction.Visible := Value;
end;

procedure TExtendedToolGroup.SetAlign(const Value: TAlign);
begin
   if Value = FAlign then exit;
   case Value of
      alTop:    ToolBar.Parent := FExForm.TopBar;
      alBottom: ToolBar.Parent := FExForm.BottomBar;
      alLeft:   ToolBar.Parent := FExForm.LeftBar;
      alRight:  ToolBar.Parent := FExForm.RightBar;
   end;
   ToolBar.Align := Value;
   FAlign := Value;
end;

procedure TExtendedToolGroup.SetButtonHeight(const Value: integer);
begin
   if ToolBar <> nil then ToolBar.ButtonHeight := Value;
end;

procedure TExtendedToolGroup.SetButtonWidth(const Value: integer);
begin
   if ToolBar <> nil then ToolBar.ButtonWidth := Value;
end;

procedure TExtendedToolGroup.SetEmpty(const Value: boolean);
begin
   ToolBar.DestroyComponents;
   ActionList.DestroyComponents;
   ImageList.Clear;
   Tools.FreeAll;
end;

procedure TExtendedToolGroup.SetexForm(const Value: TExtendedOrbis_Form);
begin
   FexForm := Value;
   if Value = nil then exit;
   ActionList := TActionList.Create(FExForm);
   ActionList.Name := 'al_' + Name;
   ImageList  := TImageList.Create(FExForm);
   ImageList.Name := 'img_' + Name;
   ActionList.Images := ImageList;
   ToolBar := TToolBar.Create(FExForm);
   ToolBar.Name := 'tb_' + Name;
   ToolBar.AutoSize := true;
   ToolBar.ShowHint := true;
   //ToolBar.EdgeBorders := [ebLeft,ebRight,ebTop,ebBottom];
   ToolBar.EdgeBorders := [];
   ToolBar.DragKind := dkDock;
   ToolBar.DragMode := dmAutomatic;
   FExForm.TopBar.InsertControl(ToolBar);
   ToolBar.Images := ImageList;
end;

procedure TExtendedToolGroup.SetFindActionName(const Value: string);
var
   i: integer;
   A: TAction;
begin
   FActualAction := nil;
   for i := 0 to ActionList.ActionCount - 1 do
   begin
      A := TAction(ActionList.Actions[i]);
      if CompareText(Value, A.Name) = 0 then
      begin
         FActualAction := A;
         break;
      end;
   end;
end;

procedure TExtendedToolGroup.SetFlat(const Value: boolean);
begin
   if ToolBar <> nil then ToolBar.Flat := Value;
end;

procedure TExtendedToolGroup.SetImageHeight(const Value: integer);
begin
   if ImageList = nil then exit;
   ImageList.Height := Value;
end;

procedure TExtendedToolGroup.SetImageList(const Value: TImagelist);
begin
   FImageList := Value;
end;

procedure TExtendedToolGroup.SetImageWidth(const Value: integer);
begin
   if ImageList = nil then exit;
   ImageList.Width := Value;
end;

procedure TExtendedToolGroup.SetNewImage(const Value: string);
var
   BM: Graphics.TBitmap;
   S: string;
   BColor: TColor;
begin
   BM := Graphics.TBitmap.Create;
   S := '';
   if pos(':',Value) = 0 then
   begin
      S := ParamStr(0);
      S := ExtractFilePath(S);
      CheckBackSlash(S);
   end;
   S := S + Value;
   try
      BM.LoadFromFile(S);
      BColor := BM.Canvas.Pixels[0,0];
   except
      BColor := clGray;
   end;
   ImageList.AddMasked(BM,BColor);
end;

procedure TExtendedToolGroup.SetNewScriptAction(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   A := TScriptAction.Create(ActionList);
   A.ActionList := ActionList;
   FActualAction := A;
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   //ToolBar.EdgeInner :=
   //ToolBar.EdgeOuter :=
   B.Action := A;
   B.Style := tbsCheck;
   A.DisableIfNoHandler := false;
end;

procedure TExtendedToolGroup.SetNewSeparator(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   if Value = '|' then B.Style := tbsDivider
                  else B.Style := tbsSeparator;
   B.Width := 10;
end;

procedure TExtendedToolGroup.SetNewToolAction(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   A := TToolAction.Create(ActionList);
   A.ActionList := ActionList;
   FActualAction := A;
   A.OnExecute := ActionExecute;
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   B.Action := A;
   B.Style := tbsCheck;
   A.DisableIfNoHandler := false;
end;


procedure TExtendedToolGroup.SetShowCaptions(const Value: boolean);
begin
   if ToolBar <> nil then ToolBar.ShowCaptions := Value;
end;

procedure TExtendedToolGroup.SetToolBar(const Value: TToolbar);
begin
  FToolBar := Value;
end;

initialization
   RegisterOrbisClass(TExtendedToolGroup);
   RegisterOrbisClass(TExtendedFormTool);
end.
