unit t_ProjectionsFrame;

interface
uses
   e_ProjectionInfo,
   Elements,
   m_Intera,
   f_NamedProjectionsForm,
   r_ProjectionsFrame,
   t_orbisFrame;
type

TProjectionsFrame = class(TOrbisFrame)
  private
    FProjections: TProjectionInfoModel;
    FForm: TNamedProjections_Form;
    function GetPFrame: TProjections_Frame;
    procedure SetProjections(const Value: TProjectionInfoModel);
    procedure SetNext(const Value: boolean);
    procedure SetPrevious(const Value: boolean);
    procedure SetSetUserProjections(const Value: boolean);
    procedure SetForm(const Value: TNamedProjections_Form);
    procedure SetDeleteNamedProjection(const Value: string);
    procedure SetNewNamedProjection(const Value: string);
    procedure SetShowNamedProjection(const Value: string);
public
   procedure   CreateFrame; override;
   procedure   On_tm_ProjectionCHanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   on_tm_Hide(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   CHangeProjection;
   procedure   SetButtons;
   property    PFrame: TProjections_Frame read GetPFrame;
   property    Form: TNamedProjections_Form read FForm write SetForm;
published
   property Projections: TProjectionInfoModel read FProjections write SetProjections;
   property Next: boolean write SetNext stored false;
   property Previous: boolean write SetPrevious stored false;
   property SetUserProjections: boolean write SetSetUserProjections stored false;
   property NewNamedProjection: string write SetNewNamedProjection stored false;
   property DeleteNamedProjection: string write SetDeleteNamedProjection stored false;
   property ShowNamedProjection: string write SetShowNamedProjection stored false;
end;

implementation
uses
   Classes,
   Forms,
   Windows,
   e_Class,
   e_Proje,
   m_Orbis;

{ TProjectionsFrame }

procedure TProjectionsFrame.CHangeProjection;
var
   P: TProjectionInfo;
begin
   if Projections = nil then exit;
   P := Projections.Projections.ActualProjectionInfo;
   Device.Projection.Center_X := P.X;
   Device.Projection.Center_Y := P.Y;
   Device.Projection.Scale := P.Scale;
   Device.Projection.VirtualScale := P.VirtualScale;
   PostMsgToOwners(tm_BreakAndReView,nil,false);
end;

procedure TProjectionsFrame.CreateFrame;
begin
   aFrame := TProjections_Frame.Create(nil);
   PFrame.Tool := Self;
end;

function TProjectionsFrame.GetPFrame: TProjections_Frame;
begin
   result := TProjections_Frame(aFrame);
end;

procedure TProjectionsFrame.HandleMessage(var aMessage: TOrbisMessage;var Handled: boolean);
begin
   inherited;
   if Handled then exit;
   case aMessage.Message of
      tm_ProjectionChanged: On_tm_ProjectionCHanged(aMessage, Handled);
      tm_Hide:   on_tm_Hide(aMessage,Handled);
   end;
end;

procedure TProjectionsFrame.on_tm_Hide(var aMessage: TOrbisMessage;var Handled: boolean);
begin
   if aMessage.Sender = Self then
   begin
      SetUserProjections := false;
      Handled := true;
   end;
end;

procedure TProjectionsFrame.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   case aMessage.Key of
      VK_F8:
      begin
         if ssShift in aMessage.Shift then
            Next := True
         else
            Previous := True;
      end;
   end;
end;

procedure TProjectionsFrame.On_tm_ProjectionCHanged(var aMessage: TOrbisMessage; var Handled: boolean);
var
   P: TProjection;
   PInf: TProjectionInfo;
begin
   if Projections = nil then exit;
   if aMessage.Data = nil then exit;
   if not(aMessage.Data is TProjection) then exit;
   P := TProjection(aMessage.Data);
   PInf := Projections.Projections.ActualProjectionInfo;
   if (PInf.X = 0) or (PInf.Y = 0) then
   begin
      PInf.X := P.Center_X;
      PInf.Y := P.Center_Y;
      PInf.Scale := P.Scale;
      PInf.VirtualScale := P.VirtualScale;
   end
   else
      Projections.Projections.NewProjection := P;
   SetButtons;
end;

procedure TProjectionsFrame.SetButtons;
begin
   if Projections = nil then exit;
   PFrame.b_Previous.Enabled := not(Projections.Projections.NoPrevious);
   PFrame.b_Next.Enabled := not(Projections.Projections.NoNext);
end;

procedure TProjectionsFrame.SetDeleteNamedProjection(const Value: string);
var
   e: TElement;
   i: integer;
begin
   if Projections = nil then exit;
   e := Projections.NamedProjections.FindElementName(Value);
   if e = nil then exit;
   Projections.NamedProjections.FreeElement(e);
   if Form = nil then exit;
   i := Form.ListBox.Items.IndexOf(Value);
   if i > -1 then
      Form.ListBox.Items.Delete(i);
end;

procedure TProjectionsFrame.SetForm(const Value: TNamedProjections_Form);
begin
  FForm := Value;
end;

procedure TProjectionsFrame.SetNewNamedProjection(const Value: string);
var
   PInf,PInfN: TProjectionInfo;
begin
   if Projections = nil then exit;
   PInf := Projections.Projections.ActualProjectionInfo;
   PInfN := TProjectionInfo.Create;
   PInfN.Name := Value;
   PInfN.X := PInf.X;
   PInfN.Y := PInf.Y;
   PInfN.Scale := PInf.Scale;
   PInfN.VirtualScale := PInf.VirtualScale;
   Projections.NamedProjections.AddElement(PInfN);
   if Form = nil then exit;
   Form.ListBox.Items.Add(Value);
end;

procedure TProjectionsFrame.SetNext(const Value: boolean);
begin
   if Projections = nil then exit;
   Projections.Projections.Next;
   ChangeProjection;
   SetButtons;
end;

procedure TProjectionsFrame.SetPrevious(const Value: boolean);
begin
   if Projections = nil then exit;
   Projections.Projections.Previous;
   ChangeProjection;
   SetButtons;
end;

procedure TProjectionsFrame.SetProjections(const Value: TProjectionInfoModel);
var
   P: TProjectionInfo;
begin
   FProjections := Value;
   P := Value.Projections.ActualProjectionInfo;
   if (P.X = 0) or (P.Y = 0) then exit;
   ChangeProjection;
   SetButtons;
end;

procedure TProjectionsFrame.SetSetUserProjections(const Value: boolean);
var
   i: integer;
begin
   if Value then
   begin
      if Form = nil then
         Form := TNamedProjections_Form.Create(Application);
      {
      Form.Left := FormLeft;
      Form.Top := FormTop;
      }
      Form.Tool := Self;
      Form.Visible := true;
      PFrame.b_UserProjections.Down := true;
      // nastaveni
      Form.ListBox.Clear;
      if Projections = nil then exit;
      if Projections.NamedProjections.NoElements then exit;
      for i := 0 to Projections.NamedProjections.LastIndex do
      begin
         Form.ListBox.Items.Add(Projections.NamedProjections.Elements[i].Name)
      end;
   end
   else
   begin
      if Form <> nil then
      begin
         {
         FormLeft := Form.Left;
         FormTop := Form.Top;
         }
         Form.Free;
      end;
      Form := nil;
      PFrame.b_UserProjections.Down := false;
   end;
end;

procedure TProjectionsFrame.SetShowNamedProjection(const Value: string);
var
   P: TProjectionInfo;
begin
   if Projections = nil then exit;
   P := TProjectionInfo(Projections.NamedProjections.FindElementName(Value));
   if P = nil then exit;
   Device.Projection.Center_X := P.X;
   Device.Projection.Center_Y := P.Y;
   Device.Projection.Scale := P.Scale;
   Device.Projection.VirtualScale := P.VirtualScale;
   PostMsgToOwners(tm_BreakAndReView,nil,false);
end;

INITIALIZATION
   RegisterOrbisClass(TProjectionsFrame);
END.
