unit t_TaskForm;

interface
uses
   Windows,
   ActnList,
   c_Actions,
   e_ToolModel,
   f_TaskForm,
   t_FormTool,
   Tools;

type


TTaskForm = class(TFormTool)
private
   FActualAction: TAction;
   function GetForm: TTask_Form;
    function GetActualCaption: string;
    function GetActualHint: string;
    function GetActualImageIndex: integer;
    function GetActualScript: string;
    function GetActualVisible: boolean;
    function GetImageHeight: integer;
    function GetImageWidth: integer;
    procedure SetActualCaption(const Value: string);
    procedure setActualHint(const Value: string);
    procedure SetActualImageIndex(const Value: integer);
    procedure SetActualScript(const Value: string);
    procedure SetActualVisible(const Value: boolean);
    procedure SetFindActionName(const Value: string);
    procedure SetImageHeight(const Value: integer);
    procedure SetImageWidth(const Value: integer);
    procedure SetNewImage(const Value: string);
    procedure SetNewScriptAction(const Value: string);
    procedure SetNewToolAction(const Value: string);
    procedure SetShowCaptions(const Value: boolean);
    function GetFlat: boolean;
    procedure SetFlat(const Value: boolean);
    function GetButtonHeight: integer;
    function GetButtonWidth: integer;
    procedure SetButtonHeight(const Value: integer);
    procedure SetButtonWidth(const Value: integer);
    function GetShowCaprions: boolean;
    function GetActualTool: TTool;
    procedure SetActualTool(const Value: TTool);
public
   procedure CreateForm; override;
   procedure CreateSelfByModel; override;
   function  CreateToolByModel(aModel: TToolModel): TTool; override;
   procedure ActionExecute(Sender: TObject);
   procedure ActionPopup(Sender: TObject; MousePos: TPoint; var Handled: Boolean);
   property Form: TTask_Form read GetForm;
published
   property   Flat: boolean read GetFlat write SetFlat;
   property   ButtonHeight: integer read GetButtonHeight write SetButtonHeight;
   property   ButtonWidth: integer read GetButtonWidth write SetButtonWidth;
   property   ImageHeight: integer read GetImageHeight write SetImageHeight;
   property   ImageWidth: integer read GetImageWidth write SetImageWidth;
   // For manual setting only:
   property   NewImage: string write SetNewImage stored false;
   property   NewScriptAction: string write SetNewScriptAction stored false;
   property   NewToolAction: string write SetNewToolAction stored false;
   property   FindActionName: string write SetFindActionName stored false;
   property   ActualVisible: boolean read GetActualVisible write SetActualVisible stored false;
   property   ActualScript: string read GetActualScript write SetActualScript stored false;
   property   ActualTool: TTool read GetActualTool write SetActualTool stored false;
   property   ActualImageIndex: integer read GetActualImageIndex write SetActualImageIndex stored false;
   property   ActualCaption: string read GetActualCaption write SetActualCaption stored false;
   property   ActualHint: string read GetActualHint write setActualHint stored false;
   property   ShowCaptions: boolean read GetShowCaprions write SetShowCaptions;
end;

implementation
uses
   {!!!}Dialogs,
   ComCtrls,
   Forms,
   Graphics,
   SysUtils,
   e_Class,
   e_Type,
   m_Orbis,
   t_InterfaceTool,
   u_String;

{ TTaskForm }

procedure TTaskForm.ActionExecute(Sender: TObject);
var
   T: TTool;
begin
   if Sender = FActualAction then
   begin
      FActualAction.Checked := false;
      FActualAction.Checked := true;
      exit;
   end;
{
   if TToolAction(Sender).Locked then
   begin
      ShowMessage('Locked :-P');
      ecxit;
   end;
}
   T := TToolAction(Sender).Tool;
   if FActualAction <> nil then
      FActualAction.Checked := false;
   if ActiveTool <> nil then
   begin
      //ShowMessage('Before Destroying');
      if ActiveTool.DynamicTool then ActiveTool.DestroyDynamicPart;
      //ShowMessage('After Destroying');
      if ActiveTool is TInterfaceTool then TInterfaceTool(ActiveTool).on_Deactivate;
      ActiveTool := nil;
      //ShowMessage('After nil');
   end;
   if T <> nil then
   begin
      if T.DynamicTool then T.CreateDynamicPart;
      ActiveTool := T;
      if T is TInterfaceTool then TInterfaceTool(T).on_Activate;
   end;
   FActualAction := TAction(Sender);
   FActualAction.Checked := true;
   //ShowMessage('Hop do GW');
   PostMsgToOwners(tm_ExternalTaskSelected,TToolAction(Sender).Tool,false);
   //PostTextMsg(tm_AddToProtocol,'SelectedTask: '+TToolAction(Sender).Name,nil);
end;

procedure TTaskForm.ActionPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
//   TToolAction(TToolButton(Sender).Action).Locked := not TToolAction(TToolButton(Sender).Action).Locked;
end;

procedure TTaskForm.CreateForm;
begin
   aForm := TTask_Form.Create(Application);
end;


procedure TTaskForm.CreateSelfByModel;
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
begin
   M := SelfModel;
   if M = nil then exit;
   inherited;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
   FActualAction := nil;
end;

function TTaskForm.CreateToolByModel(aModel: TToolModel): TTool;
begin
   result := nil;
   if aModel = nil then exit;
   result := inherited CreateToolByModel(aModel);
   if result = nil then exit;
   NewToolAction := result.Name;
   ActualTool := result;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
end;


function TTaskForm.GetActualCaption: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Name;
end;

function TTaskForm.GetActualHint: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Hint;
end;

function TTaskForm.GetActualImageIndex: integer;
begin
   result := -1;
   if FactualAction <> nil then result := FActualAction.ImageIndex;
end;

function TTaskForm.GetActualScript: string;
begin
   result := '';
   if FactualAction = nil then exit;
   if not (FActualAction is TScriptAction) then exit;
   result := TScriptAction(FActualAction).Script;
end;

function TTaskForm.GetActualTool: TTool;
begin
   result := nil;
   if FactualAction = nil then exit;
   if not (FActualAction is TToolAction) then exit;
   result := TToolAction(FActualAction).Tool;
end;

function TTaskForm.GetActualVisible: boolean;
begin
   result := false;
   if FactualAction <> nil then result := FActualAction.Visible;
end;

function TTaskForm.GetButtonHeight: integer;
begin
   result := 0;
   if Form <> nil then result := Form.ToolBar.ButtonHeight;
end;

function TTaskForm.GetButtonWidth: integer;
begin
   result := 0;
   if Form <> nil then result := Form.ToolBar.ButtonWidth;
end;

function TTaskForm.GetFlat: boolean;
begin
   result := false;
   if Form <> nil then result := Form.ToolBar.Flat;
end;

function TTaskForm.GetForm: TTask_Form;
begin
   result := TTask_Form(aForm);
end;



function TTaskForm.GetImageHeight: integer;
begin
   result := 0;
   if Form <> nil then result := Form.ImageList.Height;
end;

function TTaskForm.GetImageWidth: integer;
begin
   result := 0;
   if Form <> nil then result := Form.ImageList.Width;
end;

function TTaskForm.GetShowCaprions: boolean;
begin
   result := false;
   if Form <> nil then result := Form.ToolBar.ShowCaptions;
end;

procedure TTaskForm.SetActualCaption(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Caption := Value;
end;

procedure TTaskForm.setActualHint(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Hint := Value;
end;

procedure TTaskForm.SetActualImageIndex(const Value: integer);
begin
   if FactualAction <> nil then FActualAction.ImageIndex := Value;
end;

procedure TTaskForm.SetActualScript(const Value: string);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TScriptAction) then exit;
   TScriptAction(FActualAction).Script := Value;
end;

procedure TTaskForm.SetActualTool(const Value: TTool);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TToolAction) then exit;
   TToolAction(FActualAction).Tool := Value;
   if Value <> nil then FActualAction.Name := 'Action_'+Value.Name;
end;

procedure TTaskForm.SetActualVisible(const Value: boolean);
begin
   if FactualAction <> nil then FActualAction.Visible := Value;
end;

procedure TTaskForm.SetButtonHeight(const Value: integer);
begin
   if Form <> nil then Form.ToolBar.ButtonHeight := Value;
end;

procedure TTaskForm.SetButtonWidth(const Value: integer);
begin
   if Form <> nil then Form.ToolBar.ButtonWidth := Value;
end;

procedure TTaskForm.SetFindActionName(const Value: string);
var
   i: integer;
   A: TAction;
begin
   FActualAction := nil;
   for i := 0 to Form.ActionList.ActionCount - 1 do
   begin
      A := TAction(Form.ActionList.Actions[i]);
      if CompareText(Value, A.Name) = 0 then
      begin
         FActualAction := A;
         break;
      end;
   end;
end;

procedure TTaskForm.SetFlat(const Value: boolean);
begin
   if Form <> nil then Form.ToolBar.Flat := Value;
end;

procedure TTaskForm.SetImageHeight(const Value: integer);
begin
   if Form = nil then exit;
   Form.ImageList.Height := Value;
end;

procedure TTaskForm.SetImageWidth(const Value: integer);
begin
   if Form = nil then exit;
   Form.ImageList.Width := Value;
end;

procedure TTaskForm.SetNewImage(const Value: string);
var
   BM: Graphics.TBitmap;
   S: string;
   BColor: TColor;
begin
   BM := Graphics.TBitmap.Create;
   S := '';
   S := GetAbsolutePath(Value);
   try
      BM.LoadFromFile(S);
      BColor := BM.Canvas.Pixels[0,0];
   except
      BColor := clGray;
   end;
   Form.ImageList.AddMasked(BM,BColor);
end;

procedure TTaskForm.SetNewScriptAction(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   A := TScriptAction.Create(Form);
   A.ActionList := Form.ActionList;
   FActualAction := A;
   B := TToolButton.Create(Form);
   Form.ToolBar.InsertControl(B);
   B.Action := A;
   A.DisableIfNoHandler := false;
end;

procedure TTaskForm.SetNewToolAction(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   A := TToolAction.Create(Form);
   A.ActionList := Form.ActionList;
   FActualAction := A;
   A.OnExecute := ActionExecute;
   B := TToolButton.Create(Form);
   Form.ToolBar.InsertControl(B);
   B.Action := A;
   B.Style := tbsCheck;
   B.OnContextPopup := ActionPopup;
   A.DisableIfNoHandler := false;
end;

procedure TTaskForm.SetShowCaptions(const Value: boolean);
begin
   if Form <> nil then Form.ToolBar.ShowCaptions := Value;
end;

initialization
   RegisterOrbisClass(TTaskForm);
end.
