unit e_Orbis01Projects;

interface
uses
   Elements,
   g_Elems,
   e_Raster,
   e_LHCProject;

type

//   UserLayer Restriction
//      0 - Full Access
//      1 - Read Only
//      2 - Forbidden

TOrbis01UserLayerCounts = packed record
   ARE   : integer;
   LIN   : integer;
   LIN_K : integer;
   PNT   : integer;
   TXT   : integer;
   NOTE  : integer;  // NOTE je vlastne NOT ale aby se to nezblaznilo zejo....
end;

TOrbis01UserLayerInfo = class(TStringEntity)
  private
    FModified: boolean;
    FShared: boolean;
    FRestriction: integer;
    procedure SetModified(const Value: boolean);
    procedure SetShared(const Value: boolean);
    procedure SetRestriction(const Value: integer);
published
   property Modified: boolean read FModified write SetModified;
   property Shared: boolean read FShared write SetShared;
   property Restriction: integer read FRestriction write SetRestriction;
end;

TOrbis01Info = class(TOwnerEntity)
  private
    FSourcePath: string;
    FSourceTyp: string;
    FProjectName: string;
    procedure SetProjectName(const Value: string);
    procedure SetSourcePath(const Value: string);
    procedure SetSourceTyp(const Value: string);
published
   property SourceTyp: string read FSourceTyp write SetSourceTyp;
   property SourcePath: string read FSourcePath write SetSourcePath;
   property ProjectName: string read FProjectName write SetProjectName;
end;

TOrbis01LHCInfo = class(TOrbis01Info)
private
    FLHCNumber: integer;
    FContinual: boolean;
    FLHCYear: integer;
    FUserDataSplitted: boolean;
    FDataProfileName: string;
    FUserDataUUIDExists: boolean;
    procedure SetLHCNumber(const Value: integer);
    procedure SetContinual(const Value: boolean);
    procedure SetLHCYear(const Value: integer);
    procedure SetUserDataSplitted(const Value: boolean);
    procedure SetDataProfileName(const Value: string);
    procedure SetUserDataUUIDExists(const Value: boolean);
published
   property LHCNumber: integer read FLHCNumber write SetLHCNumber;
   property LHCYear: integer read FLHCYear write SetLHCYear;
   property Continual: boolean read FContinual write SetContinual;
   property UserDataSplitted: boolean read FUserDataSplitted write SetUserDataSplitted;
   property UserDataUUIDExists: boolean read FUserDataUUIDExists write SetUserDataUUIDExists;
   property DataProfileName: string read FDataProfileName write SetDataProfileName;
end;

TOrbis01KatastrInfo = class(TOrbis01Info)
  private
    FKatastrNumber: integer;
    procedure SetKatastrNumber(const Value: integer);
published
   property KatastrNumber: integer read FKatastrNumber write SetKatastrNumber;
end;

TOrbis01LHCEntity = class(TOwnerEntity)
public
   function GetLHCInfo(aLHC,aRok: integer): TOrbis01LHCInfo; overload;
   function GetLHCInfo(aLHC,aRok: integer; aStrict: boolean): TOrbis01LHCInfo; overload;
end;

TOrbis01KatastrEntity = class(TOwnerEntity)
public
   function GetKatastrInfo(aKatastr: integer): TOrbis01KatastrInfo;
end;

TOrbis01Projects = class(TOwnerEntity)
  private
    function GetKatastrs: TOrbis01KatastrEntity;
    function GetLHCs: TOrbis01LHCEntity;
public
   function GetLHCInfo(aLHC, aRok: integer): TOrbis01LHCInfo; overload;
   function GetLHCInfo(aLHC, aRok: integer; aStrict: boolean): TOrbis01LHCInfo; overload;
   function GetKatastrInfo(aKatastr: integer): TOrbis01KatastrInfo;
   function NewLHCInfo: TOrbis01LHCInfo;
   function NewKatastrInfo: TOrbis01KatastrInfo;
   procedure AddProject(aProject: TOrbisLHCProject;var OldSameProjectName: string; UsrDataSplitted: boolean; ProfileName: string);
   procedure AddKatastr(const SourcePath,ORBFile,ProjectDirectory: string);
published
   property LHCs: TOrbis01LHCEntity read GetLHCs stored false;
   property Katastrs: TOrbis01KatastrEntity read GetKatastrs stored false;
end;

TOrbis01RasterEntity = class(TOwnerEntity)
  private
    function GetRC(aIndex: integer): TRasterCollection;
    procedure SetRC(aIndex: integer; const Value: TRasterCollection);
public
   property RC[aIndex: integer]: TRasterCollection read GetRC write SetRC;
end;

TOrbis01RasterInfo = class(TOwnerEntity)
  private
    FVisible: boolean;
    FLHC: integer;
    function GetR1: TOrbis01RasterEntity;
    function GetR2: TOrbis01RasterEntity;
    procedure SetLHC(const Value: integer);
    procedure SetVisible(const Value: boolean);
published
   property LHC: integer read FLHC write SetLHC;
   property Visible: boolean read FVisible write SetVisible;
   property R1: TOrbis01RasterEntity read GetR1 stored false;
   property R2: TOrbis01RasterEntity read GetR2 stored false;
end;

TOrbis01LayerInfo = class(TOwnerEntity)
private
    FVisible: boolean;
    FLHC: integer;
    FDirectory: string;
    FDataExists: boolean;
    procedure SetLHC(const Value: integer);
    procedure SetVisible(const Value: boolean);
    procedure SetDirectory(const Value: string);
    procedure SetDataExists(const Value: boolean);
published
   property LHC: integer read FLHC write SetLHC;
   property Visible: boolean read FVisible write SetVisible;
   property Directory: string read FDirectory write SetDirectory;
   property DataExists: boolean read FDataExists write SetDataExists;
end;


TOrbis01KRElement = class(TRectEntity)
  private
    FNazev: string;
    function GetEmpty: boolean;
    procedure SetEmpty(const Value: boolean);
    procedure SetNazev(const Value: string);
published
   property Nazev: string read FNazev write SetNazev;
   property Empty: boolean read GetEmpty write SetEmpty stored false;
end;

TOrbis01KatastrRepository = class(TRectEntity)
  private
    FVisible: boolean;
    procedure SetVisible(const Value: boolean);
published
   property Visible: boolean read FVisible write SetVisible;
end;

implementation
uses
   SysUtils,
   e_Class,
   e_LHCModel,
   e_KatastrModel,
   u_String,
   Worlds;

{ TOrbis01LHCInfo }

procedure TOrbis01LHCInfo.SetContinual(const Value: boolean);
begin
   FContinual := Value;
end;

procedure TOrbis01LHCInfo.SetDataProfileName(const Value: string);
begin
  FDataProfileName := Value;
end;

procedure TOrbis01LHCInfo.SetLHCNumber(const Value: integer);
begin
   FLHCNumber := Value;
   if FLHCYear > 0 then
      Name := IntToStr(Value) + '_' + IntToStr(LHCYear)
   else
      Name := IntToStr(Value);
end;


procedure TOrbis01LHCInfo.SetLHCYear(const Value: integer);
begin
  FLHCYear := Value;
   if Value > 0 then
      Name := IntToStr(FLHCNumber) + '_' + IntToStr(LHCYear)
   else
      Name := IntToStr(FLHCNumber);
end;

procedure TOrbis01LHCInfo.SetUserDataSplitted(const Value: boolean);
begin
   FUserDataSplitted := Value;
end;

procedure TOrbis01LHCInfo.SetUserDataUUIDExists(const Value: boolean);
begin
  FUserDataUUIDExists := Value;
end;

{ TOrbis01Projects }

procedure TOrbis01Projects.AddKatastr(const SourcePath,ORBFile,ProjectDirectory: string);
var
   INF,INF1: TOrbis01KatastrInfo;
   e: TElement;
   en: TEntity;
   K: Tgis_Katastr;
   i: integer;
   FS: string;
begin
   e := World.ObjectModel.Element['DATA'];
   e := TEntity(e).Element['PARCELY'];
   en := TEntity(e);
   for i := 0 to en.LastIndex do
   begin
      K := Tgis_Katastr(en.Elements[i]);
      INF := GetKatastrInfo(K.Cislo);
      if INF = nil then
         INF := NewKatastrInfo
      else
      begin
         if not SameText(OrbFile,Inf.ProjectName) then
         begin
            FS := ProjectDirectory;
            CHeckBackSlash(FS);
            FS := FS + INF.ProjectName;
            FS := GetAbsolutePath(FS);
            if FIleExists(FS) then
               DeleteFile(FS);
         end;
      end;
      INF.KatastrNumber := K.Cislo;
      INF.SourceTyp := 'SHP';  // asi zatim
      INF.SourcePath := SourcePath;
      INF.ProjectName := ORBFile;
   end;
end;

procedure TOrbis01Projects.AddProject(aProject: TOrbisLHCProject;var OldSameProjectName: string; UsrDataSplitted: boolean; ProfileName: string);
var
   INF,INF1: TOrbis01LHCInfo;
   e: TElement;
   L: Tgis_LHC;
begin
   OldSameProjectName := '';
   INF := NewLHCInfo;
   e := World.ObjectModel.Element['DATA'];
   e := TEntity(e).Element['LHC'];
   L := Tgis_LHC(TEntity(e).FindElementClass(Tgis_LHC));
   INF.LHCNumber := L.Cislo;
   aProject.Name := 'LHP_'+L.Name+'_'+L.Rok.ToString;
   aProject.Year := L.ROK;
   INF.SourceTyp := aProject.Comment;
   INF.SourcePath := aProject.ExternalDataDirectory;
   INF.ProjectName := aProject.Name;
   INF.LHCYear := aProject.Year;
   INF.UserDataSplitted := UsrDataSplitted;
   INF.UserDataUUIDExists := true;
   INF.DataProfileName := ProfileName;
   INF1 := GetLHCInfo(L.Cislo,L.ROK);
   if INF1 <> INF then
   begin
      //OldSameProjectName := INF1.ProjectName;
      INF1.Free;
   end;
end;

function TOrbis01Projects.GetKatastrInfo(aKatastr: integer): TOrbis01KatastrInfo;
begin
   result := Katastrs.GetKatastrInfo(aKatastr);
end;

function TOrbis01Projects.GetKatastrs: TOrbis01KatastrEntity;
begin
   result := TOrbis01KatastrEntity(FindElementClass(TOrbis01KatastrEntity));
   if result = nil then
   begin
      result := TOrbis01KatastrEntity.Create;
      AddElement(result);
   end;
end;

function TOrbis01Projects.GetLHCInfo(aLHC,aRok: integer): TOrbis01LHCInfo;
begin
   result := LHCs.GetLHCInfo(aLHC,aRok);
end;

function TOrbis01Projects.GetLHCInfo(aLHC, aRok: integer; aStrict: boolean): TOrbis01LHCInfo;
begin
   result := LHCs.GetLHCInfo(aLHC,aRok,aStrict);
end;

function TOrbis01Projects.GetLHCs: TOrbis01LHCEntity;
begin
   result := TOrbis01LHCEntity(FindElementClass(TOrbis01LHCEntity));
   if result = nil then
   begin
      result := TOrbis01LHCEntity.Create;
      AddElement(result);
   end;
end;

function TOrbis01Projects.NewKatastrInfo: TOrbis01KatastrInfo;
begin
   result := TOrbis01KatastrInfo.Create;
   Katastrs.AddElement(result);
end;

function TOrbis01Projects.NewLHCInfo: TOrbis01LHCInfo;
begin
   result := TOrbis01LHCInfo.Create;
   LHCs.AddElement(result);
end;

{ TOrbis01LHCEntity }

function TOrbis01LHCEntity.GetLHCInfo(aLHC,aRok: integer; aStrict: boolean): TOrbis01LHCInfo;
var
   i: integer;
   e: TElement;
   LI: TOrbis01LHCInfo;
begin
   result := nil;
   if NoElements then exit;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TOrbis01LHCInfo) then continue;
      LI := TOrbis01LHCInfo(e);
      if LI.LHCNumber = aLHC then
      begin
         if aStrict then
         begin
            if LI.LHCYear = aRok then
            begin
               result := LI;
               exit;
            end
         end
         else
         begin
            if aRok = 0  then
            begin
               result := LI;
               exit;
            end
            else
            if LI.LHCYear = aRok then
            begin
               result := LI;
               exit;
            end
            else
            if LI.LHCYear = 0 then
               result := LI;
         end;

      end;
   end;
end;

function TOrbis01LHCEntity.GetLHCInfo(aLHC, aRok: integer): TOrbis01LHCInfo;
begin
   result := GetLHCInfo(aLHC,aRok,false);
end;

{ TOrbis01Info }

procedure TOrbis01Info.SetProjectName(const Value: string);
begin
  FProjectName := Value;
end;

procedure TOrbis01Info.SetSourcePath(const Value: string);
begin
  FSourcePath := Value;
end;

procedure TOrbis01Info.SetSourceTyp(const Value: string);
begin
  FSourceTyp := Value;
end;

{ TOrbis01KatastrInfo }

procedure TOrbis01KatastrInfo.SetKatastrNumber(const Value: integer);
begin
   FKatastrNumber := Value;
   Name := IntToStr(Value);
end;

{ TOrbis01KatastrEntity }

function TOrbis01KatastrEntity.GetKatastrInfo(aKatastr: integer): TOrbis01KatastrInfo;
var
   i: integer;
   e: TElement;
   LK: TOrbis01KatastrInfo;
begin
   result := nil;
   if NoElements then exit;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TOrbis01KatastrInfo) then continue;
      LK := TOrbis01KatastrInfo(e);
      if LK.KatastrNumber = aKatastr then
      begin
         result := LK;
         exit;
      end;
   end;
end;

{ TOrbis01RasterEntity }

function TOrbis01RasterEntity.GetRC(aIndex: integer): TRasterCollection;
begin
   result := nil;
   if aIndex > LastIndex then exit;
   result := TRasterCollection(Elements[aIndex]);
end;

procedure TOrbis01RasterEntity.SetRC(aIndex: integer; const Value: TRasterCollection);
var
   aRE: TOrbis01RasterEntity;
begin
   if Value = nil then exit;
   aRE := TOrbis01RasterEntity(Value.FindOwnerClass(TOrbis01RasterEntity));
   if aRE <> nil then
      aRE.RemoveElement(Value);
   if (aIndex > LastIndex) or (aIndex < 0) then
      AddElement(Value)
   else
      InsertElement(aIndex,Value);
end;

{ TOrbis01RasterInfo }

function TOrbis01RasterInfo.GetR1: TOrbis01RasterEntity;
begin
   result := TOrbis01RasterEntity(FindElementName('R1'));
   if result = nil then
   begin
      result := TOrbis01RasterEntity.Create;
      AddElement(result);
      result.Name := 'R1';
   end;
end;

function TOrbis01RasterInfo.GetR2: TOrbis01RasterEntity;
begin
   result := TOrbis01RasterEntity(FindElementName('R2'));
   if result = nil then
   begin
      result := TOrbis01RasterEntity.Create;
      AddElement(result);
      result.Name := 'R2';
   end;
end;

procedure TOrbis01RasterInfo.SetLHC(const Value: integer);
begin
  FLHC := Value;
end;

procedure TOrbis01RasterInfo.SetVisible(const Value: boolean);
begin
  FVisible := Value;
end;

{ TOrbis01KRElement }

function TOrbis01KRElement.GetEmpty: boolean;
begin
   result := NoElements;
end;

procedure TOrbis01KRElement.SetEmpty(const Value: boolean);
var
   En: TEntity;
   e: TElement;
begin
   e := FindElementClass(Tgis_ParcelModel);
   if e <> nil then
      TEntity(e).HardFreeAll;
   HardFreeAll;
end;

procedure TOrbis01KRElement.SetNazev(const Value: string);
begin
  FNazev := Value;
end;

{ TOrbis01KatastrRepository }

procedure TOrbis01KatastrRepository.SetVisible(const Value: boolean);
begin
  FVisible := Value;
end;

{ TOrbis01LayerInfo }

procedure TOrbis01LayerInfo.SetDataExists(const Value: boolean);
begin
  FDataExists := Value;
end;

procedure TOrbis01LayerInfo.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TOrbis01LayerInfo.SetLHC(const Value: integer);
begin
   FLHC := Value;
end;

procedure TOrbis01LayerInfo.SetVisible(const Value: boolean);
begin
   FVisible := Value;
end;

{ TOrbis01UserLayerInfo }

procedure TOrbis01UserLayerInfo.SetModified(const Value: boolean);
begin
  FModified := Value;
end;

procedure TOrbis01UserLayerInfo.SetRestriction(const Value: integer);
begin
  FRestriction := Value;
end;

procedure TOrbis01UserLayerInfo.SetShared(const Value: boolean);
begin
  FShared := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TOrbis01UserLayerInfo);
   RegisterOrbisClass(TOrbis01LHCInfo);
   RegisterOrbisClass(TOrbis01KatastrInfo);
   RegisterOrbisClass(TOrbis01LHCEntity);
   RegisterOrbisClass(TOrbis01KatastrEntity);
   RegisterOrbisClass(TOrbis01Projects);
   RegisterOrbisClass(TOrbis01RasterEntity);
   RegisterOrbisClass(TOrbis01RasterInfo);
   RegisterOrbisClass(TOrbis01KatastrRepository);
   RegisterOrbisClass(TOrbis01KRElement);
   RegisterOrbisClass(TOrbis01LayerInfo);
END.
