unit U_LanguagesBDE;


interface

uses BDE, Classes, Elements;



Type

  TLdSet = set of (ldTdBase, ldTPdox, ldTInformix, ldTIntrBase, ldTMsAccess,
                   ldTMsSQL, ldTOracle, ldTSybase, ldtDB2);


   TLangDriver = class (TOwnerElement)
{
Long name	Internal name	Character set	        Collation sequence
'ascii' ANSI	DBWINUS0	Windows CODE PAGE 1252	Binary
'Spanish' ANSI	DBWINES0	Windows CODE PAGE 1252	Spanish
'WEurope' ANSI	DBWINWE0	Windows CODE PAGE 1252	Multilingual Western Europe
dBASE CHS cp936	DB936CN0	DOS CODE PAGE 936	dBASE Chinese936
dBASE CSY cp852	DB852CZ0	DOS CODE PAGE 852	dBASE Czech852
}
   private
      FInternalName: String;
      FCharacterSet: String;
      FCollationSequence: String;
      FLdType: TldSet;

      function SetInternaldBaseName(aVal: String): Boolean;
      function SetInternalPdoxName(aVal: String): Boolean;
      function SetdBaseName(aName: String): Boolean;
      function SetPdoxName(aName: String): Boolean;

      procedure SetInternalName(aVal: String);

   protected
      procedure SetName(aName: String); override;
   public
      constructor Create; override;
      destructor  Destroy;  override;
   published
      property InternalName: String read FInternalName write SetInternalName;
      property CharacterSet: String read FCharacterSet;
      property CollationSequence: String read FCollationSequence;
      property LdType: TldSet read FLdType;
   end;



   TAllLangDrivers = class (TOwnerEntity)
   private
      FActualDriver: TLangDriver;
      FNewDriverName: String;

      function GetActualDriver: TLangDriver;
      procedure SetNewDriverName(aValue: string);
   public
      constructor Create; override;
      constructor CreatedBase;
      constructor CreatePdox;

      destructor  Destroy; override;

      procedure CreateAllDrivers; virtual;
      procedure CreatedBaseDrivers;
      procedure CreatePdoxDrivers;
   published
      property ActualDriver: TLangDriver read GetActualDriver;
      property NewDriverName: string write SetNewDriverName;
   end;


{
   TLangDriversdBase = class (TOwnerEntity)
   private
      FActualDriver: TLangDriver;
      FNewDriverName: String;

      function GetActualDriver: TLangDriver;
      procedure SetNewDriverName(aValue: string);
   public
      constructor Create; override;
      destructor  Destroy; override;
      procedure   CreateAllDrivers; virtual;
   published
      property ActualDriver: TLangDriver read GetActualDriver;
      property NewDriverName: string write SetNewDriverName;
   end;
 }


   TLangDrivers = class (TOwnerEntity)
   private
       FAllLangDrivers: TAllLangDrivers;
       FdBaseLangDrivers: TAllLangDrivers;
       FPdoxLangDrivers: TAllLangDrivers;

       function GetAllLangDrivers: TAllLangDrivers;
       function GetdBaseLangDrivers: TAllLangDrivers;
       function GetPdoxLangDrivers: TAllLangDrivers;
   protected
   public
      constructor Create; override;
      constructor CreateAll;
      constructor CreatedBase;
      constructor CreatePdox;
      destructor  Destroy;  override;
   published
      property AllLangDrivers: TAllLangDrivers read GetAllLangDrivers;
      property dBaseLangDrivers: TAllLangDrivers read GetdBaseLangDrivers;
      property PdoxLangDrivers: TAllLangDrivers read GetPdoxLangDrivers;
   end;




implementation
// -----------------------
//  TLangDrivers
// -----------------------

constructor TLangDrivers.Create;
var
   D: TAllLangDrivers;
   S: string;
begin
   inherited create;

   Name := 'Jazykov ovladae';

   if FAllLangDrivers = nil then
   begin
      S := 'Vechny jazykov ovladae';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.Create;
         D.Name := S;
         AddElement(D);
      end;
      FAllLangDrivers := D;
   end;

end;



constructor TLangDrivers.CreatedBase;
var
   D: TAllLangDrivers;
   S: string;
begin
   inherited create;

   Name := 'Jazykov ovladae';

   if FdBaseLangDrivers = nil then
   begin
      S := 'Jazykov ovladae dBASE';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.CreatedBase;
         D.Name := S;
         AddElement(D);
      end;
      FdBaseLangDrivers := D;
   end;

end;



constructor TLangDrivers.CreatePdox;
var
   D: TAllLangDrivers;
   S: string;
begin
   inherited create;

   Name := 'Jazykov ovladae';

   if FPdoxLangDrivers = nil then
   begin
      S := 'Jazykov ovladae Pdox';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.CreatePdox;
         D.Name := S;
         AddElement(D);
      end;
      FPdoxLangDrivers := D;
   end;

end;



constructor TLangDrivers.CreateAll;
var
   D, D1, D2: TAllLangDrivers;
   S: string;
begin
   inherited create;

   Name := 'Jazykov ovladae';

   if FAllLangDrivers = nil then
   begin
      S := 'Vechny jazykov ovladae';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.Create;
         D.Name := S;
         AddElement(D);
      end;
      FAllLangDrivers := D;
   end;

   if FdBaseLangDrivers = nil then
   begin
      S := 'Jazykov ovladae dBASE';
      D1 := TAllLangDrivers(FindElementName(S));

      if D1 = nil then
      begin
         D1 := TAllLangDrivers.CreatedBase;
         D1.Name := S;
         AddElement(D1);
      end;
      FdBaseLangDrivers := D1;
   end;

   if FPdoxLangDrivers = nil then
   begin
      S := 'Jazykov ovladae Paradox';
      D2 := TAllLangDrivers(FindElementName(S));

      if D2 = nil then
      begin
         D2 := TAllLangDrivers.CreatePdox;
         D2.Name := S;
         AddElement(D2);
      end;
      FPdoxLangDrivers := D2;
   end;
end;



destructor TLangDrivers.Destroy;
begin
   inherited destroy;
end;



function TLangDrivers.GetAllLangDrivers;
var
   D: TAllLangDrivers;
   S: string;
begin
   if FAllLangDrivers = nil then
   begin
      S := 'Vechny jazykov ovladae';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.Create;
         D.Name := S;
         AddElement(D);
      end;
      FAllLangDrivers := D;
   end;

   Result := FAllLangDrivers;
end;



function TLangDrivers.GetdBaseLangDrivers;
var
   D: TAllLangDrivers;
   S: string;
begin
   if FdBaseLangDrivers = nil then
   begin
      S := 'Jazykov ovladae dBASE';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.CreatedBase;
         D.Name := S;
         AddElement(D);
      end;
      FdBaseLangDrivers := D;
   end;

   Result := FdBaseLangDrivers;
end;



function TLangDrivers.GetPdoxLangDrivers;
var
   D: TAllLangDrivers;
   S: string;
begin
   if FPdoxLangDrivers = nil then
   begin
      S := 'Jazykov ovladae Paradox';
      D := TAllLangDrivers(FindElementName(S));

      if D = nil then
      begin
         D := TAllLangDrivers.CreatePdox;
         D.Name := S;
         AddElement(D);
      end;
      FPdoxLangDrivers := D;
   end;

   Result := FPdoxLangDrivers;
end;


// -----------------------
//  TLangDriver
// -----------------------

constructor TLangDriver.Create;
begin
   inherited Create;
end;



destructor  TLangDriver.Destroy;
begin
   inherited Destroy;
end;



procedure TLangDriver.SetName(aName: String);
begin
   inherited SetName(aName);

   if not SetdBaseName(aName) then
      if not SetPdoxName(aName) then ;

end;



function TLangDriver.SetdBaseName(aName: String): Boolean;
begin

   Result := True;
   FLdType := [ldTdBase];

   if aName = '''ascii'' ANSI' then
   begin
      FInternalName := 'DBWINUS0';
      FCharacterSet := 'Windows CODE PAGE 1252';
      FCollationSequence := 'Binary';
   end
   else if aName = '''Spanish'' ANSI' then
   begin
      FInternalName := 'DBWINES0';
      FCharacterSet := 'Windows CODE PAGE 1252';
      FCollationSequence := 'Spanish';
   end
   else if aName = '''WEurope'' ANSI' then
   begin
      // FLongName := aName;
      FInternalName := 'DBWINWE0';
      FCharacterSet := 'Windows CODE PAGE 1252';
      FCollationSequence := 'Multilingual Western Europe';
   end
   else if aName = 'dBASE CHS cp936' then
   begin
      // FLongName := aName;
      FInternalName := 'DB936CN0';
      FCharacterSet := 'DOS CODE PAGE 936';
      FCollationSequence := 'dBASE Chinese936';
   end
   else if aName = 'dBASE CSY cp852' then
   begin
      // FLongName := aName;
      FInternalName := 'DB852CZ0';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Czech852';
   end
   else if aName = 'dBASE CSY cp867' then
   begin
      // FLongName := aName;
      FInternalName := 'DB867CZ0';
      FCharacterSet := 'DOS CODE PAGE 867';
      FCollationSequence := 'dBASE Czech867';
   end
   else if aName = 'dBASE DAN cp865' then
   begin
      // FLongName := aName;
      FInternalName := 'DB865DA0';
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'dBASE Danish';
   end
   else if aName = 'dBASE DEU cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437DE0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE German';
   end
   else if aName = 'dBASE DEU cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850DE0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE German850';
   end
   else if aName = 'dBASE ELL GR437' then
   begin
      // FLongName := aName;
      FInternalName := 'db437gr0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Greek';
   end
   else if aName = 'dBASE ENG cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437UK0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE English/UK';
   end
   else if aName = 'dBASE ENG cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850UK0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE English850/UK';
   end
   else if aName = 'dBASE ENU cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437US0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE English/US';
   end
   else if aName = 'dBASE ENU cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850US0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE English/US';
   end
   else if aName = 'dBASE ESP cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437ES1';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Spanish';
   end
   else if aName = 'dBASE ESP cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850ES0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Spanish850';
   end
   else if aName = 'dBASE FIN cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437FI0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Finnish';
   end
   else if aName = 'dBASE FRA cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437FR0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE French';
   end
   else if aName = 'dBASE FRA cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850FR0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE French850';
   end
   else if aName = 'dBASE FRC cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850CF0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Canadian-French850';
   end
   else if aName = 'dBASE FRC cp863' then
   begin
      // FLongName := aName;
      FInternalName := 'DB863CF1';
      FCharacterSet := 'DOS CODE PAGE 863';
      FCollationSequence := 'dBASE Canadian-French863';
   end
   else if aName = 'dBASE HUN cp852' then
   begin
      // FLongName := aName;
      FInternalName := 'db852hdc';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Hungarian';
   end
   else if aName = 'dBASE ITA cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437IT0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Italian';
   end
   else if aName = 'dBASE ITA cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850IT1';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Italian850';
   end
   else if aName = 'dBASE JPN Dic932' then
   begin
      // FLongName := aName;
      FInternalName := 'DB932JP1';
      FCharacterSet := 'DOS CODE PAGE 932';
      FCollationSequence := 'dBASE Japan Dic932';
   end
   else if aName = 'dBASE JPN cp932' then
   begin
      // FLongName := aName;
      FInternalName := 'DB932JP0';
      FCharacterSet := 'DOS CODE PAGE 932';
      FCollationSequence := 'dBASE Japan cp932';
   end
   else if aName = 'dBASE KOR cp949' then
   begin
      // FLongName := aName;
      FInternalName := 'DB949KO0';
      FCharacterSet := 'DOS CODE PAGE 949';
      FCollationSequence := 'dBASE Korea949';
   end
   else if aName = 'dBASE NLD cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437NL0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Dutch';
   end
   else if aName = 'dBASE NLD cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850NL0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Dutch850';
   end
   else if aName = 'dBASE NOR cp865' then
   begin
      // FLongName := aName;
      FInternalName := 'DB865NO0';
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'dBASE Norwegian';
   end
   else if aName = 'dBASE PLK cp852' then
   begin
      // FLongName := aName;
      FInternalName := 'db852po0';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Polish852';
   end
   else if aName = 'dBASE PTB cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850PT0';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Brazilian Portuguese 850';
   end
   else if aName = 'dBASE PTG cp860' then
   begin
      // FLongName := aName;
      FInternalName := 'DB860PT0';
      FCharacterSet := 'DOS CODE PAGE 860';
      FCollationSequence := 'dBASE Brazilian Portuguese 860';
   end
   else if aName = 'dBASE RUS cp866' then
   begin
      // FLongName := aName;
      FInternalName := 'db866ru0';
      FCharacterSet := 'DOS CODE PAGE 866';
      FCollationSequence := 'dBASE Russian';
   end
   else if aName = 'dBASE SLO cp852' then
   begin
      // FLongName := aName;
      FInternalName := 'db852sl0';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Slovakia 852';
   end
   else if aName = 'dBASE SVE cp437' then
   begin
      // FLongName := aName;
      FInternalName := 'DB437SV0';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Swedish';
   end
   else if aName = 'dBASE SVE cp850' then
   begin
      // FLongName := aName;
      FInternalName := 'DB850SV1';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Swedish850';
   end
   else if aName = 'dBASE CHT cp950' then
   begin
      // FLongName := aName;
      FInternalName := 'DB950TW0';
      FCharacterSet := 'DOS CODE PAGE 950';
      FCollationSequence := 'dBASE Taiwan950';
   end
   else if aName = 'dBASE THA cp874' then
   begin
      // FLongName := aName;
      FInternalName := 'db874th0';
      FCharacterSet := 'DOS CODE PAGE 874';
      FCollationSequence := 'dBASE Thai';
   end
   else if aName = 'dBASE TRK cp857' then
   begin
      // FLongName := aName;
      FInternalName := 'DB857TR0';
      FCharacterSet := 'DOS CODE PAGE 857';
      FCollationSequence := 'dBASE Turkish';
   end
   else if aName = 'Hebrew dBASE' then
   begin
      // FLongName := aName;
      FInternalName := 'dbHebrew';
      FCharacterSet := '???';
      FCollationSequence := 'dBASE Hebrew';
   end
   else
   begin
      FLdType := [];
      Result := False;     {driver nebyl nalezen}
   end;
end;



function TLangDriver.SetPdoxName(aName: String): Boolean;
begin

   Result := True;
   FLdType := [ldTPdox];

   if aName = 'Paradox ''ascii''' then
   begin
      FInternalName := 'ascii';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Binary';
      FLdType := [ldTPdox, ldTIntrBase];
   end
   else if aName = 'Paradox ''hebrew''' then
   begin
      FInternalName := 'hebrew';
      FCharacterSet := 'DOS CODE PAGE 862';
      FCollationSequence := 'Binary';
   end
   else if aName = 'Paradox ''intl''' then
   begin
      FInternalName := 'intl';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Multilingual Western Europe';
   end
   else if aName = 'Paradox ''intl'' 850' then
   begin
      FInternalName := 'intl850';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'Brazilian Portuguese, French Canadian (2-level)';
   end
   else if aName = 'Paradox ''japan''' then
   begin
      FInternalName := 'japan';
      FCharacterSet := 'CODE PAGE 932';
      FCollationSequence := 'Japanese (4-level)';
   end
   else if aName = 'Paradox ''nordan''' then
   begin
      FInternalName := 'nordan';
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'Norwegian/Danish (Paradox 3.5, 2-level)';
   end
   else if aName = 'Paradox ''nordan40''' then
   begin
      FInternalName := 'nordan40';
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'Norwegian/Danish (Paradox 4.0, 5.0, 5.5, 7.0, 2-level)';
   end
   else if aName = 'Paradox ''swedfin''' then
   begin
      FInternalName := 'swedfin';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Swedish/Finnish (2-level)';
   end
   else if aName = 'Paradox ''turk''' then
   begin
      FInternalName := 'turk';
      FCharacterSet := 'CODE PAGE 857';
      FCollationSequence := 'Turkish (2-level)';
   end
   else if aName = 'Paradox ANSI HEBREW' then
   begin
      FInternalName := 'ANHEBREW';
      FCharacterSet := '1255(ANSI)';
      FCollationSequence := 'Binary';
   end
   else if aName = 'Paradox China 936' then
   begin
      FInternalName := 'china';
      FCharacterSet := 'CODE PAGE 936';
      FCollationSequence := 'China (1-level)';
   end
   else if aName = 'Paradox Cyrr 866' then
   begin
      FInternalName := 'cyrr';
      FCharacterSet := 'DOS CODE PAGE 866';
      FCollationSequence := 'Cyrillic';
   end
   else if aName = 'Paradox Czech 852' then
   begin
      FInternalName := 'czech';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Czech852 (2-level)';
   end
   else if aName = 'Paradox Czech 867' then
   begin
      FInternalName := 'cskamen';
      FCharacterSet := 'DOS CODE PAGE 867';
      FCollationSequence := 'Czech867 (2-level)';
   end
   else if aName = 'Paradox ESP 437' then
   begin
      FInternalName := 'SPANISH';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Spanish (3-level)';
   end
   else if aName = 'Paradox Greek GR437' then
   begin
      FInternalName := 'grcp437';
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Greek (2-level)';
   end
   else if aName = 'Paradox Hun 852 DC' then
   begin
      FInternalName := 'hun852dc';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Hungarian (2-level)';
   end
   else if aName = 'Paradox ISL 861' then
   begin
      FInternalName := 'iceland';
      FCharacterSet := 'DOS CODE PAGE 861';
      FCollationSequence := 'Icelandic (2-level)';
   end
   else if aName = 'Paradox Korea 949' then
   begin
      FInternalName := 'korea';
      FCharacterSet := 'CODE PAGE 949';
      FCollationSequence := 'Korea (1-level)';
   end
   else if aName = 'Paradox Polish 852' then
   begin
      FInternalName := 'polish';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Polish (2-level)';
   end
   else if aName = 'Paradox Slovene 852' then
   begin
      FInternalName := 'slovene';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Slovene (2-level)';
   end
   else if aName = 'Paradox Taiwan 950' then
   begin
      FInternalName := 'taiwan';
      FCharacterSet := 'DOS CODE PAGE 950';
      FCollationSequence := 'DOS CODE PAGE 950';
   end
   else if aName = 'Paradox Thai 874' then
   begin
      FInternalName := 'thai';
      FCharacterSet := 'DOS CODE PAGE 874';
      FCollationSequence := 'Thai (3-level)';
   end
   else if aName = 'Pdox ANSI Cyrillic' then
   begin
      FInternalName := 'ancyrr';
      FCharacterSet := '1251 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "cyrr" (2-level)';
   end
   else if aName = 'Pdox ANSI Czech' then
   begin
      FInternalName := 'anczech';
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "czech" (2-level)';
   end
   else if aName = 'Pdox ANSI Greek' then
   begin
      FInternalName := 'angreek1';
      FCharacterSet := '1253 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "greek" (2-level)';
   end
   else if aName = 'Pdox ANSI Hun. DC' then
   begin
      FInternalName := 'anhundc';
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "hung" (2-level)';
   end
   else if aName = 'Pdox ANSI Intl' then
   begin
      FInternalName := 'ANSIINTL';
      FCharacterSet := '1252 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "intl" (3-level)';
   end
   else if aName = 'Pdox ANSI Intl850' then
   begin
      FInternalName := 'ANSII850';
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'Compatible with Paradox "intl850" (3-level)';
   end
   else if aName = 'Pdox ANSI Nordan4' then
   begin
      FInternalName := 'ANSINOR4';
      FCharacterSet := '1252 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "nordan40" (2-level)';
   end
   else if aName = 'Pdox ANSI Polish' then
   begin
      FInternalName := 'anpolish';
      FCharacterSet := '1250(ANSI)';
      FCollationSequence := 'Compatible with Paradox "polish" (2-level)';
   end
   else if aName = 'Pdox ANSI Slovene' then
   begin
      FInternalName := 'ansislov';
      FCharacterSet := '1250(ANSI)';
      FCollationSequence := 'Compatible with Paradox "slovene" (2-level)';
   end
   else if aName = 'Pdox ANSI Spanish' then
   begin
      FInternalName := 'ANSISPAN';
      FCharacterSet := '1252(ANSI)';
      FCollationSequence := 'Compatible with Paradox "SPANISH" (3-level)';
   end
   else if aName = 'Pdox ANSI Swedfin' then
   begin
      FInternalName := 'ANSISWFN';
      FCharacterSet := '1252(ANSI)';
      FCollationSequence := 'Compatible with Paradox "swedfin" (3-level)';
   end
   else if aName = 'Pdox ANSI Turkish' then
   begin
      FInternalName := 'ANTURK';
      FCharacterSet := '1254(ANSI)';
      FCollationSequence := 'Compatible with Paradox "turk" (2-level)';
   end
   else if aName = 'pdx ANSI Czech ''CH''' then
   begin
      FInternalName := 'anczechw';
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := '(2-level)';
   end
   else if aName = 'pdx Czech 852 ''CH''' then
   begin
      FInternalName := 'czechw';
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := '(2-level)';
   end
   else if aName = 'pdx Czech 867 ''CH''' then
   begin
      FInternalName := 'cskamenw';
      FCharacterSet := 'DOS CODE PAGE 867';
      FCollationSequence := '(2-level)';
   end
   else if aName = 'pdx ANSI ISO L_2 CZ' then
   begin
      FInternalName := 'anil2czw';
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := 'Paradox Czech, (2-level)';
   end
   else if aName = 'pdx ISO L_2 Czech' then
   begin
      FInternalName := 'il2czw';
      FCharacterSet := 'ISO8859-2';
      FCollationSequence := 'Paradox Czech, ISO8859-2 (2-level)';
   end
   else if aName = 'Pdox ANSI Bulgarian' then
   begin
      FInternalName := 'BGPD1251';
      FCharacterSet := '1251 (ANSI)';
      FCollationSequence := 'Bulgarian (2-level)';
   end
   else if aName = 'Paradox Bulgaria 868' then
   begin
      FInternalName := 'Bulgaria';
      FCharacterSet := 'DOS CODE PAGE 868';
      FCollationSequence := 'Bulgaria (2-level)';
   end
   else
   begin
      FLdType := [];
      Result := False;     {driver nebyl nalezen}
   end;
end;



procedure TLangDriver.SetInternalName(aVal: String);
begin

   FInternalName := aVal;

   if not SetInternaldBaseName(aVal) then
      if not SetInternalPdoxName(aVal) then ;

end;



function TLangDriver.SetInternaldBaseName(aVal: String): Boolean;
begin

   Result := True;
   FLdType := [ldTdBase];

   if aVal = 'DBWINUS0' then
   begin
      inherited SetName('''ascii'' ANSI');
      FCharacterSet := 'Windows CODE PAGE 1252';
      FCollationSequence := 'Binary';
   end
   else if aVal = 'DBWINES0' then
   begin
      inherited SetName('''Spanish'' ANSI');
      FCharacterSet := 'Windows CODE PAGE 1252';
      FCollationSequence := 'Spanish';
   end
   else if aVal = 'DBWINWE0' then
   begin
      inherited SetName('''WEurope'' ANSI');
      FCharacterSet := 'Windows CODE PAGE 1252';
      FCollationSequence := 'Multilingual Western Europe';
   end
   else if aVal = 'DB936CN0' then
   begin
      inherited SetName('dBASE CHS cp936');
      FCharacterSet := 'DOS CODE PAGE 936';
      FCollationSequence := 'dBASE Chinese936';
   end
   else if aVal = 'DB852CZ0' then
   begin
      inherited SetName('dBASE CSY cp852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Czech852';
   end
   else if aVal = 'DB867CZ0' then
   begin
      inherited SetName('dBASE CSY cp867');
      FCharacterSet := 'DOS CODE PAGE 867';
      FCollationSequence := 'dBASE Czech867';
   end
   else if aVal = 'DB865DA0' then
   begin
      inherited SetName('dBASE DAN cp865');
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'dBASE Danish';
   end
   else if aVal = 'DB437DE0' then
   begin
      inherited SetName('dBASE DEU cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE German';
   end
   else if aVal = 'DB850DE0' then
   begin
      inherited SetName('dBASE DEU cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE German850';
   end
   else if aVal = 'db437gr0' then
   begin
      inherited SetName('dBASE ELL GR437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Greek';
   end
   else if aVal = 'DB437UK0' then
   begin
      inherited SetName('dBASE ENG cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE English/UK';
   end
   else if aVal = 'DB850UK0' then
   begin
      inherited SetName('dBASE ENG cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE English850/UK';
   end
   else if aVal = 'DB437US0' then
   begin
      inherited SetName('dBASE ENU cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE English/US';
   end
   else if aVal = 'DB850US0' then
   begin
      inherited SetName('dBASE ENU cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE English/US';
   end
   else if aVal = 'DB437ES1' then
   begin
      inherited SetName('dBASE ESP cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Spanish';
   end
   else if aVal = 'DB850ES0' then
   begin
      inherited SetName('dBASE ESP cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Spanish850';
   end
   else if aVal = 'DB437FI0' then
   begin
      inherited SetName('dBASE FIN cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Finnish';
   end
   else if aVal = 'DB437FR0' then
   begin
      inherited SetName('dBASE FRA cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE French';
   end
   else if aVal = 'DB850FR0' then
   begin
      inherited SetName('dBASE FRA cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE French850';
   end
   else if aVal = 'DB850CF0' then
   begin
      inherited SetName('dBASE FRC cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Canadian-French850';
   end
   else if aVal = 'DB863CF1' then
   begin
      inherited SetName('dBASE FRC cp863');
      FCharacterSet := 'DOS CODE PAGE 863';
      FCollationSequence := 'dBASE Canadian-French863';
   end
   else if aVal = 'db852hdc' then
   begin
      inherited SetName('dBASE HUN cp852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Hungarian';
   end
   else if aVal = 'DB437IT0' then
   begin
      inherited SetName('dBASE ITA cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Italian';
   end
   else if aVal = 'DB850IT1' then
   begin
      inherited SetName('dBASE ITA cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Italian850';
   end
   else if aVal = 'DB932JP1' then
   begin
      inherited SetName('dBASE JPN Dic932');
      FCharacterSet := 'DOS CODE PAGE 932';
      FCollationSequence := 'dBASE Japan Dic932';
   end
   else if aVal = 'DB932JP0' then
   begin
      inherited SetName('dBASE JPN cp932');
      FCharacterSet := 'DOS CODE PAGE 932';
      FCollationSequence := 'dBASE Japan cp932';
   end
   else if aVal = 'DB949KO0' then
   begin
      inherited SetName('dBASE KOR cp949');
      FCharacterSet := 'DOS CODE PAGE 949';
      FCollationSequence := 'dBASE Korea949';
   end
   else if aVal = 'DB437NL0' then
   begin
      inherited SetName('dBASE NLD cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Dutch';
   end
   else if aVal = 'DB850NL0' then
   begin
      inherited SetName('dBASE NLD cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Dutch850';
   end
   else if aVal = 'DB865NO0' then
   begin
      inherited SetName('dBASE NOR cp865');
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'dBASE Norwegian';
   end
   else if aVal = 'db852po0' then
   begin
      inherited SetName('dBASE PLK cp852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Polish852';
   end
   else if aVal = 'DB850PT0' then
   begin
      inherited SetName('dBASE PTB cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Brazilian Portuguese 850';
   end
   else if aVal = 'DB860PT0' then
   begin
      inherited SetName('dBASE PTG cp860');
      FCharacterSet := 'DOS CODE PAGE 860';
      FCollationSequence := 'dBASE Brazilian Portuguese 860';
   end
   else if aVal = 'db866ru0' then
   begin
      inherited SetName('dBASE RUS cp866');
      FCharacterSet := 'DOS CODE PAGE 866';
      FCollationSequence := 'dBASE Russian';
   end
   else if aVal = 'db852sl0' then
   begin
      inherited SetName('dBASE SLO cp852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'dBASE Slovakia 852';
   end
   else if aVal = 'DB437SV0' then
   begin
      inherited SetName('dBASE SVE cp437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'dBASE Swedish';
   end
   else if aVal = 'DB850SV1' then
   begin
      inherited SetName('dBASE SVE cp850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'dBASE Swedish850';
   end
   else if aVal = 'DB950TW0' then
   begin
      inherited SetName('dBASE CHT cp950');
      FCharacterSet := 'DOS CODE PAGE 950';
      FCollationSequence := 'dBASE Taiwan950';
   end
   else if aVal = 'db874th0' then
   begin
      inherited SetName('dBASE THA cp874');
      FCharacterSet := 'DOS CODE PAGE 874';
      FCollationSequence := 'dBASE Thai';
   end
   else if aVal = 'DB857TR0' then
   begin
      inherited SetName('dBASE TRK cp857');
      FCharacterSet := 'DOS CODE PAGE 857';
      FCollationSequence := 'dBASE Turkish';
   end
   else if aVal = 'dbHebrew' then
   begin
      inherited SetName('Hebrew dBASE');
      FCharacterSet := '???';
      FCollationSequence := 'dBASE Hebrew';
   end
   else
   begin
      FLdType := [];
      Result := False;    {driver nebyl nalezen};
   end;
end;



function TLangDriver.SetInternalPdoxName(aVal: String): Boolean;
begin

   Result := True;
   FLdType := [ldTPdox];

   if aVal = 'ascii' then
   begin
      inherited SetName('Paradox ''ascii''');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Binary';
   end
   else if aVal = 'hebrew' then
   begin
      inherited SetName('Paradox ''hebrew''');
      FCharacterSet := 'DOS CODE PAGE 862';
      FCollationSequence := 'Binary';
   end
   else if aVal = 'intl' then
   begin
      inherited SetName('Paradox ''intl''');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Multilingual Western Europe';
   end
   else if aVal = 'intl850' then
   begin
      inherited SetName('Paradox ''intl'' 850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'Brazilian Portuguese, French Canadian (2-level)';
   end
   else if aVal = 'japan' then
   begin
      inherited SetName('Paradox ''japan''');
      FCharacterSet := 'CODE PAGE 932';
      FCollationSequence := 'Japanese (4-level)';
   end
   else if aVal = 'nordan' then
   begin
      inherited SetName('Paradox ''nordan''');
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'Norwegian/Danish (Paradox 3.5, 2-level)';
   end
   else if aVal = 'nordan40' then
   begin
      inherited SetName('Paradox ''nordan40''');
      FCharacterSet := 'DOS CODE PAGE 865';
      FCollationSequence := 'Norwegian/Danish (Paradox 4.0, 5.0, 5.5, 7.0, 2-level)';
   end
   else if aVal = 'swedfin' then
   begin
      inherited SetName('Paradox ''swedfin''');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Swedish/Finnish (2-level)';
   end
   else if aVal = 'turk' then
   begin
      inherited SetName('Paradox ''turk''');
      FCharacterSet := 'CODE PAGE 857';
      FCollationSequence := 'Turkish (2-level)';
   end
   else if aVal = 'ANHEBREW' then
   begin
      inherited SetName('Paradox ANSI HEBREW');
      FCharacterSet := '1255(ANSI)';
      FCollationSequence := 'Binary';
   end
   else if aVal = 'china' then
   begin
      inherited SetName('Paradox China 936');
      FCharacterSet := 'CODE PAGE 936';
      FCollationSequence := 'China (1-level)';
   end
   else if aVal = 'cyrr' then
   begin
      inherited SetName('Paradox Cyrr 866');
      FCharacterSet := 'DOS CODE PAGE 866';
      FCollationSequence := 'Cyrillic';
   end
   else if aVal = 'czech' then
   begin
      inherited SetName('Paradox Czech 852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Czech852 (2-level)';
   end
   else if aVal = 'cskamen' then
   begin
      inherited SetName('Paradox Czech 867');
      FCharacterSet := 'DOS CODE PAGE 867';
      FCollationSequence := 'Czech867 (2-level)';
   end
   else if aVal = 'SPANISH' then
   begin
      inherited SetName('Paradox ESP 437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Spanish (3-level)';
   end
   else if aVal = 'grcp437' then
   begin
      inherited SetName('Paradox Greek GR437');
      FCharacterSet := 'DOS CODE PAGE 437';
      FCollationSequence := 'Greek (2-level)';
   end
   else if aVal = 'hun852dc' then
   begin
      inherited SetName('Paradox Hun 852 DC');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Hungarian (2-level)';
   end
   else if aVal = 'iceland' then
   begin
      inherited SetName('Paradox ISL 861');
      FCharacterSet := 'DOS CODE PAGE 861';
      FCollationSequence := 'Icelandic (2-level)';
   end
   else if aVal = 'korea' then
   begin
      inherited SetName('Paradox Korea 949');
      FCharacterSet := 'CODE PAGE 949';
      FCollationSequence := 'Korea (1-level)';
   end
   else if aVal = 'polish' then
   begin
      inherited SetName('Paradox Polish 852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Polish (2-level)';
   end
   else if aVal = 'slovene' then
   begin
      inherited SetName('Paradox Slovene 852');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := 'Slovene (2-level)';
   end
   else if aVal = 'taiwan' then
   begin
      inherited SetName('Paradox Taiwan 950');
      FCharacterSet := 'DOS CODE PAGE 950';
      FCollationSequence := 'DOS CODE PAGE 950';
   end
   else if aVal = 'thai' then
   begin
      inherited SetName('Paradox Thai 874');
      FCharacterSet := 'DOS CODE PAGE 874';
      FCollationSequence := 'Thai (3-level)';
   end
   else if aVal = 'ancyrr' then
   begin
      inherited SetName('Pdox ANSI Cyrillic');
      FCharacterSet := '1251 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "cyrr" (2-level)';
   end
   else if aVal = 'anczech' then
   begin
      inherited SetName('Pdox ANSI Czech');
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "czech" (2-level)';
   end
   else if aVal = 'angreek1' then
   begin
      inherited SetName('Pdox ANSI Greek');
      FCharacterSet := '1253 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "greek" (2-level)';
   end
   else if aVal = 'anhundc' then
   begin
      inherited SetName('Pdox ANSI Hun. DC');
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "hung" (2-level)';
   end
   else if aVal = 'ANSIINTL' then
   begin
      inherited SetName('Pdox ANSI Intl');
      FCharacterSet := '1252 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "intl" (3-level)';
   end
   else if aVal = 'ANSII850' then
   begin
      inherited SetName('Pdox ANSI Intl850');
      FCharacterSet := 'DOS CODE PAGE 850';
      FCollationSequence := 'Compatible with Paradox "intl850" (3-level)';
   end
   else if aVal = 'ANSINOR4' then
   begin
      inherited SetName('Pdox ANSI Nordan4');
      FCharacterSet := '1252 (ANSI)';
      FCollationSequence := 'Compatible with Paradox "nordan40" (2-level)';
   end
   else if aVal = 'anpolish' then
   begin
      inherited SetName('Pdox ANSI Polish');
      FCharacterSet := '1250(ANSI)';
      FCollationSequence := 'Compatible with Paradox "polish" (2-level)';
   end
   else if aVal = 'ansislov' then
   begin
      inherited SetName('Pdox ANSI Slovene');
      FCharacterSet := '1250(ANSI)';
      FCollationSequence := 'Compatible with Paradox "slovene" (2-level)';
   end
   else if aVal = 'ANSISPAN' then
   begin
      inherited SetName('Pdox ANSI Spanish');
      FCharacterSet := '1252(ANSI)';
      FCollationSequence := 'Compatible with Paradox "SPANISH" (3-level)';
   end
   else if aVal = 'ANSISWFN' then
   begin
      inherited SetName('Pdox ANSI Swedfin');
      FCharacterSet := '1252(ANSI)';
      FCollationSequence := 'Compatible with Paradox "swedfin" (3-level)';
   end
   else if aVal = 'ANTURK' then
   begin
      inherited SetName('Pdox ANSI Turkish');
      FCharacterSet := '1254(ANSI)';
      FCollationSequence := 'Compatible with Paradox "turk" (2-level)';
   end
   else if aVal = 'anczechw' then
   begin
      inherited SetName('pdx ANSI Czech ''CH''');
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := '(2-level)';
   end
   else if aVal = 'czechw' then
   begin
      inherited SetName('pdx Czech 852 ''CH''');
      FCharacterSet := 'DOS CODE PAGE 852';
      FCollationSequence := '(2-level)';
   end
   else if aVal = 'cskamenw' then
   begin
      inherited SetName('pdx Czech 867 ''CH''');
      FCharacterSet := 'DOS CODE PAGE 867';
      FCollationSequence := '(2-level)';
   end
   else if aVal = 'anil2czw' then
   begin
      inherited SetName('pdx ANSI ISO L_2 CZ');
      FCharacterSet := '1250 (ANSI)';
      FCollationSequence := 'Paradox Czech, (2-level)';
   end
   else if aVal = 'il2czw' then
   begin
      inherited SetName('pdx ISO L_2 Czech');
      FCharacterSet := 'ISO8859-2';
      FCollationSequence := 'Paradox Czech, ISO8859-2 (2-level)';
   end
   else if aVal = 'BGPD1251' then
   begin
      inherited SetName('Pdox ANSI Bulgarian');
      FCharacterSet := '1251 (ANSI)';
      FCollationSequence := 'Bulgarian (2-level)';
   end
   else if aVal = 'Bulgaria' then
   begin
      inherited SetName('Paradox Bulgaria 868');
      FCharacterSet := 'DOS CODE PAGE 868';
      FCollationSequence := 'Bulgaria (2-level)';
   end
   else
   begin
      FLdType := [];
      Result := False;    {driver nebyl nalezen};
   end;

end;
{
 TAllLangDrivers
}

constructor TAllLangDrivers.Create;
begin
   inherited Create;
   CreateAllDrivers;
end;



constructor TAllLangDrivers.CreatedBase;
begin
   inherited Create;
   CreatedBaseDrivers;
end;



constructor TAllLangDrivers.CreatePdox;
begin
   inherited Create;
   CreatePdoxDrivers;
end;



destructor  TAllLangDrivers.Destroy;
begin
   inherited Destroy;
end;



procedure TAllLangDrivers.SetNewDriverName(aValue: string);
var
   D: TLangDriver;
begin
   if aValue = '' then exit;
   D := TLangDriver.Create;
   D.Name := aValue;
   AddElement(D);
   FActualDriver := D;
end;



function TAllLangDrivers.GetActualDriver: TLangDriver;
var
   D: TlangDriver;
   S: string;
begin
   if FActualDriver = nil then
   begin
      S := 'dBASE CSY cp852';
      D := TlangDriver(FindElementName(S));
      if D = nil then
      begin
         D := TlangDriver.Create;
         D.Name := S;
         AddElement(D);
      end;
      FActualDriver := D;
   end;
   Result := FActualDriver;
end;



procedure TAllLangDrivers.CreateAllDrivers;
begin
   CreatedBaseDrivers;
   CreatePdoxDrivers;
end;



procedure TAllLangDrivers.CreatedBaseDrivers;
begin
   NewDriverName := '''ascii'' ANSI';
   NewDriverName := '''Spanish'' ANSI';
   NewDriverName := '''WEurope'' ANSI';
   NewDriverName := 'dBASE CHS cp936';
   NewDriverName := 'dBASE CSY cp852';
   NewDriverName := 'dBASE CSY cp867';
   NewDriverName := 'dBASE DAN cp865';
   NewDriverName := 'dBASE DEU cp437';
   NewDriverName := 'dBASE DEU cp850';
   NewDriverName := 'dBASE ELL GR437';
   NewDriverName := 'dBASE ENG cp437';
   NewDriverName := 'dBASE ENG cp850';
   NewDriverName := 'dBASE ENU cp437';
   NewDriverName := 'dBASE ENU cp850';
   NewDriverName := 'dBASE ESP cp437';
   NewDriverName := 'dBASE ESP cp850';
   NewDriverName := 'dBASE FIN cp437';
   NewDriverName := 'dBASE FRA cp437';
   NewDriverName := 'dBASE FRA cp850';
   NewDriverName := 'dBASE FRC cp850';
   NewDriverName := 'dBASE FRC cp863';
   NewDriverName := 'dBASE HUN cp852';
   NewDriverName := 'dBASE ITA cp437';
   NewDriverName := 'dBASE ITA cp850';
   NewDriverName := 'dBASE JPN Dic932';
   NewDriverName := 'dBASE JPN cp932';
   NewDriverName := 'dBASE KOR cp949';
   NewDriverName := 'dBASE NLD cp437';
   NewDriverName := 'dBASE NLD cp850';
   NewDriverName := 'dBASE NOR cp865';
   NewDriverName := 'dBASE PLK cp852';
   NewDriverName := 'dBASE PTB cp850';
   NewDriverName := 'dBASE PTG cp860';
   NewDriverName := 'dBASE RUS cp866';
   NewDriverName := 'dBASE SLO cp852';
   NewDriverName := 'dBASE SVE cp437';
   NewDriverName := 'dBASE SVE cp850';
   NewDriverName := 'dBASE CHT cp950';
   NewDriverName := 'dBASE THA cp874';
   NewDriverName := 'dBASE TRK cp857';
   NewDriverName := 'Hebrew dBASE';
end;



procedure TAllLangDrivers.CreatePdoxDrivers;
begin
   NewDriverName := 'Paradox ''ascii''';
   NewDriverName := 'Paradox ''hebrew''';
   NewDriverName := 'Paradox ''intl''';
   NewDriverName := 'Paradox ''intl'' 850';
   NewDriverName := 'Paradox ''japan''';
   NewDriverName := 'Paradox ''nordan''';
   NewDriverName := 'Paradox ''nordan40''';
   NewDriverName := 'Paradox ''swedfin''';
   NewDriverName := 'Paradox ''turk''';
   NewDriverName := 'Paradox ANSI HEBREW';
   NewDriverName := 'Paradox China 936';
   NewDriverName := 'Paradox Cyrr 866';
   NewDriverName := 'Paradox Czech 852';
   NewDriverName := 'Paradox Czech 867';
   NewDriverName := 'Paradox ESP 437';
   NewDriverName := 'Paradox Greek GR437';
   NewDriverName := 'Paradox Hun 852 DC';
   NewDriverName := 'Paradox ISL 861';
   NewDriverName := 'Paradox Korea 949';
   NewDriverName := 'Paradox Polish 852';
   NewDriverName := 'Paradox Slovene 852';
   NewDriverName := 'Paradox Taiwan 950';
   NewDriverName := 'Paradox Thai 874';
   NewDriverName := 'Pdox ANSI Cyrillic';
   NewDriverName := 'Pdox ANSI Czech';
   NewDriverName := 'Pdox ANSI Greek';
   NewDriverName := 'Pdox ANSI Hun. DC';
   NewDriverName := 'Pdox ANSI Intl';
   NewDriverName := 'Pdox ANSI Intl850';
   NewDriverName := 'Pdox ANSI Nordan4';
   NewDriverName := 'Pdox ANSI Polish';
   NewDriverName := 'Pdox ANSI Slovene';
   NewDriverName := 'Pdox ANSI Spanish';
   NewDriverName := 'Pdox ANSI Swedfin';
   NewDriverName := 'Pdox ANSI Turkish';
   NewDriverName := 'pdx ANSI Czech ''CH''';
   NewDriverName := 'pdx Czech 852 ''CH''';
   NewDriverName := 'pdx Czech 867 ''CH''';
   NewDriverName := 'pdx ANSI ISO L_2 CZ';
   NewDriverName := 'pdx ISO L_2 Czech';
   NewDriverName := 'Pdox ANSI Bulgarian';
   NewDriverName := 'Paradox Bulgaria 868';
end;



end.
