unit t_Lineextractor;

interface
uses
   Elements,
   t_Recurs;
type

TLineExtractor = class(TRecursor)
private
   FCopyLines: boolean;
    FNoDuplicates: boolean;
    FMaxLevel: integer;
    FFirstLines: TOwnerEntity;
    FInHole: boolean;
   procedure SetCopyLines(const Value: boolean);
    procedure SetNoDuplicates(const Value: boolean);
    procedure SetMaxLevel(const Value: integer);
    function GetLines: TEntity;
    procedure SetFirstLines(const Value: TOwnerEntity);
    procedure SetInHole(const Value: boolean);
protected
   procedure SetOutput(aOutput: TElement); override;
public
   procedure DoIt; override;
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   property  FirstLines: TOwnerEntity read FFirstLines write SetFirstLines;
   property  Lines: TEntity read GetLines;
   property  InHole: boolean read FInHole write SetInHole;
published
   property  CopyLines: boolean read FCopyLines write SetCopyLines;
   property  NoDuplicates: boolean read FNoDuplicates write SetNoDuplicates;
   property  MaxLevel: integer read FMaxLevel write SetMaxLevel;
end;

implementation
uses
   e_Class,
   e_Geometry,
   g_Elems,
   m_Orbis;

{ TLineExtractor }

procedure TLineExtractor.DoIt;
var
   LineCopy: TPolyLineElement;
   Line: TPolyLineElement;
   i,j: integer;
begin
   if Output = nil then
   begin
      Output := TOwnerEntity.Create;
   end
   else
   if not (Output is TEntity) then
   begin
      PostTextMsg(tm_AddToProtocol, 'ERROR: LineExtractor expected TEntity at output!', nil);
      exit;
   end;
   FirstLines := TOwnerEntity.Create;
   InHole := false;
   inherited;
   if CopyLines then
   for j := 0 to FirstLines.LastIndex do
   begin
      Line := TPolyLineElement(FirstLines.Elements[j]);
      LineCopy := TPolyLineElement(CreateElementCopy(Line));
      for i := 0 to Line.LastIndex do
      begin
         LineCopy.AddXY(Line.Xi[i],Line.Yi[i]);
      end;
      Lines.AddElement(LineCopy);
   end
   else
   begin
      Lines.AddAllFrom := FirstLines;
   end;
   FirstLines := nil;
end;

function TLineExtractor.GetLines: TEntity;
begin
   result := TEntity(Output);
end;

procedure TLineExtractor.Process(Element: TElement);
var
   Line: TPolyLineElement;
   i: integer;
   E: TElement;
begin
   if not (Element is TPolyLineElement) then exit;
   Line := TPolyLineElement(Element);
   if NoDuplicates then
   begin
      if Line.IsMyOwner(FirstLines) then exit;// u je v seznamu
   end;
   FirstLines.AddElement(Line);
end;

function TLineExtractor.QGoInside(Element: TElement): boolean;
begin
   result := false;
   if not InHole and (MaxLevel > 0) and (Level >= MaxLevel) then
      exit;
   Result:= (Element is TEntity) and not (Element is TPolyLineElement);
   if (Element is TAreaElement) and not(Element is TMultiAreaElement) and (TAreaElement(Element).Holes <> nil) then
   begin
      InHole := true;
      GoInside(TAreaElement(Element).Holes);
      InHole := false;
   end;
end;

function TLineExtractor.QProcessBefore(Element: TElement): boolean;
begin
   Result:= (Element is TPolyLineElement);
end;

procedure TLineExtractor.SetCopyLines(const Value: boolean);
begin
  FCopyLines := Value;
end;

procedure TLineExtractor.SetFirstLines(const Value: TOwnerEntity);
begin
   ExchangeFieldElement(TElement(FFirstLines), Value);
end;

procedure TLineExtractor.SetInHole(const Value: boolean);
begin
  FInHole := Value;
end;

procedure TLineExtractor.SetMaxLevel(const Value: integer);
begin
  FMaxLevel := Value;
end;

procedure TLineExtractor.SetNoDuplicates(const Value: boolean);
begin
   FNoDuplicates := Value;
end;

procedure TLineExtractor.SetOutput(aOutput: TElement);
begin
   ExchangeFieldElement(FOutput, aOutput);
end;

INITIALIZATION
   RegisterOrbisClass(TLineExtractor);
END.

