unit t_MeasureTools;

interface
uses
   t_GraphicEditors;

type

TMeasureTool = class(TPolyElementEditor)
  private
    FUserScript: string;
    function GetArea: double;
    function GetLength: double;
    procedure SetUserScript(const Value: string);
    function GetLastLength: double;
public
   constructor Create; override;
   procedure   CreateNewObject; override;
   procedure   ExecuteDialog; override;
   function    IsPossibleToDeactivate: boolean; override;
   procedure   On_DeActivate; override;
published
   property Length: double read GetLength stored false;
   property LastLength: double read GetLastLength stored false;
   property Area: double read GetArea stored false;
   property UserScript: string read FUserScript write SetUserScript;
end;

TMeasureLengthTool = class(TMeasureTool)
public
   procedure XORDraw; override;
   procedure RegisterCursors; override;
   procedure SendInfo; override;
end;

TMeasureAreaTool = class(TMeasureTool)
  private
    FUsePattern: boolean;
    procedure SetUsePattern(const Value: boolean);
public
   procedure XORDraw; override;
   procedure RegisterCursors; override;
   procedure SendInfo; override;
published
   property UsePattern: boolean read FUsePattern write SetUsePattern;
end;


implementation
uses
   Forms,
   Graphics,
   SysUtils,
   e_Class,
   e_Geometry,
   m_Proj,
   m_Orbis, Elements;

{ TMeasureTool }

constructor TMeasureTool.Create;
begin
   inherited;
   CanCreateNewObject := true;
end;

procedure TMeasureTool.CreateNewObject;
begin
   if PolyElement <> nil then XORDraw;
   GElement := nil;
   PolyElement:= TPolyLineElement.Create;
   FActualCrdIndex:= -1;
   FMovedCrdIndex:= -1;
   if PolyElement = nil then exit;
   SendInfo;
end;

procedure TMeasureTool.ExecuteDialog;
begin
   if UserScript <> '' then
   begin
      PostTextMsg(tm_Run,UserScript,nil);
      Application.ProcessMessages;
   end;
   CreateNewObject;
   if CanBeFlash and (FlashTool <> nil) then
   begin
      Application.ProcessMessages;
      PostMsgToOwners(tm_ToolSelected, FlashTool, false);
   end;
end;

function TMeasureTool.GetArea: double;
begin
   result:= abs(PolyElement.Area);
   if result < 1E-7 then
      result:= 0;
end;

function TMeasureTool.GetLastLength: double;
begin
   result:= PolyElement.LastLength;
end;

function TMeasureTool.GetLength: double;
begin
   result:= PolyElement.Length;
end;

function TMeasureTool.IsPossibleToDeactivate: boolean;
begin
   result := true;
end;

procedure TMeasureTool.On_DeActivate;
begin
   CreateNewObject;
   inherited;
end;

procedure TMeasureTool.SetUserScript(const Value: string);
begin
  FUserScript := Value;
end;

{ TMeasureLengthTool }

procedure TMeasureLengthTool.RegisterCursors;
begin
   NewCursorName('LINESELECT');
   NewCursorName('MOVE_POINT');
   NewCursorName('INSERT_POINT');
   NewCursorName('SNAP_LPOINT');
   NewCursorName('SNAP_LBODY');
   NewCursorName('NOSELECT');
end;

procedure TMeasureLengthTool.SendInfo;
begin
   PostTextMsgToOwners(tm_LengthChanged,FloatToStr(Length)+':'+FloatToStr(LastLength));
end;

procedure TMeasureLengthTool.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
begin
   if PolyElement = nil then
      Exit;

  Device.Pen.Color:= Color;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsClear;
  if PolyElement.CoordsCount > 1 then
  begin
     Device.MoveTo(PolyElement.Xi[0], PolyElement.Yi[0]);
     for i := 1 to PolyElement.LastIndex do
         Device.LineTo(PolyElement.Xi[i], PolyElement.Yi[i]);
  end;
// vykreslovn znaek
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= PolyElement.CoordsCount-1;
  for I:= 0 to K do
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen vybranho bodu
  I:= FActualCrdIndex;
  aR:= Device.Projection.GetModelLength(PixelR - 1);
  if (I >= 0) and (I <= K) then
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen hlaviky
  aR:= Device.Projection.GetModelLength(PixelR + 2);

  if FHeaderAtTale then
     K:= 0;

  if K >= 0 then
  begin
    DrawRect(PolyElement.Xi[K], PolyElement.Yi[K], aR);

    if FCloseEnds then
    begin
      Device.MoveTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]-2*aR);
      Device.MoveTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]-2*aR);
    end
  end;

  PostMsgToOwners(tm_Repaint,nil,false);
end;

{ TMeasureAreaTool }

procedure TMeasureAreaTool.RegisterCursors;
begin
   NewCursorName('SELECTAREA');
   NewCursorName('MOVE_POINT');
   NewCursorName('INSERT_POINT');
   NewCursorName('SNAP_LPOINT');
   NewCursorName('SNAP_LBODY');
   NewCursorName('NOSELECT');
end;

procedure TMeasureAreaTool.SendInfo;
begin
   PostTextMsgToOwners(tm_AreaChanged,FloatToStr(Area)+':'+FloatToStr(LastLength));
end;

procedure TMeasureAreaTool.SetUsePattern(const Value: boolean);
begin
   if Active then XORDraw;
   FUsePattern := Value;
   if Active then XORDraw;
end;

procedure TMeasureAreaTool.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
begin
  if PolyElement.Count < 1 then exit;
  Device.Pen.Color:= Color;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  if UsePattern then
     Device.Brush.Style:= bsDiagCross
  else
     Device.Brush.Style:= bsClear;
  if PolyElement.Count < 3 then
  begin
     if PolyElement.Count = 2 then
        Device.Polyline(PolyElement);
  end
  else
     Device.Polygon(PolyElement);

// vykreslovn znaek
  aR:= Device.Projection.GetModelLength(PixelR);
  K:= PolyElement.CoordsCount-1;
  for I:= 0 to K do
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen vybranho bodu
  I:= FActualCrdIndex;
  aR:= Device.Projection.GetModelLength(PixelR - 1);
  if I >= 0 then
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);
  end;

// Vykreslen hlaviky
  aR:= Device.Projection.GetModelLength(PixelR + 2);

  if FHeaderAtTale then
     K:= 0;

  if K >= 0 then
  begin
    DrawRect(PolyElement.Xi[i], PolyElement.Yi[i], aR);

    if FCloseEnds then
    begin
      Device.MoveTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]-2*aR);
      Device.MoveTo(PolyElement.Xi[K]-2*aR, PolyElement.Yi[K]+2*aR);
      Device.LineTo(PolyElement.Xi[K]+2*aR, PolyElement.Yi[K]-2*aR);
    end
  end;

  PostMsgToOwners(tm_Repaint,nil,false);
end;

INITIALIZATION
   RegisterOrbisClass(TMeasureLengthTool);
   RegisterOrbisClass(TMeasureAreaTool);
END.

