unit T_BlkLay;

interface
uses
// KERNEL
   Elements,
   Tools,
// GLIB
   G_Elems,

{$IFDEF RAS}
// RASTER
   e_Raster,
{$ENDIF}

// SPACE
   //E_Space
   e_GeometryElement;




type
// vstup - entita blok
// vstup - blok dle kritri
  TFindBlockLayer = class(TTool)
  private
    FLayerType: string;
    FLayerName: string;
  public
    procedure DoIt; override;
  published
    property LayerType: string read FLayerType write FLayerType;
    property LayerName: string read FLayerName write FLayerName;
  end;

  TBlockLayersTool = class(TTool)
  private
    procedure AddLayer(aLayers: TEntity; aLayer: TGeometryElementCollection);
   {$IFDEF RAS}
    procedure AddRaster(aLayers: TEntity; aRaster: TRaster);
   {$ENDIF}
  public
    procedure DoIt; override;
  end;

implementation
uses
  SysUtils,
// KERNEL
   e_Class,
// interpret
   e_InterpretContainer;




procedure TFindBlockLayer.DoIt;
var
  aLayers: TEntity;
  aLayer, aBlok: TElement;
  I, J: longInt;

  procedure AddAnotherBlok(aBlok: TElement);
  var
    aLayName, aLayType: string;
  begin
    aLayName:= aBlok.GetPropertyStringByName('BlockName');
    if aLayName = ''
       then exit;
    aLayName:= ExtractFileName(aLayName);
    aLayName:= copy(aLayName, 1, pos('.', aLayName)-1);
    aLayType:= aBlok.GetPropertyStringByName('TYPE');

    if (FLayerName <> '') and (CompareText(FLayerName, aLayName) <> 0) then
        exit;
    if (FLayerType <> '') and  (CompareText(FLayerType, aLayType) <> 0) then
        exit;
    TEntity(Output).AddElement(aBlok)
  end;


begin
  if not (Input is TEntity) then exit;
  if not (Output is TEntity) then exit;
  aLayers:= TEntity(Input);
  for I:= 0 to aLayers.Count-1 do
  begin
    aLayer:= aLayers.Elements[I];
    if aLayer is TInterpretLayer then
    begin
      for J:= 0 to TEntity(aLayer).Count-1 do
      begin
        aBlok:= TEntity(aLayer).Elements[J];
        AddAnotherBlok(aBlok);
      end;
    end
    else
    begin
      AddAnotherBlok(aLayer);
    end
  end;
end;



procedure TBlockLayersTool.AddLayer(aLayers: TEntity; aLayer: TGeometryElementCollection);
var
  aLayName: string;
  aEl: TElement;
  aTargetLayer: TRectEntity;
begin
  aTargetLayer:= nil;
// zjitn jmna clov vrstvy
  aLayName:= aLayer.GetPropertyStringByName('BlockName');
  aLayName:= ExtractFileName(aLayName);
  aLayName:= aLayer.GetPropertyStringByName('TYPE') + '_' +
             copy(aLayName, 1, pos('.', aLayName)-1);

// hledn clov vrstvy v aRect
  aEl:= aLayers.FindElementName(aLayName);

  if aEl = nil then
  begin
    aTargetLayer:= TRectEntity.Create;
    aTargetLayer.Name:= aLayName;
    aLayers.AddElement(aTargetLayer);
  end
  else if aEl is TRectEntity then
           aTargetLayer:= TRectEntity(aEl);

  if aTargetLayer = nil then exit;

// Pidn do clov vrstvy
  aTargetLayer.AddAndUpdateRect(aLayer);
  if TElement(aLayers) is TRectEntity then
     TRectEntity(aLayers).UpdateRect(aTargetLayer)
end;

{$IFDEF RAS}

procedure TBlockLayersTool.AddRaster(aLayers: TEntity; aRaster: TRaster);
var
  aLayName: string;
  aEl: TElement;
  aTargetLayer: TRectEntity;
begin
  aTargetLayer:= nil;
// zjitn jmna clov vrstvy
  aLayName:= 'RAS_RAK';

// hledn clov vrstvy v aRect
  aEl:= aLayers.FindElementName(aLayName);

  if aEl = nil then
  begin
    aTargetLayer:= TRectEntity.Create;
    aTargetLayer.Name:= aLayName;
    aLayers.AddElement(aTargetLayer);
  end
  else if aEl is TRectEntity then
           aTargetLayer:= TRectEntity(aEl);

  if aTargetLayer = nil then exit;

// Pidn do clov vrstvy
  aTargetLayer.AddAndUpdateRect(aRaster);
  if TElement(aLayers) is TRectEntity then
     TRectEntity(aLayers).UpdateRect(aTargetLayer)
end;


procedure TBlockLayersTool.DoIt;
var
  aIn: TEntity;
  aEl, aEl2: TElement;
  //aLay: TGeometryElementCollection;
  I, J: longInt;
begin
  if not (Input is TEntity) then exit;
  if not (Output is TEntity) then exit;
  aIn:= TEntity(Input);
  for I:= 0 to aIn.Count-1 do
  begin
    aEl:= aIn.Elements[I];
    if (aEl is TInterpretLayer) then
    begin
      for J:= 0 to TEntity(aEl).Count-1 do
      begin
        aEl2:= TEntity(aEl).Elements[J];
        if (aEl2 is TRaster) then
           AddRaster(TEntity(Output), TRaster(aEl2))
        else
        if (aEl2 is TGeometryElementCollection) then
           AddLayer(TEntity(Output), TGeometryElementCollection(aEl2))
      end;
    end
    else
    if aEl is TRaster then
      AddRaster(TEntity(Output), TRaster(aEl))
    else
    if (aEl is TGeometryElementCollection) then
      AddLayer(TEntity(Output), TGeometryElementCollection(aEl));
  end;
end;

{
procedure TBlockLayersTool.DoIt;
var
  aIn: TEntity;
  aEl, aEl2: TElement;
  aLay: TSpaceEntityString;
  I, J: longInt;
begin
  if not (Input is TEntity) then exit;
  if not (Output is TEntity) then exit;
  aIn:= TEntity(Input);
  for I:= 0 to aIn.Count-1 do
  begin
    aEl:= aIn.Elements[I];
    if (aEl is TInterpretLayer) and (TInterpretLayerthen
    begin
      for J:= 0 to TEntity(aEl).Count-1 do
      begin
        aEl2:= TEntity(aEl).Elements[J];
        if not (aEl2 is TRaster) then
           continue;
        AddRaster(TEntity(Output), TRaster(aEl2));
        continue
      end;
    end
    else
    if aEl is TRaster then
    begin
      AddRaster(TEntity(Output), TRaster(aEl));
      continue;
    end;

    if not (aEl is TSpaceEntityString) then
    begin
      if aEl is TInterpretLayer then
      begin
        for J:= 0 to TEntity(aEl).Count-1 do
        begin
          aEl2:= TEntity(aEl).Elements[J];
          if not (aEl2 is TSpaceEntityString) then
             continue;
          aLay:= TSpaceEntityString(aEl2);
          AddLayer(TEntity(Output), aLay);
        end;
      end;
      continue;
    end;
    aLay:= TSpaceEntityString(aEl);
    AddLayer(TEntity(Output), aLay);
  end;
end;
}



{$ELSE}
procedure TBlockLayersTool.DoIt;
var
  aIn: TEntity;
  aEl, aEl2: TElement;
  aLay: TGeometryElementCollection;
  I, J: longInt;
begin
  if not (Input is TEntity) then exit;
  if not (Output is TEntity) then exit;
  aIn:= TEntity(Input);
  for I:= 0 to aIn.Count-1 do
  begin
    aEl:= aIn.Elements[I];
    if not (aEl is TGeometryElementCollection) then
    begin
      if aEl is TInterpretLayer then
      begin
        for J:= 0 to TEntity(aEl).Count-1 do
        begin
          aEl2:= TEntity(aEl).Elements[J];
          if not (aEl2 is TGeometryElementCollection) then
             continue;
          aLay:= TGeometryElementCollection(aEl2);
          AddLayer(TEntity(Output), aLay);
        end;
      end;
      continue;
    end;
    aLay:= TGeometryElementCollection(aEl);
    AddLayer(TEntity(Output), aLay);
  end;
end;
{$ENDIF}


begin
  RegisterOrbisClass(TBlockLayersTool);
  RegisterOrbisClass(TFindBlockLayer);
end.
