unit T_grp;

{
[Unit_Description]
Description=Nstroj pro tdn modelu do skupin dle zadanch parametr
Admin=Olda
TechTopic=Tool
Topic=Analysis
Version=1.0

[Edit]
11.8.1997 11:32=Registrace
}



interface
uses
// KERNEL
   Elements,
   E_Type,
   Tools;

type
{ input: Zpracovvan element
  output: naplovan entita, do n pid podle vlastnost
          Elementu novou skupinu a do t pid entitu tdy OwnerClass.
          a do t pak Element.
          Pokud output=nil, tak vytvo TEntity a provede to sam.

  Pokud Criteron obsahuje vce property list, pak pro kad
  zalo dal hierarchii
}

  TPropsAndClass = class(TPropertyList)
  private
    FOwnerClassName: string;
    FSocketName: string;
    FSocketClassName: string;
  published
    property OwnerClassName: string read FOwnerClassName write FOwnerClassName;
    property SocketName: string read FSocketName write FSocketName;
    property SocketClassName: string read FSocketClassName write FSocketClassName;
  end;


  TGroupper = class(TTool)
  private
    FCriterion: TEntity;
    FOwnerClass: TElementClass;
    FAssigments: TStringListElement;
    FDoItEmpty: boolean;
    procedure SetCriterion(aVal: TEntity);
    procedure SetOwnerClass(aClass: string);
    function GetOwnerClass: string;
    function DoGroup(anEntity: TEntity; aCriterion: TPropertyList): TEntity;
    procedure UpgradeRects(anEntity: TOwnerEntity);
    procedure SetNextAssigment(aVal: string);
  public
    procedure DoIt; override;
    destructor Destroy; override;
  published
    property Criterion: TEntity read FCriterion write SetCriterion;
    property OwnerClass: string read GetOwnerClass write SetOwnerClass;
    property NextAssigment: string write SetNextAssigment;
    property DoItEmpty: boolean read FDoItEmpty write FDoItEmpty;
  end;


implementation
uses
   Forms,
// KERNEL
    e_Class,
// GLIB
   G_Elems;


procedure TGroupper.SetCriterion(aVal: TEntity);
begin
  if FCriterion <> nil then
     FCriterion.Free;
  FCriterion:= aVal;
  AddElement(aVal)
end;


destructor TGroupper.Destroy;
begin
  if FAssigments <> nil then
     FAssigments.Free;
  inherited Destroy
end;


function TGroupper.GetOwnerClass: string;
begin
  result:= FOwnerClass.ClassName;
end;

procedure TGroupper.SetOwnerClass(aClass: string);
begin
  FOwnerClass:= GetOrbisClass(aClass)
end;


function TGroupper.DoGroup(anEntity: TEntity; aCriterion: TPropertyList): TEntity;
{ v anEntity vytvo skupinu, kterou vrt }
var
  aStr: string;
  aOwnerClass: TElementClass;
  aSocketClass: TElementClass;
  aSocket: TEntity;
begin
  //result:= nil;
  aCriterion.Element:= Input;
  aStr:= aCriterion.Catenation;
  if aStr = '' then
     aStr:= '___';

  aSocketClass:= nil;
  if (aCriterion is TPropsAndClass) and
     (TPropsAndClass(aCriterion).SocketClassName <> '') then
  begin
    aSocketClass:= GetOrbisClass(TPropsAndClass(aCriterion).SocketClassName);
  end;

  result:= TEntity(anEntity.FindElementName(aStr));

  if Result <> nil then
  begin
    if aSocketClass <> nil then
    begin
      result:= TEntity(result.FindElementName(TPropsAndClass(aCriterion).SocketName));
      if Result <> nil then
         exit
    end
    else
      Exit;
  end;

  aOwnerClass:= FOwnerClass;
  if (aCriterion is TPropsAndClass) and
     (TPropsAndClass(aCriterion).OwnerClassName <> '') then
  begin
    aOwnerClass:= GetOrbisClass(TPropsAndClass(aCriterion).OwnerClassName);
  end;

  if aOwnerClass = nil then
     aOwnerClass:= TOwnerEntity;

  result:= TEntity(aOwnerClass.Create);
  result.Name:= aStr;

  anEntity.AddElement(result);

  if aSocketClass <> nil then
  begin
    aSocket:= TEntity(aSocketClass.Create);
    aSocket.Name:= TPropsAndClass(aCriterion).SocketName;
    Result.AddElement(aSocket);
    Result:= aSocket
  end
end;



procedure TGroupper.UpgradeRects(anEntity: TOwnerEntity);
var
  I: longInt;
begin
  if not (anEntity is TRectEntity) then exit;
  TRectEntity(anEntity).UpdateRect(Input);
  for I:= 0 to anEntity.OwnersCount-1 do
  begin
    UpgradeRects(TOwnerEntity(anEntity.Owners[I]))
  end
end;


procedure TGroupper.SetNextAssigment(aVal: string);
begin
  if FAssigments = nil then
  begin
    FAssigments:= TStringListElement.Create;
  end;

  FAssigments.NextNameIsValue:= aVal
end;




procedure TGroupper.DoIt;
var
  I: integer;
  anEntity: TEntity;
begin
  Application.ProcessMessages;

  if not(Input is TElement) then exit;
  if not (TElement(Criterion) is TEntity) then exit;
  if FOwnerClass = nil then
     FOwnerClass:= TNameEntity;
  if Output=nil then
  begin
    output:= TOwnerEntity.Create;
  end;

  {jednorovov grupovn}
  if Criterion is TPropertyList then
     anEntity:= DoGroup(TEntity(Output), TPropertyList(Criterion))
  else
  begin
  {hierarchick grupovn}
    anEntity:= TEntity(Output);
    for I:= 0 to Criterion.Count-1 do
    begin
      if Criterion.Elements[I] is TPropertyList then
      anEntity:= DoGroup(anEntity, TPropertyList(Criterion.Elements[I]));
      if anEntity = nil then break
    end
  end;
  if anEntity <> nil then
  begin
    if FAssigments <> nil then
    { Ppadn piazovn od Inputu do vlastnka - zprava do leva }
    begin
      for I:= 0 to FAssigments.Count-1 do
      try
        anEntity.SetPropertyStringByName
         (FAssigments.Names[I], Input.GetPropertyStringByName(FAssigments.Strings[I]))
      except
      end
    end;

    if not DoItEmpty then
    begin
      anEntity.AddElement(Input);
      if anEntity is TRectEntity then
      begin
        UpgradeRects(TOwnerEntity(anEntity))
      end
    end
  end;
end;


BEGIN
   RegisterOrbisClass(TGroupper);
   RegisterOrbisClass(TPropsAndClass);
end.
