unit e_WCS;

interface
uses
  Elements,
  e_Pale,
  g_Elems,
  Worlds;

const
  { A Constants for World Coordinate Systems }
  cs_Cartesian = 0;
  cs_JTSK      = 1;
  cs_Gauss     = 2;
type





TWCSId = byte;


TWCS = class(TElement)
  protected
    FID: TWCSID;
    FCoord1Name: String;
    FCoord2Name: String;
    FCoord3Name: String;
    function  GetID: TWCSID; virtual;
    procedure SetID(aID: TWCSID); virtual;
    function  GetCoord1Name: string; virtual;
    function  GetCoord2Name: string; virtual;
    function  GetCoord3Name: string; virtual;
    procedure SetCoord1Name(aName: string); virtual;
    procedure SetCoord2Name(aName: string); virtual;
    procedure SetCoord3Name(aName: string); virtual;
  public

    procedure GiveICS(const Crd1, Crd2, Crd3: extended;
                      var X, Y, Z: extended); virtual; abstract;
    procedure GiveWCSFromICS(const X, Y, Z: extended;
                             var Crd1, Crd2, Crd3: extended); virtual; abstract;

    function RecalculateToICS(Coord: TCoordinate): TCoordinate; virtual; abstract;
    function RecalculateFromICS(Coord: TCoordinate): TCoordinate; virtual; abstract;


    property ID: TWCSID read GetID write SetID;
    property Coord1Name: string read GetCoord1Name write SetCoord1Name;
    property Coord2Name: string read GetCoord2Name write SetCoord2Name;
    property Coord3Name: string read GetCoord3Name write SetCoord3Name;
end;



TGeographic_WCS = class(TWCS)
 public
   procedure GiveWGS84(const Crd1, Crd2, Crd3: extended;
                       var X, Y, Z: extended);virtual; abstract;

   procedure GiveWCSFromWGS84(const X, Y, Z: extended;
                              var Crd1, Crd2, Crd3: extended);virtual; abstract;
end;


TJTSK = class(TGeographic_WCS)
 protected
   function  GetID: TWCSID; override;
   function  GetCoord1Name: string; override;
   function  GetCoord2Name: string; override;
   function  GetCoord3Name: string; override;
 public

    procedure GiveICS(const Crd1, Crd2, Crd3: extended;
                      var X, Y, Z: extended); override;
    procedure GiveWCSFromICS(const X, Y, Z: extended;
                             var Crd1, Crd2, Crd3: extended); override;

    procedure GiveWGS84(const Crd1, Crd2, Crd3: extended;
                        var X, Y, Z: extended); override;

    procedure GiveWCSFromWGS84(const X, Y, Z: extended;
                               var Crd1, Crd2, Crd3: extended); override;
    function RecalculateToICS(Coord: TCoordinate): TCoordinate; override;
    function RecalculateFromICS(Coord: TCoordinate): TCoordinate; override;
end;

TGauss_Kruger = class(TGeographic_WCS)
 protected
   function  GetID: TWCSID; override;
   function  GetCoord1Name: string; override;
   function  GetCoord2Name: string; override;
   function  GetCoord3Name: string; override;
 public

    procedure GiveICS(const Crd1, Crd2, Crd3: extended;
                      var X, Y, Z: extended); override;
    procedure GiveWCSFromICS(const X, Y, Z: extended;
                             var Crd1, Crd2, Crd3: extended); override;

    procedure GiveWGS84(const Crd1, Crd2, Crd3: extended;
                        var X, Y, Z: extended); override;

    procedure GiveWCSFromWGS84(const X, Y, Z: extended;
                               var Crd1, Crd2, Crd3: extended); override;
    function RecalculateToICS(Coord: TCoordinate): TCoordinate; override;
    function RecalculateFromICS(Coord: TCoordinate): TCoordinate; override;
end;


TWCSLibrary = class(TOwnerEntity)
public
   function  GetWCSID(aID: TWCSID): TWCS;
end;


TWCSWorld = class(TWorld)
  private
    FColorPalette: TColorPalette;
    procedure SetColorPalette(const Value: TColorPalette);
protected
   FWCSLibrary: TWCSLibrary;
   function  GetWCSLibrary: TWCSLibrary;
   procedure SetWCSLibrary(aWCSLibrary: TWCSLibrary);
public
   iniFileName : string;
   constructor Create; override;
   function GetIDWCS(aID: TWCSID): TWCS;

   property WCSLibrary: TWCSLibrary read GetWCSLibrary write SetWCSLibrary;
published
   property ColorPalette: TColorPalette read FColorPalette write SetColorPalette;
end;

implementation

uses
   e_Color,
   u_String;


{ TWCS }

function  TWCS.GetID: TWCSID;
begin
   result := FID;
end;


procedure  TWCS.SetID(aID: TWCSID);
begin
   FID := aID;
end;

function  TWCS.GetCoord1Name: string;
begin
   result := FCoord1Name;
end;

function  TWCS.GetCoord2Name: string;
begin
   result := FCoord2Name;
end;

function  TWCS.GetCoord3Name: string;
begin
   result := FCoord3Name;
end;

procedure TWCS.SetCoord1Name(aName: string);
begin
   FCoord1Name := aName;
end;

procedure TWCS.SetCoord2Name(aName: string);
begin
   FCoord2Name := aName;
end;

procedure TWCS.SetCoord3Name(aName: string);
begin
   FCoord3Name := aName;
end;




{TJTSK}


function  TJTSK.GetID: TWCSID;
begin
   result := cs_JTSK;
end;


function TJTSK.GetCoord1Name: string;
begin
  Result:= 'Y'
end;

function TJTSK.GetCoord2Name: string;
begin
  Result:= 'X'
end;

function TJTSK.GetCoord3Name: string;
begin
  Result:= 'Vyska'
end;



procedure TJTSK.GiveICS(const Crd1, Crd2, Crd3: extended;
                  var X, Y, Z: extended);
begin
  X:= -Crd1; Y:= -Crd2; Z:= -Crd3
end;

procedure TJTSK.GiveWCSFromICS(const X, Y, Z: extended;
                         var Crd1, Crd2, Crd3: extended);
begin
  Crd1:= -X; Crd2:= -Y; Crd3:= -Z
end;

procedure TJTSK.GiveWGS84(const Crd1, Crd2, Crd3: extended;
                    var X, Y, Z: extended);
begin
end;

procedure TJTSK.GiveWCSFromWGS84(const X, Y, Z: extended;
                           var Crd1, Crd2, Crd3: extended);
begin
end;



function TJTSK.RecalculateToICS(Coord: TCoordinate): TCoordinate;
begin
   with Coord do
   begin
      X := -X;
      Y := -Y;
      Z := -Z;
   end;
   result := Coord;
end;

function TJTSK.RecalculateFromICS(Coord: TCoordinate): TCoordinate;
begin
   with Coord do
   begin
      X := -X;
      Y := -Y;
      Z := -Z;
   end;
   result := Coord;
end;



{ TWCSLibrary }

{ !!!!!!!!!!!!! zjednodueno !!!!!!!!!!!!!!!!}
function  TWCSLibrary.GetWCSID(aID: TWCSID): TWCS;
var
   WCS: TWCS;
   i: longInt;
begin
   { Hledn }
   WCS := nil;
   for i := 0 to Count -1 do
   begin
      if TWCS(Elements[i]).ID = aID then
      begin
         WCS := TWCS(Elements[i]);
         break;
      end;
   end;
   { vytven }
   if not assigned(WCS) then
   begin
      case aID of
         cs_JTSK:
         begin
             WCS := TJTSK.Create;
         end;
         cs_Gauss:
         begin
             WCS := TGauss_Kruger.Create;
         end;
      end;
      AddElement(WCS);
   end;
   result := WCS;
end;


{ TWCSWorld }

function  TWCSWorld.GetWCSLibrary: TWCSLibrary;
begin
   if not assigned(FWCSLibrary) then
   begin
      FWCSLibrary := TWCSLibrary.Create;
      AddElement(FWCSLibrary);
   end;
   result := FWCSLibrary;
end;

procedure TWCSWorld.SetWCSLibrary(aWCSLibrary: TWCSLibrary);
begin
   if assigned(FWCSLibrary) then
   begin
      FreeElement(FWCSLibrary);
      FWCSLibrary := nil;
   end;
   if assigned(aWCSLibrary) then FWCSLibrary := aWCSLibrary;
end;

function  TWCSWorld.GetIDWCS(aID: TWCSID): TWCS;
begin
   result := WCSLibrary.GetWCSID(aID);
end;




constructor TWCSWorld.Create;
begin
   InitSystemColorPalette;
   ColorPalette := SystemColorPalette;
   inherited;
end;

procedure TWCSWorld.SetColorPalette(const Value: TColorPalette);
begin
   FColorPalette := Value;
   AddElement(Value);
end;

{ TGauss_Kruger }

function TGauss_Kruger.GetCoord1Name: string;
begin
  Result:= 'X'
end;

function TGauss_Kruger.GetCoord2Name: string;
begin
  Result:= 'Y'
end;

function TGauss_Kruger.GetCoord3Name: string;
begin
  Result:= 'Z'
end;

function TGauss_Kruger.GetID: TWCSID;
begin
   result := cs_Gauss;
end;

procedure TGauss_Kruger.GiveICS(const Crd1, Crd2, Crd3: extended; var X, Y,
  Z: extended);
begin
  X:= Crd1; Y:= Crd2; Z:= Crd3
end;

procedure TGauss_Kruger.GiveWCSFromICS(const X, Y, Z: extended; var Crd1,
  Crd2, Crd3: extended);
begin
  Crd1:= X; Crd2:= Y; Crd3:= Z
end;

procedure TGauss_Kruger.GiveWCSFromWGS84(const X, Y, Z: extended; var Crd1,
  Crd2, Crd3: extended);
begin
end;

procedure TGauss_Kruger.GiveWGS84(const Crd1, Crd2, Crd3: extended; var X,
  Y, Z: extended);
begin
end;

function TGauss_Kruger.RecalculateFromICS(Coord: TCoordinate): TCoordinate;
begin
   //with Coord do
   //begin
   //   X := -X;
   //   Y := -Y;
   //   Z := -Z;
   //end;
   result := Coord;
end;

function TGauss_Kruger.RecalculateToICS(Coord: TCoordinate): TCoordinate;
begin
   //with Coord do
   //begin
   //   X := -X;
   //   Y := -Y;
   //   Z := -Z;
   //end;
   result := Coord;
end;

END.
