unit E_files;

interface
uses
  SysUtils,
  Elements;
//  g_Elems;

const

   err_WrongFileName = 11;
   err_FileNotFound  = 12;
   err_FileReadError = 13;

type



TFileElement = class(TOwnerEntity)
private
  FSearchRec: TSearchRec;
  FFileName: {P}String;
  function GetFileName: string;
  function GetFilePath: string;
  function GetFileExt: string;
  function GetFileSize: longInt;
  function GetFileTime: TDateTime;
  function GetFileTimeString: string;
  function GetReadOnly: boolean;
  function GetHidden: boolean;
  function GetSysFile: boolean;
  function GetVolumeID: boolean;
  function GetaDirectory: boolean;
  function GetArchive: boolean;
  function GetAttributes: byte;
  procedure SetFileExt(aValue: string);
  procedure SetFileTime(aValue: TDateTime);
  procedure SetFileTimeString(aValue: string);
  procedure SetReadOnly(aValue: boolean);
  procedure SetHidden(aValue: boolean);
  procedure SetSysFile(aValue: boolean);
  procedure SetArchive(aValue: boolean);
  procedure SetReadInfo(aValue: boolean);
  procedure SetAttributes(aValue: byte);
  procedure SetAttribute(aValue: byte; realy: boolean);
    function GetUpperExt: string;
    function GetUpperName: string;
    function GetFileCheckSum: double;
protected
  function GetExists: boolean; virtual;
  procedure SetFileName(aValue: string); virtual;
  function  GetName: string; override;
  procedure SetName(aName: string); override;
public
  property SearchRec: TSearchRec read FSearchRec write FSearchRec;
published
  property Exists: boolean read GetExists stored false;
  property UpperName: string read GetUpperName stored false;
  property UpperExt: string read GetUpperExt stored false;
  property FileName: string read GetFileName write SetFileName;
  property FilePath: string read GetFilePath stored false;
  property FileExt: string read GetFileExt write SetFileExt stored false;
  property FileSize: longInt      read GetFileSize stored false;
  property FileTime: TDateTime    read GetFileTime       write SetFileTime stored false;
  property FileTimeString: string read GetFileTimeString write SetFileTimeString stored false;
  property FileCheckSum: double   read GetFileCheckSum stored false;
  property Attributes: byte read GetAttributes write SetAttributes stored false;
  property AttrReadOnly: boolean  read GetReadOnly       write SetReadOnly stored false;
  property AttrHidden: boolean    read GetHidden         write SetHidden stored false;
  property AttrSysFile: boolean   read GetSysFile        write SetSysFile stored false;
  property AttrVolumeID: boolean  read GetVolumeID stored false;
  property AttrDirectory: boolean read GetaDirectory stored false;
  property AttrArchive: boolean   read GetArchive        write SetArchive stored false;
  property ReadInfo: boolean write SetReadInfo stored false;
end;


TDirectoryElement = class(TFileElement)
  private
    FDirOnly: boolean;
    FCheckSum: double;
    procedure SetDirOnly(const Value: boolean);
    procedure SetCheckSum(const Value: double);
protected
   function GetExists: boolean; override;
   procedure SetFileName(aValue: string); override;
published
   property DirOnly: boolean read FDirOnly write SetDirOnly;
   property CheckSum: double read FCHeckSum write SetCheckSum;
end;

TDriveType = (Unknown,DontExists,Removable,HardDisk,Network,CDROM,RAM);

TLogicalDriveElement = class(TDirectoryElement)
private
   FActive: boolean;
   FRootName: char;
   function GetDriveTypeValue: TDriveType;
   function GetDriveTypeString: string;
   procedure SetActive(aValue: boolean);
protected
   function  GetName: string; override;
   procedure SetName(aName: string); override;
published
   property Active: boolean read FActive write SetActive stored false;
   property DriveType: TDriveType read GetDriveTypeValue stored false;
   property DriveTypeString: string read GetDriveTypeString stored false;
   property RootName: char read FRootName write FRootName;
end;


TLocalComputerElement = class(TOwnerEntity)
public
   constructor Create; override;
end;


TFileModel = class(TOwnerEntity)
private
   FFileName: string;
protected
   function  GetFileName: string; virtual;
   function  GetPath: string; virtual;
   procedure SetFileName(const aValue: string); virtual;
   procedure SetPath(aValue: string); virtual;
published
   property FileName: string read GetFileName write SetFileName;
   property Path: string read GetPath write SetPath;
end;


TFilePathModel = class(TFileModel)
private
   FPath1: string;
published
   property Path1: string read FPath1 write FPath1 stored true;
end;



TFileBasedEntity = class(TOwnerEntity)
private
   FFileName: string;
   FError: longInt;
protected
   function  GetFileName: string; virtual;
   procedure SetFileName(const aValue: string); virtual;
public
   procedure LoadFromFile(aFileName: string); virtual;
   procedure SaveToFile(aFileName: string); virtual;
published
   property FileName: string read GetFileName write SetFileName;
   property Error: longInt read FError write FError;
end;

{
TBMPFileElement = class(TRectEntity)
private
  FFileName: string;
  function  GetFileName: string;
  procedure SetFileName(FN: string);
published
  property FileName: string read GetFileName write SetFileName;
end;
}

TTextFileElement = class(TFileElement)
end;

TPascalUnitElement = class(TFileElement)
private
   function GetDescription: string;
   function GetVersion: string;
   function GetAdmin: string;
   function GetTechTopic: string;
   function GetTopic: string;
   procedure SetDescription(aValue: string);
   procedure SetVersion(aValue: string);
   procedure SetAdmin(aValue: string);
   procedure SetTechTopic(aValue: string);
   procedure SetTopic(aValue: string);
published
   {
   property InterfaceUses: TOwnerEntity read GetInterfaceUses stored false;
   property ImplementationUses: TOwnerEntity read GetImplementationUses stored false;
   }
   property Description: string read GetDescription write SetDescription;
   property Version: string read GetVersion write SetVersion;
   property Admin: string read GetAdmin write SetAdmin stored false;
   property TechTopic: string read GetTechTopic write SetTechTopic stored false;
   property Topic: string read GetTopic write SetTopic stored false;
end;


TPascalUnits = class(TOwnerEntity)
private
   function GetAdmins: TOwnerEntity;
   function GetTechTopics: TOwnerEntity;
   function GetTopics: TOwnerEntity;
   function GetUnits: TOwnerEntity;
public
   procedure AddUnitFile(aValue: TFileElement);
published
   property Admins: TOwnerEntity read GetAdmins stored false;
   property TechTopics: TOwnerEntity read GetTechTopics stored false;
   property Topics: TOwnerEntity read GetTopics stored false;
   property Units: TOwnerEntity read GetUnits stored false;
   property NextUnit: TFileElement write AddUnitFile stored false;
end;

function GetDirectoryCheckSum(aPath: string): double;


implementation
uses
   Windows,
   System.DateUtils,
   {! t_Main,}
   IniFiles,
   e_Class,
   U_String;


function GetDirectoryCHeckSum(aPath: string): double;
var
   ChkSum: double;
   function CheckSubdirs(aSubPath: string): double;
   var
      S: string;
      SR: TSearchRec;
      Status: integer;
      AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond: Word;
      UN: string;
      EX: string;
   begin
      S := aSubPath;
      CheckBackSlash(S);
      Status := FindFirst(S + '*.*', faAnyFile, SR);
      while Status = 0 do
      begin
        if (SR.Name <> '.') and (SR.Name <> '..') and
           (SR.Attr and faVolumeID = 0) then
        begin
           if (SR.Attr and faDirectory = faDirectory) then
              CheckSubDirs(S + SR.Name);
           UN := UpperCase(SR.Name);
           Ex := ExtractFileExt(UN);
           if (UN <> 'MIND.MIX') and (Ex <> '.TFE') and (Ex <> '.JGE') then
           begin
              DecodeDateTime(SR.TimeStamp, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond);
              ChkSum :=   ChkSum + StringIntSum(UN) + SR.Size + AYear + AMonth + ADay + AHour + AMinute + ASecond;
           end;
        end;
        Status := FindNext(SR);
      end;
   end;
begin
   result := 0;
   ChkSum := 0;
   CheckSubDirs(aPath);
   result := CHkSum;
end;

{ TFileElement }


function TFileElement.GetAttributes: byte;
begin
   result := FSearchRec.Attr;
end;

function TFileElement.GetExists: boolean;
begin
   result := FileExists(FileName);
end;

function TFileElement.GetFileName: string;
begin
   result := FFileName;
   //result := GetPString(FFileName);
end;

function TFileElement.GetFilePath: string;
var
   S: string;
begin
   S := ExtractFilePath(FileName);
   result := CheckBackSlash(S);
end;

function TFileElement.GetFileCheckSum: double;
var
   AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond: Word;
begin
   DecodeDateTime(FileTime, AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond);
   result := FileSize + AYear + AMonth + ADay + AHour + AMinute + ASecond;
end;

function TFileElement.GetFileExt: string;
var
   S: string;
begin
   result := '';
   S := ExtractFileExt(FileName);
   if length(S) <= 1 then exit;
   result := copy(S, 2, Length(S)-1);
end;

function TFileElement.GetFileSize: longInt;
begin
   result := FSearchRec.Size;
end;

function TFileElement.GetFileTime: TDateTime;
begin
   result := FSearchRec.TimeStamp;
//   result := FileDateToDateTime(FSearchRec.Time);
end;

function TFileElement.GetFileTimeString: string;
begin
   result := DateTimeToStr(FileTime);
end;

function TFileElement.GetReadOnly: boolean;
begin
   result := FSearchRec.Attr and faReadOnly > 0;
end;

function TFileElement.GetHidden: boolean;
begin
   result := FSearchRec.Attr and faHidden > 0;
end;

function TFileElement.GetSysFile: boolean;
begin
   result := FSearchRec.Attr and faSysFile > 0;
end;

function TFileElement.GetVolumeID: boolean;
begin
   result := FSearchRec.Attr and faVolumeID > 0;
end;

function TFileElement.GetaDirectory: boolean;
begin
   result := FSearchRec.Attr and faDirectory > 0;
end;

function TFileElement.GetArchive: boolean;
begin
   result := FSearchRec.Attr and faArchive > 0;
end;

function  TFileElement.GetName: string;
begin
   result := ExtractFileName(FileName);
end;

procedure TFileElement.SetFileName(aValue: string);
begin
   FFileName := aValue;
   //SetPString(FFileName, aValue);
end;

procedure TFileElement.SetFileExt(aValue: string);
begin
   if (length(aValue) < 1) or (aValue[1] <> '.') then FileName := ChangeFileExt(FileName, '.' + aValue)
                                                 else FileName := ChangeFileExt(FileName, aValue);
end;


procedure TFileElement.SetFileTime(aValue: TDateTime);
var
   aHandle: integer;
begin
   FSearchRec.Time := DateTimeToFileDate(aValue);
   aHandle := FileOpen(FileName, fmOpenReadWrite);
   FileSetDate(aHandle, FSearchRec.Time);
   FileClose(aHandle);
end;

procedure TFileElement.SetFileTimeString(aValue: string);
begin
   try
      FileTime := StrToDateTime(aValue);
   except
   end;
end;

procedure TFileElement.SetAttributes(aValue: byte);
begin
   FSearchRec.Attr := aValue;
   FileSetAttr(FileName, FSearchRec.Attr);
end;

procedure TFileElement.SetAttribute(aValue: byte; realy: boolean);
begin
   if realy then FSearchRec.Attr := FSearchRec.Attr or aValue
   else
   begin
      FSearchRec.Attr := FSearchRec.Attr and (not aValue)
   end;
   FileSetAttr(FileName, FSearchRec.Attr);
end;


procedure TFileElement.SetReadOnly(aValue: boolean);
begin
   SetAttribute(faReadOnly, aValue);
end;

procedure TFileElement.SetHidden(aValue: boolean);
begin
   SetAttribute(faHidden, aValue);
end;

procedure TFileElement.SetSysFile(aValue: boolean);
begin
   SetAttribute(faSysFile, aValue);
end;

procedure TFileElement.SetArchive(aValue: boolean);
begin
   SetAttribute(faArchive, aValue);
end;

procedure TFileElement.SetName(aName: string);
var
   S: string;
begin
   S := ExtractFilePath(FileName);
   S := CheckBackSlash(S) + aName;
   if RenameFile(FileName, S) then
   begin
      FSearchRec.Name :=  aName;
      FileName := S;
   end;
end;

procedure TFileElement.SetReadInfo(aValue: boolean);
begin
   if aValue then
   begin
      FindFirst(FileName, faAnyFile, FSearchRec);
   end;
end;

function TFileElement.GetUpperExt: string;
begin
   result := AnsiUpperCase(FileExt);
end;

function TFileElement.GetUpperName: string;
begin
   result := AnsiUpperCase(Name);
end;

{ TDirectoryElement }


function TDirectoryElement.GetExists: boolean;
begin
   result := DirectoryExists(FileName);
end;


procedure TDirectoryElement.SetDirOnly(const Value: boolean);
begin
  FDirOnly := Value;
end;

procedure TDirectoryElement.SetFileName(aValue: string);
var
   S: string;
   SR: TSearchRec;
   Status: integer;
   F: TFileElement;
   FU: TPascalUnitElement;
   aDir: boolean;
   UN: string;
   EX: string;
begin
   if FindOwnerClass(TDirectoryElement) = nil then
   begin
      if UpperCase(aValue) = UpperCase(FileName) then exit;
      inherited SetFileName(aValue);
      FreeAll;
      S := FileName;
      CheckBackSlash(S);
      CheckSum := 0;
      if DirOnly then Status := FindFirst(S + '*.*', faDirectory, SR)
                 else Status := FindFirst(S + '*.*', faAnyFile, SR);
      while Status = 0 do
      begin
        if (SR.Name <> '.') and (SR.Name <> '..') and
           (SR.Attr and faVolumeID = 0) then
        begin
           F := nil;
           aDir := (SR.Attr and faDirectory = faDirectory);
           if aDir then
           begin
              F := TDirectoryElement.Create;
              TDirectoryElement(F).DirOnly := DirOnly;
           end
           else
           begin
              if not DirOnly then F := TFileElement.Create;
           end;
           if F <> nil then
           begin
              F.SearchRec := SR;
              F.FileName := S + SR.Name;
              if aDir then
              begin
                 CheckSum := CHeckSum + TDirectoryElement(F).CheckSum;
              end;
              UN  := UpperCase(SR.Name);
              Ex := ExtractFileExt(UN);
              if (UN <> 'MIND.MIX') and (Ex <> '.TFE') and (Ex <> '.JGE') then
                 CheckSum := CheckSum + StringIntSum(UN) + F.FileCheckSum;

              if F.FileExt = 'PAS' then
              begin
                 FU := TPascalUnitElement.Create;
                 FU.FileName := F.FileName;
                 F.Free;
                 AddElement(FU);
              end
              else AddElement(F);
           end;
        end;
        Status := FindNext(SR);
      end;
   end;
end;


procedure TDirectoryElement.SetCheckSum(const Value: Double);
begin
  FCheckSum := Value;
end;

{ TBMPFileElement }

{
function TBMPFileElement.GetFileName: string;
begin
  Result:= FFileName;
end;

procedure TBMPFileElement.SetFileName(FN: string);
begin
  FFileName := FN;
end;

}

{ TFileModel }

function  TFileModel.GetFileName: string;
begin
   Result:= FFileName;
end;

function  TFileModel.GetPath: string;
begin
   Result := ExtractFilePath(FFileName);
end;

procedure TFileModel.SetFileName(const aValue: string);
begin
   FFileName := aValue;
end;

procedure TFileModel.SetPath(aValue: string);
begin
   FFileName := CheckBackSlash(aValue) + ExtractFileName(FFileName);
end;



{ TFileBasedEntity }

function TFileBasedEntity.GetFileName: string;
begin
  Result:= FFileName;
end;

procedure TFileBasedEntity.SetFileName(const aValue: string);
begin
  FFileName := aValue;
  Error := 0;
  if aValue = '' then
  begin
     Error := err_WrongFileName;
     exit;
  end;
  if not FileExists(aValue) then
  begin
     Error := err_FileNotFound;
     exit;
  end;
  try
     LoadFromFile(aValue);
  except
     Error := err_FileReadError;
  end
end;


procedure TFileBasedEntity.LoadFromFile(aFileName: string);
begin
end;

procedure TFileBasedEntity.SaveToFile(aFileName: string);
begin
end;

{ TPascalUnitElement }

function  TPascalUnitElement.GetDescription: string;
var
   Ini: TIniFile;
begin
   Ini := TIniFile.Create(FileName);
   result := Ini.ReadString('Unit_Description','Description','');
   Ini.Free;
end;

function  TPascalUnitElement.GetVersion: string;
var
   Ini: TIniFile;
begin
   Ini := TIniFile.Create(FileName);
   result := Ini.ReadString('Unit_Description','Version','');
   Ini.Free;
end;


function  TPascalUnitElement.GetAdmin: string;
var
   Ini: TIniFile;
begin
   Ini := TIniFile.Create(FileName);
   result := Ini.ReadString('Unit_Description','Admin','');
   Ini.Free;
end;

function  TPascalUnitElement.GetTechTopic: string;
var
   Ini: TIniFile;
begin
   Ini := TIniFile.Create(FileName);
   result := Ini.ReadString('Unit_Description','TechTopic','');
   Ini.Free;
end;

function  TPascalUnitElement.GetTopic: string;
var
   Ini: TIniFile;
begin
   Ini := TIniFile.Create(FileName);
   result := Ini.ReadString('Unit_Description','Topic','');
   Ini.Free;
end;

procedure TPascalUnitElement.SetDescription(aValue: string);
begin
   {!!!}
end;

procedure TPascalUnitElement.SetVersion(aValue: string);
begin
   {!!!}
end;

procedure TPascalUnitElement.SetAdmin(aValue: string);
begin
   {!!!}
end;

procedure TPascalUnitElement.SetTechTopic(aValue: string);
begin
   {!!!}
end;

procedure TPascalUnitElement.SetTopic(aValue: string);
begin
   {!!!}
end;



{ TPascalUnits }

function  TPascalUnits.GetAdmins: TOwnerEntity;
begin
   result := TOwnerEntity(FindElementName('Admins'));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := 'Admins';
      AddElement(result);
   end;
end;

function  TPascalUnits.GetTechTopics: TOwnerEntity;
begin
   result := TOwnerEntity(FindElementName('Tech topics'));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := 'Tech topics';
      AddElement(result);
   end;
end;

function  TPascalUnits.GetTopics: TOwnerEntity;
begin
   result := TOwnerEntity(FindElementName('Topics'));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := 'Topics';
      AddElement(result);
   end;
end;

function  TPascalUnits.GetUnits: TOwnerEntity;
begin
   result := TOwnerEntity(FindElementName('Units'));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := 'Units';
      AddElement(result);
   end;
end;

procedure TPascalUnits.AddUnitFile(aValue: TFileElement);
begin
end;

{ TLogicalDriveElement }

function  TLogicalDriveElement.GetName: string;
begin
   result := FName;
end;

procedure TLogicalDriveElement.SetName(aName: string);
begin
   FName := aName; 
end;



function  TLogicalDriveElement.GetDriveTypeValue: TDriveType;
var
   S: string;
begin
   S := RootName + ':\';
   result := TDriveType(GetDriveType(PChar(S)));
end;

function  TLogicalDriveElement.GetDriveTypeString: string;
begin
   case DriveType of
      Unknown: result := 'UnKnown';
      DontExists: result := 'not found ';
      Removable: begin
                    if RootName in ['A','B'] then
                       result := 'Floppy disk'
                    else
                       result := 'Removable disk';
                 end;
      HardDisk: result := 'HD';
      Network: result := 'REMOTE disk';
      CDROM: result := 'CDROM';
      RAM: result := 'RAM Disk';
   end;
end;

procedure TLogicalDriveElement.SetActive(aValue: boolean);
begin
   if FActive = aValue then exit;
   if not aValue then
   begin
      FreeAll;
   end
   else
   begin
      FileName := RootName + ':\';
   end;
   FActive := aValue;
end;


{ TLocalComputerElement }

constructor TLocalComputerElement.Create;
var
   D,i: longInt;
   Shift: longInt;
   LDE: TLogicalDriveElement;
   Letter: char;
begin
   D := GetLogicalDrives;
   Shift := 1;
   Letter := 'A';
   for i := 0 to 31 do
   begin
      if (D and Shift) > 0 then
      begin
         LDE := TLogicalDriveElement.Create;
         LDE.RootName := Letter;
         LDE.Name := LDE.RootName + ' ' + LDE.DriveTypeString;
         AddElement(LDE);
      end;
      if i < 31 then
      begin
         Shift := Shift shl 1;
         Inc(Letter);
      end;
   end
end;



begin
   RegisterOrbisClass(TFileElement);
   RegisterOrbisClass(TDirectoryElement);
   RegisterOrbisClass(TLogicalDriveElement);
   RegisterOrbisClass(TLocalComputerElement);
   RegisterOrbisClass(TTextFileElement);
//   RegisterOrbisClass(TBMPFileElement);
   RegisterOrbisClass(TFileModel);
   RegisterOrbisClass(TFilePathModel);
   RegisterOrbisClass(TFileBasedEntity);
   RegisterOrbisClass(TPascalUnitElement);
   RegisterOrbisClass(TPascalUnits);
end.
