unit e_Stat;

interface
uses
   Elements;
type



TStatLayerInfo = class(TOwnerElement)
  private
    FIsArea: boolean;
    FIsNew: boolean;
    FNewArea: double;
    FArea: double;
    FCount: integer;
    FNewCount: integer;
    procedure SetArea(const Value: double);
    procedure SetCount(const Value: integer);
    procedure SetIsArea(const Value: boolean);
    procedure SetIsNew(const Value: boolean);
    procedure SetNewArea(const Value: double);
    procedure SetNewCount(const Value: integer);
public
   procedure CreateFor(L: TEntity);
   procedure UpdateFor(L: TEntity);
published
   property IsNew: boolean read FIsNew write SetIsNew;
   property Count: integer read FCount write SetCount;
   property NewCount: integer read FNewCount write SetNewCount;
   property IsArea: boolean read FIsArea write SetIsArea;
   property Area: double read FArea write SetArea;
   property NewArea: double read FNewArea write SetNewArea;
end;

TStatLayerInfos = class(TOwnerEntity)
public
   procedure CreateFor(L: TEntity);
   procedure UpdateFor(L: TEntity);
end;

implementation
uses
   e_Class,
   e_Geometry,
   e_GeometryElement;

{ TStatLayerInfo }

procedure TStatLayerInfo.CreateFor(L: TEntity);
var
   i: integer;
   e: TElement;
   GE: TGeometryElement;
   G: TGeometry;
   A: TAreaElement;
   SA: double;
begin
   IsArea := pos('ARE',L.Name)>0;
   Count := L.Count;
   Area := 0;
   if not IsArea then exit;
   if L.NoElements then exit;
   SA := 0;
   for i:= 0 to L.LastIndex do
   begin
      e := L.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      if not (G is TAreaElement) then continue;
      A := TAreaElement(G);
      SA := SA + A.Area;
   end;
   Area := SA/10000;
end;

procedure TStatLayerInfo.SetArea(const Value: double);
begin
  FArea := Value;
end;

procedure TStatLayerInfo.SetCount(const Value: integer);
begin
  FCount := Value;
end;

procedure TStatLayerInfo.SetIsArea(const Value: boolean);
begin
  FIsArea := Value;
end;

procedure TStatLayerInfo.SetIsNew(const Value: boolean);
begin
  FIsNew := Value;
end;

procedure TStatLayerInfo.SetNewArea(const Value: double);
begin
  FNewArea := Value;
end;

procedure TStatLayerInfo.SetNewCount(const Value: integer);
begin
  FNewCount := Value;
end;

procedure TStatLayerInfo.UpdateFor(L: TEntity);
var
   i: integer;
   e: TElement;
   GE: TGeometryElement;
   G: TGeometry;
   A: TAreaElement;
   SA: double;
begin
   NewCount := L.Count;
   NewArea := 0;
   if not IsArea then exit;
   if L.NoElements then exit;
   SA := 0;
   for i:= 0 to L.LastIndex do
   begin
      e := L.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      if not (G is TAreaElement) then continue;
      A := TAreaElement(G);
      SA := SA + A.Area;
   end;
   NewArea := SA/10000;
end;

{ StatLayerInfos }

procedure TStatLayerInfos.CreateFor(L: TEntity);
var
   i: integer;
   e: TElement;
   en: TEntity;
   SI: TStatLayerInfo;
begin
   if L.NoElements then exit;
   for i := 0 to L.LastIndex do
   begin
      e := L.Elements[i];
      if not (e is TEntity) then continue;
      en := TEntity(e);
      SI := TStatLayerInfo.Create;
      SI.Name := en.Name;
      AddElement(SI);
      SI.CreateFor(en);
   end;
end;

procedure TStatLayerInfos.UpdateFor(L: TEntity);
var
   i: integer;
   e: TElement;
   en: TEntity;
   SI: TStatLayerInfo;
begin
   if L.NoElements then exit;
   for i := 0 to L.LastIndex do
   begin
      e := L.Elements[i];
      if not (e is TEntity) then continue;
      en := TEntity(e);
      SI := TStatLayerInfo(FindElementName(en.Name));
      if SI = nil then
      begin
         SI := TStatLayerInfo.Create;
         SI.Name := en.Name;
         AddElement(SI);
         SI.CreateFor(en);
         SI.IsNew := true;
      end
      else
      begin
         SI.UpdateFor(en);
      end;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TStatLayerInfo);
   RegisterOrbisClass(TStatLayerInfos);
END.

