unit t_FindOkolniSkupiny;

interface
uses
   Elements,
   e_LHCModel,
   e_Space,
   Tools;

type

TFindOkolniSkupiny = class(TTool)
private
   function GetFrame: TPorostniSkupina;
   function GetFrames: TEntity;
   procedure SetFrames(const Value: TEntity);
public
   procedure DoIt; override;
   procedure ProcessArea(anArea: TPolyLineAreaElement; HoleCheck: boolean);
   property  Frame: TPorostniSkupina read GetFrame;
   property  Frames: TEntity read GetFrames write SetFrames;
end;

implementation
uses
   e_Class;
{ TFindOkolniSkupiny }

procedure TFindOkolniSkupiny.DoIt;
var
   Skupina: TPorostniSkupina;
   E: TEntity;
   El: TElement;
   i,j: integer;
   Are, Hole: TPolyLineAreaElement;
begin
   if Input = nil then exit;
   if not (Input is TPorostniSkupina) then exit;
   if Frames = nil then Frames := TOwnerEntity.Create;
   E := Frame.Geometry;
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if not (El is TPolyLineAreaElement) then Continue;
      Are := TPolyLineAreaElement(El);
      ProcessArea(Are,True);
      if Are.Holes <> nil then
      begin
         for j := 0 to Are.Holes.LastIndex do
         begin
            El := Are.Holes.Elements[j];
            if not (El is TPolyLineAreaElement) then Continue;
            Hole := TPolyLineAreaElement(El);
            ProcessArea(Hole,false);
         end;                    
      end;
   end;

end;

function TFindOkolniSkupiny.GetFrame: TPorostniSkupina;
begin
   result := TPorostniSkupina(Input);
end;

function TFindOkolniSkupiny.GetFrames: TEntity;
begin
   result := TEntity(Output);
end;

procedure TFindOkolniSkupiny.ProcessArea(anArea: TPolyLineAreaElement; HoleCheck: boolean);
var
   i,j: integer;
   Lin: TPolyLineElement;
   E : TElement;
   O: TEntity;
   Are: TAreaElement;
   PSK: TPorostniSKupina;
begin
   for i := 0 to anArea.LastIndex do
   begin
      E := anArea.Elements[i];
      if not(E is TPolyLineElement) then Continue;
      Lin := TPolyLineElement(E);
      for j := 0 to Lin.OwnersCount - 1 do
      begin
         O := Lin.Owners[j];
         if not (O is TAreaElement) then Continue;
         Are := TAreaElement(O);
         if Are = anArea then Continue;
         if Are.IsHole then
         begin
            if not HoleCheck then Continue;
            Are := TAreaElement(Are.HoleOwnerArea);
            if Are = nil then Continue;
         end;
         PSK := TPorostniSkupina(Are.FindClassInOwners(TPorostniSkupina));
         if PSK = nil then Continue;
         if PSK = Frame then Continue;
         if Frames.IndexOf(PSK) < 0 then
            Frames.AddElement(PSK);
      end;
   end;
end;

procedure TFindOkolniSkupiny.SetFrames(const Value: TEntity);
begin
   Output := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TFindOkolniSkupiny);
END.
