unit e_SKmap;

interface
uses
   G_Elems,
   e_Geometry,
   e_GeometryElement,
   Elements,
   i_Basic;
type
//                    Max
//      1_____________2
//      |             |       rohy m. listu
//      |             |
//      4_____________3
//    Min


TMap_SK = class(TRectEntity)
  private
    FColumn: integer;
    FQuadrant: integer;
    FSection: integer;
    FRow: integer;
    FY1: double;
    FX4: double;
    FX1: double;
    FX3: double;
    FY2: double;
    FY3: double;
    FX2: double;
    FY4: double;
    FKlad_Code: integer;
    function GetColumn_Text: string;
    function GetQuadrant_Text: string;
    procedure SetColumn(const Value: integer);
    procedure SetColumn_Text(const Value: string);
    procedure SetQuadrant(const Value: integer);
    procedure SetQuadrant_Text(const Value: string);
    procedure SetRow(const Value: integer);
    procedure SetSection(const Value: integer);
    procedure SetX1(const Value: double);
    procedure SetX2(const Value: double);
    procedure SetX3(const Value: double);
    procedure SetX4(const Value: double);
    procedure SetY1(const Value: double);
    procedure SetY2(const Value: double);
    procedure SetY3(const Value: double);
    procedure SetY4(const Value: double);
    function GetMaxX_SK: double;
    function GetMaxY_SK: double;
    function GetMinX_SK: double;
    function GetMinY_SK: double;
    procedure SetKlad_Code(const Value: integer);
    procedure SetByGeometry(const Value: TGeometry);
    function GetFullName: string;
    procedure SetFullName(const Value: string);
protected
   function  GetName: string; override;
   procedure SetName(aName: string); override;
public
   procedure   Update; override;
   function    IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
   function    IsHitByXY(X, Y: double; Tolerance: double): boolean; override;
   function    XY_SK_IsInside(aX,aY: double): boolean;
   function    CreateSection(aSection: integer): TMap_SK;
   procedure   SectionByXY(aX,aY: double);
published
   property FullName: string read GetFullName write SetFullName stored false;
   property Klad_Code: integer read FKlad_Code write SetKlad_Code;
   property Quadrant: integer read FQuadrant write SetQuadrant;
   property Quadrant_Text: string read GetQuadrant_Text write SetQuadrant_Text stored false;
   property Column: integer read FColumn write SetColumn;
   property Column_Text: string read GetColumn_Text write SetColumn_Text stored false;
   property Row: integer read FRow write SetRow;
   property Section: integer read FSection write SetSection;
   property X1: double read FX1 write SetX1;
   property Y1: double read FY1 write SetY1;
   property X2: double read FX2 write SetX2;
   property Y2: double read FY2 write SetY2;
   property X3: double read FX3 write SetX3;
   property Y3: double read FY3 write SetY3;
   property X4: double read FX4 write SetX4;
   property Y4: double read FY4 write SetY4;
   property MinX_SK: double read GetMinX_SK stored false;
   property MinY_SK: double read GetMinY_SK stored false;
   property MaxX_SK: double read GetMaxX_SK stored false;
   property MaxY_SK: double read GetMaxY_SK stored false;
   property ByGeometry: TGeometry write SetByGeometry stored false;
end;

TMaps_SK = class(TRectEntity)
  private
    FKlad_Code: integer;
    FActualMap: TMap_SK;
    FSections: boolean;
    FFindX: double;
    FFindY: double;
    procedure SetKlad_Code(const Value: integer);
    procedure SetUpdateAll(const Value: boolean);
    procedure SetNextMap(const Value: TMap_SK);
    procedure SetActualMap(const Value: TMap_SK);
    procedure SetFindMapName(const Value: string);
    procedure SetResetSections(const Value: boolean);
    procedure SetSections(const Value: boolean);
    procedure SetSort(const Value: boolean);
    procedure SetFindMapXY(const Value: boolean);
    procedure SetFindMapXY_SK(const Value: boolean);
    procedure SetFindX(const Value: double);
    procedure SetFindY(const Value: double);
public
   function GetMapByName(const Value: string): TMap_SK;
   function GetMapByXY(aX,aY: double): TMap_SK;
   function GetMapByXY_SK(aX,aY: double): TMap_SK;
   // Q Quadrant
   // C Column
   // R Row
   // S Section
   function GetQCRSByXY_SK(aX,aY: double; var Q,C,R,S: integer): boolean;
   function GetMapByQXRS(Q,C,R,S: integer): TMap_SK;
published
   property Klad_Code: integer read FKlad_Code write SetKlad_Code; // 0 Gusterberg 1 Sv.Stepan 2 Geler...
   property UpdateAll: boolean write SetUpdateAll stored false;
   property Sort: boolean write SetSort stored false;
   property NextMap: TMap_SK write SetNextMap stored false;
   property ActualMap: TMap_SK read FActualMap write SetActualMap stored false ;
   property FindMapName: string write SetFindMapName stored false;
   property ResetSections: boolean write SetResetSections stored false;
   property Sections: boolean read FSections write SetSections stored false;
   property FindX: double read FFindX write SetFindX stored false;
   property FindY: double read FFindY write SetFindY stored false;
   property FindMapXY: boolean write SetFindMapXY stored false;
   property FindMapXY_SK: boolean write SetFindMapXY_SK stored false;
end;

TInterpretMap_SK = class(TRectInterpret)
  private
    FShowSections: boolean;
    FNoSelect: boolean;
    FColor: integer;
    FSectionsColor: integer;
    procedure SetColor(const Value: integer);
    procedure SetNoSelect(const Value: boolean);
    procedure SetSectionsColor(const Value: integer);
    procedure SetShowSections(const Value: boolean);
protected
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
published
   property ShowSections: boolean read FShowSections write SetShowSections;
   property Color: integer read FColor write SetColor;
   property SectionsColor: integer read FSectionsColor write SetSectionsColor;
   property NoSelect: boolean read FNoSelect write SetNoSelect;
end;

implementation
uses
   Graphics,
   SysUtils,
   AnalInt,
   e_Class,
   t_Sort,
   u_String;

const
   Sah=1.896484;

{ TMap_SK }

function TMap_SK.CreateSection(aSection: integer): TMap_SK;
var
   dx1,dy1, dx2, dy2: double;
   aX1,aY1,aX2,aY2: double;
   c,r,i: integer;
begin
   result := nil;
   if (aSection < 1) or (aSection > 20) then exit;
   result := TMap_SK.Create;
   result.Name := Name;
   result.Section := aSection;
   r := (aSection-1) div 4;
   c := aSection-(r*4) - 1;
   // sloupec
   dx1 := (X2-X1)/4;
   dy1 := (Y2-Y1)/4;
   dx2 := (X3-X4)/4;
   dy2 := (Y3-Y4)/4;
   result.X1 := X1+c*dx1;
   result.Y1 := Y1+c*dy1;
   result.X2 := result.X1+dx1;
   result.Y2 := result.Y1+dy1;
   result.X4 := X4+c*dx2;
   result.Y4 := Y4+c*dy2;
   result.X3 := result.X4+dx2;
   result.Y3 := result.Y4+dy2;
   // radek
   dx1 := (result.X4-result.X1)/5;
   dy1 := (result.Y4-result.Y1)/5;
   dx2 := (result.X3-result.X2)/5;
   dy2 := (result.Y3-result.Y2)/5;

   result.X1 := result.X1+r*dx1;
   result.Y1 := result.Y1+r*dy1;
   result.X4 := result.X1+dx1;
   result.Y4 := result.Y1+dy1;
   result.X2 := result.X2+r*dx2;
   result.Y2 := result.Y2+r*dy2;
   result.X3 := result.X2+dx2;
   result.Y3 := result.Y2+dy2;
   result.Update;
end;

function TMap_SK.GetColumn_Text: string;
begin
   result := IntegerToRome(Column);
   if result = '' then
      result := '?';
end;

function TMap_SK.GetFullName: string;
begin
   result := Name;
   if Section > 0 then
      result :=result + '-' + IntToStr(Section);
end;

function TMap_SK.GetMaxX_SK: double;
begin
   result := 0;
   case Quadrant of
      1,4: result := Column  * 4000 * Sah;
      2,3: result := (Column - 1) * -4000 * Sah;
   end;
end;

function TMap_SK.GetMaxY_SK: double;
begin
   result := MinY_SK + 4000 * Sah;
end;

function TMap_SK.GetMinX_SK: double;
begin
   result := 0;
   case Quadrant of
      1,4: result := (Column - 1) * 4000 * Sah;
      2,3: result := Column * -4000 * Sah;
   end;
end;

function TMap_SK.GetMinY_SK: double;
var
   StartRow: integer;
begin
   StartRow := 45;
   Case Klad_Code of
      0: StartRow := 45;
      1: StartRow := 33;
      2: StartRow := 32;
   end;
   result := (StartRow - Row) * 4000 * Sah;
end;

function TMap_SK.GetName: string;
begin
   result := Quadrant_Text + '-' + Column_Text + '-' + IntToStr(Row);
end;

function TMap_SK.GetQuadrant_Text: string;
begin
   case Quadrant of
      1: result := 'VS';
      2: result := 'ZS';
      3: result := 'ZJ';
      4: result := 'VJ';
   else
      result := '?';
   end;
end;

function TMap_SK.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
   result := false;
   if CoordEl = nil then exit;
   result := IsHitByXY(CoordEl.X, CoordEl.Y, Tolerance);
end;

function TMap_SK.IsHitByXY(X, Y, Tolerance: double): boolean;
var
   Corns: TCorns;
begin
   Corns[1].X := X1;
   Corns[2].X := X2;
   Corns[3].X := X4;
   Corns[4].X := X3;
   Corns[1].Y := Y1;
   Corns[2].Y := Y2;
   Corns[3].Y := Y4;
   Corns[4].Y := Y3;
   result := InsideCorns(X,Y,Corns);
end;

procedure TMap_SK.SectionByXY(aX, aY: double);
var
   dx1,dy1, dx2, dy2: double;
   aX1,aY1,aX2,aY2: double;
   c,r,i: integer;
begin
   Section := 0;
   // sloupec
   dx1 := (X2-X1)/4;
   dy1 := (Y2-Y1)/4;
   dx2 := (X3-X4)/4;
   dy2 := (Y3-Y4)/4;
   c := 0;
   for i := 1 to 4 do
   begin
      aX1 := X1+i*dx1;
      aY1 := Y1+i*dy1;
      aX2 := X4+i*dx2;
      aY2 := Y4+i*dy2;
      if JeVlevoOdPrimky(aX2,aY2,aX1,aY1,aX,aY) then  // 1 a 2 prehozeno .. zdola nahoru
      begin
         c := i;
         break;
      end;
   end;
   if c = 0 then exit; // haha sloupec nenalezen
   // radek
   dx1 := (X4-X1)/5;
   dy1 := (Y4-Y1)/5;
   dx2 := (X3-X2)/5;
   dy2 := (Y3-Y2)/5;
   r := 0;
   for i := 1 to 5 do
   begin
      aX1 := X1+i*dx1;
      aY1 := Y1+i*dy1;
      aX2 := X2+i*dx2;
      aY2 := Y2+i*dy2;
      if JeVlevoOdPrimky(aX1,aY1,aX2,aY2,aX,aY) then
      begin
         r := i;
         break;
      end;
   end;
   if r = 0 then exit; // hahahaha radek nenalezen
   Section := (r-1)*4+c
end;

procedure TMap_SK.SetByGeometry(const Value: TGeometry);
var
   PE: TPolyElement;
   i: integer;
   aX,aY: double;
begin
   if not (Value is TPolyElement) then exit;
   PE :=TPolyELement(Value);
   Y1 := -5000000;
   X2 := -5000000;
   Y3 := 0;
   X4 := 0;
   for i := 0 to PE.CoordsCount - 1 do
   begin
      aX := PE.Xi[i];
      aY := PE.Yi[i];
      if aY > Y1 then
      begin
         X1 := aX;
         Y1 := aY;
      end;
      if aX > X2 then
      begin
         X2 := aX;
         Y2 := aY;
      end;
      if aY < Y3 then
      begin
         X3 := aX;
         Y3 := aY;
      end;
      if aX < X4 then
      begin
         X4 := aX;
         Y4 := aY;
      end;
   end;
   Update;
end;

procedure TMap_SK.SetColumn(const Value: integer);
begin
  FColumn := Value;
end;

procedure TMap_SK.SetColumn_Text(const Value: string);
begin
   Column := RomeToInteger(Value);
end;

procedure TMap_SK.SetFullName(const Value: string);
begin
   Name := Value;
end;

procedure TMap_SK.SetKlad_Code(const Value: integer);
begin
  FKlad_Code := Value;
end;

procedure TMap_SK.SetName(aName: string);
var
   S: string;
   i: integer;
   Delimiter: char;
begin
   i := 1;
   Quadrant := 0;
   Column := 0;
   Row := 0;
   Section := 0;
   S := ReadToDelimiterWithoutWS(aName,['-'],i,Delimiter,i);
   if S = '' then exit;
   Quadrant_Text := S;
   if Delimiter <> '-' then exit;
   S := ReadToDelimiterWithoutWS(aName,['-'],i,Delimiter,i);
   if S = '' then exit;
   Column_Text := S;
   if Delimiter <> '-' then exit;
   S := ReadToDelimiterWithoutWS(aName,['-'],i,Delimiter,i);
   if S = '' then exit;
   Row := StrToIntDef(S,0);
   if Delimiter <> '-' then exit;
   S := ReadToDelimiterWithoutWS(aName,['-'],i,Delimiter,i);
   if S = '' then exit;
   Section := StrToIntDef(S,0);
end;

procedure TMap_SK.SetQuadrant(const Value: integer);
begin
  FQuadrant := Value;
end;

procedure TMap_SK.SetQuadrant_Text(const Value: string);
var
   S: string;
begin
   Quadrant := 0;
   if length(Value) < 2 then exit;
   S := UpperCase(Value);
   case S[1] of
      'Z':
      case S[2] of
         'S': Quadrant := 2;
         'J': Quadrant := 3;
      end;
      'V':
      case S[2] of
         'S': Quadrant := 1;
         'J': Quadrant := 4;
      end;
   end;
end;

procedure TMap_SK.SetRow(const Value: integer);
begin
  FRow := Value;
end;

procedure TMap_SK.SetSection(const Value: integer);
begin
  FSection := Value;
end;

procedure TMap_SK.SetX1(const Value: double);
begin
  FX1 := Value;
end;

procedure TMap_SK.SetX2(const Value: double);
begin
  FX2 := Value;
end;

procedure TMap_SK.SetX3(const Value: double);
begin
  FX3 := Value;
end;

procedure TMap_SK.SetX4(const Value: double);
begin
  FX4 := Value;
end;

procedure TMap_SK.SetY1(const Value: double);
begin
  FY1 := Value;
end;

procedure TMap_SK.SetY2(const Value: double);
begin
  FY2 := Value;
end;

procedure TMap_SK.SetY3(const Value: double);
begin
  FY3 := Value;
end;

procedure TMap_SK.SetY4(const Value: double);
begin
  FY4 := Value;
end;

procedure TMap_SK.Update;
begin
   MinX := X4;
   MinY := Y3;
   MaxX := X2;
   MaxY := Y1;
end;

function TMap_SK.XY_SK_IsInside(aX, aY: double): boolean;
begin
   result:= (aX >= MinX_SK) and (aX <= MaxX_SK) and
            (aY >= MinY_SK) and (aY <= MaxY_SK);
end;

{ TMaps_SK }

function TMaps_SK.GetMapByName(const Value: string): TMap_SK;
var
   S: string;
   i: integer;
   Delimiter: char;
   Quad,Col,Row,Section : integer;
begin
   result := nil;
   ActualMap := nil;
   i := 1;
   S := ReadToDelimiterWithoutWS(Value,['-'],i,Delimiter,i);
   if S = '' then exit;
   Quad := 0;
   if length(S) >= 2 then
   begin
      S := UpperCase(Value);
      case S[1] of
         'Z':
         case S[2] of
            'S': Quad := 2;
            'J': Quad := 3;
         end;
         'V':
         case S[2] of
            'S': Quad := 1;
            'J': Quad := 4;
         end;
      end;
   end;
   if Delimiter <> '-' then exit;
   S := ReadToDelimiterWithoutWS(Value,['-'],i,Delimiter,i);
   if S = '' then exit;
   Col := RomeToInteger(S);
   if Delimiter <> '-' then exit;
   S := ReadToDelimiterWithoutWS(Value,['-'],i,Delimiter,i);
   if S = '' then exit;
   Row := StrToIntDef(S,0);
   Section := 0;
   if Delimiter = '-' then
   begin
      S := ReadToDelimiterWithoutWS(Value,['-'],i,Delimiter,i);
      if S = '' then
         Section := 0
      else
         Section := StrToIntDef(S,0);
   end;
   result := GetMapByQXRS(Quad,Col,Row,Section);
end;

function TMaps_SK.GetMapByQXRS(Q, C, R, S: integer): TMap_SK;
var
   Q1,C1,R1: integer;
   Qi,Ci,Ri: integer;
   QE,CE: TRectEntity;
   M: TMap_SK;
begin
   M := ActualMap;
   if M <> nil then
   begin
      if (M.Quadrant = Q) and (M.Column = C) and (M.Row = R) then
      begin
         result := M;
         M.Section := S;
         exit;
      end;
   end;
   result := nil;
   ActualMap := nil;
   Q1 := TRectEntity(Elements[0]).Index;
   Qi := Q - Q1;
   if (Qi < 0) or (Qi > LastIndex) then
      exit;
   QE := TRectEntity(Elements[Qi]);

   C1 := TRectEntity(QE.Elements[0]).Index;
   Ci := C - C1;
   if (Ci < 0) or (Ci > QE.LastIndex) then
      exit;
   CE := TRectEntity(QE.Elements[Ci]);

   R1 := TMap_SK(CE.Elements[0]).Row;
   Ri := R - R1;
   if (Ri < 0) or (Ri > CE.LastIndex) then
      exit;
   M := TMap_SK(CE.Elements[Ri]);
   M.Section := S;
   result := M;
   ActualMap := M;
end;

function TMaps_SK.GetMapByXY(aX, aY: double): TMap_SK;
var
   QE,CE: TRectEntity;
   M: TMap_SK;
   //E: TElement;
   iQ,iC,iM: integer;
begin
   M := ActualMap;
   if M <> nil then
   if M.IsHitByXY(aX,aY,0) then
   begin
      result := M;
      if Sections then
      begin
         M.SectionByXY(aX,aY);
         // nastaveni sekce podle souradnic
      end;
      exit;
   end;
   result := nil;
   ActualMap := nil;
   if PointIsInside(aX, aY) then
   for iQ := 0 to LastIndex do
   begin
      QE := TRectEntity(Elements[iQ]);
      if QE.PointIsInside(aX, aY) then
      for iC := 0 to QE.LastIndex do
      begin
         CE := TRectEntity(QE.Elements[iC]);
         if CE.PointIsInside(aX,aY) then
         for iM := 0 to CE.LastIndex do
         begin
            M := TMap_SK(CE.Elements[iM]);
            if M.IsHitByXY(aX,aY,0) then
            begin
               result := M;
               ActualMap := M;
               if Sections then
               begin
                  // nastaveni sekce podle souradnic
                  M.SectionByXY(aX,aY);
               end;
               exit;
            end;
         end;
      end;
   end;
end;

function TMaps_SK.GetMapByXY_SK(aX, aY: double): TMap_SK;
var
   M: TMap_SK;
   Q,C,R,S: integer;
begin
   M := ActualMap;
   if M <> nil then
   begin
      if M.XY_SK_IsInside(aX,aY) then
      begin
         result := M;
         exit;
      end;
   end;
   GetQCRSByXY_SK(aX,aY,Q,C,R,S);
   result := GetMapByQXRS(Q,C,R,S);
end;

function TMaps_SK.GetQCRSByXY_SK(aX, aY: double; var Q, C, R, S: integer): boolean;
var
   XX,YY: double;
   StartRow: integer;
   dX,dY: double;
begin
   StartRow := 45;
   case Klad_Code of
      0: StartRow := 45;
      1: StartRow := 33;
      2: StartRow := 32;
   end;
   result := false;
   XX := Abs(aX);
   YY := Abs(aY);
   if aX < 0 then
   begin
      if aY < 0 then
         Q := 3
      else
         Q := 2;
   end
   else
   begin
      if aY < 0 then
         Q := 4
      else
         Q := 1;
   end;
   C := Trunc(XX / (4000 * Sah)) + 1;
   R := StartRow - Trunc(YY / (4000 * Sah));
   if Sections then
   begin
   end
   else
      S := 0;
   result := true;
end;

procedure TMaps_SK.SetActualMap(const Value: TMap_SK);
begin
  FActualMap := Value;
end;

procedure TMaps_SK.SetFindMapName(const Value: string);
begin
   GetMapByName(Value);
end;

procedure TMaps_SK.SetFindMapXY(const Value: boolean);
begin
   if not Value then exit;
   GetMapByXY(FindX,FindY);
end;

procedure TMaps_SK.SetFindMapXY_SK(const Value: boolean);
begin
   if not Value then exit;
   GetMapByXY_SK(FindX,FindY);
end;

procedure TMaps_SK.SetKlad_Code(const Value: integer);
begin
  FKlad_Code := Value;
end;

procedure TMaps_SK.SetNextMap(const Value: TMap_SK);
var
   E: TElement;
   QE,CE: TRectEntity;
begin
   if Value = nil then exit;
   E := Value;
   if not (E is TMap_SK) then exit;
   QE := TRectEntity(FindOrCreateElementName(TRectEntity,Value.Quadrant_Text));
   if QE = nil then exit;
   QE.Index := Value.Quadrant;
   CE := TRectEntity(QE.FindOrCreateElementName(TRectEntity,Value.Column_Text));
   if CE = nil then exit;
   CE.Index := Value.Column;
   CE.AddElement(Value);
   Value.Klad_Code := Klad_Code;
end;

procedure TMaps_SK.SetResetSections(const Value: boolean);
var
   QE,CE: TRectEntity;
   M: TMap_SK;
   iQ,iC,iM: integer;
begin
   if not Value then exit;
   for iQ := 0 to LastIndex do
   begin
      QE := TRectEntity(Elements[iQ]);
      for iC := 0 to QE.LastIndex do
      begin
         CE := TRectEntity(QE.Elements[iC]);
         for iM := 0 to CE.LastIndex do
         begin
            M := TMap_SK(CE.Elements[iM]);
            M.Section := 0;
         end;
      end;
   end;
end;

procedure TMaps_SK.SetSections(const Value: boolean);
begin
  FSections := Value;
end;

procedure TMaps_SK.SetSort(const Value: boolean);
var
   QE,CE: TRectEntity;
   iQ,iC: integer;
   SE: TSortElementsByInteger;
   PropName: string;
begin
   if not Value then exit;
   PropName := 'Index';
   SE := TSortElementsByInteger.Create;
   SE.Input := Self;
   SE.PropertyName := PropName;
   SE.DoIt;
   for iQ := 0 to LastIndex do
   begin
      QE := TRectEntity(Elements[iQ]);
      SE.Input := QE;
      SE.PropertyName := PropName;
      SE.DoIt;
      for iC := 0 to QE.LastIndex do
      begin
         CE := TRectEntity(QE.Elements[iC]);
         SE.Input := CE;
         SE.PropertyName := 'Row';
         SE.DoIt;
      end;
   end;
   SE.Free;
end;

procedure TMaps_SK.SetUpdateAll(const Value: boolean);
var
   QE,CE,ME: TRectEntity;
   //E: TElement;
   iQ,iC,iM: integer;
begin
   for iQ := 0 to LastIndex do
   begin
      QE := TRectEntity(Elements[iQ]);
      for iC := 0 to QE.LastIndex do
      begin
         CE := TRectEntity(QE.Elements[iC]);
         for iM := 0 to CE.LastIndex do
         begin
            ME := TRectEntity(CE.Elements[iM]);
            ME.Update;
         end;
         CE.Update;
      end;
      QE.Update;
   end;
   Update;
end;

procedure TMaps_SK.SetFindX(const Value: double);
begin
  FFindX := Value;
end;

procedure TMaps_SK.SetFindY(const Value: double);
begin
  FFindY := Value;
end;

{ TInterpretMap_SK }

function TInterpretMap_SK.IsItsClass(AObject: TObject): boolean;
begin
   Result := (AObject.ClassType = TMaps_SK) or (AObject.ClassType = TMap_SK);
end;

procedure TInterpretMap_SK.Process(Element: TElement);
var
   M: TMap_SK;
   MS: TMaps_SK;
   A: TAreaElement;
   dx1,dy1, dx2, dy2: double;
   aX1,aY1,aX2,aY2: double;
   c,r,i: integer;
begin
   if Element.ClassType = TMaps_SK then
   begin
      if not NoSelect then
      begin
         MS := TMaps_SK(Element);
         if MS.ActualMap = nil then
            exit;
         M := MS.ActualMap;
         A := TAreaElement.Create;
         A.AddXY(M.X1,M.Y1);
         A.AddXY(M.X2,M.Y2);
         A.AddXY(M.X3,M.Y3);
         A.AddXY(M.X4,M.Y4);
         A.AddXY(M.X1,M.Y1);
         A.Update;
         Device.Brush.Color:= 255;
         Device.Pen.Style:= psClear;
         Device.PolyPolygon(A);
         A.Free;
      end;
      exit;
   end;
   if not (Element.ClassType = TMap_SK) then
      exit;
   M := TMap_SK(Element);
   if not M.IsOverRegion(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY) then
      exit;
   if ShowSections then
   begin
      Device.Pen.Style:= psSolid;
      Device.Pen.Color := SectionsColor;
      Device.Pen.Width := 1;
      // sloupec
      dx1 := (M.X2-M.X1)/4;
      dy1 := (M.Y2-M.Y1)/4;
      dx2 := (M.X3-M.X4)/4;
      dy2 := (M.Y3-M.Y4)/4;
      c := 0;
      for i := 1 to 3 do
      begin
         aX1 := M.X1+i*dx1;
         aY1 := M.Y1+i*dy1;
         aX2 := M.X4+i*dx2;
         aY2 := M.Y4+i*dy2;
         Device.MoveTo(aX1,aY1);
         Device.LineTo(aX2,aY2);
      end;
      // radek
      dx1 := (M.X4-M.X1)/5;
      dy1 := (M.Y4-M.Y1)/5;
      dx2 := (M.X3-M.X2)/5;
      dy2 := (M.Y3-M.Y2)/5;
      for i := 1 to 4 do
      begin
         aX1 := M.X1+i*dx1;
         aY1 := M.Y1+i*dy1;
         aX2 := M.X2+i*dx2;
         aY2 := M.Y2+i*dy2;
         Device.MoveTo(aX1,aY1);
         Device.LineTo(aX2,aY2);
      end;
   end;
   Device.Pen.Style:= psSolid;
   Device.Pen.Color := Color;
   Device.Pen.Width := 1;
   Device.MoveTo(M.X1, M.Y1);
   Device.LineTo(M.X2, M.Y2);
   Device.LineTo(M.X3, M.Y3);
   Device.LineTo(M.X4, M.Y4);
   Device.LineTo(M.X1, M.Y1);
end;

function TInterpretMap_SK.QProcessBefore(Element: TElement): boolean;
begin
   Result := (Element.ClassType = TMaps_SK) or (Element.ClassType = TMap_SK);
end;

procedure TInterpretMap_SK.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TInterpretMap_SK.SetNoSelect(const Value: boolean);
begin
  FNoSelect := Value;
end;

procedure TInterpretMap_SK.SetSectionsColor(const Value: integer);
begin
  FSectionsColor := Value;
end;

procedure TInterpretMap_SK.SetShowSections(const Value: boolean);
begin
  FShowSections := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMap_SK);
   RegisterOrbisClass(TMaps_SK);
   RegisterOrbisClass(TInterpretMap_SK);
END.
