unit E_view;
{****************************************************************************}
{                                                                            }
{          ORBIS ... View - okno z jednoho svta do druhho                  }
{                    ViewProjection - projekce okna                          }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface

uses
// KERNEL
   Elements,
   E_Type,
// GLIB
   g_Elems,
// DEVICE
   e_Proje,
// GISLAYER
   E_LayScr;

type



TView = class;

TViewProjection = class(TProjection)
private
   FView: TView;
   function  GetModelView: TDoubleRect; override;
   procedure SetCenterPoint(const aCenterPoint: TDoubleCoord); override;
   procedure SetCenterX(const aValue: double); override;
   procedure SetCenterY(const aValue: double); override;
   procedure SetScale(const aScale: double); override;
   procedure SetModelView(const aRect: TDoubleRect); override;
   procedure SetView(const aView: TView);
public
   procedure ChangeProjection (const aCenterPoint: TDoubleCoord; const aScale: double;
                               const aView: TView);
   procedure ChangeProjectionByRect (const aModelRect: TDoubleRect;const aView: TView);
   function  GetViewX(const ModelX: double): double;
   function  GetViewY(const ModelY: double): double;
   function  GetModelX(const ViewX: double): double;
   function  GetModelY(const ViewY: double): double;
   function  GetViewLength(const ModelLength: double): double;
   function  GetViewPoint(const ModelX, ModelY: double): TDoubleCoord;
   function  GetViewRect(const MinX, MinY, MaxX, MaxY: double): TDoubleRect;
   function  GetModelPoint(const ViewX, ViewY: double): TDoubleCoord;
   function  GetModelRect(const MinX, MinY, MaxX, MaxY: double): TDoubleRect;

   property  View: TView read FView write SetView;
end;



Tview = class(TDrawRectangle)
private
   FControlViewName: string; { jmno View, na kterm je tento view zvisl }
   FDeltaX, FDeltaY: double;
   FReadAfter: boolean;
    FDataDirectory: string;
    FNoMIND: boolean;

   function  GetControlView: TView;

   function GetModelRectBoundary: TDoubleRect;
   function  GetRealCenterX: double;
   function  GetRealCenterY: double;
   function  GetScale  : double;
   function  GetVirtualScale  : double;
   procedure SetRealCenterX(aValue: double);
   procedure SetRealCenterY(aValue: double);

   function GetMinX: double;
   function GetMaxX: double;
   function GetMinY: double;
   function GetMaxY: double;

   procedure SetScale  (aValue: double);
   procedure SetVirtualScale  (aValue: double);
   function  GetProjection: TViewProjection;
    function GetAngle: double;
    procedure SetAngle(const Value: double);
    function GetRotateCenterX: double;
    function GetRotateCenterY: double;
    procedure SetRotateCenterX(const Value: double);
    procedure SetRotateCenterY(const Value: double);
    procedure SetDataDirectory(const Value: string);
    procedure SetNoMIND(const Value: boolean);
//   procedure GraphicsReader(aType, aPath: string; Props: TPropertyList);

 protected
   FModel: TElement;
   FProjection: TViewProjection;
   function  CreateModel: TRectEntity; virtual;
   procedure SetModel(aModel: TElement);
   procedure SetProjection(aProjection: TViewProjection);
   procedure DoReadGraphics(aB: boolean); virtual;
   function  ReadElement(ParStr: string;  Props: TPropertyList): TElement;
 public
   constructor Create; override;
   procedure AddToModelScript(aElement: TElement);
   procedure AddToModel(Element: TElement);
   function  GetModelScript: TModelScript;
   procedure ResetBoundary;
   property Projection: TViewProjection read GetProjection write SetProjection;
   property ModelRectBoundary: TDoubleRect read GetModelRectBoundary;
   procedure ReadGraphics;
 published
   property Model: TElement read FModel write SetModel stored false;
   property ModelScript: TModelScript read GetModelScript stored false;
   property ModelCenterX: double read GetRealCenterX write SetRealCenterX;
   property ModelCenterY: double read GetRealCenterY write SetRealCenterY;
   property ModelMinX: double read GetMinX stored false;
   property ModelMaxX: double read GetMaxX stored false;
   property ModelMinY: double read GetMinY stored false;
   property ModelMaxY: double read GetMaxY stored false;
   property Scale: double   read GetScale   write SetScale;

   property Angle: double   read GetAngle write SetAngle;
   property RotateCenterX: double   read GetRotateCenterX write SetRotateCenterX;
   property RotateCenterY: double   read GetRotateCenterY write SetRotateCenterY;
   property VirtualScale: double read GetVirtualScale write SetVirtualScale;
   property ControlViewName: string read FControlViewName write FControlViewName;
   property ModelDeltaX: double read FDeltaX write FDeltaX;
   property ModelDeltaY: double read FDeltaY write FDeltaY;
   property ReadModel: boolean write DoReadGraphics stored false;
   property ReadAfter: boolean read FReadAfter write FReadAfter;
   property DataDirectory: string read FDataDirectory write SetDataDirectory;
   property NoMIND: boolean read FNoMIND write SetNoMIND;
end;


implementation
uses
  Classes,
  Graphics,
  SysUtils, FileCtrl,
// KERNEL
  e_Class,
  U_File,
  u_Props,
  U_String,
// TOPOL
  T_TopImp,
//GISLAYER
  T_LmCre;

{$IFDEF BINRAS}
  u_Ras, E_TopRas,
{$ENDIF}
//  E_TopMdl,
  //T_PZ,
  {!T_Main,}





{ TView }

constructor TView.Create;
begin
   inherited Create;
   FProjection := TViewProjection.Create;
   AddElement(FProjection);
   FProjection.View:= self;
   BrushColor:= clWhite;
   BrushStyle:= bsClear;
   PenStyle:= psDot;
   PenColor:= clSilver;
end;





function TView.ReadElement(ParStr: string;  Props: TPropertyList): TElement;
var
  I: integer;
  PropName: string;
begin
  result:= CreateElementByName(ParStr);
  if result = nil then exit;

  if Props <> nil then
  begin
    for I:= 0 to Props.Count-1 do
    begin
      PropName:= Props.Names[I];
      if PropName[1] = '@' then
         PropName:= copy(PropName, 2, 255);
      SetPropStringByName(result, PropName, Props.Strings[I]);
    end
  end
end;



procedure TView.AddToModel(Element: TElement);
var
  aModel: TElement;
begin
  aModel:= self.Model;
  if aModel = nil then
  begin
     aModel:= TRectEntity.Create;
     aModel.Name:= 'VIEWMODEL'
  end;

  if (aModel is TRectEntity) then
     TRectEntity(aModel).AddAndUpdateRect(Element)
  else
  if aModel is TEntity then
       TEntity(aModel).AddElement(Element);

  self.Model:= aModel
end;





procedure TView.SetModel(aModel: TElement);
begin
   if assigned(FModel) then RemoveElement(FModel);
   FModel := nil;
   FModel := aModel;
   AddElement(FModel);
end;

procedure Tview.SetNoMIND(const Value: boolean);
begin
  FNoMIND := Value;
end;

function  TView.GetProjection: TViewProjection;
begin
  if FProjection = nil then
  begin
    FProjection := TViewProjection.Create;
    AddElement(FProjection);
    FProjection.View:= self
  end;
  result:= FProjection
end;


function  TView.GetControlView: TView;
begin
  result:= nil;
  if ControlViewName = '' then exit;
  if OwnersCount = 0 then exit;
  result:= TView(Owners[0].FindElementName(ControlViewName))
end;

function  TView.GetRealCenterX: double;
begin
  if GetControlView = nil then
     result:= Projection.Center_X+ModelDeltaX
  else
  begin
     result:= GetControlView.ModelCenterX+ModelDeltaX;
     Projection.Center_X:= result
  end
end;

function  TView.GetRealCenterY: double;
begin
  if GetControlView = nil then
     result:= Projection.Center_Y+ModelDeltaY
  else
  begin
     result:= GetControlView.ModelCenterY+ModelDeltaY;
     Projection.Center_Y:= result
  end
end;


function  TView.GetScale  : double;
begin
  result:= Projection.Scale
end;

function  TView.GetVirtualScale: double;
begin
  result:= Projection.VirtualScale
end;

procedure TView.SetRealCenterX(aValue: double);
begin
  Projection.Center_X:= aValue
end;

procedure TView.SetRealCenterY(aValue: double);
begin
  Projection.Center_Y:= aValue
end;

function TView.GetMinX: double;
var
  DR: TDoubleRect;
begin
  DR:= GetModelRectBoundary;
  Result:= DR.MinX
end;

function TView.GetMaxX: double;
var
  DR: TDoubleRect;
begin
  DR:= GetModelRectBoundary;
  Result:= DR.MaxX
end;

function TView.GetMinY: double;
var
  DR: TDoubleRect;
begin
  DR:= GetModelRectBoundary;
  Result:= DR.MinY
end;


function TView.GetMaxY: double;
var
  DR: TDoubleRect;
begin
  DR:= GetModelRectBoundary;
  Result:= DR.MaxY
end;


procedure TView.SetScale  (aValue: double);
begin
  Projection.Scale:= aValue
end;

procedure TView.SetVirtualScale  (aValue: double);
begin
  Projection.VirtualScale:= aValue
end;




procedure TView.SetProjection(aProjection: TViewProjection);
begin
   if assigned(FProjection) then RemoveElement(FProjection);
   FProjection := nil;
   FProjection := aProjection;
   if assigned(FProjection) then
   begin
      AddElement(FProjection);
      FProjection.View := Self;
   end;
end;


function TView.GetModelRectBoundary: TDoubleRect;
begin
  result.MinX:= 0;
  result.MinY:= 0;
  result.MaxX:= 0;
  result.MaxY:= 0;
  if Scale = 0 then exit;
  Projection.Center_X:= ModelCenterX;
  Projection.Center_Y:= ModelCenterY;
  result:= Projection.GetModelRect(0, 0, MaxX-MinX, MaxY-MinY)
  //result:= Projection.GetModelRect(Minx, Miny, MaxX, MaxY);
end;

procedure TView.ResetBoundary;
begin
  GetModelRectBoundary
end;



function  TView.GetModelScript: TModelScript;
begin
  result:= TModelScript(FindElementClass(TModelScript));
  if result = nil then
  begin
    result:= TModelScript.Create;
    result.Name:= 'Model Script';
    AddElement(result);
  end;
end;


function TView.CreateModel: TRectEntity;
begin
  result:= TRectEntity.Create;
  result.Name:= 'Pedmt zjmu';
  self.Model:= result
end;




procedure TView.AddToModelScript(aElement: TElement);
begin
{  Varianta, kdy se to ukldalo do pedmtu zjmu
   zruit po 6.10.96
  aModel:= self.Model;
  ModelScript:= nil;

  if aModel = nil then
     aModel:= CreateModel;

  if aModel = nil then exit;

  ModelScript:= TModelScript(TEntity(aModel).FindElementClass(TModelScript));
  if ModelScript = nil then
  begin
    ModelScript:= TModelScript.Create;
    ModelScript.Name:= 'Model Script';
    TEntity(aModel).AddElement(ModelScript);
  end;

  if ModelScript <> nil then
     ModelScript.AddElement(aElement)
}

  if ModelScript <> nil then
     ModelScript.AddElement(aElement)
end;



(* O.S. 19980918 Zruit po 199810
procedure TView.ReadGraphics;
var
  I, J: longInt;
  aModelScript: TModelScript;
  aLayerScript: TLayerScript;
  aLayerProps: TPropertyList;
  aPathEl: TElement;
  El: TElement;
begin
  ResetBoundary;
  aModelScript:= TModelScript(FindElementClass(TModelScript));

  if aModelScript <> nil then
  for I:= 0 to aModelScript.Count - 1 do
  begin
    aLayerScript:= TLayerScript(aModelScript.Elements[I]);
    if TElement(aLayerScript) is TLayerScript then
    begin
      aLayerProps:= aLayerScript.GetProperties;

      if aLayerScript.Name = 'ELEMENT' then
      begin
        El:= ReadElement(aLayerScript.Description, aLayerProps);
        AddToModel(El);
        continue
      end;

      for J:= 0 to aLayerScript.Count-1 do
      { cyklus pes vechny cesty }
      begin
        aPathEl:= aLayerScript.Elements[J];
        if TElement(aPathEl) is TPathElement then
        begin
          GraphicsReader(aLayerScript.Name, TPathElement(aPathEl).Text, aLayerProps)
        end
      end;
    end
  end;

  if not Projection.IsOK then
  begin
  { nastaven stedu projekce podle stedu natench dat}
    if TElement(Model) is TRectEntity then
    begin
      Projection.Center_X:= TRectEntity(Model).CenterX;
      Projection.Center_Y:= TRectEntity(Model).CenterY
    end
  end
end;

*)



procedure TView.ReadGraphics;
var
  aModelScript: TModelScript;
  MBSC: TModelByScriptCreator;

begin
  ResetBoundary;
  aModelScript:= TModelScript(FindElementClass(TModelScript));

  if aModelScript = nil then exit;

  MBSC:= TModelByScriptCreator.Create;
  MBSC.Input:= aModelScript;
  if Projection.IsOK then
  begin
    MBSC.MinX:= ModelMinX;
    MBSC.MinY:= ModelMinY;
    MBSC.MaxX:= ModelMaxX;
    MBSC.MaxY:= ModelMaxY;
  end;
  MBSC.CutLayer:= true;
  MBSC.Compatibility30 := true;   // nacitani primo collections bez interpretLayers
  MBSC.DoIt;
  Self.Model:= MBSC.Output;
  Self.Model.Name:= 'VIEWMODEL';
  MBSC.Free;

  if not Projection.IsOK then
  begin
  { nastaven stedu projekce podle stedu natench dat}
    if TElement(Model) is TRectEntity then
    begin
      Projection.Center_X:= TRectEntity(Model).CenterX;
      Projection.Center_Y:= TRectEntity(Model).CenterY
    end
  end
end;



procedure TView.DoReadGraphics(aB: boolean);
begin
  if aB then
     ReadGraphics;
  if Name = 'VEKTOR' then
     TestedElement := TOwnerElement(Model);
end;


(*
procedure  TView.GraphicsReader
          (aType, aPath: string; Props:TPropertyList);
var
  BlokReader: TTopolBlokReader;
  El: TElement;
  PropName: string;
  I: longInt;
  Rect: TRectEntity;
  RasRect: TDoubleRect;
begin
  Rect:= nil;
{$IFDEF BINRAS}
  if aType = 'RAS' then
  begin
     Rect:= TRectEntity.Create;
     Rect.SetByDoubleRect(ModelRectBoundary);
     RasRect:= GetRasRect(aPath);
     if Rect.IsOverRegion
        (RasRect.Min.X, RasRect.Min.Y, RasRect.Max.X, RasRect.Max.Y) then
     begin
       El:= ReadTopolRaster(aPath);
       if El <> nil then
       begin
         TTopRas(El).SetProps(Props);
         AddToModel(El)
       end
     end;
     Rect.Free;
     exit
  end
  else
{$ENDIF}
  if aType = 'ELEMENT' then
  begin
    El:= ReadElement(aPath, Props);
    AddToModel(El);
    exit
  end;

  if not DirectoryExists(aPath) then exit;

  if Projection.IsOK then
  begin
    Rect:= TRectEntity.Create;
    Rect.SetByDoubleRect(ModelRectBoundary);

    if not BlokInRect(aPath, Rect) then
    begin
      Rect.Free;
      Rect:= nil;
      exit
    end;
  end;

  BlokReader:= TTopolBlokReader.CreateBlock(aPath);
  try
    BlokReader.FiltrDruhu:= CreateFiltrDruhu(aType, Props);
    BlokReader.SetProps(Props);

    if Projection.IsOK then
    begin
      BlokReader.RectLimits:= Rect;
    end;

    if aType = 'LIN' then
       El:= BlokReader.ReadLines
    else
    if aType = 'PNT' then
       El:= BlokReader.ReadPoints
    else
    if aType = 'ARE' then
       El:= BlokReader.ReadAreas
    else
    if aType = 'TXT' then
       El:= BlokReader.ReadTexts
    else
    if aType = 'LINSYM' then
       El:= BlokReader.ReadLineSymbols;


    if El <> nil then
    begin
      if Props <> nil then
      begin
        for I:= 0 to Props.Count-1 do
        begin
          PropName:= Props.Names[I];
          if PropName[1] = '@' then
             PropName:= copy(PropName, 2, 255);
          SetPropStringByName(El, PropName, Props.Strings[I]);
        end
      end;

      AddToModel(El)
    end
  finally
    try
      if Rect <> nil then
      begin
        Rect.Free;
        Rect:= nil
      end;
    except
    end;
    BlokReader.Free
  end;
end;

*)




function Tview.GetAngle: double;
begin
   result := Projection.Angle;
end;

procedure Tview.SetAngle(const Value: double);
begin
   Projection.angle := Value;
end;

function Tview.GetRotateCenterX: double;
begin
   result := Projection.RotateCenter_x;
end;

function Tview.GetRotateCenterY: double;
begin
   result := Projection.RotateCenter_y;
end;

procedure Tview.SetRotateCenterX(const Value: double);
begin
   Projection.RotateCenter_X := Value;
end;

procedure Tview.SetRotateCenterY(const Value: double);
begin
   Projection.RotateCenter_Y := Value;
end;

procedure Tview.SetDataDirectory(const Value: string);
begin
  FDataDirectory := Value;
end;

{ TViewProjection }

function  TViewProjection.GetModelView: TDoubleRect;
begin
  result.MinX:= 0;
  result.MinY:= 0;
  result.MaxX:= 0;
  result.MaxY:= 0;
  if Scale = 0 then exit;
  result:= GetModelRect(0, 0, View.Width, View.Height)
   (*
   result.Minx := GetModelX(View.Rect.Min.x);
   result.Miny := GetModely(View.Rect.Min.y);
   result.Maxx := GetModelX(View.Rect.Max.x);
   result.Maxy := GetModely(View.Rect.Max.y);
   *)
end;

procedure TViewProjection.SetCenterPoint(const aCenterPoint: TDoubleCoord);
begin
   ChangeProjection(aCenterPoint, Scale, View);
end;

procedure TViewProjection.SetCenterX(const aValue: double);
begin
  FCenterPoint.X:= aValue;
  ChangeProjection(FCenterPoint, Scale, View);
end;

procedure TViewProjection.SetCenterY(const aValue: double);
begin
  FCenterPoint.Y:= aValue;
  ChangeProjection(FCenterPoint, Scale, View);
end;

procedure TViewProjection.SetScale(const aScale: double);
begin
   ChangeProjection(CenterPoint, aScale, View);
end;

procedure TViewProjection.SetView(const aView: TView);
begin
   FView := aView;
   ChangeProjection(CenterPoint, Scale, View);
end;


procedure TViewProjection.ChangeProjection (const aCenterPoint: TDoubleCoord;
                                            const aScale: double;const aView: TView);
begin
   if View = nil then exit;
   FCenterPoint := aCenterPoint;
   FScale       := aScale;
   FView        := aView;

   MultX := Scale;
   MultY := Scale;
   AddX := CenterPoint.x - (((View.Rect.Max.X - View.Rect.Min.X) / 2) * MultX);
   AddY := CenterPoint.Y - (((View.Rect.Max.Y - View.Rect.Min.Y) / 2) * MultY);

end;

procedure TViewProjection.ChangeProjectionByRect (const aModelRect: TDoubleRect;
                                                  const aView: TView);
begin
   FView := aView;
   ModelView := aModelRect;
end;


procedure TViewProjection.SetModelView(const aRect: TDoubleRect);
var
   aCenterPoint: TDoubleCoord;
   aScale: double;
   ScaleX: double;
   ScaleY: double;
   dX,dY: double;
begin
   dX := aRect.MaxX - aRect.MinX;
   dY := aRect.MaxY - aRect.MinY;
   ScaleX := dX/View.Rect.Width;
   ScaleY := dY/View.Rect.Height;
   if ScaleX > ScaleY then aScale := ScaleX
                      else aScale := ScaleY;
   aCenterPoint.x :=  aRect.MinX + dX/2;
   aCenterPoint.y :=  aRect.MinY + dY/2;
   ChangeProjection(aCenterPoint, aScale, View);
end;



function  TViewProjection.GetModelX(const ViewX: double): double;
begin
   Result:= MultX*ViewX + AddX ;
end;


function  TViewProjection.GetModelY(const ViewY: double): double;
begin
   Result:= MultY*ViewY + AddY ;
end;

function TViewProjection.GetViewX(const ModelX: double): double;
begin
   Result:= (ModelX - AddX)/MultX;
end;


function TViewProjection.GetViewY(const ModelY: double): double;
begin
   Result:= (ModelY - AddY)/MultY;
end;


function TViewProjection.GetViewLength(const ModelLength: double): double;
begin
   Result:= ModelLength / MultX;
end;


function  TViewProjection.GetViewPoint(const ModelX, ModelY: double): TDoubleCoord;
begin
   Result.X:= (ModelX - AddX)/MultX;
   Result.Y:= (ModelY - AddY)/MultY;
end;


function  TViewProjection.GetViewRect(const MinX, MinY, MaxX, MaxY: double): TDoubleRect;
begin
   Result.Min := GetViewPoint(MinX, MinY);
   Result.Max := GetViewPoint(MaxX, MaxY);
end;

function  TViewProjection.GetModelPoint(const ViewX, ViewY: double): TDoubleCoord;
begin
   Result.x := MultX*ViewX + AddX;
   Result.y := MultY*ViewY + AddY;
end;


function  TViewProjection.GetModelRect(const MinX, MinY, MaxX, MaxY: double): TDoubleRect;
begin
   result.Min := GetModelPoint(MinX, MinY);
   result.Max := GetModelPoint(MaxX, MaxY);
end;






BEGIN
   RegisterOrbisClass(TViewProjection);
   RegisterOrbisClass(TView);
END.
