unit t_Print;

interface
uses
// KERNEL
   Tools,
// DEVICE
   e_Device,
// INTERPRET
   i_Basic;

type

TPrint = Class(TTool)
private
   FBackupDevice: TDevice;
   FInterpret: TInterpret;
    FFileName: string;
    FSilentMode: boolean;
    FTitle: string;
    procedure SetSilentMode(const Value: boolean);
    procedure SetTitle(const Value: string);
public
   procedure DoIt; override;
   procedure PrintDesktop;
   procedure PrintMultiPageDesktop;
   property BackupDevice: TDevice read FBackupDevice write FBackupDevice;
   property Interpret: TInterpret read FInterpret write FInterpret;
   property FileName: string read FFileName write FFileName;
   property SilentMode: boolean read FSilentMode write SetSilentMode;
   property Title: string read FTitle write SetTitle;
end;

implementation
uses
   Dialogs,
   Printers_f,
   SysUtils,
   Elements,
   e_PrnPg,
   e_DeskTP,
   e_Type,
   G_Elems,
   m_Orbis,
   t_PrnDlg;


// TPrint

procedure TPrint.DoIt;
var
   PrintDevice:  TPrintDevice;
   aCenterPoint: TDoubleCoord;
   aScale: double;
begin
  if Input = nil then exit;
  if not (Input is TInterpret) then exit;
  Interpret := TInterpret(Input);

  if Interpret.Device = nil then
  begin
    ShowMessage('Nen nastaveno zazen interpretru');
    exit
  end;

  if Printer.Printing then
  begin
    ShowMessage('Dosud se tiskne')
  end
  else
  try
    Printer.FileName := FileName;
    { dialog nastaven tisku }
    BackupDevice := Interpret.Device;


    PrintDevice:= TPrintDevice.CreateTitle(Title);
    if PrintDevice = nil then exit;
    Interpret.Printing:= true;

    aCenterPoint := BackupDevice.Projection.CenterPoint;
    aScale := BackupDevice.Projection.Scale;

    PrintDevice.Projection.ChangeProjection
    (aCenterPoint, aScale, PrintDevice.Page);

    PrintDevice.Projection.VirtualScale:=BackupDevice.Projection.VirtualScale;

    Interpret.Device:= PrintDevice;

    try
      if (Interpret.Input is TEntity) then
      begin
         if (TEntity(Interpret.Input).FindElementClass(TDesktop) <> nil) then
            PrintDeskTop
         else
         if (TEntity(Interpret.Input).FindElementClass(TMultiPageDesktop) <> nil) then
            PrintMultiPageDeskTop
         else
            Interpret.DoIt
      end
      else
         Interpret.DoIt
    except
      on E: exception do
      begin
        Printer.Abort;
        ShowMessage('Problmy s tiskem: ' + E.Message)
      end;
    end;

    Interpret.Device:= BackupDevice;

    PrintDevice.Free;
    Interpret.Printing:= false;
    Interpret.PostMsgToOwners(tm_EndOfPrint, nil, DontKill);
  except
    on E: exception do
    begin
      ShowMessage('Problmy s ppravou tisku: ' + E.Message)
    end;
  end;
end;


procedure TPrint.PrintDeskTop;
var
   PrintDlg: TPrintDialogTool;
   PrintPages: TPrintPages;
   i: longInt;
   PCount: longInt;
   DeskTop: TDeskTop;
begin
  DeskTop:= TDeskTop(TEntity(Interpret.Input).FindElementClass(TDesktop));
  PrintDlg := TPrintDialogTool.Create;
  PrintDlg.PrintDevice := TPrintDevice(Interpret.Device);
  PrintDlg.Input := DeskTop;
  PrintDlg.SilentMode := SilentMode;
  PrintDlg.FileName := FileName;
  PrintDlg.DoIt;
  PrintPages := TPrintPages(PrintDlg.Output);

  if PrintPages = nil then
  begin
    PrintDlg.Free; { jinak se uvolni samo}
    Printer.Abort;
    Exit;
  end;

  PrintDlg.StartTime:= Now;
//  Application.ProcessMessages;
  PCount := PrintPages.PagesCount -1;
  for  i := 0 to  PCount do
  begin
     {
     if i > 0 then Printer.BeginDoc;
     }
     if PCount > 0 then
        PrintDlg.PageNo:='Tisk strany . ' + InTToStr(Printer.PageNumber) +
                        ' (' + IntToStr(PCount+1) + ')';

     Interpret.Device.Projection.ChangeProjection
     (PrintPages.PrintPage[i].Rect.CenterCoord, PrintPages.Scale, Interpret.Device.Page);
     Interpret.DoIt;
     if i < PCount then Printer.NewPage;
  end;
  PrintPages.Free;
end;





procedure TPrint.PrintMultiPageDesktop;
var
   PrintDlg: TPrintDialogTool;
   PrintPages: TPrintPages;
   i: longInt;
   PCount: longInt;
   MPDeskTop: TMultiPageDeskTop;
   Desktop: TDesktop;
   OInput: TElement;
   Coord: TDoubleCoord;
   dx,dy: double;
   OnlyPages: boolean;
   IL: TIntegerListElement;
begin
  OInput := Interpret.Input;
  try
     MPDeskTop:= TMultiPageDeskTop(TEntity(Interpret.Input).FindElementClass(TMultiPageDesktop));
     Desktop := MPDesktop.DesktopPage[0];
     PrintDlg := TPrintDialogTool.Create;
     PrintDlg.MultiPage := true;
     PrintDlg.PrintDevice := TPrintDevice(Interpret.Device);
     PrintDlg.Input := DeskTop;
     PrintDlg.DoIt;
     OnlyPages := PrintDlg.Form.r_Pages.Checked;
     if OnlyPages then
     begin
        IL := TIntegerListElement.Create;
        IL.ValuesString := PrintDlg.Form.ed_Pages.Text;
     end;
     PrintPages := TPrintPages(PrintDlg.Output);

     if PrintPages = nil then
     begin
       PrintDlg.Free; { jinak se uvolni samo}
       Printer.Abort;
       Exit;
     end;
     dx := PrintPages.PrintPage[i].CenterX - Desktop.CenterX;
     dy := PrintPages.PrintPage[i].CenterY - Desktop.CenterY;
     PrintDlg.StartTime:= Now;
   //  Application.ProcessMessages;

     for  i := 0 to  MPDesktop.LastIndex do
     begin
        MPDesktop.ActualPage := MPDesktop.DesktopPage[i];
        Desktop := MPDesktop.ActualPage;
        if OnlyPages then
        begin
           if Not IL.ValueExists(MPDesktop.ActualNumber) then Continue;
        end;
        Coord := Desktop.Rect.CenterCoord;
        Coord.X := Coord.X + dx;
        Coord.Y := Coord.Y + dy;
        {
        if i > 0 then Printer.BeginDoc;
        }
        PrintDlg.PageNo:='Tisk strany . ' + InTToStr(MPDesktop.ActualNumber) +
                           ' (' + IntToStr(MPDesktop.Count) + ')';

        Interpret.Device.Projection.ChangeProjection
        (Coord, PrintPages.Scale, Interpret.Device.Page);
        Interpret.Input := Desktop;
        Interpret.DoIt;
        if i < MPDesktop.LastIndex then Printer.NewPage;
     end;
     PrintPages.Free;
  except

  end;
  if OnlyPages then IL.Free;
  Interpret.Input := OInput;
end;

procedure TPrint.SetSilentMode(const Value: boolean);
begin
  FSilentMode := Value;
end;

procedure TPrint.SetTitle(const Value: string);
begin
  FTitle := Value;
end;

END.
