unit E_Class;
{
[Unit_Description]
Description=Elementy pro registraci td ORBISu
Admin=Petr
TechTopic=Elements
Topic=System
Version=1.0
}

interface
uses
   TypInfo,
   Elements;

type

TOrbisClassProperty = class(TNameElement)
private
   FPropInfo: PPropInfo;
   FIndex: integer;
   FTrueIndex: integer;
   procedure SetPropInfo(aPropInfo: PPropInfo);
   function  GetTypeKind: TTypeKind;
public
   property PropInfo: PPropInfo read FPropInfo write SetPropInfo;
   property TrueIndex: integer read FTrueIndex write FTrueIndex;
published
   property PropIndex: integer read FIndex write FIndex;
   property TypeKind: TTypeKind read GetTypeKind;
end;



TOrbisClass = class(TOwnerEntity)
private
   FElementClass: TElementClass;
   FPropList: PPropList;
   FPropCount: integer;
   function  GetElementClassName: string;
   function  GetAncestorClassName: string;
   procedure SetElementClass(aValue: TElementClass);
   function  GetPropertyName(aIndex: integer): string;
   function  GetPropertyTypeKind (aIndex: integer): TTypeKind;
   function  GetPropertyStored(aIndex: integer): boolean; virtual;
   function  GetPropInfo(aIndex: integer): PPropInfo;
public
   destructor Destroy; override;
   function   IsClassName(const aName: string): boolean;
   procedure  ShowProperties; virtual;
   property   ElementClass: TElementClass read FElementClass write SetElementClass;
   property   PropertyName[aIndex: integer]: string read GetPropertyName;
   property   PropertyTypeKind[aIndex: integer]: TTypeKind read GetPropertyTypeKind;
   property   PropertyStored[aIndex: integer]: boolean read GetPropertyStored;
   property   PropList: PPropList read FPropList;
   property   PropInfo[aIndex: integer]: PPropInfo read GetPropInfo;
published
   property   ElementClassName: string read GetElementClassName;
   property   AncestorClassName: string read GetAncestorClassName;
   property   PropCount: integer read FPropCount;
end;


TOrbisInstance = class(TOrbisClass)
private
   FInstance: TElement;
   procedure SetInstance(aValue: TElement);
   function  GetPropertyStored(aIndex: integer): boolean; override;
public
   procedure SetPropertyValue(aIndex: integer; var aValue);
   property Instance: TElement read FInstance write SetInstance;
   property PropertyStored[aIndex: integer]: boolean read GetPropertyStored;
end;


TOrbisClasses = class(TOwnerEntity)
private
   FActualClass: TOrbisClass;
   FShowProperties: boolean;
   FSortByName: boolean;
   FIndex: longInt;
   procedure SetSortByName(aValue: boolean);
protected
   function  FindIndex(const aName: string; var NearestRight: longInt): longInt;
public
   procedure AddOrbisClass(aClass: TOrbisClass); virtual;
   procedure AddClass(aClass: TElementClass); virtual;
   function  GetOrbisClassByName(const aName: string): TOrbisClass; virtual;
   function  GetClass (const aClassName: string): TElementClass; virtual;
   function  FindClass(const aClassName: string): TElementClass;  virtual;
   property  ActualClass: TOrbisClass read FActualClass;
published
   property  ShowProperties: boolean read FShowProperties write FShowProperties;
   property  SortByName: boolean read FSortByName write SetSortByname;
end;


const OrbisClasses: TOrbisClasses = nil;


procedure RegisterOrbisClass(aClass: TElementClass);
function  GetOrbisClass (const aClassName: string): TElementClass;
function  FindOrbisClass(const aClassName: string): TElementClass;


implementation

uses
  {!t_Main,}
   Classes,
   SysUtils;


procedure RegisterOrbisClass(aClass: TElementClass);
begin
   if OrbisClasses = nil then
   begin
      OrbisClasses := TOrbisClasses.Create;
      OrbisClasses.ShowProperties := true;
   end;
   OrbisClasses.AddClass(aClass);
end;

function  GetOrbisClass (const aClassName: string): TElementClass;
begin
   result := OrbisClasses.GetClass(aClassName);
end;

function  FindOrbisClass(const aClassName: string): TElementClass;
begin
   result := OrbisClasses.FindClass(aClassName);
end;


{ TOrbisClassProperty }


procedure TOrbisClassProperty.SetPropInfo(aPropInfo: PPropInfo);
begin
  FPropInfo:= aPropInfo;
  Name:= FPropInfo^.Name
end;


function TOrbisClassProperty.GetTypeKind: TTypeKind;
begin
  result:= tkUnknown;
  if FPropInfo = nil then exit;
  result:= FPropInfo^.PropType^.Kind;
end;



{ TOrbisClass }




destructor TOrbisClass.Destroy;
begin
  if FPropList <> nil then
     FreeMem(FPropList, Count * SizeOf(Pointer));
  inherited destroy
end;

function  TOrbisClass.GetPropertyName(aIndex: integer): string;
begin
   result := FPropList^[aIndex]^.Name;
end;

function  TOrbisClass.GetPropertyTypeKind (aIndex: integer): TTypeKind;
begin
   result := FPropList^[aIndex]^.PropType^.Kind;
end;

function  TOrbisClass.GetPropInfo(aIndex: integer): PPropInfo;
begin
   result := FPropList^[aIndex];
end;

function  TOrbisClass.GetPropertyStored(aIndex: integer): boolean;
var
//   PropI: PPropInfo;
   L: longInt;
   res: longint;
begin
   result:= true;
   if FPropList = nil then exit;
   L := longInt(PropInfo[aIndex]^.StoredProc);
   res := L and $0FFFFFF00;
   if not longBool(res) then
   { constant value }
   begin
      result := longBool(L and $000000FF);
   end;
end;



function  TOrbisClass.IsClassName(const aName: string): boolean;
begin
   result := AnsiCompareText(aName, Name) = 0;
end;


function  TOrbisClass.GetElementClassName: string;
begin
   result := '';
   if ElementClass = nil then exit;
   result := ElementClass.ClassName;
end;

function  TOrbisClass.GetAncestorClassName: string;
begin
   result := '';
   if ElementClass = nil then exit;
   result := ElementClass.ClassParent.ClassName;
end;

procedure TOrbisClass.SetElementClass(aValue: TElementClass);
begin
   FElementClass := aValue;
   if FPropList <> nil then
      FreeMem(FPropList, FPropCount * SizeOf(Pointer));
   FPropList:=nil;

   if aValue = nil then exit;
   Name:= FElementClass.ClassName;

   FPropCount:= GetTypeData(FElementClass.ClassInfo)^.PropCount;
   if FPropCount > 0 then
   begin
     GetMem(FPropList, FPropCount*sizeOf(Pointer));
     try
       GetPropInfos(FElementClass.ClassInfo, FPropList);
     except
       if FPropList <> nil then
          FreeMem(FPropList, FPropCount * SizeOf(Pointer));
       FPropList:= nil;
       FPropCount:= 0;
     end
   end
end;


procedure  TOrbisClass.ShowProperties;
var
  I: integer;
  OCP: TOrbisClassProperty;
begin
  FreeAll;
  if FPropList = nil then exit;
  for I:= 0 to PropCount-1 do
  begin
     OCP := TOrbisClassProperty.Create;
     OCP.PropInfo := FPropList^[I];
     AddElement(OCP);
  end;
end;

{ TOrbisInstance }

procedure TOrbisInstance.SetInstance(aValue: TElement);
begin
   FInstance := aValue;
   ElementClass := TElementCLass(FInstance.ClassType);
end;

function  TOrbisInstance.GetPropertyStored(aIndex: integer): boolean;
begin
  result:= false;
  if FPropList = nil then exit;
  if FInstance = nil then exit;
  result:= IsStoredProp(FInstance, FPropList^[aIndex]);
end;


procedure TOrbisInstance.SetPropertyValue(aIndex: integer; var aValue);
var
   P: PPropInfo;
begin
   P := FPropList^[aIndex];
   case P^.PropType^.Kind of
       tkString
       {$IFDEF Win32}
       , tkLString
       {$ENDIF}
                : SetStrProp(Instance, P, string(aValue));
       tkInteger,tkEnumeration:
       begin
          SetOrdProp(Instance, P, integer(aValue));
       end;
       tkFloat:
       begin
          SetFloatProp(Instance, P, Extended(aValue));
       end;
       tkClass:
       begin
           SetOrdProp(Instance, P, longInt(aValue));
       end;
   end; { case }

end;


{ TOrbisClasses }



procedure TOrbisClasses.SetSortByName(aValue: boolean);
begin
end;



function  TOrbisClasses.FindIndex(const aName: string; var NearestRight: longInt): longInt;
var
   L,R,C: longInt;
   LResult, RResult, CResult: integer;
begin
   result := -1;
   NearestRight := -1;
   if NoElements then exit;
   L := 0;
   R := Count - 1;
   RResult := AnsiCompareText(aName, Elements[R].Name);
   if (RResult > 0) then exit;
   if RResult = 0 then
   begin
      result := R;
      exit;
   end;
   if R > L then
   begin
      LResult := AnsiCompareText(aName, Elements[L].Name);
      if (LResult < 0) then
      begin
         NearestRight := L;
         exit;
      end;
      if LResult = 0 then
      begin
         result := L;
         NearestRight := L + 1;
         exit;
      end;
   end
   else
   begin
      NearestRight := L;
      exit;
   end;
   repeat
      NearestRight := R;
      if R - L <= 1 then exit;
      C := ((R - L) div 2) + L;
      CResult := AnsiCompareText(aName, Elements[C].Name);
      if CResult < 0 then R := C else
      if CResult > 0 then L := C else
      begin
         result := C;
         NearestRight := C + 1;
         exit;
      end;
   until L >= R;
end;



procedure TOrbisClasses.AddOrbisClass(aClass: TOrbisClass);
var
   i: longInt;
   S: string;
begin
   S := aClass.Name;
   FindIndex(S, i);
   if i > -1 then InsertElement(i, aClass)
             else AddElement(aClass);
   FActualClass := aClass;
   aClass.ID := FIndex;
   inc(FIndex);
end;



procedure TOrbisClasses.AddClass(aClass: TElementClass);
var
   OC: TOrbisClass;
   i: longInt;
   S: string;
begin
   OC := TOrbisClass.Create;
   OC.ElementClass := aClass;
   S := aClass.ClassName;
   OC.Name := S;
   OC.ID := FIndex;
   inc(FIndex);
   if ShowProperties then OC.ShowProperties;
   FindIndex(S, i);
   if i > -1 then InsertElement(i, OC)
             else AddElement(OC);
   FActualClass := OC;
end;


function  TOrbisClasses.GetOrbisClassByName(const aName: string): TOrbisClass;
var
   i, ri: longInt;
begin
   result := nil;
   i := FindIndex(aName, ri);
   if i > -1 then
   begin
      result := TOrbisClass(Elements[i]);
      FActualClass := result;
   end;
end;


function  TOrbisClasses.GetClass (const aClassName: string): TElementClass;
var
   i, ri: longInt;
   EC: TOrbisClass;
begin
   result := nil;
   i := FindIndex(aClassName, ri);
   if i > -1 then
   begin
      EC := TOrbisClass(Elements[i]);
      result := EC.ElementClass;
      FActualClass := EC;
   end;
end;

function  TOrbisClasses.FindClass(const aClassName: string): TElementClass;
begin
  result := GetClass(aClassName);
  if result = nil then raise EClassNotFound.Create('Orbis nenael tdu '+ aClassName);
end;





INITIALIZATION
   RegisterOrbisClass(TOrbisClass);
   RegisterOrbisClass(TOrbisClassProperty);
END.
