unit e_RASTER;

interface
uses
   // KERNEL
   Elements,
   // GLIB
   e_GFile,
   // RASTER
   Rasdata_TLB,
   TOPOCXLib_TLB;

type

TRaster = class(TGraphicFileElement)
private
   FColor: integer;
   FSrvRas: IRaster;
   FViewEntry: IViewEntry;
   FHorizDPI: integer;
   FVertDPI: integer;
protected
   function GetFileName: string; override;
   procedure SetFileName(aValue: string); override;
   function GetColor: integer;
   procedure SetColor(aValue: integer);
   function GetRCS: integer;
   procedure SetRCS(aValue: integer);
   function  GetFileSize: integer;
   function  GetRasTyp: SmallInt;
   function  GetCompres: SmallInt;
   function  GetTiled: boolean;
   function  GetPixelWidth: integer;
   function  GetPixelHeight: integer;
   function  GetVertRes: double;
   function  GetHorizRes: double;
   function  GetGeoRaster: boolean;
   procedure SetHorizDPI(aValue: integer);
   procedure SetVertDPI(aValue: integer);
public
   constructor  Create; override;
   destructor   Destroy; override;
   property SrvRas: IRaster read FSrvRas write FSrvRas;
   property ViewEntry: IViewEntry read FViewEntry write FViewEntry;
published
   property Color: integer read GetColor write SetColor;
   property RCS: integer read GetRCS write SetRCS;
   property FileSize: integer read GetFileSize stored false;
   property RasTyp: SmallInt read GetRasTyp stored false;
   property Compres: SmallInt read GetCompres stored false;
   property Tiled: boolean read GetTiled stored false;
   property PixelWidth: integer read GetPixelWidth stored false;
   property PixelHeight: integer read GetPixelHeight stored false;
   property VertRes: double read GetVertRes stored false;
   property HorizRes: double read GetHorizRes stored false;
   property GeoRaster: boolean read GetGeoRaster stored false;
   property HorizDPI: integer read FHorizDPI write SetHorizDPI;
   property VertDPI: integer read FVertDPI write SetVertDPI;
end;

const
  UseRasters: Boolean = True;


implementation
uses
// KERNEL
   e_Class,
// RASTER
   UseLic;

constructor  TRaster.Create;
var
  hr: hresult;

const
  TLRasterLicKey: widestring        = '=0FJA=0C4kVIB4AE0URF=T[:';

begin
//   SrvRas := CoRaster.Create;
  hr:= CreateInprocObjLic('TopoL.Raster', TLRasterLicKey,
                          IRaster, IUnknown(FSrvRas));
   //SrvRas.SoSys := 2;
   ViewEntry := CoViewEntry.Create;
   ViewEntry.Type_ := 1;
   ViewEntry.ViewObj := SrvRas;
end;

destructor   TRaster.Destroy;
begin
   if SrvRas.IsOpen then SrvRas.CloseRas(false);
   SrvRas := nil;
   ViewEntry := nil;
end;

function  TRaster.GetFileSize: integer;
begin
   result := SrvRas.FileSize;
end;

function  TRaster.GetRasTyp: SmallInt;
begin
   result := SrvRas.RasTyp;
end;

function  TRaster.GetCompres: SmallInt;
begin
   result := SrvRas.Compres;
end;

function  TRaster.GetTiled: boolean;
begin
   result := SrvRas.Tiled;
end;

function  TRaster.GetPixelWidth: integer;
begin
   result := SrvRas.Width;
end;

function  TRaster.GetPixelHeight: integer;
begin
   result := SrvRas.Height;
end;

function  TRaster.GetVertRes: double;
begin
   result := SrvRas.VertRes;
end;

function  TRaster.GetHorizRes: double;
begin
   result := SrvRas.HorzRes;
end;

function  TRaster.GetGeoRaster: boolean;
begin
   result := not(((SrvRas.y1 = 0) and (SrvRas.x1 = 0))and(((HorizRes = 1) and (VertRes = 1))
             or((HorizRes = 0) and (VertRes = 0))));
end;

procedure TRaster.SetHorizDPI(aValue: integer);
begin
   if GeoRaster then exit;
   FHorizDPI := aValue;
   if aValue = 0 then Width := SrvRas.dy
   else
   begin
      Width := 0.0254*(PixelWidth/aValue);
   end;
end;

procedure TRaster.SetVertDPI(aValue: integer);
begin
   if GeoRaster then exit;
   FVertDPI := aValue;
   if aValue = 0 then Height := SrvRas.dx
   else
   begin
      Height := 0.0254*(PixelHeight/aValue);
   end;
end;


function  TRaster.GetFileName: string;
begin
   result := SrvRas.FileName;
end;

procedure TRaster.SetFileName(aValue: string);
var
   S: string;
begin
   if SrvRas.IsOpen then SrvRas.CloseRas(false);
   SrvRas.FileName := aValue;
   //if not FileExists(aValue) then exit;
   SrvRas.OpenRas(aValue, 1,2);
   S := SrvRas.ErrorStr;
   if GeoRaster then
   begin
      Minx := -SrvRas.y1-SrvRas.dy;
      MinY := -SrvRas.x1-SrvRas.dx;
      Width := SrvRas.dy;
      Height := SrvRas.dx;
       { TODO -oPPK -cRASTER : Tohle je tu jen kvli dotahovn rastr ... jinak mi nesedly }
      {
      Miny := Miny-SrvRas.VertRes/2;
      Minx := Minx-SrvRas.HorzRes/2;;
      }
   end
   else
   begin
      Minx := SrvRas.y1;
      MinY := SrvRas.x1;
      Width := SrvRas.Width;
      Height := SrvRas.Height;
   end;
end;

function TRaster.GetColor: integer;
begin
   result := FColor;
   //result := ViewEntry.P1;
end;

procedure TRaster.SetColor(aValue: integer);
begin
   FColor := aValue;
   //ViewEntry.P1 := aValue;
end;

function TRaster.GetRCS: integer;
begin
   result := SrvRas.CoordOri;
end;

procedure TRaster.SetRCS(aValue: integer);
begin
   SrvRas.CoordOri := aValue;
end;


initialization
   RegisterOrbisClass(TRaster);
end.
