unit t_CHeck_LIN_ARE;

interface
uses
   Elements,
   e_QTree,
   t_Chk;

type

TCheck_LIN_ARE = class (TCheckTool)
private
    FAreas: TEntity;
    FLines: TEntity;
    FInside: boolean;
    FLevelQT: TRectEntityQTree;
    procedure SetAreas(const Value: TEntity);
    procedure SetLines(const Value: TEntity);
    procedure SetInside(const Value: boolean);
    procedure SetLevelQT(const Value: TRectEntityQTree);
public
   procedure DoIt; override;
   procedure PrepareQTree;
   procedure ProcessLine;
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
published
   property Lines: TEntity read FLines write SetLines;
   property Areas: TEntity read FAreas write SetAreas;
   property Inside: boolean read FInside write SetInside;
end;

implementation
uses
   Forms,
   Dialogs,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   t_QTreeTools,
   t_SpatialAnalysis;

{ TCheck_LIN_ARE }

procedure TCheck_LIN_ARE.DoIt;
var
   i: integer;
   e: TElement;
begin
   if (Lines = nil) or Lines.NoElements then
   begin
      ShowMessage(Name + ': vrstva Lines nen definovna nebo je przdn!');
      exit;
   end;
   if Areas = nil then
   begin
      ShowMessage(Name + ': Areas nen definovno!');
      exit;
   end;
   StartProgress(Name,0,Lines.LastIndex);
   progressText := 'Pracovn struktura ploch';
   Application.ProcessMessages;
   PrepareQTree;
   progressText := 'Line Checking...';
   Application.ProcessMessages;
   for i := 0 to Lines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lines.Elements[i];
      Input := e;
      ProcessLine;
   end;
end;

procedure TCheck_LIN_ARE.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if Areas = nil then exit;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Areas;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
end;

procedure TCheck_LIN_ARE.ProcessLine;
var
   GE,GE1: TGeometryElement;
   G,G1: TGeometry;
   E,O: TEntity;
   El: TElement;
   SAT: TSpatialAnalysisTool;
   W: TQTreeWalker;
   i: integer;
   Rel: boolean;
   ARE: TAreaElement;
begin
   if Input = nil then exit;
   if not (Input is TGeometryElement) then exit;
   GE := TGeometryElement(Input);
   if GE.NoGeometry then exit;
   G := GE.Geometry;
   // hledn, zda je linie hranic v plochch
   for i := 0 to G.OwnersCount - 1 do
   begin
      // hledme plochy
      O := G.Owners[i];
      if not (O is TAreaElement) then continue; // neni to plocha tak co
      ARE := TAreaElement(O);
      if ARE.IsHole then
         ARE := TAreaElement(ARE.HoleOwnerArea); // u dry eme jejho vlastnka
      E := nil;
      if ARE <> nil then
         E := ARE.FindClassWithOwnerInOwners(TGeometryElement,Areas);
      if E <> nil then exit; // je hranic plochy ... teda asi = nen co eit kofola !
   end;
   //nen hranic, nastv fze II
   SAT := TSpatialAnalysisTool.Create;
   W := TQTreeWalker.Create;
   E := TQuickEntity.Create;
   El := W.GetFirstElementForRectangle(LevelQT,G.Minx,G.MinY,G.MaxX,G.MaxY,1);
   while El <> nil do
   begin
      E.AddElement(El);
      El := W.GetNextElement;
   end;
   W.Free;
   Rel := false;
   if E.NoElements then          // dn plochy v dohledu, me to bt chyba
   begin
      if not Inside then
         AddErrorElement;
   end
   else
   for i := 0 to E.LastIndex do  //plochy jsou, tak je sfouknem
   begin
      El := E.Elements[i];
      if not (El is TGeometryElement) then continue; // a co by to asi bylo
      GE1 := TGeometryElement(El);
      if GE1.NoGeometry then continue;
      G1 := GE1.Geometry;
      Rel := Rel or (SAT.Relate(G,G1) and SAT.Inside);
      if Inside and Rel then
      begin                     // hledm linie v ploe a to je ona
         AddErrorElement;
         Break;
      end;
   end;
   if not Inside and not Rel then
   begin                        // hledm linie mimo plochy, tady jedna asi je
      AddErrorElement;
   end;
   E.Clear;
   E.Free;
   SAT.Free;
end;

procedure TCheck_LIN_ARE.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

procedure TCheck_LIN_ARE.SetInside(const Value: boolean);
begin
  FInside := Value;
end;

procedure TCheck_LIN_ARE.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

procedure TCheck_LIN_ARE.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheck_LIN_ARE);
END.

