unit t_ChkDruhy;
(*
   ---------------------------------------------------------------------------------------
   Kontrola nesouladu druh, barev a znaek v zvislosti na defininm seznamu (znacky.dbf)
   ppk 2003
   ---------------------------------------------------------------------------------------
*)
interface
uses
   e_TopolDruhy,
   t_ChkSp,
   Elements;

type

TCheck_Druhy = class(TCheckSpaceEntity)
private
    FComment: string;
    FLayer: TEntity;
    FDruhy: TTopolDruhCollection;
    FZnackaPropertyName: string;
    FBarvaPropertyName: string;
    FDruhPropertyName: string;
    FEString: string;
    procedure SetComment(const Value: string);
    procedure SetDruhy(const Value: TTopolDruhCollection);
    procedure SetLayer(const Value: TEntity);
    procedure SetBarvaPropertyName(const Value: string);
    procedure SetDruhPropertyName(const Value: string);
    procedure SetZnackaPropertyName(const Value: string);
    procedure SetEString(const Value: string);
protected
  function GetErrorMsg: string; override;
  property EString: string read FEString write SetEString;
public
   procedure DoIt; override;
published
   property Layer: TEntity read FLayer write SetLayer;
   property Druhy: TTopolDruhCollection read FDruhy write SetDruhy;
   property Comment: string read FComment write SetComment;
   property DruhPropertyName: string read FDruhPropertyName write SetDruhPropertyName;
   property BarvaPropertyName: string read FBarvaPropertyName write SetBarvaPropertyName;
   property ZnackaPropertyName: string read FZnackaPropertyName write SetZnackaPropertyName;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_GeometryElement;


{ TCheck_Druhy }

procedure TCheck_Druhy.DoIt;
var
   i: integer;
   aDruh,aBarva,aZnacka: integer;
   e: TElement;
   GE: TGeometryElement;
   asiError: boolean;
begin
   if (Layer = nil) or not (Layer is TEntity) or Layer.NoElements then
   begin
      ShowMessage('Check Druhy - Vrstva neexistuje, nebo je przdn !!!!');
      exit;
   end;
   e := Druhy;
   if (e = nil) or not (e is TTopolDruhCollection) then
   begin
      ShowMessage('Check Druhy - Nejsou definovny druhy pro kontrolu !!!!');
      exit;
   end;
   if DruhPropertyName = '' then DruhPropertyName := 'Druh';
   if BarvaPropertyName = '' then BarvaPropertyName := 'Barva';
   if ZnackaPropertyName = '' then ZnackaPropertyName := 'Znacka';

   StartProgress(Comment,0,Layer.LastIndex);

   ProgressText := 'hledme nesrovnalosti...';
   //MaxProgress := MasterLines.LastIndex;
   Progress := 0;
   Application.ProcessMessages;

   for i := 0 to Layer.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Layer.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      Input := GE;
      aDruh := GE.GetPropertyLongIntByName(DruhPropertyName);
      aBarva := GE.GetPropertyLongIntByName(BarvaPropertyName);
      aZnacka := GE.GetPropertyLongIntByName(ZnackaPropertyName);
      Druhy.FindDruhKod := aDruh;
      AsiError := false;
      EString := '';
      if Druhy.ActualDruh = nil then
      begin
         EString := 'Druh?-D:'+IntToStr(aDruh)+',B:'+IntToStr(aBarva)+',Z:'+IntToStr(aZnacka);
         AsiError := true; // Druh neexistuje;
      end
      else
      begin
         if Druhy.ActualDruh.Barva <> aBarva then
            AsiError := true; // Barva nesedi
         if Druhy.ActualDruh.Znacka <> aZnacka then
            AsiError := true; // Znacka nesedi
         if AsiError then
            EString := 'D:'+IntToStr(aDruh)+',B:'+IntToStr(aBarva)+'('+IntToStr(Druhy.ActualDruh.Barva)+'),Z:'+IntToStr(aZnacka)+'('+IntToStr(Druhy.ActualDruh.Znacka)+')';
      end;
      if AsiError then
      begin
         Error:= GetError;
         AddErrorElement;
         {
         if not FullCheck then
            break; // na dalsi linii
         }
      end;
   end;
end;

function TCheck_Druhy.GetErrorMsg: string;
begin
   result := EString;
end;

procedure TCheck_Druhy.SetBarvaPropertyName(const Value: string);
begin
  FBarvaPropertyName := Value;
end;

procedure TCheck_Druhy.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCheck_Druhy.SetDruhPropertyName(const Value: string);
begin
  FDruhPropertyName := Value;
end;

procedure TCheck_Druhy.SetDruhy(const Value: TTopolDruhCollection);
begin
  FDruhy := Value;
end;

procedure TCheck_Druhy.SetEString(const Value: string);
begin
  FEString := Value;
end;

procedure TCheck_Druhy.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

procedure TCheck_Druhy.SetZnackaPropertyName(const Value: string);
begin
  FZnackaPropertyName := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheck_Druhy);
END.

