unit t_DirectoryHCheck;

interface
uses
// KERNEL
   Elements,
// DEVICE
   t_Device,
// REVIZOR
   e_HierTempl;

type

TDirectoryHCheckNode = class(TOwnerEntity)
private
    FLostDirText: string;
    FNewDirText: string;
    procedure SetTemplate(const Value: THierarchyTemplate);
    function GetTemplate: THierarchyTemplate;
    procedure SetLostDirText(const Value: string);
    procedure SetNewDirText(const Value: string);
published
   property Template: THierarchyTemplate read GetTemplate write SetTemplate stored false;
   property LostDirText: string read FLostDirText write SetLostDirText;
   property NewDirText: string read FNewDirText write SetNewDirText;
end;

TDirectoryHCheckTool = class(TDeviceTool)
  private
    FNode: TDirectoryHCheckNode;
    FDirectory: string;
    procedure SetNode(const Value: TDirectoryHCheckNode);
    procedure SetDirectory(const Value: string);
public
   procedure DoIt; override;
published
   property Node: TDirectoryHCheckNode read FNode write SetNode;
   property Directory: string read FDirectory write SetDirectory;
end;

implementation

uses
   e_Class,
   e_Files;
{ TDirectoryHCheckNode }

function TDirectoryHCheckNode.GetTemplate: THierarchyTemplate;
begin
   result := THierarchyTemplate(FindElementClass(THierarchyTemplate));
   if result = nil then
   begin
      result := THierarchyTemplate.Create;
      result.Name := 'root';
      AddElement(result);
   end;
end;

procedure TDirectoryHCheckNode.SetLostDirText(const Value: string);
begin
  FLostDirText := Value;
end;

procedure TDirectoryHCheckNode.SetNewDirText(const Value: string);
begin
  FNewDirText := Value;
end;

procedure TDirectoryHCheckNode.SetTemplate(
  const Value: THierarchyTemplate);
var
   T: THierarchyTemplate;
begin
   T := THierarchyTemplate(FindElementClass(THierarchyTemplate));
   if T <> nil then FreeElement(T);
   Value.Name := 'root';
   AddElement(Value);
end;

{ TDirectoryHCheckTool }

procedure TDirectoryHCheckTool.DoIt;
var
   Dir: TDirectoryElement;
   NewE: TEntity;
   ClearE: TEntity;
   i: integer;
   HT: THierarchyTemplate;
begin
   if Device = nil then exit;
   Device.WritelnText(' ');
   Device.WritelnText('    Kontrola Adresov struktury ');
   Device.WriteLnText('sloka: '+Directory);
   Device.WritelnText('******************************************');
   //Device.WritelnText(' ');
   if Node = nil then
   begin
      exit;
   end;
   Dir := TDirectoryElement.Create;
   Dir.DirOnly := true;
   Dir.FileName := Directory;
   Node.Template.HierarchyElement := Dir;
   Dir.Free;
   ClearE := nil;
   NewE := nil;
   Node.Template.GetClearList(ClearE);
   Node.Template.GetNewList(NewE);
   if ClearE.NoElements and NewE.NoElements then
   begin
      Device.WritelnText(' O.K.');
   end
   else
   begin
      for i := 0 to ClearE.LastIndex do
      begin
         HT := THierarchyTemplate(ClearE.Elements[i]);
         Device.WritelnText('Chyb:  '+HT.FullPath);
      end;
      for i := 0 to NewE.LastIndex do
      begin
         HT := THierarchyTemplate(NewE.Elements[i]);
         Device.WritelnText('Navc:  '+HT.FullPath);
      end;
   end;
   ClearE.Free;
   NewE.Free;
   Node.Template.Reset;
   //Device.WritelnText(' ');
   //Device.WritelnText('    Konec Kontroly Adresov struktury ');
   Device.WritelnText('******************************************');
   Device.WritelnText(' ');
end;

procedure TDirectoryHCheckTool.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TDirectoryHCheckTool.SetNode(const Value: TDirectoryHCheckNode);
begin
  FNode := Value;
end;

initialization
   RegisterOrbisClass(TDirectoryHCheckNode);
   RegisterOrbisClass(TDirectoryHCheckTool);
end.
