{$D+}
unit T_shpimp;

interface
uses
  classes,
  Elements,
  e_GeometryElement,
  E_Reg,
  e_SHPFileElement,
  Tools,
  C_Shp;

type

TShapeFileImport = class(TTool)
private
  FFileName: string;
  FCondition: string;
  FMemStream: TMemoryStream;
  FFileHeader: TShpFileHeader;
  FRecHeader: TShpFileRecHeader;
  //FBuffer: pointer;
  FRegister: TBlockRegister;
  FLastPos: Integer;
    FSType: string;
    FiType: integer;
  procedure SetFileName(aName: string);
  procedure ReadFileHeader;
  procedure ReadRecHeader;
  function  GetFileLength: longInt;
  function  GetRegister: TBlockRegister; virtual;
  function  ConditionIsTrue(aObject: TElement): boolean;
    procedure SetSType(const Value: string);
    procedure SetiType(Value: integer);


public
  BeginPos: longInt;
  function NumberToPropertyName(aNumber: longInt): string;override;
  procedure DoIt; override;
  destructor Destroy; override;
  procedure SkipNextRec;
  function ReadAreas: TAreaCollection;
  function ReadArea(aRecNo, aShpType: longInt): TSHPAreaElement;
  function ReadLines: TPolyLineCollection;
  function ReadLine(aRecNo, aShpType: longInt): TSHPLineElement;
  function ReadPoints: TPointCollection;
  function ReadPoint(aRecNo, aShpType: longInt): TSHPPointElement;
published
  property FileName: string read FFileName write SetFileName stored false;
  property SType: string read FSType write SetSType;
  property iType: integer read FiType write SetiType;
  property Condition: string read FCondition write FCondition stored false;
  property FileLength: longInt read GetFileLength stored false;
  property Register: TBlockRegister read GetRegister stored false;
end;


implementation
uses
  Forms, Dialogs, SysUtils,
  Controls,
  Creacond,
  e_Class,
  e_Geometry,
  e_TopolElements,
  G_Elems,
  t_oper,
// SPACETOOL
  T_Applac,
  e_ShpReg,
  u_Global,
  Worlds;


destructor TShapeFileImport.Destroy;
begin
  Screen.Cursor:= crDefault;


  if FMemStream <> nil then
     FMemStream.Free;
  if FFileHeader <> nil then
     FFileHeader.Free;
  if FRecHeader <> nil then
     FRecHeader.Free;

  inherited Destroy
end;



function TShapeFileImport.GetRegister: TBlockRegister;
var
  TR: TRegister;
begin
  result:= FRegister;
  if result <> nil then exit;

  TR:= TRegister(World.ObjectModel.FindElementClass(TRegister));
  if TR = nil then
  begin
    TR:= TRegister.Create;
    World.ObjectModel.AddElement(TR);
  end;

  if TR = nil then exit;
  FRegister:= TR.GetAndCreateBlockRegister(FileName, 'TShapeBlockRegister');
  result:= FRegister;
end;





procedure TShapeFileImport.ReadFileHeader;
begin
  if FMemStream = nil then exit;
  if FFileHeader <> nil then
     FFileHeader.Free;

  FFileHeader:= TShpFileHeader.Create;
  FMemStream.Seek(0, 0);
  FMemStream.Read(FFileHeader.FFileCode, 100);
  BeginPos:= FMemStream.Position - 100;
{
  ShowMessage(IntToStr(Reader.Position-BeginPos) + #13 +
              IntToStr(Header.FileLength) + ' ' +
              FloatToStr(Header.XMin) + ' ' + FloatToStr(Header.XMax) + #13 +
              FloatToStr(Header.YMin) + ' ' + FloatToStr(Header.YMax));
}
  FMemStream.Seek(100, 0);
end;



procedure TShapeFileImport.ReadRecHeader;
begin
  if FRecHeader <> nil then
     FRecHeader.Free;
  FRecHeader:= TShpFileRecHeader.Create;
  FMemStream.Read(FRecHeader.FRecNo, 12);
  FLastPos  := FMemStream.Position;
end;



procedure TShapeFileImport.SetFileName(aName: string);
begin
  FFileName:= aName;
  if FMemStream <> nil then
     FMemStream.Free;
  FMemStream:= TMemoryStream.Create;
  FMemStream.LoadFromFile(aName);
  ReadFileHeader;
  SType := 'N';
  if FFileHeader <> nil then
  begin
     iType := FFileHeader.ShapeType;
     if FFileHeader.ShapeType in [sht_arc,sht_PolyLineZ,sht_PolyLineM] then
        SType := 'L'
     else if FFileHeader.ShapeType in [sht_polygon, sht_polygonZ] then
        SType := 'A'
     else if FFileHeader.ShapeType = sht_point then
        SType := 'P';
  end;
{  ReadRecHeader }
end;



function  TShapeFileImport.GetFileLength: longInt;
begin
  result:= 0;
  if FFileHeader = nil then exit;
  result:= FFileHeader.FileLength
end;




function TShapeFileImport.ConditionIsTrue(aObject: TElement): boolean;
var
  CC: TCondCreator;
  aCondition: TCondition;

begin
  result:= true;
  if Condition = '' then exit;
  if aObject = nil then exit;

  try
    CC := TCondCreator.Create;
    CC.Command:=Condition;
    CC.DoIt;
    aCondition := TCondition(CC.Output);
    CC.Free;

    if aCondition = nil then
       exit;

    aCondition.ParamItem[0]:=aObject;

    result:= aCondition.IsTrue;

    aCondition.Free;
  except
    ShowMessage('Condition processing Error' + #13+#10+ Condition)
  end
end;






function TShapeFileImport.ReadArea(aRecNo, aShpType: longInt): TSHPAreaElement;
const
  MaxPoints = (MaxInt div Sizeof(TShpPoint)) - 100;
type
   TParts = array[0..1023]of longInt;
   TPoints = array[0..MaxPoints]of TShpPoint;
var
   Poly: TShpPolygonRec;
   Parts, Points: pointer;
   I, J: longInt;
   SHPArea: TSHPAreaElement;
   GArea: TAreaElement;
   LastArea: TAreaElement;
   //PointPlacer: TAreaPointPlacer;
   StartPoint, EndPoint: longInt;
   aHole: TAreaElement;
begin
   //result:= nil;
   Poly:= TShpPolygonRec.Create;
   FMemStream.Read(Poly.FXmin, 40);

   getMem(pointer(Parts), Poly.NumParts * 4);
   FMemStream.Read(Parts^, Poly.NumParts * 4);

   if Poly.NumPoints > MaxPoints then
      Poly.NumPoints:= MaxPoints;

   getMem(Points, Poly.NumPoints * 16);
   FMemStream.Read(Points^, Poly.NumPoints * 16);
 {
   ShowMessage(FloatToStr(Poly.Xmin) + ' ' +  FloatToStr(Poly.Ymin) + ' ' +
               FloatToStr(Poly.Xmax) + ' ' +  FloatToStr(Poly.Ymax) + #13 +
               IntToStr(Poly.NumParts) + ' ' + IntToStr(Poly.NumPoints) + #13 +
               IntToStr(TParts(Parts^)[0]) + ' ' + IntToStr(TParts(Parts^)[1]) + ' ' +
               IntToStr(TParts(Parts^)[2]));
 }
   if Poly.NumParts = 1 then
   begin
      GArea := TAreaElement.Create;
      for I:= 0 to Poly.NumPoints-1 do
      begin
        try
           GArea.AddXY(TPoints(Points^)[I].X, TPoints(Points^)[I].Y)
        except
          showMessage(inttostr(I) + #13 + #10 +
                      floatToStr(TPoints(Points^)[I].X) + '; ' + floatToStr(TPoints(Points^)[I].Y))
        end;
        Application.ProcessMessages;
      end;
      GArea.MinX:= Poly.Xmin;
      GArea.MinY:= Poly.YMin;
      GArea.MaxX:= Poly.Xmax;
      GArea.MaxY:= Poly.Ymax;
//      if GArea.Area < 0 then
//      begin
//         Error := 1;
//         GArea.SwapBorder := true;
//      end;
   end
   else
   begin
      GArea:= TAreaElement.Create;

      StartPoint:= TParts(Parts^)[0];
      EndPoint:= TParts(Parts^)[1]-1;
      for I:= StartPoint to EndPoint do
      begin
         GArea.AddXY(TPoints(Points^)[I].X, TPoints(Points^)[I].Y)
      end;
      LastArea := GArea;
      GArea.MinX:= Poly.Xmin;
      GArea.MinY:= Poly.YMin;
      GArea.MaxX:= Poly.Xmax;
      GArea.MaxY:= Poly.Ymax;
//      if GArea.Area < 0 then
//      begin
//         Error := 1;
//         GArea.SwapBorder := true;
//      end;

      { Vkldn enklv }
      for I:= 1 to Poly.NumParts-1 do
      begin
         StartPoint:= TParts(Parts^)[I];
         if I = Poly.NumParts-1 then
            EndPoint:= Poly.NumPoints-1
         else
            EndPoint:= TParts(Parts^)[I+1]-1;
         if EndPoint > StartPoint+2 then
         begin
            aHole:= TAreaElement.Create;
            for J:= StartPoint to EndPoint do
            begin
               aHole.AddXY(TPoints(Points^)[J].X, TPoints(Points^)[J].Y)
            end;
//            if aHole.Area > 0 then
//            begin
//               Error := 1;
//               aHole.SwapBorder := true;
//            end;
            aHole.RecalculateRect;
            if aHole.RealArea < 0 then
               LastArea.AddHole(aHole)
            else
            begin
               if not (GArea is TMultiAreaElement) then
               begin
                  GArea := TMultiAreaElement.Create;
                  GArea.AddAndUpdateRect(LastArea);
                  GArea.AbsRefX := LastArea.AbsRefX;
                  GArea.AbsRefY := LastArea.AbsRefY;
               end;
               LastArea := aHole;
               GArea.AddAndUpdateRect(LastArea);
               GArea.AbsRefX := LastArea.AbsRefX;
               GArea.AbsRefY := LastArea.AbsRefY;
            end;
         end;
      end;
   end;
   SHPArea := TSHPAreaElement.Create;
   result  := SHPArea;
   SHPArea.RecNo:= aRecNo;
   SHPArea.ShapeType:= aShpType;
   SHPArea.Barva:= 10;
   SHPArea.Znacka:= 1;
   SHPArea.Geometry := GArea;

 {
   PointPlacer:= TAreaPointPlacer.Create;
   PointPlacer.Input := result;
   PointPlacer.DoIt;
   PointPlacer.Free;
 }

   Register.RegisterObject(result);
   if not ConditionIsTrue(result) then
   begin
     result.Free;
     result:= nil;
     exit
   end;

 {
   Form1.Memo.Clear;
   for I:= 0 to Poly.NumPoints-1 do
   begin
     Form1.Memo.Lines.Add(FloatToStr(TPoints(Points^)[I].X) + ' ' + FloatToStr(TPoints(Points^)[I].Y));
   end;
 }
   freeMem(Points, Poly.NumPoints * 16);
   freeMem(Parts, Poly.NumParts*4);
   Poly.Free;
end;



procedure TShapeFileImport.DoIt;
var
   En: TEntity;
   Delta, MinVal{, I}: longInt;
   anArea: TSHPAreaElement;
   TA: TTopolAreaElement;
   aLine: TSHPLineElement;
   TL: TTopolLineElement;
   aPoint: TSHPPointElement;
   TP: TTopolPointElement;
begin
   Error := 0;
   if FMemStream = nil then exit;
   if FFileHeader = nil then exit;
   if Output = nil then
   begin
      En := TGeometryElementCollection.Create;
      Output := EN;
   end
   else
   begin
      if not(output is TEntity) then
         exit;
      En := TEntity(Output);
   end;
   Screen.Cursor:= crHourGlass;
   StartProgress('SHP import: '+FileName,0,FileLength);
   Delta:= FileLength div 100;
   MinVal:= 0;
   //I:= 0;

try
   while (FMemStream.Position-BeginPos < FileLength) do
   begin
      ReadRecHeader;
      if (FMemStream.Position > MinVal) then
      begin
         Progress:= FMemStream.Position;
         ProgressText := 'ten: ' + FileName + '; ' +
                       intToStr(FMemStream.Position) +
                          '(' + IntToStr(FileLength) + ')';
         Application.ProcessMessages;
         //inc(I);
         MinVal:= MinVal + Delta;
      end;
      if FRecHeader.ShapeType in [sht_polygon,sht_PolygonZ] then
      begin
         AnArea:= ReadArea(FRecHeader.RecNo, FRecHeader.ShapeType);
         if TElement(anArea) is TSHPAreaElement then
         if SHPasTopol then
         begin
            TA := TTopolAreaElement.Create;
            TA.Barva  := anArea.Barva;
            TA.Znacka := anArea.Znacka;
            TA.Druh   := 1;
            TA.Geometry := anArea.Geometry;
            TA.Properties := anArea;
            anArea.Geometry := nil;
            En.AddElement(TA);
         end
         else
            En.AddElement(anArea);
      end
      else
      if FRecHeader.ShapeType in [sht_arc,sht_PolyLineZ,sht_PolyLineM] then
      begin
         ALine:= ReadLine(FRecHeader.RecNo, FRecHeader.ShapeType);
         if TElement(aLine) is TSHPLineElement then
         if SHPasTopol then
         begin
            TL := TTopolLineElement.Create;
            TL.Barva := aLine.Barva;
            TL.Znacka := aLine.Znacka;
            TL.Geometry := aLine.Geometry;
            TL.Properties := aLine;
            aLine.Geometry := nil;
            En.AddElement(TL)
         end
         else
            En.AddElement(aLine);
      end
      else
      if FRecHeader.ShapeType = sht_point then
      begin
         APoint:= ReadPoint(FRecHeader.RecNo, FRecHeader.ShapeType);
         if TElement(aPoint) is TSHPPointElement then
         if SHPasTopol then
         begin
            TP := TTopolPointElement.Create;
            TP.Barva := aPoint.Barva;
            TP.Znacka := aPoint.Znacka;
            TP.Geometry := aPoint.Geometry;
            TP.Properties := aPoint;
            aPoint.Geometry := nil;
            En.AddElement(TP);
         end
         else
            En.AddElement(aPoint);
      end;
//      else
      SkipNextRec;
   end;
   if (En is TRectEntity) then
      TRectEntity(En).Update;
   EndProgress;
except
   ShowMessage('XXXX')
end;
end;



procedure TShapeFileImport.SkipNextRec;
var
  Pom: Word;
  I: longInt;
begin
   FMemStream.Seek(FLastPos + ((FRecHeader.RecLength - 4)), 0 );
  {
  for I:= 1 to FRecHeader.RecLength-4 do
      FMemStream.Read(Pom, 1);
   }
end;




function TShapeFileImport.ReadAreas: TAreaCollection;
var
  //AreaS: TAreaString;
  anArea: TSHPAreaElement;
  Delta, MinVal{, I}: longInt;
  TP: TTopolAreaElement;
begin
  result:= nil;
  if FMemStream = nil then exit;
  if FFileHeader = nil then exit;
{  if FRecHeader = nil then exit; }

  Screen.Cursor:= crHourGlass;
  StartProgress('SHP import: '+FileName,0,FileLength);
  Delta:= FileLength div 100;
  MinVal:= 0;
  //I:= 0;

  result:= TAreaCollection.Create;
  result.Name:= FileName;
try
  while (FMemStream.Position-BeginPos < FileLength) do
  begin
    ReadRecHeader;

    if (FMemStream.Position > MinVal) then
    begin
       Progress:= FMemStream.Position;
       ProgressText := 'ten: ' + FileName + '; ' +
                             intToStr(FMemStream.Position) +
                             '(' + IntToStr(FileLength) + ')';
       Application.ProcessMessages;
       //inc(I);
       MinVal:= MinVal + Delta;
    end;

    if FRecHeader.ShapeType in [sht_polygon, sht_polygonZ] then
    begin
      AnArea:= ReadArea(FRecHeader.RecNo, FRecHeader.ShapeType);
      if TElement(anArea) is TSHPAreaElement then
         if SHPasTopol then
         begin
            TP := TTopolAreaElement.Create;
            TP.Barva  := anArea.Barva;
            TP.Znacka := anArea.Znacka;
            TP.Druh   := 1;
            TP.Geometry := anArea.Geometry;
            TP.Properties := anArea;
            anArea.Geometry := nil;
            result.AddAndUpdateRect(TP);
         end
         else
            result.AddAndUpdateRect(anArea)
    end;
//    else
    SkipNextRec;
  end;
  EndProgress;
except
  ShowMessage('XXXX')
end;
end;



function TShapeFileImport.ReadLine(aRecNo, aShpType: Integer): TSHPLineElement;
const
  MaxPoints = $7FFF;
type
   TParts = array[0..1023]of longInt;
   TPoints = array[0..MaxPoints]of TShpPoint;
var
   Poly: TShpPolygonRec;
   Parts, Points: pointer;
   I, J: longInt;
   SHPLine: TSHPLineElement;
   GLine: TPolyLineElement;
   MLine: TMultiPolyLineElement;
   //PointPlacer: TAreaPointPlacer;
   StartPoint, EndPoint: longInt;
   //aHole: TAreaElement;
begin
   //result:= nil;
   Poly:= TShpPolygonRec.Create;
   FMemStream.Read(Poly.FXmin, 40);

   getMem(pointer(Parts), Poly.NumParts * 4);
   FMemStream.Read(Parts^, Poly.NumParts * 4);

   if Poly.NumPoints > MaxPoints then
      Poly.NumPoints:= MaxPoints;

   getMem(Points, Poly.NumPoints * 16);
   FMemStream.Read(Points^, Poly.NumPoints * 16);
 {
   ShowMessage(FloatToStr(Poly.Xmin) + ' ' +  FloatToStr(Poly.Ymin) + ' ' +
               FloatToStr(Poly.Xmax) + ' ' +  FloatToStr(Poly.Ymax) + #13 +
               IntToStr(Poly.NumParts) + ' ' + IntToStr(Poly.NumPoints) + #13 +
               IntToStr(TParts(Parts^)[0]) + ' ' + IntToStr(TParts(Parts^)[1]) + ' ' +
               IntToStr(TParts(Parts^)[2]));
 }
   if Poly.NumParts = 1 then
   begin
      GLine := TPolyLineElement.Create;
      for I:= 0 to Poly.NumPoints-1 do
      begin
        try
           GLine.AddXY(TPoints(Points^)[I].X, TPoints(Points^)[I].Y)
        except
          showMessage(inttostr(I) + #13 + #10 +
                      floatToStr(TPoints(Points^)[I].X) + '; ' + floatToStr(TPoints(Points^)[I].Y))
        end;
        Application.ProcessMessages;
      end;
      GLine.MinX:= Poly.Xmin;
      GLine.MinY:= Poly.YMin;
      GLine.MaxX:= Poly.Xmax;
      GLine.MaxY:= Poly.Ymax;
   end
   else
   begin
      MLine := TMultiPolyLineElement.Create;
      GLIne:= TPolyLineElement.Create;

      StartPoint:= TParts(Parts^)[0];
      EndPoint:= TParts(Parts^)[1]-1;
      for I:= StartPoint to EndPoint do
      begin
         GLine.AddXY(TPoints(Points^)[I].X, TPoints(Points^)[I].Y)
      end;

      GLine.MinX:= Poly.Xmin;
      GLine.MinY:= Poly.YMin;
      GLine.MaxX:= Poly.Xmax;
      GLine.MaxY:= Poly.Ymax;
      MLine.AddPolyLine(GLine,true);
      { Vkldn enklv }
      for I:= 1 to Poly.NumParts-1 do
      begin
         StartPoint:= TParts(Parts^)[I];
         if I = Poly.NumParts-1 then
            EndPoint:= Poly.NumPoints-1
         else
            EndPoint:= TParts(Parts^)[I+1]-1;
         if EndPoint > StartPoint+2 then
         begin
            GLine:= TPolyLineElement.Create;
            for J:= StartPoint to EndPoint do
            begin
               GLine.AddXY(TPoints(Points^)[J].X, TPoints(Points^)[J].Y)
            end;
            GLine.RecalculateRect;
            MLine.AddPolyLine(GLine,true);
         end;
      end;
      GLine := MLine;
   end;
   SHPLine := TSHPLineElement.Create;
   result  := SHPLine;
   SHPLine.RecNo:= aRecNo;
   SHPLine.ShapeType:= aShpType;
   SHPLine.Barva:= 13;
   SHPLine.Znacka:= 0;
   SHPLine.Geometry := GLine;

 {
   PointPlacer:= TAreaPointPlacer.Create;
   PointPlacer.Input := result;
   PointPlacer.DoIt;
   PointPlacer.Free;
 }

   Register.RegisterObject(result);
   if not ConditionIsTrue(result) then
   begin
     result.Free;
     result:= nil;
     exit
   end;

 {
   Form1.Memo.Clear;
   for I:= 0 to Poly.NumPoints-1 do
   begin
     Form1.Memo.Lines.Add(FloatToStr(TPoints(Points^)[I].X) + ' ' + FloatToStr(TPoints(Points^)[I].Y));
   end;
 }
   freeMem(Points, Poly.NumPoints * 16);
   freeMem(Parts, Poly.NumParts*4);
   Poly.Free;
end;

function TShapeFileImport.ReadLines: TPolyLineCollection;
var
  //AreaS: TAreaString;
  aLine: TSHPLineElement;
  TL: TTopolLineElement;
  Delta, MinVal{, I}: longInt;
begin
  result:= nil;
  if FMemStream = nil then exit;
  if FFileHeader = nil then exit;
{  if FRecHeader = nil then exit; }

  Screen.Cursor:= crHourGlass;
  StartProgress('SHP import: '+FileName,0,FileLength);
  Delta:= FileLength div 100;
  MinVal:= 0;
  //I:= 0;

  result:= TPolyLineCollection.Create;
  result.Name:= FileName;
try
  while (FMemStream.Position-BeginPos < FileLength) do
  begin
    ReadRecHeader;

    if (FMemStream.Position > MinVal) then
    begin
       Progress:= FMemStream.Position;
       ProgressText := 'ten: ' + FileName + '; ' +
                             intToStr(FMemStream.Position) +
                             '(' + IntToStr(FileLength) + ')';
       Application.ProcessMessages;
       //inc(I);
       MinVal:= MinVal + Delta;
    end;

    if FRecHeader.ShapeType in [sht_arc,sht_PolyLineZ,sht_PolyLineM] then
    begin
      ALine:= ReadLine(FRecHeader.RecNo, FRecHeader.ShapeType);
      if TElement(aLine) is TSHPLineElement then
         if SHPasTopol then
         begin
            TL := TTopolLineElement.Create;
            TL.Barva := aLine.Barva;
            TL.Znacka := aLine.Znacka;
            TL.Geometry := aLine.Geometry;
            TL.Properties := aLine;
            aLine.Geometry := nil;
            result.AddAndUpdateRect(TL)
         end
         else
            result.AddAndUpdateRect(aLine)
    end;
    //else
    SkipNextRec;
  end;
  EndProgress;
except
  ShowMessage('XXXX')
end;
end;

function TShapeFileImport.ReadPoint(aRecNo,aShpType: Integer): TSHPPointElement;
var
   Pnt: TShpPoint;
   GPoint: TPointElement;
begin
   //result:= nil;
   FMemStream.Read(Pnt, SizeOf(Pnt));
   GPoint := TPointElement.Create;
   GPoint.X := Pnt.X;
   GPoint.Y := Pnt.Y;
   result := TSHPPointElement.Create;
   result.RecNo:= aRecNo;
   result.ShapeType:= aShpType;
   result.Barva:= 13;
   result.Znacka:= 0;
   result.Geometry := GPoint;

   Register.RegisterObject(result);
   if not ConditionIsTrue(result) then
   begin
     result.Free;
     result:= nil;
     exit
   end;
end;

function TShapeFileImport.ReadPoints: TPointCollection;
var
  //AreaS: TAreaString;
  aPoint: TSHPPointElement;
  TP: TTopolPointElement;
  Delta, MinVal{, I}: longInt;
begin
  result:= nil;
  if FMemStream = nil then exit;
  if FFileHeader = nil then exit;
{  if FRecHeader = nil then exit; }

  Screen.Cursor:= crHourGlass;
  StartProgress('SHP import: '+FileName,0,FileLength);
  Delta:= FileLength div 100;
  MinVal:= 0;
  //I:= 0;

  result:= TPointCollection.Create;
  result.Name:= FileName;
try
  while (FMemStream.Position-BeginPos < FileLength) do
  begin
    ReadRecHeader;

    if (FMemStream.Position > MinVal) then
    begin
       Progress:= FMemStream.Position;
       ProgressText := 'ten: ' + FileName + '; ' +
                             intToStr(FMemStream.Position) +
                             '(' + IntToStr(FileLength) + ')';
       Application.ProcessMessages;
       //inc(I);
       MinVal:= MinVal + Delta;
    end;

    if FRecHeader.ShapeType = sht_point then
    begin
      APoint:= ReadPoint(FRecHeader.RecNo, FRecHeader.ShapeType);
      if TElement(aPoint) is TSHPPointElement then
         if SHPasTopol then
         begin
            TP := TTopolPointElement.Create;
            TP.Barva := aPoint.Barva;
            TP.Znacka := aPoint.Znacka;
            TP.Geometry := aPoint.Geometry;
            TP.Properties := aPoint;
            aPoint.Geometry := nil;
            result.AddAndUpdateRect(TP)
         end
         else
            result.AddAndUpdateRect(aPoint)
    end;
//    else
    SkipNextRec;
  end;
  EndProgress;
except
  ShowMessage('XXXX')
end;
end;

function TShapeFileImport.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0:result:='FileName';
      else result:='Output';
      end;{case}
end;

procedure TShapeFileImport.SetSType(const Value: string);
begin
  FSType := Value;
end;

procedure TShapeFileImport.SetiType(Value: integer);
begin
  FiType := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TShapeFileImport);
END.
