unit T_applac;

interface
uses
   Wintypes,
// KERNEL
   Elements,
   Tools,
   T_Recurs,
// SPACE
   e_Geometry,
   //e_Space,
// DEVICE
   e_Device,
// SYMBOL
   e_PlZn;

type

TAnalDevice = class(TCanvasDevice)
  private
    F_PageSize: integer;
    procedure Set_PageSize(const Value: integer);
 public
   procedure SetupPage(Sender: TObject); override;
   property  _PageSize: integer read F_PageSize write Set_PageSize;
end;


TIntegerPolygon = class(TElement)
 private
   FCount   : integer;         { Pocet Lomovych bodu plochy }
   FSize    : longInt;
   FPoints  : PVrcholy;        { Buffer s Lomovymi body }
 public
   destructor  Destroy; override;
   property Count: integer read FCount write FCount default 0;
   property Size: longInt read FSize write FSize;
   property Points: PVrcholy read FPoints write FPoints;
end;

TPolygonList = class(TEntity)
 protected
   function GetPointsCount: longInt;
 public
   property PointsCount: longInt read GetPointsCount;
end;

TAnalArea = class(TPlosnaZnacka)
 public
   function  FindCentroid (PolygonList: TPolygonList): TPoint;
   procedure UvodniZpracovaniPolyHranice (PolygonList: TPolygonList; var PocetHran, NejnizsiY : integer);
end;


TAreaPointPlacer = class(TTool)
private
   FNotRectify: boolean;
 protected
   FDevice: TAnalDevice;
   FPolygonList: TPolygonList;
   FAnalArea: TAnalArea;
   function  GetArea: TAreaElement;
   function  GetPolygonList: TPolygonList;
   procedure SetPolygonList(aPolygonList: TPolygonList);
   function  GetAnalArea: TAnalArea;
   procedure SetAnalArea(aAnalArea: TAnalArea);
   function  GetDevice: TAnalDevice;
   procedure SetDevice(aDevice: TAnalDevice);
   procedure CreateDevice(aPageSize: integer);
   procedure CreatePolygonList;
   procedure FreePolygonList;
   procedure SetPolygonPoint;
 public
   procedure DoIt; override;
   property  Area: TAreaElement read GetArea;
   property  Device: TAnalDevice read GetDevice write SetDevice;
   property  PolygonList: TPolygonList read GetPolygonList write SetPolygonList;
   property  AnalArea: TAnalArea read GetAnalArea write SetAnalArea;
   property  NotRectify: boolean read FNotRectify write FNotRectify;
end;

TMultiAreaPointPlacer = class(TRecursor)
  private
    FProcessAreaCount: integer;
    procedure SetProcessAreaCount(const Value: integer);
 protected
   FPointPlacer: TAreaPointPlacer;
   function  GetPointPlacer: TAreaPointPlacer;
   procedure SetPointPlacer(aPointPlacer: TAreaPointPlacer);
 public
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   procedure DoIt; override;
   property  ProcessAreaCount: integer read FProcessAreaCount write SetProcessAreaCount;
   property  PointPlacer: TAreaPointPlacer read GetPointPlacer write SetPointPlacer;
end;


implementation
uses
   Dialogs, SysUtils,
   classes, Forms, Controls,
// KERNEL
   e_Class,
// DEVICE
   e_Page,
// GLIB
   m_Orbis,
   AnalInt;

{ TAnalDevice }

procedure TAnalDevice.SetupPage(Sender: TObject);
var
   HResX, HResY: double;
begin
   if not assigned(FPage) then
   begin
      FPage := T_Page.Create;
      AddElement(FPage);
   end;
   //Page.Rect := Rect(0,0,500,500);
   //Page.Rect := Rect(0,0,1000,1000);
   if _PageSize < 500 then _PageSize := 500;
   Page.Rect := Rect(0,0,_PageSize,_PageSize);
   Page.ClipRegion:= Page.Rect;

   HResX := 1000;
   HResY := 1000;
   HResX := HResX * 200 / SizeOfInch;
   HResY := HResY * 200 / SizeOfInch;
   Page.SetResolution(HResX, HResY);
   Projection.ChangeProjection
    (Projection.CenterPoint, Projection.Scale, Page)
end;

procedure TAnalDevice.Set_PageSize(const Value: integer);
begin
  F_PageSize := Value;
end;

{ TIntegerPolygon }

destructor  TIntegerPolygon.Destroy;
begin
   FreeMem(Points, Size);
   inherited Destroy
end;

{ TPolygonList }

function TPolygonList.GetPointsCount: longInt;
var
   i: longInt;
begin
   result := 0;
   for i := 0 to count -1 do
   begin
      inc(result, TIntegerPolygon(Elements[i]).Count);
   end;
end;


{ TAnalArea }

function TAnalArea.FindCentroid (PolygonList: TPolygonList): TPoint;
var
  PocetHran, PrvniHrana, PosledniHrana,
  radek, NejnizsiY, PocetPruseciku : integer;
  PocetVrcholu: longInt;
  k, Index_Hrany: longInt;
  HalfY: longInt;
  dx, dxx, x1, xx1, x2, xx2: longInt;
begin {Scan_Line_Fill}
  PocetVrcholu := PolygonList.PointsCount;
  GetMem(Hranice, PocetVrcholu * SizeOf(Hrana));
  if not Assigned(Hranice) then exit;

  UvodniZpracovaniPolyHranice (PolygonList, PocetHran, NejnizsiY);
  if PocetHran = 0 then
  begin
     FreeMem(Hranice, PocetVrcholu * SizeOf(Hrana));
     result.X := 0;
     result.Y := 0;
     exit;
  end;

  PrvniHrana := 1;              { inicializace ukazatelu }
  PosledniHrana := 1;           { do setrideneho seznamu }
  radek := Hranice^[1].y_horni;
  HalfY := NejnizsiY + (radek - NejnizsiY) div 2;
  while radek > {NejnizsiY} HalfY do
  begin
     AktualizujSeznamHran ( PocetHran, radek, PrvniHrana, PosledniHrana);
     UpravSeznamHran (PrvniHrana, PosledniHrana);
     Dec(radek);
  end; {for radek}
  AktualizujSeznamHran ( PocetHran, radek, PrvniHrana, PosledniHrana);
  NalezniPrusecikyY ( radek, PocetPruseciku, PrvniHrana, PosledniHrana);
  result.y := radek;

  dx := 0;
  x1 := 0;
  x2 := 0;
  Index_Hrany := PrvniHrana;
  for k := 1 to PocetPruseciku div 2 do
  begin
     while hranice^[index_hrany].DeltaY = 0 do inc(index_hrany);
     xx1 := hranice^[index_hrany].PrusecikX div UniverzalniDelitel;
     inc(index_hrany);
     while hranice^[index_hrany].DeltaY = 0 do inc(index_hrany);
     xx2 := hranice^[index_hrany].PrusecikX div UniverzalniDelitel;
     dxx := xx2 - xx1;
     if dxx > dx then
     begin
        dx := dxx;
        x1 := xx1;
        x2 := xx2;
     end;
     inc(index_hrany);
  end; {for k}

  result.x := x1 + (x2 - x1) div 2;

  FreeMem(Hranice, PocetVrcholu * SizeOf(Hrana));
end; {Scan_Line_Fill}



procedure TAnalArea.UvodniZpracovaniPolyHranice (PolygonList: TPolygonList; var PocetHran, NejnizsiY : integer);
var
  k, x1, y1 : longInt;
  i: longInt;
  P: TIntegerPolygon;

  {LF}
  function Najdi_dalsi_y (k: longInt) : integer;
  var
    i:longInt;
    hotovo : boolean;
  begin
       {nalezne v seznamu vrcholu dalsi vrchol,
       jehoz souradnice y neni rovna y[k]}
     i:=k;
     hotovo := false;
     while not (hotovo) do
        if P.Points^[i].y <> P.Points^[k].y then hotovo := true
        else
        begin
           inc(i);
           if i > P.Count then i := 1;
        end;
     Najdi_dalsi_y := P.Points^[i].y
  end; {Najdi_dalsi_y}

  {LF}
  procedure Vloz_do_seznamu_hran (polozka, x1, y1, x2, y2, dalsi_y : longInt);
  var
    max_y : longInt;
    x2_prac, zmena_x_prac : single;
    Zarazeno : boolean;
  begin
     { uprava souradnic vrcholu pri kraceni hrany }
     { ------------------------------------------ }
     zmena_x_prac := (x2 - x1) / (y2 - y1) ;
     x2_prac := x2;
     if (y2 > y1) and (y2 < dalsi_y) then
     begin
        y2 := y2 - 1;
        x2_prac := x2_prac - zmena_x_prac;
     end
     else
     if (y2 < y1) and (y2 > dalsi_y) then
     begin
        y2 := y2 + 1;
        x2_prac := x2_prac + zmena_x_prac;
     end;

     { zatrideni do seznamu hran podle max_y dane hrany }
     { ------------------------------------------------ }
     if y1 > y2 then max_y := y1 else max_y := y2;
     Zarazeno := false;
     while not(Zarazeno) and (polozka > 1) do
     begin
        if (max_y <= hranice^[polozka - 1].y_horni) then
           Zarazeno := true
        else
        begin
           hranice^[polozka] := hranice^[polozka-1];
           polozka := polozka - 1
        end;
     end; {while}
     with hranice^[polozka] do
     begin
          y_horni := max_y;
          DeltaY := abs(y2 - y1) + 1;
          if y1 > y2 then PrusecikX := longint(x1)*UniverzalniDelitel
                     else PrusecikX := round(x2_prac*UniverzalniDelitel);
          ZmenaXNaRadek := round(zmena_x_prac*UniverzalniDelitel){ * VeleKrok};
     end; {with}
  end; {Vloz_do_seznamu_hran}




{MAIN}
begin {vodni_zpracovani_hranice }
   PocetHran := 0;
   NejnizsiY := 10000;

   for i := 0 to PolygonList.Count -1 do
   begin
      P := TIntegerPolygon(PolygonList.Elements[i]);

      x1    := P.Points^[P.Count].x;
      y1    := P.Points^[P.Count].y;
      for k := 1 to P.Count do
      begin
         if y1 = P.Points^[k].y then        { vodorovna hrana je hned vykreslena }
         begin
         end
         else                    { umisteni nevodorovne hrany do seznamu }
         begin
           inc(PocetHran);
           Vloz_do_seznamu_hran (PocetHran, x1, y1,
                                 P.Points^[k].x,
                                 P.Points^[k].y,
                                 Najdi_dalsi_y(k));
         end;
         y1 := P.Points^[k].y;
         x1 := P.Points^[k].x; { priprava pro dalsi hranu }
         if y1 < NejnizsiY then NejnizsiY := y1;
      end; {for k}
   end; {for i}
end; { vodni_zpracovani_hranice }




function GoodCenter(aX, aX1, aX2: double): boolean;
var
  aMin, aPom: double;
begin
  aMin:= abs(aX1-aX);
  aPom:= abs(aX2-aX);
  if aPom < aMin then aMin:= aPom;
  result:= abs(aX1-aX2) < (100*aMin)
end;





procedure TAreaPointPlacer.SetPolygonPoint;
var
   P: TPoint;
   X1, X2: double;
   CrossCount: longInt;
   PomArea: TAreaElement;
   _Inside: boolean;
   function TryToFindCentroid(aPageSize: integer): boolean;
   begin
      if aPageSize > 500 then
      begin
         FreePolygonList;
         //PostTextMsg(tm_AddToProtocol,'Iterace umstn pop.bodu u plochy ' + IntToStr(aPageSize),nil);
      end;
      CreateDevice(aPageSize);
      CreatePolygonList;
      P := AnalArea.FindCentroid(PolygonList);
      Area.AbsRefX:= Device.Projection.GetModelX(P.x);
      Area.AbsRefY:= Device.Projection.GetModelY(P.Y);
      result := Area.XYIsInside(Area.AbsRefX, Area.AbsRefY);
      _Inside := result;
   end;
begin
  try
    Area.RefCoords.X := 0;
    Area.RefCoords.Y := 0.5;
    if not TryToFindCentroid(500) then
       if not TryToFindCentroid(1000) then
          if not TryToFindCentroid(5000) then
              TryToFindCentroid(10000);
    if not _Inside then
    begin
      PomArea:= Area;
      if Area is TPolyLineAreaElement then
         PomArea:= TPolyLineAreaElement(Area).CreateAreaElement;
      Area.RefCoords.X := 0;
      Area.RefCoords.Y := 0.5;
      Nearest2PointsRight(PomArea, Area.AbsRefX-1, Area.AbsRefY, X1, X2, CrossCount);
      if (CrossCount > 1) and not odd(CrossCount) then
      begin
        Area.AbsRefX:= (X1 + X2) / 2;
        if not Area.XYIsInside(Area.AbsRefX, Area.AbsRefY) then
        begin
           PostTextMsg(tm_AddToProtocol,'Chyba umstn pop.bodu u plochy ' + #13 + #10 +
                            FloatToStr(Area.AbsRefX) + ' ; ' + FloatToStr(Area.AbsRefY),nil);
          (*
          {!!!} ShowMessage('Chyba umstn pop.bodu u plochy ' + #13 + #10 +
                            FloatToStr(Area.AbsRefX) + ' ; ' + FloatToStr(Area.AbsRefY));
           *)
        end;
      end
      else
      begin
        Area.RelRefY := 0.3;
        Nearest2PointsRight(PomArea, Area.AbsRefX-1E10, Area.AbsRefY, X1, X2, CrossCount);
        if (CrossCount > 1) and not odd(CrossCount) then
        begin
          Area.AbsRefX:= (X1 + X2) / 2;
          if not Area.XYIsInside(Area.AbsRefX, Area.AbsRefY) then
          begin
             PostTextMsg(tm_AddToProtocol,'Chyba umstn pop.bodu u plochy ' + #13 + #10 +
                              FloatToStr(Area.AbsRefX) + ' ; ' + FloatToStr(Area.AbsRefY),nil);
             (*
            {!!!} ShowMessage('Chyba umstn pop.bodu u plochy ' + #13 + #10 +
                              FloatToStr(Area.AbsRefX) + ' ; ' + FloatToStr(Area.AbsRefY));
             *)
          end;
        end
      end;
      if Area is TPolyLineAreaElement then
         PomArea.Free;
    end
(*
     else
     if not NotRectify then
     begin
       PomArea:= Area;
       if Area is TPolyLineAreaElement then
          PomArea:= TPolyLineAreaElement(Area).CreateAreaElement;
       Nearest2PointsLeftRight(PomArea, Area.AbsRefX, Area.AbsRefY, X1, X2, CrossCount);
       if (CrossCount > 1) and not odd(CrossCount) then
       begin
         Area.AbsRefX:= (X1 + X2) / 2;
         if not Area.XYIsInside(Area.AbsRefX, Area.AbsRefY) then
         begin
           {!!!} ShowMessage('Chyba umstn pop.bodu u plochy ' + #13 + #10 +
                             FloatToStr(Area.AbsRefX) + ' ; ' + FloatToStr(Area.AbsRefY));
         end;
       end
       else
       begin
         X1:= X1 + X2 -X2;
       end;

       if Area is TPolyLineAreaElement then
          PomArea.Free;
     end
*)
     ;

    FreePolygonList;

  except
    Area.AbsRefX:= Area.CenterX;
    Area.AbsRefY:= Area.CenterY;
  end
end;





{ TAreaPointPlacer }

{ TODO -oPPK -cDodlat : popisove body jsou porad nejake divne }

procedure TAreaPointPlacer.DoIt;
var
   X1, X2: double;
   CrossCount: longInt;
   PomArea: TAreaElement;
begin
  Output := Input;
  if Area = nil then exit;
  try
    //ppk
    SetPolygonPoint;
    exit;
    //ppk
    if Area.XYIsInside(Area.CenterX, Area.CenterY) then
    begin
      Area.AbsRefX:= Area.CenterX;
      Area.AbsRefY:= Area.CenterY;
      if not NotRectify then
      begin
        PomArea:= Area;
        if Area is TPolyLineAreaElement then
           PomArea:= TPolyLineAreaElement(Area).CreateAreaElement;
        Nearest2PointsLeftRight(PomArea, Area.AbsRefX, Area.AbsRefY, X1, X2, CrossCount);

        if not GoodCenter(Area.AbsRefX, X1, X2) then
        begin
          if (CrossCount > 1) and not odd(CrossCount) then
          begin
            Area.AbsRefX:= (X1 + X2) / 2;
            if not Area.XYIsInside(Area.AbsRefX, Area.AbsRefY) then
            begin
              SetPolygonPoint
            end;
          end
          else
          begin
            X1:= X1 + X2 -X2;
          end;
        end;

        if Area is TPolyLineAreaElement then
           PomArea.Free;
      end;


      exit;
    end;

    SetPolygonPoint;

  except
    //ShowMessage('Popisov bod ! ... exception');
    Area.AbsRefX:= Area.CenterX;
    Area.AbsRefY:= Area.CenterY;
  end
end;


function  TAreaPointPlacer.GetArea: TAreaElement;
begin
   result := TAreaElement(Input);
end;


function  TAreaPointPlacer.GetPolygonList: TPolygonList;
begin
   if not assigned(FpolygonList) then
   begin
      FpolygonList := TPolygonList.Create;
      AddElement(FPolygonList);
   end;
   result := FPolygonList;
end;

procedure TAreaPointPlacer.SetPolygonList(aPolygonList: TPolygonList);
begin
   if assigned(FPolygonList) then
   begin
      FreeElement(FPolygonList);
      FpolygonList := nil;
   end;
   FPolygonList := aPolygonList;
   AddElement(FPolygonList);
end;


function  TAreaPointPlacer.GetAnalArea: TAnalArea;
begin
   if not assigned(FAnalArea) then
   begin
      FAnalArea := TAnalArea.Create;
      AddElement(FAnalArea);
   end;
   result := FAnalArea;
end;

procedure TAreaPointPlacer.SetAnalArea(aAnalArea: TAnalArea);
begin
   if assigned(FAnalArea) then
   begin
      FreeElement(FAnalArea);
      FAnalArea := nil;
   end;
   FAnalArea := aAnalArea;
   AddElement(FAnalArea);
end;


function  TAreaPointPlacer.GetDevice: TAnalDevice;
begin
   if not assigned(FDevice) then
   begin
      FDevice := TAnalDevice.Create;
      AddElement(FDevice);
   end;
   result := FDevice;
end;

procedure TAreaPointPlacer.SetDevice(aDevice: TAnalDevice);
begin
   if assigned(FDevice) then
   begin
      FreeElement(FDevice);
      FDevice := nil;
   end;
   FDevice := aDevice;
   AddElement(FDevice);
end;



procedure TAreaPointPlacer.CreateDevice(aPageSize: integer);
begin
   try
      Device._PageSize := aPagesize;
      Device.SetupPage(nil);
      Device.Projection.ChangeProjectionByRectangle(Area.Rect, Device.Page);
   except
   end;
end;


procedure TAreaPointPlacer.CreatePolygonList;
var
   BArea, SubArea: TAreaElement;
   Polygon: TIntegerPolygon;
   i: longInt;
   Buffer: Pointer;
   Size: integer;
   cc : integer;
begin
   BArea := nil;
   Polygon := TIntegerPolygon.Create;
   PolygonList.AddElement(Polygon);

   if Area is TPolyLineAreaElement then
   begin
      BArea := TPolyLineAreaElement(Area).CreateAreaElement;
      Device.FillBuffer(BArea, Buffer, Size, cc);
   end
   else
     Device.FillBuffer(Area, Buffer, Size, cc);

   Polygon.Points := Buffer;
   Polygon.Size   := Size;
   Polygon.Count  := cc;

{   BArea.Clear; odstranil O.S. 980104 - fillBuffer si bere kopii !}
   if Area is TPolyLineAreaElement then
      BArea.Free;

   if Area.Holes = nil then exit;

   if Area.Holes.Count = 0 then exit;
   for i := 0 to Area.Holes.LastIndex do
   begin
      SubArea:= TAreaElement(Area.Holes.Elements[i]);
      if SubArea.NoElements then continue;   // prazdna plocha
      Polygon := TIntegerPolygon.Create;
      PolygonList.AddElement(Polygon);
      if SubArea is TPolyLineAreaElement then
      begin
        BArea := TPolyLineAreaElement(SubArea).CreateAreaElement;
        Device.FillBuffer(BArea, Buffer, Size, cc);
      end
      else
        Device.FillBuffer(SubArea, Buffer, Size, cc);

      Polygon.Points := Buffer;
      Polygon.Size   := Size;
      Polygon.Count  := cc;
{      BArea.Clear; odstranil O.S. 980104}
      if SubArea is TPolyLineAreaElement then
         BArea.Free;
   end;
end;


procedure TAreaPointPlacer.FreePolygonList;
begin
   PolygonList.FreeAll;
end;


{ TMultiAreaPointPlacer }




procedure TMultiAreaPointPlacer.Process(Element: TElement);
var
  Are: TAreaElement;
begin
  if Element is TAreaElement then
  begin
    Are := TAreaElement(Element);
    //Plocha:= TAreaElement(Element);
    try
      PointPlacer.NotRectify:= true;
      PointPlacer.Input := Element;
      PointPlacer.DoIt;
      ProcessAreaCount := ProcessAreaCount + 1;
    except
    end;
    if not Are.IsHitByXY(Are.X,Are.Y,0) then
       //ShowMessage('POPOPO');
       {PostTextMsg(tm_AddToProtocol,' *Internal popopo err',nil)};
  end;
end;


function  TMultiAreaPointPlacer.QGoInside(Element: TElement): boolean;
begin
  result:= Element is TEntity
end;

function  TMultiAreaPointPlacer.QProcessBefore(Element: TElement): boolean;
begin
  result:= (Element is TAreaElement) and not (Element is TMultiAreaElement);
end;




procedure TMultiAreaPointPlacer.DoIt;
begin
   Screen.Cursor:= crHourGlass;
   //if not (Input is TAreaString) then exit;
   if not (Input is TEntity) then exit;
   FPointPlacer:= TAreaPointPlacer.Create;
   ProcessAreaCount := 0;
   try
     inherited DoIt;
   except
   end;

   FPointPlacer.Free;
   Screen.Cursor:= crDefault
end;


function  TMultiAreaPointPlacer.GetPointPlacer: TAreaPointPlacer;
begin
   if not assigned(FPointPlacer) then
   begin
      FPointPlacer := TAreaPointPlacer.Create;
      AddElement(FPointPlacer);
   end;
   result := FPointPlacer;
end;

procedure TMultiAreaPointPlacer.SetPointPlacer(aPointPlacer: TAreaPointPlacer);
begin
   if assigned(FPointPlacer) then
   begin
      FreeElement(FPointPlacer);
      FPointPlacer := nil;
   end;
   FPointPlacer := aPointPlacer;
   AddElement(FPointPlacer);
end;



procedure TMultiAreaPointPlacer.SetProcessAreaCount(const Value: integer);
begin
   FProcessAreaCount := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMultiAreaPointPlacer);
END.





