unit T_Topol;

{ vytvareni topologie }
{ TTopolNodeBranch          vetev uzlu - element
  TTopolNode                vlastni uzel - OwnerEntity
  TPutLinesIntoNodes        vkldn lini do uzl v NODES v dan entit.
  TProchazecListuQTree      pedek prochaze qtree, vyhledv a pot list
  TPutNodesIntoQTree        projde vechny listy a pouv TPutLinesIntoNodes
  TNodeConector             vyhled pro dan uzel vechny protiuzly
  TNodesConector            vyhled vechny uzly v QTree a pouv TNodeConector
  TCreateAreaForBranch = class(TTool) vytvo plochu pro vtev uzlu
  TCreateAreasForQTree = class(TProchazecListuQTreeGauge)
       Pro vechny listy QTree a jejich uzly a jejich vetve vytvari plochy

{ 1. Jednoduse
     z linii, ktere jiz byly topologicky zpracovany
     a) vytvoreni vazeb mezi liniemi navzajem
     b) vytvoreni ploch i s enklavami
Reseni
  Vytvome strukturu uzl. Kad uzel je entita drc
  Souadnici uzlu
  Vtve uzlu                     Branches
         Vstupujc linie,       Line
         Potek linie           LineBegin
         Plocha vpravo           RightArea
         Ukazatel na uzel        Node

  Uzly se vytv v cyklu pes vechny doten linie.
     Jak zjistm, zda pro tuto linii ji existuje uzel ? Uzel je ve stejnem
     QTree jako linie, takze pouhym porovnanim. Navic linie je uzlem vlastnena,
     takze muze projit vsechny vlastniky linie a najit mezi nimi jeden az dva
     uzly.
  Po vytvoen st je mon definovat plochy.

     Cyklus pes vechny uzly.
     Zjisteni vsech vetvi, zda nejsou vetve se stejnym uhlem.
     Prohledv se uzel zda m vude plochy. Kdy ne, tak se obhne dokola.

     }

interface
uses
// KERNEL
  Elements,
  Tools,
  T_Recurs,
// GLIB
  G_Elems,
// SPACE
  e_Geometry,
  //E_Space,
  e_QTree,
// SPACETOOL
  E_Node,
  T_ToTool;

const
  MaxBranchCount = 10000;
  MinRegularArea = 1E-3;

type



TNodeFinderForLine = class(TTool)
{ hledn uzlu pro linii }
private
  FLineBegin: boolean;
public
  procedure DoIt; override;
published
  property LineBegin: boolean read FLineBegin write FLineBegin;
end;


TPutLinesIntoNodes = class(TProchazecLinii{Gauge})
{ obdr na Inputu TSpaceLineBegEndCell obsahujc linie. Pro kadou linii najde nebo vytvo
  uzel do kterho ji ulo. Uzly vlo do TOwnerEntity NODES }
private
  FNodes: TOwnerEntity;
  FPutBothEnds, FPutBegin: boolean;

  function  FindNode(X, Y: double): TNode;
  procedure PutIntoNodes(Lin: TPolyLineElement; aBegin: boolean);
public
  procedure DoIt; override;
  function  QGoInside(Element: TElement): boolean; override;

  procedure Process(Element: TElement); override;
end;





TProchazecListuQTree = class(TProchazecObjektu)
private
public
  FPocetListu: longInt;
  function  QGoInside(Element: TElement): boolean; override;
  function  QProcessBefore(Element: TElement): boolean; override;
  procedure Process(Element: TElement); override;
  procedure DoIt; override;
published
  property LeavesCount: longInt read FPocetListu;
end;



TProchazecListuQTreeGauge = class(TProchazecListuQTree)
public
  procedure DoIt; override;
end;


TPutNodesIntoQTree = class(TProchazecListuQTreeGauge)
{ Pro vechny listy QTree vytvori pro dane linie uzly }
public
  procedure Process(Element: TElement); override;
end;


TRemoveSingleLinesInQTree = class(TProchazecListuQTreeGauge)
{ Pro vechny listy QTree zrusi odkazy na linie bez vazby na druhem konci }
public
  procedure Process(Element: TElement); override;
end;



TDeleteDirectRelationsToLinesInQTree = class(TProchazecListuQTreeGauge)
{ Pro vechny listy QTree zrusi LineBegins a LineEnds }
public
  procedure Process(Element: TElement); override;
end;


TNodeConector = class(TTool)
private
  FJoinDifferentLines: boolean;
  FDeletePseudoNode: boolean;
  FMaxCoordsCount: longInt;
  function OKJoinLinesInPseudoNode(aNode: TNode): boolean;
public
  procedure DoIt; override;
published
  property JoinDifferentLines: boolean read FJoinDifferentLines write FJoinDifferentLines;
  property DeletePseudoNode: boolean read FDeletePseudoNode write FDeletePseudoNode;
  property MaxCoordsCount: longInt read FMaxCoordsCount write FMaxCoordsCount;
end;


TNodesConector = class(TProchazecListuQTreeGauge)
{ Pro vechny listy QTree a jejich uzly nastav propojen na jin uzly }
private
  FJoinDifferentLines: boolean;
  FDeletePseudoNodes: boolean;
  FMaxCoordsCount: longInt;
public
  procedure Process(Element: TElement); override;
published
  property JoinDifferentLines: boolean read FJoinDifferentLines write FJoinDifferentLines;
  property DeletePseudoNodes: boolean read FDeletePseudoNodes write FDeletePseudoNodes;
  property MaxCoordsCount: longInt read FMaxCoordsCount write FMaxCoordsCount;
end;



TCreateAreaForBranch = class(TTool)
{ Input ... Branch, OutPut ... Plocha }
private
  FAreaValue: double;
public
  procedure DoIt; override;
published
  property AreaValue: double read FAreaValue;
end;





TCreateAreasForQTree = class(TProchazecListuQTreeGauge)
{ Pro vechny listy QTree a jejich uzly a jejich vetve vytvari plochy }
private
  UkladacPloch: TRectEntityQTree;
  FAreaCount: longInt;
public
  procedure Process(Element: TElement); override;
  procedure DoIt; override;
end;



implementation

uses
  Forms, SysUtils, Dialogs,
// KERNEL
  e_Class,
  M_Orbis,
  Worlds,
// GLIB
  MathLib,
// SPACETOOL
  T_ApPlac;

//  F_Ploch,




                       { TNodeFinderForLine }


procedure TNodeFinderForLine.DoIt;
var
  Lin: TPolyLineElement;
  I: longInt;
  PBranch: TNodeBranch;
begin
  Output:= nil;
  if input is TPolyLineElement then
  begin
    Lin:= TPolyLineElement(Input);
    for I:= 0 to Lin.OwnersCount-1 do
    begin
      if Lin.Owners[I] is TNode then
      begin
        PBranch:= TNode(Lin.Owners[I]).
                  GetBranchOfLine(Lin, FLineBegin);
        if (PBranch <> nil) then
        begin
          Output:= TNode(Lin.Owners[I]);
        end
      end { TNode }
    end { for I }
  end { if Input is TPolyLineElement}
end;




                         { TPutLinesIntoNodes }


function  TPutLinesIntoNodes.QGoInside(Element: TElement): boolean;
//var
  //Trida: string;
begin
  Result:= (Element is TRectEntity)  and not (Element is TPolyLineElement);

  if result then
  begin
    if (Element is TPolyLineString) and
       ((Element.Name = 'BEGINS') or (Element.Name = 'ENDS')) then
    begin
      FPutBothEnds:= false;
      FPutBegin:= (Element.Name = 'BEGINS')
    end
    else
    begin
      FPutBothEnds:= true
    end;
  end
end;




function TPutLinesIntoNodes.FindNode(X, Y: double): TNode;
var
  Node: TNode;
  Nasel: boolean;
  I: longINt;

begin
  Node := nil;
  result:= nil;
  Nasel:= false;
  I:= 0;
  while (I < FNodes.Count) and not Nasel do
  begin
    Node:= TNode(FNodes.Elements[I]);
    Nasel:= (abs(Node.X - X) < 0.0001) and (abs(Node.Y - Y) < 0.0001);
    if not Nasel then inc(I)
  end;
  if Nasel then result:= Node
end;



procedure TPutLinesIntoNodes.PutIntoNodes(Lin: TPolyLineElement; aBegin: boolean);
var
  Node: TNode;
  X, Y: double;
begin
  Lin.GetOrientedCoord(0, aBegin, X, Y);
  Node:= FindNode(X, Y);
  if Node = nil then
  begin
    Node:= TNode.Create;
    FNodes.AddElement(Node)
  end;

  if Node <> nil then
     Node.AddLine(Lin, aBegin)
end;




procedure TPutLinesIntoNodes.Process(Element: TElement);
var
  //Linie, PodLinie: TPolyLineString;
  Lin: TPolyLineElement;
  //RectEnt: TRectEntity;
  //LC: longINt;
begin
  if (Element is TPolyLineElement) then
  begin
    Lin:= TPolyLineElement(Element);
    //LC:= Lin.SpaceId;

    Application.ProcessMessages;

    if FNodes = nil then
    begin
      FNodes:= TOwnerEntity.Create;
      FNodes.Name:= 'NODES';
      TSpaceLineBegEndCell(Input).AddElement(FNodes)
    end;

    if FNodes <> nil then
    begin
      if FPutBothEnds then
      begin
        PutIntoNodes(Lin, true);
        PutIntoNodes(Lin, false);
      end
      else
      begin
        PutIntoNodes(Lin, FPutBegin);
      end;
{      GaugeSet(FPocetLinii) }
    end; { povedlo se vytvorit Node }

    Lin.IsDone:= true;
    inc(FPocetLinii)
  end;
end;



procedure TPutLinesIntoNodes.DoIt;
begin
  if Input is TSpaceLineBegEndCell then
     inherited DoIt;
end;



     { TProchazecListuQTree }

function  TProchazecListuQTree.QGoInside(Element: TElement): boolean;
begin
  Result:= (Element is TSpaceLineBegEndCell)
end;



function  TProchazecListuQTree.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element is TSpaceLineBegEndCell) and
           TSpaceLineBegEndCell(Element).Last
end;


procedure TProchazecListuQTree.Process;
begin
  inc(FPocetListu)
end;



procedure TProchazecListuQTree.DoIt;
begin
  FPocetListu:= 0;
  inherited DoIt;
end;





procedure TProchazecListuQTreeGauge.DoIt;
var
  Prochazec: TProchazecListuQTree;
begin

  Prochazec:= TProchazecListuQTree.Create;
  Prochazec.Input:= Input;
  Prochazec.DoIt;
  if Prochazec.FPocetListu > 100 then
  begin
     StartProgress(Self.Name,0,Prochazec.FPocetListu);
    //GaugeMaxValue(Prochazec.FPocetListu);
    //GaugeShow;
  end;
  Prochazec.Free;

  inherited DoIt;
  //GaugeHide
end;




procedure TPutNodesIntoQTree.Process(Element: TElement);
var
  DoNodes: TPutLinesIntoNodes;
begin
  inc(FPocetListu);

  DoNodes:= TPutLinesIntoNodes.Create;
  DoNodes.Input:= Element;
  DoNodes.DoIt;
  DoNodes.Free;

  GaugeSet(FPocetListu)
end;




procedure TDeleteDirectRelationsToLinesInQTree.Process(Element: TElement);
var
  Cell: TSpaceLineBegEndCell;
begin
  inc(FPocetListu);
  Cell:= TSpaceLineBegEndCell(Element);
  if Cell.FLineBegins <> nil then
  begin
    Cell.FLineBegins.Free;
    Cell.FLineBegins:= nil
  end;

  if Cell.FLineEnds <> nil then
  begin
    Cell.FLineEnds.Free;
    Cell.FLineEnds:= nil
  end;
  GaugeSet(FPocetListu)
end;



function TNodeConector.OKJoinLinesInPseudoNode(aNode: TNode): boolean;
var
  Branch1, Branch2, OtherBranch1, OtherBranch2: TNodeBranch;
  Line1, Line2: TPolyLineElement;
  Node1, Node2: TNode;
  Begin1, Begin2: boolean;
  CrdEl1, CrdEl2: T2DCoordinate;
  I: longInt;
begin
  Branch1 := nil;
  Branch2 := nil;
  result:= false;
  if aNode.Elements[0] is TNodeBranch then
     Branch1:= TNodeBranch(aNode.Elements[0]);
  if aNode.Elements[1] is TNodeBranch then
     Branch2:= TNodeBranch(aNode.Elements[1]);

  if (Branch1 = nil) or (Branch2 = nil) then exit;

  Line1:= Branch1.Line;
  Line2:= Branch2.Line;

  if MaxCoordsCount > 0 then
  begin
    if (Line1.Count + Line2.Count - 1) > MaxCoordsCount then
       exit
  end;

  Begin1:= Branch1.LineBegin;
  Begin2:= Branch2.LineBegin;
  { styn souadnice }
  CrdEl1:= Line1.GetOrientedCoordElement(0, Begin1);
  CrdEl2:= Line2.GetOrientedCoordElement(0, Begin2);

  Branch1.GiveLineSecondNodeBranch(Node1, OtherBranch1);
  Branch2.GiveLineSecondNodeBranch(Node2, OtherBranch2);

  if Branch1 = OtherBranch2 then exit; { je to uzaven jedna linie }

  if JoinDifferentLines or
     (Line1.GetPropertyLongIntByName('Druh') =
      Line2.GetPropertyLongIntByName('Druh')) then
  try
  begin
    result:= true;

    for I:= 1 to Line2.Count-1 do
    { cyklus vyjmn souadnic z druh linie}
    begin
      if Begin1 then
         Line1.InsertElement(0, Line2.GetOrientedCoordElement(I, Begin2))
      else
         Line1.AddElement(Line2.GetOrientedCoordElement(I, Begin2))
    end;

    { prvn souadnice nebyly petaeny, mus se uvolnit extra, me
      se vak stt, e ob souadnice alokuj stejnou pam (pokud
      pochzej ze stejn linie) }
    if CrdEl1 <> CrdEl2 then
    begin
      CrdEl2.Free;
    end
    else
    begin
{          ShowMessage(FloatToStr(CrdEl2.X) + ' ' + FloatToStr(CrdEl2.Y)) }
    end;

    Line2.Clear;
    { nejdve se musely souadnice vyhodit z entity, jinak by byly
      uvolnny z pamti, kde vak jsou nyn pro Line1 }
    Line2.Free;

    Line1.RecalculateRect;
    Line1.CalculateLabelProps;

    Node2.FreeElement(OtherBranch2);
    OtherBranch2:= Node2.AddLine(Line1, Begin1);
    OtherBranch2.Node:= Node1;
  end
  except
    on E:Exception do
       ShowMessage('Chyba pi spojovn v pseudouzlu linie .' + Line1.GetPropertyStringByName('Cislo') + ': '
                   + E.Message);
  end;
end;





procedure TNodeConector.DoIt;
var
  Node, Node2: TNode;
  Branch, OtherBranch: TNodeBranch;
  I{, Cislo}: longInt;
begin
  try
  if Input is TNode then
  begin
    Node:= TNode(Input);
    if DeletePseudoNode and (Node.BranchCount = 2) and
       OKJoinLinesInPseudoNode(Node) then
       Node.Free
    else
    for I:= 0 to Node.Count-1 do
    begin
      if Node.Elements[I] is TNodeBranch then
      begin
        Branch:= TNodeBranch(Node.Elements[I]);
        //Cislo:= Branch.Line.SpaceId;
        if Branch.Node = nil then
        begin
          Branch.GiveLineSecondNodeBranch(Node2, OtherBranch);
          Branch.Node:= Node2;
          if OtherBranch <> nil then
             OtherBranch.Node:= Node
        end
      end { Element is TNodeBranch}
    end { for I}
  end
  except
    showMessage('TNodeConector.Process: Chyba');
  end
end;





procedure TNodesConector.Process(Element: TElement);
var
  Nodes: TOwnerEntity;
  DoNode: TNodeConector;
  I: longInt;
  PrevCount: longInt;
begin
  try
  inc(FPocetListu);
  GaugeSet(FPocetListu);
  Nodes:= TOwnerEntity(TEntity(Element).FindElementName('NODES'));
  if Nodes <> nil then
  begin
    DoNode:= TNodeConector.Create;
    DoNode.JoinDifferentLines:= JoinDifferentLines;
    DoNode.DeletePseudoNode:= DeletePseudoNodes;
    DoNode.MaxCoordsCount:= MaxCoordsCount;

    I:= 0;
    while I < Nodes.Count do
    begin
      PrevCount:= Nodes.Count;
      if Nodes.Elements[I] is TNode then
      begin
        DoNode.Input:= Nodes.Elements[I];
        DoNode.DoIt;
      end;
      {Pokud ubyl uzel tak nezvyuji I}
      if PrevCount = Nodes.Count then
         inc(I)
    end;

    DoNode.Free
  end
  except
    showMessage('TNodesConector.Process: Chyba');
  end
end;



procedure TCreateAreaForBranch.DoIt;
var
  NextNode: TNode;
  Branch, NextBranch: TNodeBranch;
  BranchCount: longInt;
  Plocha: TPolyLineAreaElement;
  OKej: boolean;
  aHole: boolean;

  PointPlacer: TAreaPointPlacer;

begin
  Plocha:= nil;
  Output:= nil;
  //aHole:= false;

  if Input is TNodeBranch then
  begin
    Branch:= TNodeBranch(Input);

    if (Branch.Line <> nil) and (Branch.Node <> nil) and
       (Branch.RightArea = nil) and not Branch.NotRegularArea then
    begin
      Branch.GiveRightAreaPropertyAndSet(FAreaValue, OKej, BranchCount, true {setregularity});

      if OKej and (BranchCount < 5000) and (abs(FAreaValue) > MinRegularArea) then
      begin
        aHole:= FAreaValue < 0;

        Branch.NotRegularArea:= false;
        BranchCount:= 1;
        FAreaValue:= Branch.Line.GetSubAreaValue(Branch.LineBegin);
        NextBranch:= Branch.GetNextNodeBranchRight(NextNode);

        if NextBranch <> nil then
        try
        begin
          Plocha:= TPolyLineAreaElement.Create;
          Plocha.LinesOrigines[0]:= Branch.LineBegin;
          Plocha.AddAndUpdateRect(Branch.Line);

          if not aHole then
             Branch.RightArea:= Plocha
          else
          begin
             Branch.RightArea:= nil;
             Plocha.IsHole:= true
          end;

          { hlavn cyklus tvorby plochy obhnm vtv }
          while (NextBranch <> Branch) and (BranchCount < MaxBranchCount) do
          begin
            NextBranch.NotRegularArea:= false;
            NextBranch.RightArea:= Plocha;
            Plocha.LinesOrigines[BranchCount]:= NextBranch.LineBegin;
            Plocha.AddAndUpdateRect(NextBranch.Line);
            FAreaValue:= FAreaValue + NextBranch.Line.GetSubAreaValue
                                                       (NextBranch.LineBegin);

            NextBranch:= NextBranch.GetNextNodeBranchRight(NextNode);
            inc(BranchCount);
            Application.ProcessMessages;
          end;

          if abs(FAreaValue) > MinRegularArea then
          begin
            if FAreaValue > 0 then
            begin
              PointPlacer:= TAreaPointPlacer.Create;
              PointPlacer.Input := Plocha;
              PointPlacer.DoIt;
              PointPlacer.Free;
            end
            else
              Plocha.IsHole:= true;

            Output:= Plocha;
            //Plocha.SavedArea := FAreaValue;
            Plocha.FixedArea := Abs(FAreaValue);
          end
          else
          begin
            Plocha.Free
          end;
        end { plocha }
        except
        begin
          if Plocha <> nil then
             Plocha.Free;
        end
        end
      end { regulerni plocha }
      else
        BranchCount:= BranchCount
    end {smysluplna vetev}
  end {if Input is TNodeBranch}
end;



procedure TCreateAreasForQTree.Process(Element: TElement);
var
  Nodes: TOwnerEntity;
  I, J: longInt;
  DoArea: TCreateAreaForBranch;
  Plocha: TAreaElement;
  NOde: TNode;
  Branch: TNodeBranch;
  //aLC: longINt;
begin
  inc(FPocetListu);
  GaugeSet(FPocetListu);
try
  Nodes:= TOwnerEntity(TEntity(Element).FindElementName('NODES'));
  if Nodes <> nil then
  begin

    DoArea:= TCreateAreaForBranch.Create;

    for I:= 0 to Nodes.Count-1 do
    begin
      if Nodes.Elements[I] is TNode then
      begin
        Node:= TNode(Nodes.Elements[I]);

        for J:= 0 to Node.Count-1 do
        if Node.Elements[J] is TNodeBranch then
        begin
          Branch:= TNodeBranch(Node.Elements[J]);
          if (Branch.Line <> nil) and (Branch.RightArea = nil) and
             not (Branch.NotRegularArea) then
          begin
            //aLC:= Branch.Line.SpaceId;
            DoArea.Input:= Branch;

            DoArea.DoIt;
            Application.ProcessMessages;

            Plocha:= TAreaElement(DoArea.Output);
            if Plocha <> nil then
            begin
              if Plocha.IsHole then;
              inc(FAreaCount);
              UkladacPloch.NextRectEntity:= Plocha
            end
          end
        end

      end
    end;

    DoArea.Free

  end;
except
  ShowMessage('Chyba v CreateAreaForQtree');
end
end;



procedure TCreateAreasForQTree.DoIt;
begin
  if not (input is TOwnerEntity) then exit;

  UkladacPloch:= TRectEntityQTree.Create;

  FAreaCount:= 0;
  inherited DoIt;
  Output:= UkladacPloch
end;






{ TRemoveSingleLinesInQTree }

procedure TRemoveSingleLinesInQTree.Process(Element: TElement);
var
  Nodes: TOwnerEntity;
  I, J: longInt;
  Node, sNode: TNode;
  Branch, sBranch: TNodeBranch;
  //aLC: longINt;
  Zrusit: Boolean;

begin
  inc(FPocetListu);
  GaugeSet(FPocetListu);
try
  Nodes:= TOwnerEntity(TEntity(Element).FindElementName('NODES'));
  if Nodes <> nil then
  begin
    for I:= Nodes.Count-1 downto 0 do
    begin
      if Nodes.Elements[I] is TNode then
      begin
        Zrusit:= False;
        Node:= TNode(Nodes.Elements[I]);

        if Node.BranchCount = 1 then
        begin
          for J:= Node.Count-1 downto 0 do
          if Node.Elements[J] is TNodeBranch then
          begin
            Branch:= TNodeBranch(Node.Elements[J]);
            if (Branch.Line <> nil) then
            begin
              Branch.GiveLineSecondNodeBranch(sNode, sBranch);
              sNode.FreeElement(sBranch);
//              Branch.Line.Free;
              Zrusit:= true
            end
          end
        end;

        if Zrusit then
           Node.Free;
      end
    end;
  end;
except
  ShowMessage('Chyba v CreateAreaForQtree');
end
end;



begin
   RegisterOrbisClass(TProchazecListuQTree);
   RegisterOrbisClass(TPutLinesIntoNodes);
   RegisterOrbisClass(TPutNodesIntoQtree);
   RegisterOrbisClass(TNodesConector);
   RegisterOrbisClass(TDeleteDirectRelationsToLinesInQTree);
   RegisterOrbisClass(TCreateAreaForBranch);
   RegisterOrbisClass(TCreateAreasForQTree);
end.

