unit e_LinZn;
{
****************************************************************
**                                                            **
**                 ORBIS ... model TopoL                      **
**  Jednotka pro praci s uzasne slozitymi liniovymi znackami, **
**  jako jsou treba burty apod.                               **
**                                                            **
**                                                -PPK-94-    **
****************************************************************
}

INTERFACE

uses
   Classes,
   Graphics,
// KERNEL
   CtiPar,
   Elements,
// GLIB
   g_Elems,
// SPACE
   e_Geometry,
   //E_Space,
// DEVICE
   e_Device,
// SYMBOL
   e_BGIF;

const

  { Status TLinZnacky }

  znNatacetDleLinie  = $0001; { Nataceni podle linie je-li vypnuto, znacka se
                                vykresluje stale pod zadanym uhlem }
  znLamatZnacku      = $0002; { Moznost zlomeni v lomovem bode }
  znDokonalyLom      = $0004; { Transformace zlomene znacky .. hladke napojeni }
  znLamatMezeru      = $0008; { DTTO s mezerou }
  znCentrovat        = $0010; { znacku nelamat, jen centrovat - t.zn. umistit
                                pod patricnym uhlem stredem na lomovem bode }
  znKreslitPosledni  = $0020; { oznamuje zda se ma znacka vykreslit jako pos-
                                ledni v pripade, ze neni zlomitelna }
  znKreslitZbytek    = $0040; { u zlomitelne znacky na konci linie vyzaduje
                                vykresleni zbytku znacky ... dotazeni }
  znB1               = $0100; { Vykreslovat v prvnim bode linie }
  znB2               = $0200; { Vykreslovat v poslednim bode linie }
  znLB               = $0400; { Vykreslovat v kazdem lomovem bode linie }
  znLB0              = $0800; { Vykreslovat na zacatku vertexu }
  znLB1              = $1000; { Vykreslovat na konci vertexu }

  znKompMaska        = $FF00; { maska pro kontrolu predchozich nastaveni }

  { Status TRetezovezZnacky }

  znUpravovatMezeru  = $0002; { Moznost zvetseni ci zmens. posledni mezery }
  znKreslitPrvni     = $0004; { Kreslit prvni znacku linie }
  znHarmonika        = $0008; { Natahovaci znacka .... ze tri casti }

  { Vysledky operaci lamani apod. }

  opZnacka = $0001;  { Definovani Zbyleho objektu ... Znacka nebo mezera}
  opMezera = $0002;
  opCela   = $0004;  { Informace, zda zbyl cely objekt nebo jeho cast }
  opCast   = $0008;

  polVlevo  = True;
  polVPravo = False;

type

  TLongPoint = record
     x,y: longInt;
  end;



  { Bod realne definice znacky }
  { ************************** }

  TBodZnacky = class
   public
     Spoj  : boolean;      { Vykresleni nebo jen presun pera }
     Sour  : TDoubleCoord; { Skutecne sour. definice znacky }
     OtSour: TDoubleCoord; { Otocene sour. pro kazdy vertex zvlast }
     Dalsi : TBodZnacky;   { Dalsi podobny  nesmyslny bod }
     procedure Assign(const aBod: TBodZnacky);
  end;





  { Bod celociselne definice znacky }
  { ******************************* }
  {  ... zatim pouzito jen pro nabidkove objekty }

  TIntBodZnacky = class
     Spoj  : boolean;        { Vykresleni nebo jen presun pera }
     Sour  : TLongPoint;     { Skutecne sour. definice znacky }
     Dalsi : TIntBodZnacky;  { Dalsi podobny  nesmyslny bod }
  end;





  { Zakladni predek liniovych znacek umoznujici definici cary }
  { ********************************************************* }

  TLinZnacka = class(TElement)
   private
    FTrueColor: TColor;
    FByTrueColor: boolean;
   protected

   public
    Velikost   : double;    { Delka(!) znacky v m .. zadani v mm }
    Sila       : double;    { Sila je v mm }
    Barva      : Integer;   { Barva ... to je jasny }
    Uhel       : double;    { Uhel v radianech .. zadani ovsem ve stupnich }
    Vzdalenost : double;    { Vzdalenost k nasledujici zn. (v m .. zad. v mm)}
    Mezera     : TDoubleCoord; { Sour rozdil k nasledujici znacce }
    Status     : word;      { Informace o zpusobu vykreslovani zn. }
    Dalsi      : TLinZnacka;{ Ukazatel na dalsi zn. v seznamu }
    Definice   : TBodZnacky;{ Definice znacky v realnych souradnicich vzhledem
                              ke vztaznemu meritku }
    Invisible  : boolean;   { informace o tom, ze clanek ma nulovou delku a
                              nebude se zobrazovat }
    Generalizovat: boolean; { clanek bude vykreslen pouze carou }
    Constructor  Create; override;
    Destructor   Destroy; override;
    procedure    Assign(Source: TPersistent); override;
    procedure    NastavSilu(ASila: double);
    procedure    NastavUhel(Uh: double);
    {
    procedure    NastavVelikostVmm(Vel: double);
    }
    procedure    NastavVelikostVm (Vel: double);
    procedure    NastavVzdalenost(Vzd: double);
    procedure    OtocDefinici(Uh: double); virtual;
    procedure    PrekrojZnacku ( Kde: double; var LevaC, PravaC: TLinZnacka;
                                 Op: word; const PLin: TRectEntity);virtual;
    procedure    PreskocMezeru(Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord);
    procedure    VyrobDefinici; virtual;
    function     VyrobKopiiDefinice: TBodZnacky; virtual;
    procedure    Zobraz(aDevice: TDevice; Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord); virtual;
    procedure    ZrusDefinici;
  published
    property     Color: Integer read Barva write Barva;
    property     TrueColor: TColor read FTrueColor write FTrueColor;
    property     ByTrueColor: boolean read FByTrueColor write FByTrueColor;
  end;



  { Objekt pro praci s petrklicovymi znackami }
  { ***************************************** }

  TPetrklicZn = class(TLinZnacka)
   protected
    FSoubor: String;                  { Soubor obsahujici definici znacky }
    function     GetSoubor: string;
    procedure    SetSoubor(aSoubor: string);
   public
    Kod    : byte;       { Kod znacky }
    constructor  Create; override;
    destructor   Destroy; override;
    procedure    Assign(Source: TPersistent); override;
    procedure    NastavKod(AKod: Byte);
    procedure    NastavSoubor(const ASoubor: string);
    procedure    OtocDefinici(Uh: double); override;
    procedure    PrekrojZnacku ( Kde: double; var LevaC, PravaC: TLinZnacka;
                                 Op: word; const PLin: TRectEntity);override;
    {
    procedure    VystupDoHPGL(Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord); virtual;
    }
    procedure    Zobraz(aDevice: TDevice; Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord); override;

    property     Soubor: string read GetSoubor write SetSoubor;
  end;




  { Objekt umoznujici retezeni jednoduchych znacek }
  { ********************************************** }

  TRetezovaZnacka = class(TLinZnacka)
   public
    ZakladniZnacka: Integer;      { Moznost def.jedn.zn. misto sloziteho ret.}
    Vlevo: boolean;               { Poloha retezu vzhledem k hlavni linii }
    OchrannaZona: double;         { Buffer pro zobrazeni ci potlaceni jinych obektu }
    Znacky: TLinZnacka;           { Seznam znacek }
    ShowAll: boolean;
    constructor  Create; override;
    destructor   Destroy; override;

    {procedure    NastavOchrannouZonu(Z: double);}
    procedure    NastavPolohu(Kde: boolean);
    procedure    OtocDefinici(Uh: double); override;
    procedure    PridejZnacku(AZnacka: TLinZnacka); virtual;
    function     VratDelkuCeleZnacky: double;
    function     VratDelkuZbytkuZnacky ( Znacka: TLinZnacka;
                                         Op: word; d: double): double;
    function     VratRidiciLinii ( Vpravo: boolean;
                                   aLin: TRectEntity; var Mazat: boolean): TRectEntity;
    procedure     NastavGeneralizace(const aDevice: TDevice);
    procedure    ZobrazCelouZnacku(aDevice: TDevice; Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord);
    procedure    ZobrazPodelLinie (aDevice: TDevice; PuvodniBarva: TColor;
                                   DLin: TPolyLineElement; Orez: boolean; Rect: TDoubleRect); virtual;
    procedure    ZobrazZakladniZnacku ( aDevice: TDevice;
                                        PuvodniBarva: TColor; PLin: TRectEntity);
    procedure    ZobrazZbytekZnacky (aDevice: TDevice; Vychbod: TDoubleCoord; Znacka: TLinZnacka;
                                      var PoslBod: TDoubleCoord);

  end;




  { Objekt umoznujici ponekud komplikovane retezeni jednoduchych znacek }
  { ******************************************************************* }

(*
  TKomplikada = class(TRetezovaZnacka)
   public
    B1   : TLinZnacka;    { Seznam znacek pro prvni bod linie }
    B2   : TLinZnacka;    { Seznam znacek pro posledni bod linie }
    LB   : TLinZnacka;    { Znacky lezici na kazdem lomovem bodu }
    LB0  : TLinZnacka;    { Znacky na sudych (0,2,4...) lom. b.}
    LB1  : TLinZnacka;    { Znacky na lichych (1,3,5...) lom. b.}
                          { Ve znackach je ta vata }
    B1Buf: double;        { Ochranna zona prvniho bodu }
    B2Buf: double;        { Ochranna zona posledniho bodu }
    LBBuf: double;        { Ochranna zona bezneho lom. bodu }
    LU   : double;        { Posledni zjisteny uhel .... nic je -1000000 }
    constructor  Create;
    destructor   Destroy; override;
    procedure    OtocDefinici(Uh: double); override;
    procedure    OtocDefiniciKraje(Uh: double; aStat: word); virtual;
    procedure    PridejZnacku(AZnacka: TLinZnacka); override;
    procedure    PreskocMezeru(Vychbod: TDoubleCoord; aDelka, aUhel: double; var PoslBod: TDoubleCoord);
    {
    procedure    VystupRetezuDoHPGL(DLin: TDynLiniePtr; Vyrez: TObdelnik; Meritko: double); virtual;
    }
    procedure    ZobrazPodelLinie (aDevice: TDevice; DLin: TTopolLin; Orez: boolean); override;
    procedure    ZobrazB   (aDevice: TDevice; Vychbod: TDoubleCoord; AStat: word);
    procedure    ZobrazLB0 (aDevice: TDevice; Vychbod: TDoubleCoord; AStat: word);
    procedure    ZobrazLB1 (aDevice: TDevice; Vychbod: TDoubleCoord; AStat: word);
    procedure    ZobrazLB  (aDevice: TDevice; Vychbod: TDoubleCoord; AStat: word);
  end;

*)


  { Kompletni obj. umoznujici paralelni spojeni retezovych znacek }
  { ************************************************************* }

  TLiniovaZnacka = class(TElement)
   private
    FByTrueColor: boolean;
   public
    NahradniZnacka: integer;          { Nahradni znacka pro draft rezim }
    Kod           : longInt;          { Kodove oznaceni zn. }
    Nazev         : string[30];       { Jeji nazev }
    Znacky        : TRetezovaZnacka;  { Seznam retezovych znacek }
    ShowAll       : boolean;
    constructor Create; override;
    destructor  Destroy; override;
    procedure   NastavKod(AKod: longInt);
    procedure   NastavNahradniZnacku(AZnacka: integer);
    procedure   NastavNazev(ANazev: String);
    procedure   PridejZnacku(AZnacka: TRetezovaZnacka);
    procedure   ZobrazLinii (adevice:TDevice; PuvodniBarva: TColor;
                                       Linie: TPolyLineElement; Orez: boolean; Rect: TDoubleRect);
   published
    property    ByTrueColor: boolean read FByTrueColor write FByTrueColor;
  end;


  { Typ zaznamu Souboru Liniovych znacek }
  { ************************************ }

  TSouborLinZn = class(TOwnerElement)
  public
    Kod: longInt;     { Kod pripojeni v knihovne znacek }
  end;



  { Kolekce predstavujici knihovnu vsech znackovych souboru }
  { ******************************************************* }

  TLinZnLibrary = class(TEntity)
    constructor CreateAll(const ASoubor: string);
    procedure   VratSouborCislo(ACislo: longInt; var Soubor: string);
  end;




  { Kolekce liniovych znacek }
  { ************************ }

  TLinioveZnacky = class(TEntity)
   public
    Knihovna: TLinZnLibrary;{ Knihovna liniovych znacek}
    KodNabidky: longInt;    { Kod pripojeni aktualniho zn. souboru }
    Nabidka: boolean;       { Informace o tom, ze se jede v nabidkovem rezimu}
    OK: boolean;            { }
    Device: TDevice;
    Rect: TDoubleRect;
    constructor CreateAll(const ASoubor: string);
    destructor  Destroy; override;
    function    PridejZnackuDleKodu(AKod: longInt): TLiniovaZnacka;
    function    VratZnackuDleKodu(AKod: longInt): TLiniovaZnacka;
  end;





  { Potomek BGIFontu umoznujici zakladni operace s Petrklicoidy }
  { *********************************************************** }

  TPaklicNaPetrklic = class(TBGIFont)
  public
    JmenoSouboru: string;
    constructor Create; override;
    function    NovyFontJmenem(NazevFontu: string): Boolean;
    function    VratRealnouDefiniciZnaku(KodZnaku: byte;
                                         Velikost,
                                         ASmer: double): TBodZnacky;
  end;

  { Objekt pro nacitani strukturovane znacky z textoveho souboru }
  { ************************************************************ }

  TCteniLiniovychZnacek = class(TCteniParametru)
   public
    Device: TDevice;
    function  CtiLiniovouZnacku(Kod: longInt): TLiniovaZnacka;
    function  CtiRetez(Paklic: TPaklicNaPetrklic): TRetezovaZnacka;
    function  CtiClanek: TLinZnacka;
    function  CtiPetrklic(Paklic: TPaklicNaPetrklic): TPetrklicZn;
    function  mm_m(mm: double): double;
    function  DejBoolParametr: boolean;
    function  DejIntParametr: Integer;
    function  DejVlevo: boolean;
  end;

  {
var

  LinioveZnacky: TLinioveZnacky;
}
IMPLEMENTATION

uses
// KERNEL
   CtiRadek,
   u_String,
// GLIB
   MathLib,
   e_Color,
   //e_TplPrp,
// SYMBOL
   t_Paral,
   u_Symbol;

const

  Ano = True;
  Ne  = False;
  Sour00: TDoubleCoord = (x: 0; y: 0);
  NabKoef = 3;    { Koeficient pro prepocet mm na  body nabidky }
  NoAngle = -1000000;

  lzB1Buffer       = 'B1BUFFER';
  lzB2Buffer       = 'B2BUFFER';
  lzLBBuffer       = 'LBBUFFER';
  lzBarva          = 'BARVA';
  lzClanek         = 'CLANEK';
  lzEnd            = 'END';
  lzHarmonika      = 'HARMONIKA';
  lzKod            = 'KOD';
  lzKomplikace     = 'KOMPLIKACE';
  lzLiniovaZnacka  = 'LINIOVAZNACKA';
  lzLomBod         = 'LOMBOD';
  lzLomBod0        = 'LOMBOD0';
  lzLomBod1        = 'LOMBOD1';
  lzNahradniZnacka = 'NAHRADNIZNACKA';
  lzNazev          = 'NAZEV';
  lzPetrklic       = 'PETRKLIC';
  lzPosledniBod    = 'POSLEDNIBOD';
  lzPrvniBod       = 'PRVNIBOD';
  lzRetez          = 'RETEZ';
  lzSila           = 'SILA';
  lzSoubor         = 'SOUBOR';
  lzStrana         = 'STRANA';
  lzTyp            = 'TYP';
  lzUhel           = 'UHEL';
  lzVelikost       = 'VELIKOST';
  lzVzdalenost     = 'VZDALENOST';
  lzZakladniZnacka = 'ZAKLADNIZNACKA';

  lzCentrovat      = 'CENTROVAT';
  lzKreslitPosledni= 'KRESLITPOSLEDNI';
  lzKreslitPrvni   = 'KRESLITPRVNI';
  lzKreslitZbytek  = 'KRESLITZBYTEK';
  lzLamatMezeru    = 'LAMATMEZERU';
  lzLamatZnacku    = 'LAMATZNACKU';
  lzNatacet        = 'NATACET';
  lzOchrannaZona   = 'OCHRANNAZONA';

{............ Tela metod TLinioveZnacky ...................}
{
  **********************************************************
  *                                                        *
  *                    Lokalni funkce                      *
  *                                                        *
  **********************************************************
}
{ Otoceni bodu Bod o uhel Uh v soustave JTSK ... vysledek v OtBod }
procedure   OtocBodJTSK(Bod: TDoubleCoord; Uh: double; var OtBod: TDoubleCoord);
var
  SinU, CosU: double;
begin
  if Uh <> 0 then
  begin
     SinU := Sin(Uh);
     CosU := Cos(Uh);
     OtBod.x := Bod.x*CosU - Bod.y*SinU;
     OtBod.y := Bod.x*SinU + Bod.y*CosU;
  end
  else OtBod := Bod;
end;


{............................................}
procedure   ZrusSeznamBoduZnacek(var PSeznam: TBodZnacky);
var
  P,PP: TBodZnacky;
begin
  P := PSeznam;
  while P <> nil do
  begin
     PP := P.Dalsi;
     P.Free;
     P  := PP;
  end;
  PSeznam := nil;
end;

{............................................}
(*
procedure   ZrusSeznamIntBoduZnacek(var PSeznam: TIntBodZnacky);
var
  P,PP: TIntBodZnacky;
begin
  P := PSeznam;
  while P <> nil do
  begin
     PP := P.Dalsi;
     P.Free;
     P  := PP;
  end;
  PSeznam := nil;
end;

*)
{............................................}


procedure TBodZnacky.Assign(const aBod: TBodZnacky);
begin
   Spoj  := aBod.Spoj;
   Sour  := aBod.Sour;
   OtSour:= aBod.OtSour;
   Dalsi := nil;
end;
{
  **********************************************************
  *                                                        *
  *                Tela metod TLinZnacka                   *
  *                                                        *
  **********************************************************
}
Constructor  TLinZnacka.Create;
begin
  inherited Create;
  Velikost   := 0;
  Sila       := 1;
  Barva      := -1;
  Uhel       := 0;
  Vzdalenost := 0;
  Dalsi      := nil;
  Mezera     := Sour00;
  Definice   := nil;
  Invisible  := Ne;
  Status     := znNatacetDleLinie + znLamatZnacku + znLamatMezeru + znKreslitZbytek;
end;

{-----------------------------------------------------------}
Destructor   TLinZnacka.Destroy;
begin
  ZrusDefinici;
end;


{-----------------------------------------------------------}

procedure    TLinZnacka.Assign(Source: TPersistent);
begin
   if Source is TLinZnacka then
   begin
      Velikost      := TLinZnacka(Source).Velikost;
      Sila          := TLinZnacka(Source).Sila;
      Barva         := TLinZnacka(Source).Barva;
      Uhel          := TLinZnacka(Source).Uhel;
      Vzdalenost    := TLinZnacka(Source).Vzdalenost;
      Mezera        := TLinZnacka(Source).Mezera;
      Status        := TLinZnacka(Source).Status;
      Dalsi         := TLinZnacka(Source).Dalsi;
      Definice      := TLinZnacka(Source).Definice;
      Invisible     := TLinZnacka(Source).Invisible;
      Generalizovat := TLinZnacka(Source).Generalizovat;
      TrueColor     := TLinZnacka(Source).TrueColor;
      ByTrueColor   := TLinZnacka(Source).ByTrueColor;
   end;
end;


{-----------------------------------------------------------}
procedure    TLinZnacka.NastavSilu(ASila: double);
begin
  Sila := ASila;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.NastavUhel(Uh: double);
begin
  Uhel := DejRadZeStupnu(Uh);
end;

{-----------------------------------------------------------}
{
procedure    TLinZnacka.NastavVelikostVmm(Vel: double);
begin
  Velikost  := Vel/1000 * LinioveZnacky.Device.Projection.VirtualScale;
  Invisible := Velikost < 0.001;
end;
}
{-----------------------------------------------------------}
procedure    TLinZnacka.NastavVelikostVm (Vel: double);
begin
  Velikost  := Vel;
  Invisible := Velikost < 0.001;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.NastavVzdalenost(Vzd: double);
begin
  Vzdalenost := Vzd;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.OtocDefinici(Uh: double);
var
  SinU, CosU: double;
  P : TBodZnacky;
  //Vzdal: double;
begin
  { Otoceni definice znacky }
  { ----------------------- }
  SinU := Sin(Uh);
  CosU := Cos(Uh);
  P := Definice;
  if P <> nil then
  begin
     P.OtSour.x := P.Sour.x*CosU - P.Sour.y*SinU;
     P.OtSour.y := P.Sour.x*SinU + P.Sour.y*CosU;
     if (Uhel <> 0) then
     begin
        P.Dalsi.OtSour.x := P.Dalsi.Sour.x*CosU - P.Dalsi.Sour.y*SinU;
        P.Dalsi.OtSour.y := P.Dalsi.Sour.x*SinU + P.Dalsi.Sour.y*CosU;
     end;
  end;
  { Otoceni mezery }
  { -------------- }
  if Status and znLB1 <> 0 then
  begin
     Uh := Uh + Pi;
     SinU := Sin(Uh);
     CosU := Cos(Uh);
  end;


  Mezera.x :=  (Vzdalenost) * CosU;
  Mezera.y :=  (Vzdalenost) * SinU;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.PrekrojZnacku ( Kde: double; var LevaC, PravaC: TLinZnacka;
                                        Op: word; const PLin: TRectEntity);
begin
  if Op and opMezera <> 0 then

  { Budeme krajet mezeru - nuda }
  { --------------------------- }
  begin
     PravaC := TLinZnacka.Create;
     PravaC.Velikost   := 0;
     PravaC.Sila       := Sila;
     PravaC.Uhel       := Uhel;
     PravaC.Vzdalenost := Vzdalenost - Kde;
     PravaC.Status     := Status;
     PravaC.Dalsi      := Dalsi;
     PravaC.Definice   := nil;
  end

  else

  { Znacka je hustsi }
  { ---------------- }
  begin
     { Leva cast }
     LevaC  := TLinZnacka.Create;
     LevaC.Velikost   := Kde;
     LevaC.Sila       := Sila;
     LevaC.Uhel       := Uhel;
     LevaC.Vzdalenost := 0;
     LevaC.Status     := Status;
     LevaC.Dalsi      := nil;
     LevaC.Definice   := nil;
     LevaC.VyrobDefinici;
     LevaC.Generalizovat := Generalizovat;
     { Prava cast }
     PravaC  := TLinZnacka.Create;
     PravaC.Velikost   := Velikost - Kde;
     PravaC.Sila       := Sila;
     PravaC.Uhel       := Uhel;
     PravaC.Vzdalenost := Vzdalenost;
     PravaC.Status     := Status;
     PravaC.Dalsi      := Dalsi;
     PravaC.Definice   := nil;
     PravaC.VyrobDefinici;
     PravaC.Generalizovat := Generalizovat;
  end;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.PreskocMezeru(Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord);
begin
  PoslBod.x := VychBod.x + Mezera.x;
  PoslBod.y := VychBod.y + Mezera.y;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.VyrobDefinici;
var
  //RealVel: double;
  PomSour: TDoubleCoord;
begin
  { Nastaveni definice znacky }
  { ------------------------- }
  Definice := TBodZnacky.Create;

  Definice.Spoj := Ano;
  PomSour.x := Velikost;
  PomSour.y := 0;
  OtocBodJTSK(PomSour, Uhel, Definice.Sour);
  if Uhel <> 0 then
  begin
     Definice.Dalsi := TBodZnacky.Create;
     Definice.Dalsi.Spoj  := Ne;
     Definice.Dalsi.Sour  := PomSour;
     Definice.Dalsi.Dalsi := nil;
  end
  else Definice.Dalsi := nil;
  { Nastaveni mezery }
  { ---------------- }
  Mezera.x := Vzdalenost;
  Mezera.y := 0;
end;

{-----------------------------------------------------------}
function    TLinZnacka.VyrobKopiiDefinice: TBodZnacky;
var
  BB,Pb,P : TBodZnacky;
begin
  BB := nil;
  Pb := nil;
  if Assigned(Definice) then
  begin
     P  := Definice;
     while P <> nil do
     begin
        if BB = nil then
        begin
           BB := TBodZnacky.Create;
           Pb := BB;
        end
        else
        begin
           Pb.Dalsi := TBodZnacky.Create;;
           Pb := Pb.Dalsi;
        end;
        Pb.Assign(P);
        Pb.Dalsi := nil;
        P := P.Dalsi;
     end;
  end;
  result := BB;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.Zobraz(aDevice: TDevice; Vychbod: TDoubleCoord; var PoslBod: TDoubleCoord);
begin
  if Invisible then PoslBod := VychBod
  else
  if Assigned(Definice) then
  begin
     aDevice.MoveTo(VychBod.x, VychBod.y);
     PoslBod.x := VychBod.x + Definice.OtSour.x;
     PoslBod.y := VychBod.y + Definice.OtSour.y;

     aDevice.LineTo(PoslBod.x, PoslBod.y);
     if Uhel > 0 then
     begin
        PoslBod.x := VychBod.x + Definice.Dalsi.OtSour.x;
        PoslBod.y := VychBod.y + Definice.Dalsi.OtSour.y;
     end;
  end;
end;

{-----------------------------------------------------------}
procedure    TLinZnacka.ZrusDefinici;
begin
  ZrusSeznamBoduZnacek(Definice);
end;

{-----------------------------------------------------------}
{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *              Tela metod TPetrklicZn                    *
  *                                                        *
  **********************************************************
}

function     TPetrklicZn.GetSoubor: string;
begin
   result := FSoubor;
end;

procedure    TPetrklicZn.SetSoubor(aSoubor: string);
begin
   FSoubor := aSoubor;
end;



constructor  TPetrklicZn.Create;
begin
  inherited Create;
  Status := znNatacetDleLinie + znLamatZnacku + znLamatMezeru
          + znDokonalyLom;
  Kod    := 0;
end;


destructor   TPetrklicZn.Destroy;
begin
   Soubor := '';
   inherited Destroy;
end;


procedure    TPetrklicZn.Assign(Source: TPersistent);
begin
   if Source is TPetrklicZn then
   begin
       Kod    := TPetrklicZn(Source).Kod;
       Soubor := TPetrklicZn(Source).Soubor;
   end;
   inherited Assign(Source);
end;


{-----------------------------------------------------------}
procedure    TPetrklicZn.NastavKod(AKod: Byte);
begin
  Kod := AKod;
end;

{-----------------------------------------------------------}
procedure    TPetrklicZn.NastavSoubor(const ASoubor: string);
begin
  Soubor := ASoubor;
end;

{-----------------------------------------------------------}
procedure    TPetrklicZn.OtocDefinici(Uh: double);
var
  SinU, CosU: double;
  P : TBodZnacky;
//  Vzdal: double;
begin
  SinU := Sin(Uh);
  CosU := Cos(Uh);
  if Assigned(Definice) then
  begin
     { Otoceni definice znacky }
     { ----------------------- }
     P := Definice;
     while P <> nil do
     begin
        P.OtSour.x := P.Sour.x*CosU - P.Sour.y*SinU;
        P.OtSour.y := P.Sour.x*SinU + P.Sour.y*CosU;
        P := P.Dalsi;
     end;
  end;
  { Otoceni mezery }
  { -------------- }
  if Status and znLB1 <> 0 then
  begin
     Uh := Uh + Pi;
     SinU := Sin(Uh);
     CosU := Cos(Uh);
  end;

  Mezera.x :=  (Vzdalenost) * CosU;
  Mezera.y :=  (Vzdalenost) * SinU;
end;

{-----------------------------------------------------------}
procedure    TPetrklicZn.PrekrojZnacku ( Kde: double; var LevaC, PravaC: TLinZnacka;
                                         Op: word; const PLin: TRectEntity);
type
  PIDBody = ^TIDBody;
  TIDBody = Array[1..8] of TDoubleCoord;
var
  P,PL,PP : TBodZnacky;
  PomP    : TBodZnacky;
  PomSour : TDoubleCoord;
  JeVlevo : boolean;
  BylVlevo: boolean;
  Prvni   : boolean;
  UhelLomu: double;
//  IDBody  : PIDBody;
  SinLU,
  CosLU   : double;
//  PomR    : double;

  {............................................}
  procedure Umisteni(var Co,Kam: TBodZnacky; var Cast: TLinZnacka);
  begin
     if not Assigned(Kam) then
     begin
        Kam := TBodZnacky.Create;
        Kam.Assign(Co);
        Cast.Definice := Kam;
     end
     else
     begin
        Kam.Dalsi := TBodZnacky.Create;
        Kam  := Kam.Dalsi;
        Kam.Assign(Co);
     end;
  end;
  {............................................}
  procedure RozdelUsecku(Sour1,Sour2: TDoubleCoord; var Vysl: TDoubleCoord);
  begin
     Vysl.x := Kde;
     Vysl.y := Sour1.y + ((Sour2.y-Sour1.y)/abs(Sour2.x-Sour1.x)*abs(Kde-Sour1.x));
  end;
  {............................................}
  procedure SoupniHoDoleva(Koho: TBodZnacky; Kontrola: boolean);
  begin
     if Kontrola and not BylVlevo then
     begin
        PomSour   := PP.Sour;
        PomSour.x := PomSour.x + Kde;
        RozdelUsecku(Koho.Sour,PomSour,PomP.Sour);
        PomP.Spoj := Ne;
        Umisteni(PomP, PL, LevaC);
        PomP.Spoj := Ano;
        Umisteni(PomP, PP, PravaC);
        PP.Sour.x := 0;
     end;
     Umisteni(Koho, PL, LevaC);
  end;
  {............................................}
  { Cela prava cast se musi posunout doleva }
  procedure SoupniHoDoprava(Koho: TBodZnacky; Kontrola: boolean);
  begin
     if Kontrola and BylVlevo then
     begin
        RozdelUsecku(PL.Sour,Koho.Sour,PomP.Sour);
        PomP.Spoj := Ano;
        Umisteni(PomP, PL, LevaC);
        PomP.Spoj := Ne;
        Umisteni(PomP, PP, PravaC);
        PP.Sour.x := 0;
     end;
     Umisteni(Koho, PP, PravaC);
     PP.Sour.x := PP.Sour.x - Kde;
  end;
  {............................................}


begin
  if Op and opMezera <> 0 then

  { Budeme krajet mezeru - nuda }
  { --------------------------- }
  begin
     LevaC := TPetrklicZn.Create;
     LevaC.Velikost   := 0;
     LevaC.Sila       := Sila;
     LevaC.Uhel       := Uhel;
     LevaC.Vzdalenost := Kde;
     LevaC.Status     := Status;
     LevaC.Dalsi      := nil;
     LevaC.Definice   := nil;

     PravaC := TPetrklicZn.Create;
     PravaC.Velikost   := 0;
     PravaC.Sila       := Sila;
     PravaC.Uhel       := Uhel;
     PravaC.Vzdalenost := Vzdalenost - Kde;
     PravaC.Status     := Status;
     PravaC.Dalsi      := Dalsi;
     PravaC.Definice   := nil;

  end

  else

  { Znacka je hustsi }
  { ---------------- }
  begin
     { Leva cast }
     { --------- }
     LevaC := TPetrklicZn.Create;
     LevaC.Velikost   := Kde;
     LevaC.Sila       := Sila;
     LevaC.Uhel       := Uhel;
     LevaC.Vzdalenost := 0;
     LevaC.Status     := Status;
     LevaC.Dalsi      := nil;
     LevaC.Definice   := nil;
     LevaC.Generalizovat := Generalizovat;

     { Prava cast }
     { ---------- }
     PravaC := TPetrklicZn.Create;
     PravaC.Velikost   := Velikost - Kde;
     PravaC.Sila       := Sila;
     PravaC.Uhel       := Uhel;
     PravaC.Vzdalenost := Vzdalenost;
     PravaC.Status     := Status;
     PravaC.Dalsi      := Dalsi;
     PravaC.Definice   := nil;
     PravaC.Generalizovat := Generalizovat;

     { ...A krajime jako zbesili }
     { ------------------------- }
     PomP := TBodZnacky.Create;
     Prvni := Ano;
     P     := Definice;
     PL    := nil;
     PP    := nil;
     Kde   := Kde;
     while P <> nil do
     begin
        JeVlevo := P.Sour.x < Kde;
        if JeVlevo then
           SoupniHoDoleva(P, not Prvni and P.Spoj)
        else
           SoupniHoDoPrava(P, not Prvni and P.Spoj);

        BylVlevo := JeVlevo;
        P := P.Dalsi;
        if Prvni then Prvni := Ne;
     end;
     PomP.Free;

     { Pokud je treba, provedeme transformaci }
     { -------------------------------------- }
     if (Status and ZnDokonalyLom <> 0) and
        (PLin.Index + 2 < PLin.Count) then
     begin

        UhelLomu := CoordUhel2Usecek ( T2DCoordinate(PLin.Elements[PLin.Index]),
                                  T2DCoordinate(PLin.Elements[PLin.Index+1]),
                                  T2DCoordinate(PLin.Elements[PLin.Index+2]));
        if UhelLomu = 0 then UhelLomu := PI
        else
        begin
           if CoordBodVpravoOdOrientPrimky( T2DCoordinate(PLin.Elements[PLin.Index]),
                                            T2DCoordinate(PLin.Elements[PLin.Index+1]),
                                            T2DCoordinate(PLin.Elements[PLin.Index+2]))
               then  UhelLomu := DvaPi - UhelLomu;
           UhelLomu := (UhelLomu -Pi) / 2;
        end;
        SinLU    := Sin(UhelLomu);
        CosLU    := Cos(UhelLomu);

        P := LevaC.Definice;
        while P <> nil do
        begin
           with P.Sour do
           begin
              x := x + ((y * (SinLU/CosLU)) / Abs(LevaC.Velikost) * Abs(x));
           end;
           P := P.Dalsi
        end;

        P := PravaC.Definice;
        while P <> nil do
        begin
           with P.Sour do
           begin
              x := x - ((y * (SinLU/CosLU)) / Abs(PravaC.Velikost) * (Abs(PravaC.Velikost)-Abs(x)));
           end;
           P := P.Dalsi
        end;
     end;


  end;
end;

{-----------------------------------------------------------}
procedure    TPetrklicZn.Zobraz ( aDevice: TDevice; Vychbod: TDoubleCoord;
                                                var PoslBod: TDoubleCoord);
var
  P : TBodZnacky;
  Sour: TDoubleCoord;
begin
  if Assigned(Definice) then
  begin
     aDevice.MoveTo(VychBod.x, VychBod.y);
     P := Definice;
     while P.Dalsi <> nil do
     begin
        Sour.y := VychBod.y + P.OtSour.y;
        Sour.x := VychBod.x + P.OtSour.x;
        if not Generalizovat then
        begin
            if P.Spoj then
               aDevice.LineTo(Sour.x, Sour.y)
            else
               aDevice.MoveTo(Sour.x, Sour.y);
        end;
        P := P.Dalsi;
     end;
     PoslBod.x := VychBod.x + P.OtSour.x;
     PoslBod.y := VychBod.y + P.OtSour.y;
     if P.Spoj or Generalizovat then
        aDevice.LineTo(PoslBod.x, PoslBod.y);
  end;
end;

{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *              Tela metod TRetezovaZnacka                *
  *                                                        *
  **********************************************************
}
constructor  TRetezovaZnacka.Create;
begin
  inherited Create;
  ZakladniZnacka := 0;
  OchrannaZona   := 0;
  Vlevo  := polVlevo;
  Znacky := nil;
  Status := znKreslitPrvni;
end;

{-----------------------------------------------------------}
destructor   TRetezovaZnacka.Destroy;
var
  P, DalsiP: TLinZnacka;
begin
  P := Znacky;
  while P <> nil do
  begin
     DalsiP := P.Dalsi;
     P.Free;
     P := DalsiP;
  end;
end;

{-----------------------------------------------------------}
{
procedure    TRetezovaZnacka.NastavOchrannouZonu(Z: double);
begin
  OchrannaZona := Z/1000 * LinioveZnacky.Device.Projection.VirtualScale;
end;
}
{-----------------------------------------------------------}
procedure    TRetezovaZnacka.NastavPolohu(Kde: boolean);
begin
  Vlevo := Kde;
end;

{-----------------------------------------------------------}
procedure    TRetezovaZnacka.OtocDefinici(Uh: double);
var
//  Delka: double;
  P: TLinZnacka;
begin
  //Delka := 0;
  P := Znacky;
  while P <> nil do
  begin
     P.OtocDefinici(Uh);
     P := P.Dalsi;
  end;
end;

{-----------------------------------------------------------}
procedure    TRetezovaZnacka.PridejZnacku(AZnacka: TLinZnacka);
var
  P: TLinZnacka;
begin
  if Assigned(AZnacka) then
  begin
     AZnacka.Dalsi := nil;
     if Assigned(Znacky) then
     begin
        P := Znacky;
        while P.Dalsi <> nil do P := P.Dalsi;
        P.Dalsi := AZnacka;
     end
     else Znacky := AZnacka;
  end;
end;

{-----------------------------------------------------------}
function     TRetezovaZnacka.VratDelkuCeleZnacky: double;
var
  Delka: double;
  P: TLinZnacka;
begin
  Delka := 0;
  P := Znacky;
  { Jinak vsechno }
  { ------------- }
  while P <> nil do
  begin
     Delka := Delka + P.Velikost + P.Vzdalenost;
     P     := P.Dalsi;
  end;
  result := Delka;
end;

{-----------------------------------------------------------}
function     TRetezovaZnacka.VratDelkuZbytkuZnacky ( Znacka: TLinZnacka;
                                                     Op: word; d: double): double;
var
  //Delka: double;
  P: TLinZnacka;
begin
  //result := 0;
  P := Znacka;
  { Znacka }
  { ------ }
  if Op and opZnacka <> 0 then
  begin
     { Cela }
     { ---- }
     if (Op and opCela) <> 0 then
     begin
        result := P.Velikost + P.Vzdalenost;
     end
     else
     { Cast }
     { ---- }
     begin
        result := P.Velikost - d + P.Vzdalenost;
     end;
  end
  else
  { Mezera }
  { ------ }
  begin
     { Cela }
     { ---- }
     if (Op and opCela) <> 0 then
     begin
        result := P.Vzdalenost;
     end
     else
     { Cast }
     { ---- }
     begin
        result := P.Vzdalenost - d;
     end;
  end;

  { Delka zbylych clanku }
  { -------------------- }
  P := P.Dalsi;
  while P <> nil do
  begin
     result := result + P.Velikost + P.Vzdalenost;
     P     := P.Dalsi;
  end;

  //result := Delka;
end;

{-----------------------------------------------------------}
function     TRetezovaZnacka.VratRidiciLinii ( Vpravo: boolean;
                                               aLin: TRectEntity; var Mazat: boolean): TRectEntity;
var
   Para: TParalel;         { Paralelni linie ... ridici cara retezu }
begin
   Mazat := false;
   if Vzdalenost > 0 then
   { ParaLinie s patricnym odstupem }
   { ------------------------------ }
   begin
      Para := TParalel.Create;
      Para.Setup (Vzdalenost,Vpravo,0);
      Para.Input := aLin;
      Para.DoIt;
      result := TRectEntity(Para.Output);
      Para.Free;
      Mazat := true;
   end
   { Jinak je pouzita primo vlastni linie }
   { ------------------------------------ }
   else result := aLin;
end;

{-----------------------------------------------------------}


{-----------------------------------------------------------}
procedure    TRetezovaZnacka.ZobrazCelouZnacku (aDevice: TDevice; Vychbod: TDoubleCoord;
                                             var PoslBod: TDoubleCoord);
var
  P: TLinZnacka;
begin
  PoslBod := VychBod;
  P       := Znacky;
  while P <> nil do
  begin
     P.Zobraz(aDevice, PoslBod, PoslBod);
     P.PreskocMezeru(PoslBod, PoslBod);
     P := P.Dalsi;
  end;
end;

{-----------------------------------------------------------}

procedure JTSKNaPolarni(A,B: T2DCoordinate; var Delka: double;var Uhel: double);
var
  dX,dY: double;
  PomR : double;
begin
   dX := B.x - A.x;
   dY := B.y - A.y;
   begin
      if dX = 0 then
      begin
         if not(dY < 0) then
           Uhel := PI/2
         else
           Uhel := 1.5*PI;
         Delka := Abs(dY);
      end
      else
      begin
         if dY < 0 then
         begin
            if dX < 0 then PomR := PI
                      else PomR := 0;
         end
         else
         begin
            if dX > 0 then PomR := 0
                      else PomR := PI;
         end;
         Uhel  := ArcTAN(dY/dX);
         Uhel  := PomR + Uhel;
         Delka := SQRT(SQR(dY)+SQR(dX));
      end;
   end;
end;

{-----------------------------------------------------------}

procedure    TRetezovaZnacka.ZobrazPodelLinie(aDevice: TDevice; PuvodniBarva: TColor;
                                              DLin: TPolyLineElement; Orez: boolean; Rect: TDoubleRect);
var
  Dk: double;                  { Delka vertexu }
  d: double;                   { Zbytek delky pred lomovym bodem }
  Uk: double;                  { Smerovy uhel vertexu }
  PLin: TRectEntity;           { Linie s Indexem }
  DelkaCeleZnacky: double;     { Delka kompletniho retezu clanku }
  PocetCelychZnacek: longInt;  { Pocet kompletnich znacek }
  DelkaZbytku2: double;        { Delka zbyle casti retezu pro prenos dale }
  ZbylaZnacka: TLinZnacka;     { Zbyla cast znacky pro vykresleni }
  Zbytek1, Zbytek2: TLinZnacka;{ Casti zbyle znacky po zlomeni }
  i: longInt;                  { Index pro cykly }
  Bod, PomBod: TDoubleCoord;   { Pomocny bod ... zacatek zbytku (treba) }
  OpStatus: word;              { Vysledky operaci }
  KyblNaZnacku: TLinZnacka;    { Odpadovy ukazatel na znacku hodnou vymazani }
  VynestKybl: boolean;         { Pokyn k vycisteni odpadoveho kose }
  Zobrazovat: boolean;         { Info o tom, zda se bude aktualni segment
                                 Vykreslovat }
  IntSila: Integer;            { Sila prepoctena z mm na body }
  Znacka1, Znacka2: TLinZnacka;{ Prvni a posledni znacka pro harmoniku }
  Poprve: boolean;             { Kontrola prvniho prubehu pro harmoniku }
  MaxIndex: longInt;
  Mazat: boolean;              { Mazat ridici linii }
  {............................................}
   //PocetLomu:  longInt;
   //PocetPretazeni: longInt;

  {............................................}
  procedure  VynesKybl;
  begin
     if KyblNaZnacku <> nil then
     begin
        KyblNaZnacku.Free;
        KyblNaZnacku := nil;
        VynestKybl   := Ne;
     end;
  end;

  {............................................}
begin
  Zobrazovat := true;
  Znacka1 := nil;
  Znacka2 := nil;
  { Nastaveni ridici linie pro retezovou znacku }
  { ------------------------------------------- }

  PLin := VratRidiciLinii(not Vlevo, DLin, Mazat);

  {**************************}
   {
   aDevice.Pen.Color := clRed;
   aDevice.Pen.Width := 1;
   aDevice.MoveTo(T2DCoordinate(PLin.Elements[0]).x, T2DCoordinate(PLin.Elements[0]).y);
   for i := 1 to PLin.Count -1 do
   begin
      aDevice.LineTo(T2DCoordinate(PLin.Elements[i]).x, T2DCoordinate(PLin.Elements[i]).y);
      aDevice.Pen.Color := clGreen;
   end;
   }
   //PocetLomu := 0;
   //PocetPretazeni := 0;
  {**************************}


  if PuvodniBarva <> aDevice.Page.Color then
  begin
    if ByTrueColor then
       PuvodniBarva := TrueColor
    else
    if (Barva > -1) or (Barva < -1) then
    begin
       PuvodniBarva := DejBarvicku(Barva)
    end;
  end;
  (* Zmna ... P.K. kvli TGeometryElementm
  if PuvodniBarva <> aDevice.Page.Color then
  begin
    if Barva > -1 then
    begin
       if ByTrueColor then PuvodniBarva := TrueColor
                      else PuvodniBarva := DejBarvicku(Barva)
    end
    else
    if PuvodniBarva = -1 then
    begin
       if ByTrueColor then PuvodniBarva := TrueColor
                      else
                      begin
                        PuvodniBarva := DejBarvicku(DLin.GetPropertyLongIntByName('Barva'));
                      end;
    end;
  end;
  *)
  { Bude se Vykreslovat Zakladni znacka }
  { ----------------------------------- }
  if (Znacky = nil) or Generalizovat then ZobrazZakladniZnacku(aDevice, PuvodniBarva, PLin)
  else
  { Kreslime clanky }
  { --------------- }
  begin
     { Nastaveni typu cary - otazka je, zda nenastavovat silu pro kazdou zn. }
     { --------------------------------------------------------------------- }
     IntSila := aDevice.Projection.GetPageLengthHoriz1(Sila/1000 * aDevice.Projection.VirtualScale);
     {
     if IntSila < 1 then IntSila := 1;
     }
     aDevice.Pen.Width := IntSila;
     aDevice.Pen.Color := PuvodniBarva;

     { Pozor, Harmonika }
     { ---------------- }
     if Status and znHarmonika <> 0 then
     begin
        Znacka1       := Znacky;
        if not Assigned(Znacka1.Dalsi) then exit;
        Znacky        := Znacky.Dalsi;
        Znacka2       := Znacky.Dalsi;
        Znacky.Dalsi  := nil;
     end;

     Poprve          := Ano;
     Zbytek1         := nil;
     Zbytek2         := nil;
     DelkaZbytku2    := 0;
     OpStatus        := 0;
     DelkaCeleZnacky := VratDelkuCeleZnacky;
     KyblNaZnacku    := nil;
     VynestKybl      := Ne;

     PLin.Index := 0;
     MaxIndex :=  PLin.Count - 1;
     if not (PLin.NoElements) then
     while (PLin.Index < MaxIndex){ and not UserAbort }do
     begin
        if Orez then Zobrazovat := CoordPrekryvObdelnikuSouradnicemi (
                                       T2dCoordinate(PLin.Elements[PLin.Index]),
                                       T2dCoordinate(PLin.Elements[PLin.Index+1]),
                                       Rect.Min, Rect.Max, OchrannaZona)
                else Zobrazovat := Ano;

        { Uhel a Delka Vertexu }
        { -------------------- }
        {!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! pouze JTSK }
        JTSKNaPolarni(T2dCoordinate(PLin.Elements[PLin.Index]),
                      T2dCoordinate(PLin.Elements[PLin.Index+1]), Dk, Uk);
        Bod.x := T2dCoordinate(PLin.Elements[PLin.Index]).x;
        Bod.y := T2dCoordinate(PLin.Elements[PLin.Index]).y;
        if Zobrazovat then
        begin
           if Status and znHarmonika <> 0 then
           begin
              if Poprve then
              begin
                 Znacka1.OtocDefinici(Uk);
                 Znacka1.Zobraz(aDevice, Bod, PomBod);
                 Poprve := Ne;
              end;
           end;
           OtocDefinici(Uk);
        end;
        { Vejde se tam pripadny zbytek? }
        { ----------------------------- }
        if  Dk >= DelkaZbytku2 then
        { Jo ! }
        begin
           Dk := Dk - DelkaZbytku2;
           PocetCelychZnacek := Round(Int(Dk / DelkaCeleZnacky));
           if PocetCelychZnacek < 0 then PocetCelychZnacek := 0;
           d   := Dk - PocetCelychZnacek * DelkaCeleZnacky;

           { Zobrazeni zbytku predchoziho vertexu }
           { ------------------------------------ }
           if Zbytek2 <> nil then
           begin
              if Zobrazovat then
              begin
                 Zbytek2.OtocDefinici(Uk);
                 ZobrazZbytekZnacky(aDevice, Bod, Zbytek2, Bod);
              end;
              ZBytek2.Free;
              Zbytek2 := nil;
           end;

           { Zobrazeni celych znacek }
           { ----------------------- }
           if Zobrazovat then
           if PocetCelychZnacek > 0 then for i := 1 to PocetCelychZnacek do
           begin
              ZobrazCelouZnacku(aDevice, Bod, Bod);
           end;
           ZbylaZnacka := Znacky;
        end
        else
        { Ne ! }
        begin
           Zbytek2.OtocDefinici(Uk);
           KyblNaZnacku := Zbytek2;
           VynestKybl   := Ano;
           ZbylaZnacka  := Zbytek2;
           d            := Dk;
          {**************************}
          //inc(PocetPretazeni); { !!!!!!!!!!!!!!!!!!!!!!!!!!!! }
          {**************************}
        end;


        { Zobrazeni zbylych celych kousku }
        { ------------------------------- }
        while ZbylaZnacka <> nil do
        begin
           if d < ZbylaZnacka.Velikost then
           begin
             OpStatus := opZnacka;
             Break;
           end
           else if Zobrazovat then ZbylaZnacka.Zobraz(aDevice, Bod, Bod);
           d := d - ZbylaZnacka.Velikost;

           if d < ZbylaZnacka.Vzdalenost then
           begin
             OpStatus := opMezera;
             Break;
           end
           else if Zobrazovat then ZbylaZnacka.PreskocMezeru(Bod, Bod);
           d := d - ZbylaZnacka.Vzdalenost;

           ZbylaZnacka := ZbylaZnacka.Dalsi;
        end;

        if not Assigned(ZbylaZnacka) then
        begin
           exit;
        end;

        { Totalni zbytky ... lamani a tak }
        { ------------------------------- }
        { Zbyva znacka }
        if OpStatus and opZnacka <> 0 then
        begin
           { Lamani }
           if ZbylaZnacka.Status and ZnLamatZnacku <> 0 then
           begin
              OpStatus := OpStatus + opCast;
              ZbylaZnacka.PrekrojZnacku(d,Zbytek1,Zbytek2,OpStatus,PLin);
              {**************************}
              //inc(PocetLomu); { !!!!!!!!!!!!!!!!!!!!!!!!!!!! }
              {**************************}
           end
           else
           { Kresleni posledni znacky .... asi bude precuhovat }
           if  (ZbylaZnacka.Status and ZnKreslitPosledni) <> 0 then
           begin
              OpStatus := opZnacka + opCast;
              if Zobrazovat then ZbylaZnacka.Zobraz(aDevice, Bod, Bod);

              Zbytek2  := TLinZnacka.Create;

              Zbytek2.Velikost  := 0;
              Zbytek2.Invisible := Ano;
              Zbytek2.Vzdalenost:= ZbylaZnacka.Velikost -
                                    d + ZbylaZnacka.Vzdalenost;
              Zbytek2.Dalsi     := ZbylaZnacka.Dalsi;
              Zbytek2.Definice  := nil;
              Zbytek2.Mezera.x := Vzdalenost;
              Zbytek2.Mezera.Y := 0;

              {
              if TypeOf(ZbylaZnacka^) = TypeOf(TLinZnacka) then
                 Zbytek2  := New(PLinZnacka, Init) else
                 Zbytek2  := New(PPetrklicZn, Init);
              Zbytek2^ := ZbylaZnacka^;
              Zbytek2^.Definice := nil;
              }
           end
           else
           { Nelame se ani nekresli posledni znacka }
           begin
              OpStatus := OpStatus + opCela;
              if ZbylaZnacka.ClassType = TLinZnacka then
                 Zbytek2  := TLinZnacka.Create else
                 Zbytek2  := TPetrklicZn.Create;
              Zbytek2.Assign(ZbylaZnacka);
              Zbytek2.Definice := ZbylaZnacka.VyrobKopiiDefinice;
           end;
        end
        else
        { Zbyva mezera }
        begin
           { Zatim zvysoka kaslu na lamani mezer }
           OpStatus := opMezera + opCast;
           ZbylaZnacka.PrekrojZnacku(d,Zbytek1,Zbytek2,OpStatus,nil);
        end;
        if Assigned(Zbytek1) then
        begin
           if Zobrazovat and
           ((PLin.Index < MaxIndex - 1) or
           (ZbylaZnacka.Status and ZnKreslitZbytek <> 0))then
           begin
              Zbytek1.OtocDefinici(Uk);
              Zbytek1.Zobraz(aDevice, Bod, Bod);
           end;
           Zbytek1.Free;
           Zbytek1 := nil;
        end;
        DelkaZbytku2 := VratDelkuZbytkuZnacky ( ZbylaZnacka, OpStatus, d);
        if VynestKybl then VynesKybl;
        PLin.Index := PLin.Index + 1;
        {
        KeepWindowsAlive;
        }
     end;
     if Zbytek2 <> nil then
     begin
        Zbytek2.Free;
        Zbytek2 := nil;
     end;

     { Pohov, Harmonika }
     { ---------------- }
     if Status and znHarmonika <> 0 then
     begin
        Bod.x := T2dCoordinate(PLin.Elements[PLin.Index]).x;
        Bod.y := T2dCoordinate(PLin.Elements[PLin.Index]).y;
        if Zobrazovat then
        begin
           Znacka2.OtocDefinici(Uk);
           Znacka2.Zobraz(aDevice, Bod, PomBod);
        end;
        Znacky.Dalsi := Znacka2;
        Znacky       := Znacka1;
     end;
  end;
  if Mazat then PLin.Free;
end;


{-----------------------------------------------------------}
procedure    TRetezovaZnacka.ZobrazZakladniZnacku ( aDevice: TDevice;
                                                    PuvodniBarva: TColor; PLin: TRectEntity);
var
   IntSila: integer;
   MaxIndex: longInt;
begin
   if not wordbool(ZakladniZnacka) or
      (ZakladniZnacka > 999) then
   begin
      IntSila := aDevice.Projection.GetPageLengthHoriz1(Sila/1000 * aDevice.Projection.VirtualScale);
      {
      if IntSila < 1 then IntSila := 1;
      }
      aDevice.Pen.Width := IntSila;
      aDevice.Pen.Color := PuvodniBarva;
   end
   else
   begin
      aDevice.Pen.Width := 1;
      aDevice.Pen.Color := PuvodniBarva;
   end;
   if PLin <> nil then
  begin
    PLin.Index := 0;
    aDevice.MoveTo(T2dCoordinate(PLin.Elements[PLin.Index]).x,
                   T2dCoordinate(PLin.Elements[PLin.Index]).y);
    MaxIndex := PLin.Count - 1;
    while (PLin.Index < MaxIndex){ and not UserAbort }do
    begin
      PLin.Index := PLin.Index + 1;
      aDevice.LineTo(T2dCoordinate(PLin.Elements[PLin.Index]).x,
                     T2dCoordinate(PLin.Elements[PLin.Index]).y);
      {KeepWindowsAlive;}
    end;
  end;
end;

{-----------------------------------------------------------}
procedure    TRetezovaZnacka.ZobrazZbytekZnacky ( aDevice: TDevice;
                                                  Vychbod: TDoubleCoord;
                                                  Znacka: TLinZnacka;
                                              var PoslBod: TDoubleCoord);
var
  P: TLinZnacka;
begin
  PoslBod := VychBod;
  P       := Znacka;
  while (P <> nil){ and not UserAbort }do
  begin
     P.Zobraz(aDevice, PoslBod, PoslBod);
     P.PreskocMezeru(PoslBod, PoslBod);
     P := P.Dalsi;
     {KeepWindowsAlive;}
  end;
end;

procedure     TRetezovaZnacka.NastavGeneralizace(const aDevice: Tdevice);
var
   P: TLinZnacka;
   PomVel: double;
begin
   P := Znacky;
   Generalizovat := true;
   if ShowAll then
      Generalizovat := false;
   while assigned(P) do
   begin
      PomVel := aDevice.Projection.GetPageLengthHoriz1(P.Velikost);
      if ShowALl then
         P.Generalizovat := false
      else
         P.Generalizovat := PomVel < 3;
      {P^.Generalizovat := false;}
      if Generalizovat then
      begin
         Generalizovat := P.Generalizovat;
         if Generalizovat then
         begin
            PomVel := aDevice.Projection.GetPageLengthHoriz1(P.Vzdalenost);
            Generalizovat := PomVel < 2;
         end;
      end;
      P := P.Dalsi;
   end;
end;


{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *              Tela metod TKomplikada                *
  *                                                        *
  **********************************************************
}

(*
constructor  TKomplikada.Init;
begin
  inherited Init;
  B1   := nil;
  B2   := nil;
  LB   := nil;
  LB0  := nil;
  LB1  := nil;
  B1Buf:= 0;
  B2Buf:= 0;
  LBBuf:= 0;
  LU   := NoAngle;
end;

{-----------------------------------------------------------}
destructor   TKomplikada.Done;
var
  P, DalsiP: PLinZnacka;
begin
  P := B1;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
  P := B2;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
  P := LB;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
  P := LB0;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
  P := LB1;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
  P := Znacky;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
end;

{-----------------------------------------------------------}
procedure    TKomplikada.OtocDefinici(Uh: double);
var
  P: PLinZnacka;
  PomU: Double;
begin
  P := LB0;
  while P <> nil do
  begin
     P^.OtocDefinici(Uh);
     P := P^.Dalsi;
  end;
  P := LB1;
  while P <> nil do
  begin
     P^.OtocDefinici(Uh);
     P := P^.Dalsi;
  end;
  if LU <> NoAngle then PomU := (Uh+(Lu-Uh)/2)
                   else PomU := Uh;
  P := LB;
  while P <> nil do
  begin
     P^.OtocDefinici(PomU);
     P := P^.Dalsi;
  end;
  P := Znacky;
  while P <> nil do
  begin
     P^.OtocDefinici(Uh);
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}
procedure    TKomplikada.OtocDefiniciKraje(Uh: double; aStat: word);
var
  P: PLinZnacka;
  PomU: Double;
begin
  P := B1;
  while P <> nil do
  begin
     if P^.Status and aStat <> 0 then P^.OtocDefinici(Uh);
     P := P^.Dalsi;
  end;

  P := B2;
  while P <> nil do
  begin
     if P^.Status and aStat <> 0 then P^.OtocDefinici(Uh);
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}
procedure    TKomplikada.PridejZnacku(AZnacka: PLinZnacka);
type
  PZn = ^PLinZnacka;
var
  P : PLinZnacka;
  Zn: PZn;
begin
  if AZnacka^.Status and znLB  <> 0 then Zn := @LB  else
  if AZnacka^.Status and znLB0 <> 0 then Zn := @LB0 else
  if AZnacka^.Status and znLB1 <> 0 then Zn := @LB1 else
  if AZnacka^.Status and znB1  <> 0 then Zn := @B1  else
  if AZnacka^.Status and znB2  <> 0 then Zn := @B2  else
     Zn := @Znacky;

  if Assigned(AZnacka) then
  begin
     AZnacka^.Dalsi := nil;
     if Assigned(Zn^) then
     begin
        P := Zn^;
        while P^.Dalsi <> nil do P := P^.Dalsi;
        P^.Dalsi := AZnacka;
     end
     else Zn^ := AZnacka;
  end;
end;

{-----------------------------------------------------------}

procedure    TKomplikada.PreskocMezeru(Vychbod: TSour; aDelka, aUhel: double; var PoslBod: TSour);
var
  dx, dy: double;
begin
  case SourSystem of
     JTSK:
     begin
        dy :=  (0-aDelka) * Cos(aUhel);
        dx :=  (0-aDelka) * Sin(aUhel);
     end;
     Gauss, Matematicky:
     begin
        dy :=  (aDelka) * Cos(aUhel);
        dx :=  (aDelka) * Sin(aUhel);
     end;
  end;
  PoslBod.y := VychBod.y + dy;
  PoslBod.x := VychBod.x + dx;
end;

{-----------------------------------------------------------}



{-----------------------------------------------------------}
procedure    TKomplikada.ZobrazPodelLinie(DC: HDC; const Okno: PWindowsObject;
                                               var Pero,OldPero: HPen; PuvodniBarva: TColorRef;
                                              DLin: TDynLiniePtr; BMP: boolean; Orez: boolean);
var
  Dk: double;                    { Delka vertexu }
  d: double;                     { Zbytek delky pred lomovym bodem }
  Uk: double;                    { Smerovy uhel vertexu }
  PLin: TLomBodPtr;            { Ukazatel na zacatek vertexu }
  DelkaCeleZnacky: double;       { Delka kompletniho retezu clanku }
  PocetCelychZnacek: longInt;  { Pocet kompletnich znacek }
  DelkaZbytku2: double;          { Delka zbyle casti retezu pro prenos dale }
  ZbylaZnacka: PLinZnacka;     { Zbyla cast znacky pro vykresleni }
  Zbytek1, Zbytek2: PLinZnacka;{ Casti zbyle znacky po zlomeni }
  i: longInt;                  { Index pro cykly }
  Bod, PomBod: TSour;          { Pomocny bod ... zacatek zbytku (treba) }
  OpStatus: word;              { Vysledky operaci }
  KyblNaZnacku: PLinZnacka;    { Odpadovy ukazatel na znacku hodnou vymazani }
  VynestKybl: boolean;         { Pokyn k vycisteni odpadoveho kose }
  Zobrazovat: boolean;         { Info o tom, zda se bude aktualni segment
     3                            Vykreslovat }
  IntSila: Integer;            { Sila prepoctena z mm na body }
  Znacka1, Znacka2: PLinZnacka;{ Prvni a posledni znacka pro harmoniku }
  Poprve: boolean;             { Kontrola prvniho vertexu }
  Naposled: boolean;           { indikator posledniho vertexu }
  PomD: double;
  PPero: PLogPen;


  {............................................}
  procedure  VynesKybl;
  begin
     if KyblNaZnacku <> nil then
     begin
        KyblNaZnacku^.Free;
        KyblNaZnacku := nil;
        VynestKybl   := Ne;
     end;
  end;

  {............................................}
begin
  { Nastaveni ridici linie pro komplikadu }
  { ------------------------------------- }
  PLin := VratRidiciLinii(not Vlevo, DLin);
  if PuvodniBarva <> DejSkutBarvu(BarvaPozadi) then
  begin
    ZrusPero(DC, Pero, OldPero);
    if Barva > -1 then PuvodniBarva := DejSkutBarvu(Barva)
    else
    if PuvodniBarva = -1 then  PuvodniBarva := DLin^.Zaznam.Barva;
                  {else PuvodniBarva := DLin^.Zaznam.Barva};
  end;

  { Nastaveni typu cary - otazka je, zda nenastavovat silu pro kazdou zn. }
  { --------------------------------------------------------------------- }
  IntSila := round(Sila/1000 * VztazneMeritko*PVektorOkno(Okno)^.KVert);
  ZrusPero(DC, Pero, OldPero);
  NastavSirkaPero(DC, IntSila, PuvodniBarva, Pero, OldPero);


  Poprve          := Ano;
  Naposled        := Ne;
  Zbytek1         := nil;
  Zbytek2         := nil;
  DelkaZbytku2    := 0;
  OpStatus        := 0;
  DelkaCeleZnacky := VratDelkuCeleZnacky;
  KyblNaZnacku    := nil;
  VynestKybl      := Ne;

  if PLin <> nil then
  while (PLin^.Dalsi <> nil) and not UserAbort do
  begin
     if Orez then Zobrazovat := PrekryvObdelnikuSouradnicemi (
                                    PLin^.Sour,
                                    PLin^.Dalsi^.Sour,
                                    PVektorOkno(Okno)^.RozmerJTSK.PH,
                                    PVektorOkno(Okno)^.RozmerJTSK.LD,
                                    OchrannaZona)
             else Zobrazovat := Ano;

     if not Assigned(PLin^.Dalsi^.Dalsi) then Naposled := Ano;
     { Uhel a Delka Vertexu }
     { -------------------- }
     JTSKNaPolarni(PLin^.Sour, PLin^.Dalsi^.Sour, Dk, Uk);
     Bod := PLin^.Sour;


     { Vyreseni prvniho bodu, obecne pak LB0 a LB }
     { ------------------------------------------ }
     PomD := 0;

     if Zobrazovat then
     begin
        OtocDefinici(Uk);
        if Poprve then
        begin
           { Prvni }
           { ----- }
           if Assigned(B1) then
           begin
              OtocDefiniciKraje(Uk, znB1);
              ZobrazB(DC, Okno, BMP, Bod, znB1);
           end;
           OpStatus := znB1;

           PomD := B1Buf;
        end
        else
        begin
           PomD     := LBBuf;
           OpStatus := znLB + znLB0;
        end;

        Dk := Dk - PomD;

        if Naposled then
        begin
           Dk := Dk - B2Buf;
        end
        else Dk := Dk - LBBuf;


        { Sude LB0 }
        { -------- }
        if Assigned(LB0) then
        begin
           ZobrazLB0(DC, Okno, BMP, Bod, opStatus);
        end;

        { LB }
        { -- }
        if Assigned(LB) then
        begin
           ZobrazLB(DC, Okno, BMP, Bod, OpStatus);
        end;

        Poprve := Ne;
     end;

     PreskocMezeru(Bod, PomD, Uk, Bod);

     if (Dk > 0) and (DelkaCeleZnacky > 0) then
     begin
        PocetCelychZnacek := Round(Int(Dk / DelkaCeleZnacky));
        if PocetCelychZnacek < 0 then PocetCelychZnacek := 0;
        d   := Dk - PocetCelychZnacek * DelkaCeleZnacky;

        { Zobrazeni celych znacek }
        { ----------------------- }
        if Zobrazovat then
        if PocetCelychZnacek > 0 then for i := 1 to PocetCelychZnacek do
        begin
           ZobrazCelouZnacku(DC, Okno, Bod, BMP, Bod);
        end;
        ZbylaZnacka := Znacky;


        { Zobrazeni zbylych celych kousku }
        { ------------------------------- }
        while ZbylaZnacka <> nil do
        begin
           if d < ZbylaZnacka^.Velikost then
           begin
             OpStatus := opZnacka;
             Break;
           end
           else if Zobrazovat then ZbylaZnacka^.Zobraz(DC, Okno, Bod, BMP, Bod);
           d := d - ZbylaZnacka^.Velikost;

           if d < ZbylaZnacka^.Vzdalenost then
           begin
             OpStatus := opMezera;
             Break;
           end
           else if Zobrazovat then ZbylaZnacka^.PreskocMezeru(Bod, Bod);
           d := d - ZbylaZnacka^.Vzdalenost;

           ZbylaZnacka := ZbylaZnacka^.Dalsi;
        end;

        if not Assigned(ZbylaZnacka) then
        begin
           exit;
        end;

        { Totalni zbytky ... lamani a tak }
        { ------------------------------- }
        { Zbyva znacka }
        if OpStatus and opZnacka <> 0 then
        begin
           { Lamani }
           if ZbylaZnacka^.Status and ZnLamatZnacku <> 0 then
           begin
              OpStatus := OpStatus + opCast;
              ZbylaZnacka^.PrekrojZnacku(d,Zbytek1,Zbytek2,OpStatus,PLin);
           end
           else
           { Kresleni posledni znacky .... asi bude precuhovat }
           if Zobrazovat and (ZbylaZnacka^.Status and ZnKreslitPosledni <> 0) then
           begin
              OpStatus := opZnacka + opCast;
              Zobraz(DC, Okno, Bod, BMP, Bod);

              Zbytek2  := New(PLinZnacka, Init);

              Zbytek2^.Velikost  := 0;
              Zbytek2^.Invisible := Ano;
              Zbytek2^.Vzdalenost:= ZbylaZnacka^.Velikost -
                                    d + ZbylaZnacka^.Vzdalenost;
              Zbytek2^.Dalsi     := ZbylaZnacka^.Dalsi;
              Zbytek2^.Definice  := nil;
              case SourSystem of
                 JTSK              : Zbytek2^.Mezera.y := 0 - Vzdalenost;
                 Gauss, Matematicky: Zbytek2^.Mezera.y := Vzdalenost;
              end;
              Zbytek2^.Mezera.x := 0;

              {
              if TypeOf(ZbylaZnacka^) = TypeOf(TLinZnacka) then
                 Zbytek2  := New(PLinZnacka, Init) else
                 Zbytek2  := New(PPetrklicZn, Init);
              Zbytek2^ := ZbylaZnacka^;
              Zbytek2^.Definice := nil;
              }
           end;
        end;

        if Assigned(Zbytek1) then
        begin
           if Zobrazovat and
           (ZbylaZnacka^.Status and ZnKreslitZbytek <> 0)then
           begin
              Zbytek1^.OtocDefinici(Uk);
              Zbytek1^.Zobraz(DC, Okno, Bod, BMP, Bod);
           end;
           Zbytek1^.Free;
           Zbytek1 := nil;
        end;

        if Assigned(Zbytek2) then
        begin
           Zbytek2^.Free;
           Zbytek2 := nil;
        end;
     end;

     { Zobrazeni druheho konce vertexu }
     { ------------------------------- }
     Bod := PLin^.Dalsi^.Sour;
     if Zobrazovat then
     begin
        if Naposled then
        begin
           { Posledni }
           { -------- }
           OtocDefiniciKraje(Uk, znB2);
           ZobrazB(DC, Okno, BMP, Bod, znB2);
           OpStatus := znB2;
        end
        else
           OpStatus := znLB1;

        { Liche LB1 }
        { --------- }
        if Assigned(LB1) then
        begin
           ZobrazLB1(DC, Okno, BMP, Bod, opStatus);
        end;

        { LB }
        { -- }
        if Assigned(LB) then
        begin
           ZobrazLB(DC, Okno, BMP, Bod, OpStatus);
        end;

     end;

     PLin := PLin^.Dalsi;
     LU := Uk;
     KeepWindowsAlive;
  end;
  if Zbytek2 <> nil then
  begin
     Zbytek2^.Free;
     Zbytek2 := nil;
  end;

end;


{-----------------------------------------------------------}

procedure    TKomplikada.ZobrazB   ( DC: HDC; const Okno: PWindowsObject; BMP: boolean;
                                     Vychbod: TSour; AStat: word );
var
  P: PLinZnacka;
  PomBod: TSour;
begin
  P := B1;
  while P <> nil do
  begin
     if P^.Status and AStat <> 0 then
     begin
        P^.Zobraz(DC, Okno, VychBod, BMP, PomBod);
     end;
     P := P^.Dalsi;
  end;
  P := B2;
  while P <> nil do
  begin
     if P^.Status and AStat <> 0 then
     begin
        P^.Zobraz(DC, Okno, VychBod, BMP, PomBod);
     end;
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}

procedure    TKomplikada.ZobrazLB0 ( DC: HDC; const Okno: PWindowsObject; BMP: boolean;
                                     Vychbod: TSour; AStat: word);
var
  P: PLinZnacka;
  PomBod: TSour;
begin
  P := LB0;
  while P <> nil do
  begin
     if (P^.Status and AStat) <> 0 then
     begin
        P^.PreskocMezeru(VychBod, PomBod);
        P^.Zobraz(DC, Okno, PomBod, BMP, PomBod);
     end;
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}
procedure    TKomplikada.ZobrazLB1 ( DC: HDC; const Okno: PWindowsObject; BMP: boolean;
                                     Vychbod: TSour; AStat: word);
var
  P: PLinZnacka;
  PomBod: TSour;
begin
  P := LB1;
  while P <> nil do
  begin
     if (P^.Status and AStat) <> 0 then
     begin
        P^.PreskocMezeru(VychBod, PomBod);
        P^.Zobraz(DC, Okno, VychBod, BMP, PomBod);
     end;
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}
procedure    TKomplikada.ZobrazLB  ( DC: HDC; const Okno: PWindowsObject; BMP: boolean;
                                     Vychbod: TSour; AStat: word);

var
  P: PLinZnacka;
  PomBod: TSour;
begin
  P := LB;
  while P <> nil do
  begin
     if (P^.Status and AStat) <> 0 then
     begin
        P^.Zobraz(DC, Okno, VychBod, BMP, PomBod);
     end;
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *           Tela metod TNabRetezovaZnacka                *
  *                                                        *
  **********************************************************
}
constructor  TNabRetezovaZnacka.Init;
begin
  inherited Init;
  ZakladniZnacka := 0;
  Vlevo  := polVlevo;
  Znacky := nil;
end;

{-----------------------------------------------------------}
destructor   TNabRetezovaZnacka.Done;
var
  P, DalsiP: PNabLinZnacka;
begin
  P := Znacky;
  while P <> nil do
  begin
     DalsiP := P^.Dalsi;
     P^.Free;
     P := DalsiP;
  end;
end;

{-----------------------------------------------------------}
procedure    TNabRetezovaZnacka.NastavPolohu(Kde: boolean);
begin
  Vlevo := Kde;
end;

{-----------------------------------------------------------}
procedure    TNabRetezovaZnacka.PridejZnacku(AZnacka: PNabLinZnacka);
var
  P: PNabLinZnacka;
begin
  if Assigned(AZnacka) then
  begin
     AZnacka^.Dalsi := nil;
     if Assigned(Znacky) then
     begin
        P := Znacky;
        while P^.Dalsi <> nil do P := P^.Dalsi;
        P^.Dalsi := AZnacka;
     end
     else Znacky := AZnacka;
  end;
end;

{-----------------------------------------------------------}
function     TNabRetezovaZnacka.VratDelkuCeleZnacky: longInt;
var
  Delka: longInt;
  P: PNabLinZnacka;
begin
  Delka := 0;
  P := Znacky;
  while P <> nil do
  begin
     Delka := Delka + P^.Velikost + P^.Vzdalenost;
     P     := P^.Dalsi;
  end;
  VratDelkuCeleZnacky := Delka;
end;


{-----------------------------------------------------------}
procedure    TNabRetezovaZnacka.ZobrazCelouZnacku ( DC: HDC;Vychbod: TLongPoint;
                                                var PoslBod: TLongPoint);
var
  P: PNabLinZnacka;
begin
  PoslBod := VychBod;
  P       := Znacky;
  while P <> nil do
  begin
     P^.Zobraz(DC,}PoslBod, PoslBod);
     P^.PreskocMezeru(PoslBod, PoslBod);
     P := P^.Dalsi;
  end;
end;

{-----------------------------------------------------------}
procedure    TNabRetezovaZnacka.ZobrazPodelLinie( DC: HDC;
                                             x1, x2, y: longint; ABarva: longInt);
var
  Dk: longInt;                    { Delka vertexu }
  d: longInt;                     { Zbytek delky pred lomovym bodem }
  IntSila: longInt;
  PomBarva: longInt;
  DelkaCeleZnacky: longInt;       { Delka kompletniho retezu clanku }
  PocetCelychZnacek: longInt;     { Pocet kompletnich znacek }
  ZbylaZnacka: PNabLinZnacka;     { Zbyla cast znacky pro vykresleni }
  i: longInt;                     { Index pro cykly }
  Bod: TLongPoint;                { Pomocny bod ... zacatek zbytku (treba) }
  OpStatus: word;                 { Vysledky operaci }

  Pero,OldPero: HPen;


begin
  { Nastaveni ridici linie pro retezovou znacku }
  { ------------------------------------------- }
  if Vlevo then dec(y, Vzdalenost)
           else inc(y, Vzdalenost);

  { Nastaveni barvy }
  { --------------- }
  if Barva > -1 then PomBarva := DejSkutBarvu(Barva)
                else PomBarva := ABarva;

  { Bude se Vykreslovat Zakladni znacka }
  { ----------------------------------- }
  if Znacky = nil then ZobrazZakladniZnacku(DC, PomBarva,x1, x2, y)
  else
  { Kreslime clanky }
  { --------------- }
  begin
     { Nastaveni typu cary - otazka je, zda nenastavovat silu pro kazdou zn. }
     { --------------------------------------------------------------------- }

     NastavSirkaPero(DC, Sila, {DejSkutBarvu}(PomBarva), Pero, OldPero);

     DelkaCeleZnacky := VratDelkuCeleZnacky;

     { Delka Vertexu a pocatecni bod }
     { ----------------------------- }
     Dk    := x2 - x1;
     Bod.x := x1;
     Bod.y := y;

     PocetCelychZnacek := Dk div  DelkaCeleZnacky;
     if PocetCelychZnacek < 0 then PocetCelychZnacek := 0;
     d   := Dk - PocetCelychZnacek * DelkaCeleZnacky;

     { Zobrazeni celych znacek }
     { ----------------------- }
     if PocetCelychZnacek > 0 then for i := 1 to PocetCelychZnacek do
     begin
        ZobrazCelouZnacku(DC, Bod, Bod);
     end;
     ZbylaZnacka := Znacky;


     { Zobrazeni zbylych celych kousku }
     { ------------------------------- }
     while ZbylaZnacka <> nil do
     begin
        if d < ZbylaZnacka^.Velikost then Break
        else ZbylaZnacka^.Zobraz(DC, Bod, Bod);
        d := d - ZbylaZnacka^.Velikost;

        if d < ZbylaZnacka^.Vzdalenost then Break
        else ZbylaZnacka^.PreskocMezeru(Bod, Bod);
        d := d - ZbylaZnacka^.Vzdalenost;

        ZbylaZnacka := ZbylaZnacka^.Dalsi;
     end;
     ZrusPero(DC, Pero, OldPero);
  end;
end;

{-----------------------------------------------------------}



procedure    TNabRetezovaZnacka.ZobrazZakladniZnacku(DC: HDC; ABarva: longInt;
                                                      x1, x2, y: longint);
var
   IntSila: Integer;
begin
   if not wordbool(ZakladniZnacka) or (ZakladniZnacka > 999) then
   begin
      NastavSirkaPero(DC, Sila, {DejSkutBarvu}(ABarva), Pero, OldPero);
   end
   else NastavPero(DC, ZakladniZnacka, {DejSkutBarvu}(ABarva), Pero, OldPero);
   MoveTo(DC, x1, y);
   LineTo(DC, x2, y);
   ZrusPero(DC, Pero, OldPero);
end;

*)

{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *              Tela metod TLiniovaZnacka                 *
  *                                                        *
  **********************************************************
}

constructor TLiniovaZnacka.Create;
begin
  inherited Create;
  NahradniZnacka := 0;
  Kod            := -1;
  Nazev          := '';
  Znacky         := nil;
end;

{-----------------------------------------------------------}
destructor  TLiniovaZnacka.Destroy;
var
  P, DalsiP: TRetezovaZnacka;
begin
  P := Znacky;
  while P <> nil do
  begin
     DalsiP := TRetezovaZnacka(P.Dalsi);
     P.Free;
     P := DalsiP;
  end;
end;

{-----------------------------------------------------------}
procedure   TLiniovaZnacka.NastavKod(AKod: longInt);
begin
  Kod := AKod;
end;

{-----------------------------------------------------------}
procedure   TLiniovaZnacka.NastavNahradniZnacku(AZnacka: integer);
begin
  NahradniZnacka := AZnacka;
end;

{-----------------------------------------------------------}
procedure   TLiniovaZnacka.NastavNazev(ANazev: String);
begin
  Nazev := ANazev;
end;

{-----------------------------------------------------------}
procedure   TLiniovaZnacka.PridejZnacku(AZnacka: TRetezovaZnacka);
begin
  if Assigned(AZnacka) then
  begin
     AZnacka.Dalsi := Znacky;
     Znacky := AZnacka;
  end;
end;

{-----------------------------------------------------------}
procedure   TLiniovaZnacka.ZobrazLinii(adevice:TDevice; PuvodniBarva: TColor;
                                       Linie: TPolyLineElement; Orez: boolean; Rect: TDoubleRect);
var
   Znacka: TRetezovaZnacka;
   Ore: boolean;
begin
   if ShowAll then Ore := false
              else Ore := Orez;
   Znacka := Znacky;
   while (Znacka <> nil) {and not UserAbort }do
   begin
      Znacka.ShowAll := ShowAll;
      Znacka.NastavGeneralizace(aDevice);
      Znacka.ZobrazPodelLinie(aDevice, PuvodniBarva, Linie, Ore, Rect);
      Znacka := TRetezovaZnacka(Znacka.Dalsi);
      {KeepWindowsAlive;}
   end;
end;

{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *              Tela metod TNabLiniovaZnacka              *
  *                                                        *
  **********************************************************
}
(*
constructor TNabLiniovaZnacka.Init;
begin
  inherited Init;
  Kod            := -1;
  Nazev          := '';
  Znacky         := nil;
end;

{-----------------------------------------------------------}
destructor  TNabLiniovaZnacka.Done;
var
  P, DalsiP: PNabRetezovaZnacka;
begin
  P := Znacky;
  while P <> nil do
  begin
     DalsiP := PNabRetezovaZnacka(P^.Dalsi);
     P^.Free;
     P := DalsiP;
  end;
end;

{-----------------------------------------------------------}
procedure   TNabLiniovaZnacka.NastavKod(AKod: longInt);
begin
  Kod := AKod;
end;

{-----------------------------------------------------------}
procedure   TNabLiniovaZnacka.NastavNazev(ANazev: String);
begin
  Nazev := ANazev;
end;

{-----------------------------------------------------------}
procedure   TNabLiniovaZnacka.PridejZnacku(AZnacka: PNabRetezovaZnacka);
begin
  if Assigned(AZnacka) then
  begin
     AZnacka^.Dalsi := Znacky;
     Znacky := AZnacka;
  end;
end;

{-----------------------------------------------------------}
procedure   TNabLiniovaZnacka.ZobrazLinii(DC: HDC;
                                          x1, x2, y: longint; ABarva: longInt);
var
   Znacka: PNabRetezovaZnacka;
begin
   Znacka := Znacky;
   while Znacka <> nil do
   begin
      Znacka^.ZobrazPodelLinie(DC,x1, x2, y, ABarva);
      Znacka := PNabRetezovaZnacka(Znacka^.Dalsi);
   end;
end;

*)
{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *              Tela metod TLinZnLibrary                  *
  *                                                        *
  **********************************************************
}


constructor TLinZnLibrary.CreateAll(const ASoubor: string);
var
  {
  F: text;
  Radek: string;
  CtiR: TCtiRadek;
  }
  P: TSouborLinZn;

begin
  inherited Create;
{No LIB ..... 24.2.1995 }
  P := TSouborLinZn.Create;
  P.Kod:= 0;
  P.Name:= ASoubor;
  AddElement(P);

(*  LIB ..... 24.2.1995  odstraneno kvuli kompatidebilite s DOSem
  P := nil;

  if ASoubor <> '' then
  begin
     assign(F, ASoubor);
     fileMode:= moOpenRONN;
     {$I-} reset(F); {$I+}
     fileMode:= moStandard;


     if IOResult = 0 then
     begin
        while not eof(F) do
        begin
           readLn(F, Radek);
           if Radek <> '' then
           begin
              New(P);
              CtiR.Init(Radek);
              P^.Kod:= CtiR.DejInteger;
              CtiR.PreskocMezery;
   {$IFDEF WINDOWS}
              StrPCopy(P^.Soubor, BezVnejsichMezer(CtiR.DejStringPoZnak(' ')));
   {$ELSE}
              P^.Soubor:= BezVnejsichMezer(CtiR.DejStringPoZnak(' '));
   {$ENDIF}
              if (P^.Kod >= 0) and (P^.Kod < 99) and (P^.Soubor <> '') then Insert(P)
                                                                       else dispose(P);
           end;
        end
     end
     else
     begin
        Fail;
     end;
  end
*)
end;


{-----------------------------------------------------------}

procedure  TLinZnLibrary.VratSouborCislo(ACislo: longInt;
                                        var Soubor: string);
var
  PSouborLZ: TSouborLinZn;

begin
  PSouborLZ := TSouborLinZn(Elements[0]);
  if Assigned(PSouborLZ) then Soubor := PSouborLZ.Name
                         else Soubor := '';
end;



{-----------------------------------------------------------}

{
  **********************************************************
  *                                                        *
  *              Tela metod TLinioveZnacky                 *
  *                                                        *
  **********************************************************
}


constructor TLinioveZnacky.CreateAll(const ASoubor: string);
begin
  inherited Create;
  OK := True;
  Knihovna := TLinZnLibrary.CreateAll(ASoubor);
  if not Assigned(Knihovna) then OK := False;
  KodNabidky := -1;
  Nabidka    := Ne;
end;


{-----------------------------------------------------------}
destructor  TLinioveZnacky.Destroy;
begin
  if Assigned(Knihovna) then Knihovna.Free;
  inherited Destroy;
end;

{-----------------------------------------------------------}
(*
procedure TLinioveZnacky.NastavNabidku(AKod: longInt);
begin
  if (not Nabidka) or (KodNabidky <> AKod) then
  begin
     FreeAll;
     { No LIB .....}
     NactiNabZnacky(0);
     { .... }
     Nabidka := Ano;
     KodNabidky := AKod;
  end;
end;

{-----------------------------------------------------------}
procedure TLinioveZnacky.ZrusNabidku;
begin
  if Nabidka then
  begin
     FreeAll;
     Nabidka    := Ne;
     KodNabidky := -1;
  end;
end;

{-----------------------------------------------------------}

procedure TLinioveZnacky.NactiNabZnacky(AKod: longInt);
var
  Ctenar       : TCteniLiniovychZnacek;
  OKa          : boolean;
  LinZnacka: PNabLiniovaZnacka;
  Soubor : PathStr;
begin
  if not OK then exit;
  FreeAll;
  Nabidka    := Ano;
  KodNabidky := AKod;
  Knihovna^.VratSouborCislo(AKod, Soubor);
  Ctenar.Init(Soubor, OKa);
  if OKa then with Ctenar do
  repeat
     LinZnacka := CtiDalsiNabLiniovouZnacku;
     if Assigned(LinZnacka) then
     begin
        LinZnacka^.Kod := LinZnacka^.Kod { LIB .... + (AKod * 10000) };
        Insert(LinZnacka);
     end;
  until  not Assigned(LinZnacka);
  Ctenar.Done;
end;


*)
{-----------------------------------------------------------}
function  TLinioveZnacky.PridejZnackuDleKodu(AKod: longInt): TLiniovaZnacka;
var
  Ctenar       : TCteniLiniovychZnacek;
  OKa          : boolean;
  LiniovaZnacka: TLiniovaZnacka;
  Soubor       : string;
begin
  PridejZnackuDleKodu := nil;
  if not OK then exit;
  Knihovna.VratSouborCislo(0{ LIB ...AKod div 10000}, Soubor );
  if Soubor <> '' then
  begin
     Ctenar := TCteniLiniovychZnacek.Create(Soubor, OKa);
     Ctenar.Device := Device;
     if OKa then with Ctenar do
     begin
        LiniovaZnacka := CtiLiniovouZnacku(AKod { LIB .... mod 10000 });
        if Assigned(LiniovaZnacka) then
        begin
           AddElement(LiniovaZnacka);
           LiniovaZnacka.Kod := AKod;
        end;
        result := LiniovaZnacka;
     end;
     Ctenar.Free;
  end;
end;
(*
{-----------------------------------------------------------}
function  TLinioveZnacky.PridejNabZnackuDleKodu(AKod: longInt): PNabLiniovaZnacka;
var
  Ctenar       : TCteniLiniovychZnacek;
  OKa          : boolean;
  LiniovaZnacka: PNabLiniovaZnacka;
  Soubor : PathStr;
begin
  PridejNabZnackuDleKodu := nil;
  if not OK then exit;
  Knihovna^.VratSouborCislo(0{ LIB ... AKod div 10000}, Soubor);
  Ctenar.Init(Soubor, OKa);
  if OKa then with Ctenar do
  begin
     LiniovaZnacka := CtiNabLiniovouZnacku(AKod { LIB ... mod 10000});
     Insert(LiniovaZnacka);
     if Assigned(LiniovaZnacka) then LiniovaZnacka^.Kod := AKod;
     PridejNabZnackuDleKodu := LiniovaZnacka;
  end;
  Ctenar.Done;
end;


*)
{-----------------------------------------------------------}
function  TLinioveZnacky.VratZnackuDleKodu(AKod: longInt): TLiniovaZnacka;
var
  Znacka: TLiniovaZnacka;
  i: longInt;

  {............................................}
  function KodSedi(Item: Pointer): Boolean; far;
    begin
       KodSedi := TLiniovaZnacka(Item).Kod = AKod;
    end;
  {............................................}
begin
   {
   ZrusNabidku;
   }
   Znacka := nil;
   for i := 0 to Count -1 do
   begin
      Znacka := TLiniovaZnacka(Elements[i]);
      if Znacka.Kod = AKod then break;
      Znacka := nil;
   end;
   if not assigned(Znacka) then Znacka := PridejZnackuDleKodu(AKod);
   result := Znacka;
end;


(*
{-----------------------------------------------------------}
function  TLinioveZnacky.VratNabZnackuDleKodu(AKod: longInt): PNabLiniovaZnacka;
var
  Znacka: PNabLiniovaZnacka;

  {............................................}
  function KodSedi(Item: Pointer): Boolean; far;
    begin
       KodSedi := PNabLiniovaZnacka(Item)^.Kod = AKod;
    end;
  {............................................}
begin
   NastavNabidku(-1);
   Znacka := FirstThat(@KodSedi);
   if Znacka = nil then Znacka := PridejNabZnackuDleKodu(AKod);
   VratNabZnackuDleKodu := Znacka;
end;



{-----------------------------------------------------------}
function  TLinioveZnacky.VratZnackuDleNazvu(ANazev: String): PLiniovaZnacka;
  {............................................}
  function NazevSedi(Item: Pointer): Boolean; far;
    begin
       NazevSedi := PLiniovaZnacka(Item)^.Nazev = ANazev;
    end;
  {............................................}
begin
   ZrusNabidku;
   VratZnackuDleNazvu := FirstThat(@NazevSedi);
end;
{-----------------------------------------------------------}
*)
{
  **********************************************************
  *                                                        *
  *             Tela metod TCteniLiniovychZnacek           *
  *                                                        *
  **********************************************************
}

function  TCteniLiniovychZnacek.mm_m(mm: double): double;
begin
   result := mm/1000 * Device.Projection.VirtualScale;
end;



function  TCteniLiniovychZnacek.CtiLiniovouZnacku(Kod: longInt): TLiniovaZnacka;
var
  LiniovaZnacka: TLiniovaZnacka;
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Paklic: TPaklicNaPetrklic;

begin
  Paklic := TPaklicNaPetrklic.Create;
  LiniovaZnacka := TLiniovaZnacka.Create;

  { Nacitame Znacku s konkretnim kodem - ta se musi ovsem nejdriv najit }
  { ------------------------------------------------------------------- }
  if longBool(Kod)  then
  begin
     LiniovaZnacka.Kod := -1;
     repeat
        { Hledam Liniovou Znacku }
        { ---------------------- }
        repeat
           DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                      NazevParametru, HodnotaParametru);
        until (JeToInfo and (HodnotaParametru = lzLiniovaZnacka)) or PosledniRadek;
        { Kontrola kodu }
        if not PosledniRadek then
        begin
           DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                      NazevParametru, HodnotaParametru);
           if JeToParametr and (NazevParametru = lzKod) then
              LiniovaZnacka.Kod :=  DejLongIntParametr else
           if JeToParametr and (NazevParametru = lzNazev) then
              LiniovaZnacka.Nazev :=  HodnotaParametru ;
        end;
     until (LiniovaZnacka.Kod = Kod) or PosledniRadek ;
     if PosledniRadek then
     begin
        { Zadna takova znacka tady nikdy nebyla }
        { ------------------------------------- }
        LiniovaZnacka.Free;
        CtiLiniovouZnacku := nil;
        Paklic.Free;
        exit;
     end;
  end;
  { Vlastni nacitani znacky }
  { ----------------------- }
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Retez }
     { ----- }
     if JeToInfo and (HodnotaParametru = lzRetez) then
        LiniovaZnacka.PridejZnacku(CtiRetez(Paklic))
     (*
     else
     { Komplikace }
     { ----- }
     if JeToInfo and (HodnotaParametru = lzKomplikace) then
        LiniovaZnacka.PridejZnacku(CtiKomplikaci(Paklic))
     *)
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
        LiniovaZnacka.Kod :=  DejLongIntParametr
     else
     { Nazev }
     { ----- }
     if JeToParametr and (NazevParametru = lzNazev) then
        LiniovaZnacka.Nazev :=  HodnotaParametru
     else
     { NahradniZnacka }
     { -------------- }
     if JeToParametr and (NazevParametru = lzNahradniZnacka) then
        LiniovaZnacka.NahradniZnacka :=  DejIntParametr;
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (LiniovaZnacka.Kod > -1) and Assigned(LiniovaZnacka.Znacky) then
     CtiLiniovouZnacku := LiniovaZnacka
  else
  begin
     CtiLiniovouZnacku := nil;
     LiniovaZnacka.Free;
  end;
  Paklic.Free;
end;


(*
{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiNabLiniovouZnacku(Kod: longInt): TNabLiniovaZnacka;
var
  LiniovaZnacka: TNabLiniovaZnacka;
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Paklic: TPaklicNaPetrklic;

begin
  Paklic := TPaklicNaPetrklic.Create;
  LiniovaZnacka := TNabLiniovaZnacka.Create;

  { Nacitame Znacku s konkretnim kodem - ta se musi ovsem nejdriv najit }
  { ------------------------------------------------------------------- }
  if longBool(Kod) then
  begin
     LiniovaZnacka.Kod := -1;
     repeat
        { Hledam Liniovou Znacku }
        { ---------------------- }
        repeat
           DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                      NazevParametru, HodnotaParametru);
        until (JeToInfo and (HodnotaParametru = lzLiniovaZnacka)) or PosledniRadek;
        { Kontrola kodu }
        if not PosledniRadek then
        begin
           DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                      NazevParametru, HodnotaParametru);
           if JeToParametr and (NazevParametru = lzKod) then
              LiniovaZnacka.Kod :=  DejLongIntParametr else
           if JeToParametr and (NazevParametru = lzNazev) then
              LiniovaZnacka.Nazev :=  HodnotaParametru ;
        end;
     until (LiniovaZnacka.Kod = Kod) or PosledniRadek ;
     if PosledniRadek then
     begin
        { Zadna takova znacka tady nikdy nebyla }
        { ------------------------------------- }
        LiniovaZnacka.Free;
        CtiNabLiniovouZnacku := nil;
        Taklic.Free;
        exit;
     end;
  end;
  { Vlastni nacitani znacky }
  { ----------------------- }
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Retez }
     { ----- }
     if JeToInfo and (HodnotaParametru = lzRetez) then
        LiniovaZnacka.PridejZnacku(CtiNabRetez(Paklic))
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
        LiniovaZnacka.Kod :=  DejLongIntParametr
     else
     { Nazev }
     { ----- }
     if JeToParametr and (NazevParametru = lzNazev) then
        LiniovaZnacka.Nazev :=  HodnotaParametru;
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (LiniovaZnacka.Kod > -1) and Assigned(LiniovaZnacka^.Znacky) then
     CtiNabLiniovouZnacku := LiniovaZnacka
  else
  begin
     CtiNabLiniovouZnacku := nil;
     LiniovaZnacka^.Free;
  end;
  dispose(Paklic, Done);
end;



{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiDalsiNabLiniovouZnacku: PNabLiniovaZnacka;
var
  LiniovaZnacka: PNabLiniovaZnacka;
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Paklic: PPaklicNaPetrklic;

begin
  Paklic := new(PPaklicNaPetrklic, Init);
  LiniovaZnacka := new(PNabLiniovaZnacka,Init);
  LiniovaZnacka^.Kod := -1;
  { Hledam Liniovou Znacku }
  { ---------------------- }
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
  until (JeToInfo and (HodnotaParametru = lzLiniovaZnacka)) or PosledniRadek;

  if PosledniRadek then
  begin
     { Zadna dalsi znacka uz neni }
     { -------------------------- }
     LiniovaZnacka^.Free;
     CtiDalsiNabLiniovouZnacku := nil;
     dispose(Paklic, Done);
     exit;
  end;

  { Vlastni nacitani znacky }
  { ----------------------- }
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Retez }
     { ----- }
     if JeToInfo and (HodnotaParametru = lzRetez) then
        LiniovaZnacka^.PridejZnacku(CtiNabRetez(Paklic))
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
        LiniovaZnacka^.Kod :=  DejLongIntParametr
     else
     { Nazev }
     { ----- }
     if JeToParametr and (NazevParametru = lzNazev) then
        LiniovaZnacka^.Nazev :=  HodnotaParametru;
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (LiniovaZnacka^.Kod > -1) and Assigned(LiniovaZnacka^.Znacky) then
     CtiDalsiNabLiniovouZnacku := LiniovaZnacka
  else
  begin
     CtiDalsiNabLiniovouZnacku := nil;
     LiniovaZnacka^.Free;
  end;
  dispose(Paklic, Done);
end;

*)

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiRetez(Paklic: TPaklicNaPetrklic): TRetezovaZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Retez: TRetezovaZnacka;
begin
  Retez := TRetezovaZnacka.Create;
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Clanek }
     { ------ }
     if JeToInfo and (HodnotaParametru = lzClanek) then
        Retez.PridejZnacku(CtiClanek)
     else
     { Petrklic }
     { -------- }
     if JeToInfo and (HodnotaParametru = lzPetrklic) then
        Retez.PridejZnacku(CtiPetrklic(Paklic))
     else
     { ZakladniZnacka }
     { -------------- }
     if JeToParametr and (NazevParametru = lzZakladniZnacka) then
        Retez.ZakladniZnacka := DejIntParametr
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Retez.NastavVzdalenost(mm_m(DejRealParametr))
     else
     { Strana }
     { ------ }
     if JeToParametr and (NazevParametru = lzStrana) then
        Retez.Vlevo := DejVlevo
     else
     { Barva }
     { ------ }
     if JeToParametr and (NazevParametru = lzBarva) then
        Retez.Barva :=  DejIntParametr
     else
     { Status - KreslitPrvni }
     { --------------------- }
     if JeToParametr and (NazevParametru = lzKreslitPrvni) then
     begin
        if DejBoolParametr then
           Retez.Status := Retez.Status or znKreslitPrvni
        else
           if Retez.Status and znKreslitPrvni <> 0 then
              Retez.Status := Retez.Status xor znKreslitPrvni;
     end
     else
     { Status - Harmonika }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzHarmonika) then
     begin
        if DejBoolParametr then
           Retez.Status := Retez.Status or znHarmonika
        else
           if Retez.Status and znHarmonika <> 0 then
              Retez.Status := Retez.Status xor znHarmonika;
     end
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Retez.Sila :=  DejRealParametr;
  until JeToInfo and (HodnotaParametru = lzEnd);
  if (Retez.ZakladniZnacka > -1) or Assigned(Retez.Znacky) then
     CtiRetez := Retez
  else
  begin
     result := nil;
     Retez.Free;
  end;
end;


(*
{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiKomplikaci(Paklic: PPaklicNaPetrklic): PKomplikada;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Retez: PKomplikada;
begin
  Retez := New(PKomplikada,Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Clanek }
     { ------ }
     if JeToInfo and (HodnotaParametru = lzClanek) then
        Retez^.PridejZnacku(CtiClanek)
     else
     { Petrklic }
     { -------- }
     if JeToInfo and (HodnotaParametru = lzPetrklic) then
        Retez^.PridejZnacku(CtiPetrklic(Paklic))
     else
     { ZakladniZnacka }
     { -------------- }
     if JeToParametr and (NazevParametru = lzZakladniZnacka) then
        Retez^.ZakladniZnacka := DejIntParametr
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Retez^.NastavVzdalenost(DejRealParametr)
     else
     { Strana }
     { ------ }
     if JeToParametr and (NazevParametru = lzStrana) then
        Retez^.Vlevo := DejVlevo
     else
     { Barva }
     { ------ }
     if JeToParametr and (NazevParametru = lzBarva) then
        Retez^.Barva :=  DejIntParametr
     else
     { Status - KreslitPrvni }
     { --------------------- }
     if JeToParametr and (NazevParametru = lzKreslitPrvni) then
     begin
        if DejBoolParametr then
           Retez^.Status := Retez^.Status or znKreslitPrvni
        else
           if Retez^.Status and znKreslitPrvni <> 0 then
              Retez^.Status := Retez^.Status xor znKreslitPrvni;
     end
     else
     { B1Buf }
     { ----- }
     if JeToParametr and (NazevParametru = lzB1Buffer) then
        Retez^.B1Buf := mm_m(DejRealParametr)
     else
     { B2Buf }
     { ----- }
     if JeToParametr and (NazevParametru = lzB2Buffer) then
        Retez^.B2Buf := mm_m(DejRealParametr)
     else
     { LBBuf }
     { ----- }
     if JeToParametr and (NazevParametru = lzLBBuffer) then
        Retez^.LBBuf := mm_m(DejRealParametr)
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Retez^.Sila :=  DejRealParametr;
  until JeToInfo and (HodnotaParametru = lzEnd);
  if (Retez^.ZakladniZnacka > -1) or Assigned(Retez^.Znacky) then
     CtiKomplikaci := Retez
  else
  begin
     CtiKomplikaci := nil;
     Retez^.Free;
  end;
end;





{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiNabRetez(Paklic: PPaklicNaPetrklic): PNabRetezovaZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Retez: PNabRetezovaZnacka;
begin
  Retez := New(PNabRetezovaZnacka,Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Clanek }
     { ------ }
     if JeToInfo and (HodnotaParametru = lzClanek) then
        Retez^.PridejZnacku(CtiNabClanek)
     else
     { Petrklic }
     { -------- }
     if JeToInfo and (HodnotaParametru = lzPetrklic) then
        Retez^.PridejZnacku(CtiNabPetrklic(Paklic))
     else
     { ZakladniZnacka }
     { -------------- }
     if JeToParametr and (NazevParametru = lzZakladniZnacka) then
        Retez^.ZakladniZnacka := DejIntParametr
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Retez^.NastavVzdalenost(DejRealParametr)
     else
     { Strana }
     { ------ }
     if JeToParametr and (NazevParametru = lzStrana) then
        Retez^.Vlevo := DejVlevo
     else
     { Barva }
     { ------ }
     if JeToParametr and (NazevParametru = lzBarva) then
        Retez^.Barva :=  DejIntParametr
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Retez^.NastavSiluVmm(DejRealParametr);
  until JeToInfo and (HodnotaParametru = lzEnd);
  if (Retez^.ZakladniZnacka > -1) or Assigned(Retez^.Znacky) then
     CtiNabRetez := Retez
  else
  begin
     CtiNabRetez := nil;
     Retez^.Free;
  end;
end;


*)


{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiClanek: TLinZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Clanek: TLinZnacka;
  //PomBool: boolean;
begin
  Clanek := TLinZnacka.Create;
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Velikost }
     { -------- }
     if JeToParametr and (NazevParametru = lzVelikost) then
        Clanek.NastavVelikostVm(mm_m(DejRealParametr))
     else
     { Uhel }
     { ---- }
     if JeToParametr and (NazevParametru = lzUhel) then
        Clanek.NastavUhel(DejRealParametr)
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Clanek.NastavVzdalenost(mm_m(DejRealParametr))
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Clanek.Sila :=  DejRealParametr
     else
     { Status - Centrovat }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzCentrovat) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znCentrovat
        else
           if Clanek.Status and znCentrovat <> 0 then
              Clanek.Status := Clanek.Status xor znCentrovat;
     end
     else
     { Status - Natacet }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzNatacet) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znNatacetDleLinie
        else
           if Clanek.Status and znNatacetDleLinie <> 0 then
              Clanek.Status := Clanek.Status xor znNatacetDleLinie;
     end
     else
     { Status - LamatZnacku }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzLamatZnacku) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znLamatZnacku
        else
           if Clanek.Status and znLamatZnacku <> 0 then
              Clanek.Status := Clanek.Status xor znLamatZnacku;
     end
     else
     { Status - LamatMezeru }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzLamatMezeru) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znLamatMezeru
        else
           if Clanek.Status and znLamatMezeru <> 0 then
              Clanek.Status := Clanek.Status xor znLamatMezeru;
     end
     else
     { Status - B1 }
     { ----------- }
     if JeToParametr and (NazevParametru = lzPrvniBod) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znB1
        else
           if Clanek.Status and znB1 <> 0 then
              Clanek.Status := Clanek.Status xor znB1;
     end
     else
     { Status - B2 }
     { ----------- }
     if JeToParametr and (NazevParametru = lzPosledniBod) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znB2
        else
           if Clanek.Status and znB2 <> 0 then
              Clanek.Status := Clanek.Status xor znB2;
     end
     else
     { Status - LB }
     { ----------- }
     if JeToParametr and (NazevParametru = lzLomBod) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znLB
        else
           if Clanek.Status and znLB <> 0 then
              Clanek.Status := Clanek.Status xor znLB;
     end
     else
     { Status - LB0 }
     { ------------ }
     if JeToParametr and (NazevParametru = lzLomBod0) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znLB0
        else
           if Clanek.Status and znLB0 <> 0 then
              Clanek.Status := Clanek.Status xor znLB0;
     end
     else
     { Status - LB1 }
     { ------------ }
     if JeToParametr and (NazevParametru = lzLomBod1) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znLB1
        else
           if Clanek.Status and znLB1 <> 0 then
              Clanek.Status := Clanek.Status xor znLB1;
     end
     else
     { Status - KreslitPosledni }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzKreslitPosledni) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znKreslitPosledni
        else
           if Clanek.Status and znKreslitPosledni <> 0 then
              Clanek.Status := Clanek.Status xor znKreslitPosledni;
     end
     else
     { Status - KreslitZbytek}
     { ------------------ }
     if JeToParametr and (NazevParametru = lzKreslitZbytek) then
     begin
        if DejBoolParametr then
           Clanek.Status := Clanek.Status or znKreslitZbytek
        else
           if Clanek.Status and znKreslitZbytek <> 0 then
              Clanek.Status := Clanek.Status xor znKreslitZbytek;
     end;
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (Clanek.Velikost <> 0) or (Clanek.Vzdalenost <> 0) then
  begin
     Clanek.VyrobDefinici;
     result := Clanek;
  end
  else
  begin
     result := nil;
     Clanek.Free;
  end;
end;



(*

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiNabClanek: PNabLinZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Clanek: PNabLinZnacka;
  PomBool: boolean;
  Uhel: double;
begin
  Clanek := New(PNabLinZnacka,Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Velikost }
     { -------- }
     if JeToParametr and (NazevParametru = lzVelikost) then
        Clanek^.NastavVelikostVmm(DejRealParametr)
     else
     { Uhel }
     { ---- }
     if JeToParametr and (NazevParametru = lzUhel) then
        Uhel := DejRadZeStupnu(DejRealParametr)
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Clanek^.NastavVzdalenost(DejRealParametr)
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Clanek^.NastavSiluVmm(DejRealParametr);
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (Clanek^.Velikost <> 0) or (Clanek^.Vzdalenost <> 0) then
  begin
     Clanek^.VyrobIntDefinici(Uhel);
     CtiNabClanek := Clanek;
  end
  else
  begin
     CtiNabClanek := nil;
     Clanek^.Free;
  end;
end;


*)

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiPetrklic(Paklic: TPaklicNaPetrklic): TPetrklicZn;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Petrklic: TPetrklicZn;
  PomInt: longInt;
begin
  Petrklic := TPetrklicZn.Create;
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Soubor }
     { ------ }
     if JeToParametr and (NazevParametru = lzSoubor) then
        Petrklic.Soubor := BezVnejsichMezer (HodnotaParametru)
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
     begin
        PomInt := DejLongIntParametr;
        if PomInt in [1..255] then Petrklic.Kod := PomInt
                              else Petrklic.Kod := 0;
     end
     else
     { Velikost }
     { -------- }
     if JeToParametr and (NazevParametru = lzVelikost) then
        Petrklic.NastavVelikostVm(mm_m(DejRealParametr))
     else
     { Uhel }
     { ---- }
     if JeToParametr and (NazevParametru = lzUhel) then
        Petrklic.NastavUhel(DejRealParametr)
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Petrklic.NastavVzdalenost(mm_m(DejRealParametr))
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Petrklic.Sila :=  DejRealParametr
     else
     { Status - Centrovat }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzCentrovat) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znCentrovat
        else
           if Petrklic.Status and znCentrovat <> 0 then
              Petrklic.Status := Petrklic.Status xor znCentrovat;
     end
     else
     { Status - Natacet }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzNatacet) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znNatacetDleLinie
        else
           if Petrklic.Status and znNatacetDleLinie <> 0 then
              Petrklic.Status := Petrklic.Status xor znNatacetDleLinie;
     end
     else
     { Status - LamatZnacku }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzLamatZnacku) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znLamatZnacku
        else
           if Petrklic.Status and znLamatZnacku <> 0 then
              Petrklic.Status := Petrklic.Status xor znLamatZnacku;
     end
     else
     { Status - B1 }
     { ----------- }
     if JeToParametr and (NazevParametru = lzPrvniBod) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znB1
        else
           if Petrklic.Status and znB1 <> 0 then
              Petrklic.Status := Petrklic.Status xor znB1;
     end
     else
     { Status - B2 }
     { ----------- }
     if JeToParametr and (NazevParametru = lzPosledniBod) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znB2
        else
           if Petrklic.Status and znB2 <> 0 then
              Petrklic.Status := Petrklic.Status xor znB2;
     end
     else
     { Status - LB }
     { ----------- }
     if JeToParametr and (NazevParametru = lzLomBod) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znLB
        else
           if Petrklic.Status and znLB <> 0 then
              Petrklic.Status := Petrklic.Status xor znLB;
     end
     else
     { Status - LB0 }
     { ------------ }
     if JeToParametr and (NazevParametru = lzLomBod0) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znLB0
        else
           if Petrklic.Status and znLB0 <> 0 then
              Petrklic.Status := Petrklic.Status xor znLB0;
     end
     else
     { Status - LB1 }
     { ------------ }
     if JeToParametr and (NazevParametru = lzLomBod1) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znLB1
        else
           if Petrklic.Status and znLB1 <> 0 then
              Petrklic.Status := Petrklic.Status xor znLB1;
     end
     else
     { Status - KreslitPosledni }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzKreslitPosledni) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znKreslitPosledni
        else
           if Petrklic.Status and znKreslitPosledni <> 0 then
              Petrklic.Status := Petrklic.Status xor znKreslitPosledni;
     end;
     { Status - KreslitZbytek}
     { ------------------ }
     if JeToParametr and (NazevParametru = lzKreslitZbytek) then
     begin
        if DejBoolParametr then
           Petrklic.Status := Petrklic.Status or znKreslitZbytek
        else
           if Petrklic.Status and znKreslitZbytek <> 0 then
              Petrklic.Status := Petrklic.Status xor znKreslitZbytek;
     end;

  until JeToInfo and (HodnotaParametru = lzEnd);

  if (Petrklic.Kod in [1..255]) and (Petrklic.Velikost > 0) then
  with Petrklic do
  begin
     Paklic.NovyFontJmenem(PathOfLibraries + '\' + Soubor);
     Definice    := Paklic.VratRealnouDefiniciZnaku(Kod, Velikost, Uhel);
     result := Petrklic;
  end
  else
  begin
     result := nil;
     Petrklic.Free;
  end;
end;

(*
{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiNabPetrklic(Paklic: PPaklicNaPetrklic): PNabLinZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Petrklic: PNabLinZnacka;
  PomInt: longInt;
  Kod: byte;
  Uhel: double;
{$IFDEF WINDOWS}
  Soubor : TPathStr;
{$ELSE}
  Soubor : PathStr;
{$ENDIF}
begin
  Petrklic := New(PNabLinZnacka, Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Soubor }
     { ------ }
     if JeToParametr and (NazevParametru = lzSoubor) then
{$IFDEF WINDOWS}
        StrPCopy(Soubor, BezVnejsichMezer (HodnotaParametru))
{$ELSE}
        Soubor := BezVnejsichMezer (HodnotaParametru)
{$ENDIF}
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
     begin
        PomInt := DejLongIntParametr;
        if PomInt in [1..255] then Kod := PomInt
                              else Kod := 0;
     end
     else
     { Velikost }
     { -------- }
     if JeToParametr and (NazevParametru = lzVelikost) then
        Petrklic^.NastavVelikostVmm(DejRealParametr)
     else
     { Uhel }
     { ---- }
     if JeToParametr and (NazevParametru = lzUhel) then
        Uhel := DejRadZeStupnu(DejRealParametr)
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Petrklic^.NastavVzdalenost(DejRealParametr)
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Petrklic^.NastavSiluVmm(DejRealParametr);

  until JeToInfo and (HodnotaParametru = lzEnd);

  if (Kod in [1..255]) and (Petrklic^.Velikost > 0) then
  with Petrklic^ do
  begin
     Paklic^.NovyFontJmenem(Soubor);
     Definice    := Paklic^.VratNabDefiniciZnaku(Kod, Velikost);
     Petrklic^.OtocIntDefinici(Uhel);
     CtiNabPetrklic := Petrklic;
  end
  else
  begin
     CtiNabPetrklic := nil;
     Petrklic^.Free;
  end;
end;


*)
{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.DejBoolParametr: boolean;
var
  CtiR: TCtiRadekClass;

begin
  CtiR:= TCtiRadekClass.Create(Hodnota);
  DejBoolParametr:= CtiR.DejBoolean;
  CtiR.Free
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.DejIntParametr: Integer;
var
  CtiR: TCtiRadekClass;

begin
  CtiR:= TCtiRadekClass.Create(Hodnota);
  result:= CtiR.DejInteger;
  CtiR.Free
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.DejVlevo: boolean;
var
  Znak: char;
  i: integer;

begin
  for i := 1 to Length(Hodnota) do
  begin
     Znak := UpCase(Hodnota[i]);
     if Znak in ['L','P','R'] then
     begin
        result := Znak = 'L';
        exit;
     end;
  end;
  DejVlevo := Ne;
end;

{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *             Tela metod TPaklicNaPetrklic               *
  *                                                        *
  **********************************************************
}


constructor TPaklicNaPetrklic.Create;
begin
  TOwnerElement.Create;
  JmenoSouboru := '';
end;

{-----------------------------------------------------------}

function    TPaklicNaPetrklic.NovyFontJmenem(NazevFontu: string): Boolean;
const
{ Overovaci znacka BGI fontu }
  BGICode = $08084B50;
var
  FontF: file;
  Magic: LongInt;
  //Nalezl: boolean;

begin
  NovyFontJmenem := Ano;
  OuKej    := Ano;
  if NazevFontu <> JmenoSouboru then begin
     if PFont <> NIL then begin
        FreeMem(PFont,DelkaFontu);
        AktualniFont := 0;
        PFont := NIL;
     end;
     {$I-}
     JmenoSouboru := '';
     System.Assign (FontF, NazevFontu);
     fileMode:= moOpenRONN;
     Reset  (FontF, 1);
     fileMode:= moStandard;
     {$I+}
     if IOResult <> 0 then Begin
        Chyba(errCannotOpenVectorFont,NazevFontu);
        Oukej          := Ne;
        NovyFontJmenem := Ne;
        exit;
     end
     else begin
        DelkaFontu := FileSize (FontF)-$80;
        GetMem (PFont,DelkaFontu);
        if PFont = NIL then begin
           Chyba(errOutOfMemory,'');
           OuKej          := Ne;
           NovyFontJmenem := Ne;
        end
        else begin
           { Je to vubec BGI format? }
           { ----------------------- }
           Seek (FontF,0);
           BlockRead (FontF, Magic, SizeOf(longInt));
           if Magic <> BGICode then
           { Prdlajs! }
           { -------- }
           begin
              OuKej          := Ne;
              NovyFontJmenem := Ne;
           end
           else
           begin
              Seek (FontF,$80);
              BlockRead (FontF, PFont^,DelkaFontu);
              Udaje        := PFont;
              JmenoSouboru := NazevFontu;
           end;
        end;
     end;
     Close (FontF);
  end;
end;

{-----------------------------------------------------------}
function    TPaklicNaPetrklic.VratRealnouDefiniciZnaku ( KodZnaku: byte;
                                                         Velikost,
                                                         ASmer: double): TBodZnacky;
type
   PDelky = ^TDelky;
   TDelky = Array[0..65520] of byte;
var
  Vektor       : PVektory;
  Index        : Word;
  //i            : Byte;
  xB,yB        : Byte;
  x,y,
  PomKoefVysky : double;
  PomkoefSirky : double;
  //Kladna,
  Spoj         : Boolean;
  PosledniZnak : Byte;
  BZ, Pbz      : TBodZnacky;
  Delky        : PDelky;
  Delka        : byte;
  SinSM,CosSm  : double;

  procedure Uprav;
  var
    hX : double;
    begin
       hX := {0 - }(x*CosSm - y*SinSm);
       y  := {0 - }(x*SinSm + y*CosSm);
       x  := hX;
    end;

begin
  Pbz := nil;
  result := nil;
  if OuKej then
  begin
    Vektor := VratVektory;
    Delky := Pointer(NativeInt(@Udaje^.Adresy)+(Udaje^.PocetZnaku shl 1));
    Delka := Delky^[KodZnaku-Udaje^.PrvniZnak];

    SinSm := Sin(ASmer);
    CosSm := Cos(ASmer);

    if Delka > 0 then
       PomKoefVysky  := Velikost/Delka
    else PomKoefVysky:= 1;
    PomKoefSirky := PomKoefVysky;
    PosledniZnak := Udaje^.PrvniZnak + Udaje^.PocetZnaku - 1;
    if not (KodZnaku in [Udaje^.PrvniZnak..PosledniZnak]) then exit;
    Index := KodZnaku - Udaje^.PrvniZnak;
    Index := Udaje^.Adresy[Index];
    BZ    := nil;
    While (Vektor^[Index] OR Vektor^[Index+1]) <> 0 do begin
      if BZ = nil then
      begin
         BZ := TBodZnacky.Create;
         BZ.Dalsi := nil;
         Pbz := BZ;
      end
      else
      begin
         Pbz.Dalsi := TBodZnacky.Create;
         Pbz := Pbz.Dalsi;
         Pbz.Dalsi := nil;
      end;
      //Kladna := True;
      xB := Vektor^[Index];
      yB := Vektor^[index+1];
      {$IFNDEF WIN64}
      asm
         mov    al,xB         {Kopiruji 6.bit (znamenko)}
         mov    ah,al
         and    ah,01111111B
         and    al,01000000B  {na pozici 7. ... pak se  }
         shl    al,1          {s cislem zachazi jako s  }
         or     ah,al         {normalnim ShortIntem.    }
         mov    xB,ah
      end;
      {$ELSE}
      xB := (xB and 127) or ((xB and 64) shl 1);
      {$ENDIF}
      x := PomKoefSirky * ShortInt(xB);
      {$IFNDEF WIN64}
       asm
          mov    al,yB
          mov    ah,al
          and    ah,10000000B
          mov    Spoj,ah

          mov    ah,al
          and    ah,01111111B
          and    al,01000000B
          shl    al,1
          or     ah,al
          mov    yB,ah
       end;
       {$ELSE}
       Spoj := (yB and 128) <> 0;
       yB := (yB and 127) or ((yB and 64) shl 1);
       {$ENDIF}

      y := PomKoefVysky * ShortInt(yB);
      Uprav;

      Pbz.Spoj := Spoj;
      Pbz.Sour.x := x;
      Pbz.Sour.y := y;

      Inc(Index,2);
    end;
    { Nastaveni mezery }
    { ---------------- }
    if Pbz <> nil then
    begin
       Pbz.Sour.x :=  Velikost;
       Pbz.Sour.y :=  0;
    end;
    result := BZ;
  end
  else result := nil;
end;


(*
{-----------------------------------------------------------}
function    TPaklicNaPetrklic.VratNabDefiniciZnaku ( KodZnaku: byte;
                                       Velikost: longInt): PIntBodZnacky;
type
   PDelky = ^TDelky;
   TDelky = Array[0..65520] of byte;
var
  Vektor       : PVektory;
  Index        : Word;
  i            : Byte;
  xB,yB        : Byte;
  x,y,
  PomKoefVysky : longInt;
  PomkoefSirky : longInt;
  Kladna,
  Spoj         : Boolean;
  PosledniZnak : Byte;
  BZ, Pbz      : PIntBodZnacky;
  Delky        : PDelky;
  Delka        : byte;

begin
  if OuKej then
  begin
    Vektor := VratVektory;
    Delky := Pointer(LongInt(@Udaje^.Adresy)+(Udaje^.PocetZnaku shl 1));
    Delka := Delky^[KodZnaku-Udaje^.PrvniZnak];
    Velikost := Velikost * UniverzalniDelitel;
    if Delka > 0 then
       PomKoefVysky  := Velikost div Delka
    else PomKoefVysky:= 1;
    PomKoefSirky := PomKoefVysky;
    PosledniZnak := Udaje^.PrvniZnak + Udaje^.PocetZnaku - 1;
    if not (KodZnaku in [Udaje^.PrvniZnak..PosledniZnak]) then exit;
    Index := KodZnaku - Udaje^.PrvniZnak;
    Index := Udaje^.Adresy[Index];
    BZ    := nil;
    While (Vektor^[Index] OR Vektor^[Index+1]) <> 0 do begin
      if BZ = nil then
      begin
         New(BZ);
         BZ^.Dalsi := nil;
         Pbz := BZ;
      end
      else
      begin
         New(Pbz^.Dalsi);
         Pbz := Pbz^.Dalsi;
         Pbz^.Dalsi := nil;
      end;
      Kladna := True;
      xB := Vektor^[Index];
      yB := Vektor^[index+1];
      asm
         mov    al,xB         {Kopiruji 6.bit (znamenko)}
         mov    ah,al
         and    ah,01111111B
         and    al,01000000B  {na pozici 7. ... pak se  }
         shl    al,1          {s cislem zachazi jako s  }
         or     ah,al         {normalnim ShortIntem.    }
         mov    xB,ah
      end;

      x := PomKoefVysky * ShortInt(xB);

      asm
         mov    al,yB
         mov    ah,al
         and    ah,10000000B
         mov    Spoj,ah

         mov    ah,al
         and    ah,01111111B
         and    al,01000000B
         shl    al,1
         or     ah,al
         mov    yB,ah
      end;

      y := PomKoefSirky * ShortInt(yB);

      Pbz^.Spoj := Spoj;
      Pbz^.Sour.x := x div UniverzalniDelitel;
      Pbz^.Sour.y := y div UniverzalniDelitel;

      Inc(Index,2);
    end;
    { Nastaveni mezery }
    { ---------------- }
    if Pbz <> nil then
    begin
       Pbz^.Sour.x :=  Velikost div UniverzalniDelitel;
       Pbz^.Sour.y :=  0;
    end;
    VratNabDefiniciZnaku := BZ;
  end
  else VratNabDefiniciZnaku := nil;
end;
*)
{-----------------------------------------------------------}
{
              **********************
              *   private Methods  *
              **********************
}
{-----------------------------------------------------------}


End.

