unit T_dbfexp;
{@ pedky td pro export topolskch objekt s databzemi }
interface
uses
// KERNEL
  Elements,
  Tools,
// SPACE
  //E_Space,
  e_GeometryElement,
  E_Props,
  E_Reg,
// TOPOL
  e_TopolElements,
  T_AreLin,
  E_tplreg,
  C_TLDBF;


type

  TExportDBFClass = class(TTool)
  {@ export do vnitn databze topolskch blok }
  private
    FExpBlockName: string;
    FControlDir: string;
    //FCreateFile: string;
    FCreateFromSource: boolean;
    FDBF: TTopolDBFTable;
    FObjectType: char;
    FDruh: smallInt;
    FDynamicProps: TDynamicProperties;
    FPlochyVHa: boolean;
    FBothSources: boolean;
    FRoundArea: integer;
    function GetDBFName: string;
    function OKCreateDBF(aSpaceObj: TTopoLElement): boolean;
    procedure SetIntegerField(aFieldName: string; aVal: longInt);
    procedure SetFloatField(aFieldName: string; aVal: double);
    procedure SetObjectTypeSpecificValues(aObject: TTopoLElement); virtual;
    procedure SetRoundArea(const Value: integer);
  public
    destructor Destroy; override;
    procedure DoIt; override;
  published
    property ExpBlockName: string read FExpBlockName write FExpBlockName;
    {@ pln jmno bloku do kterho se exportuje }
    property ControlDir: string read FControlDir write FControlDir;
    {@ pln jmno dcho adrese ve kterm se hledaj CRT a TDS soubory }
    property ObjectType: char read FObjectType write FObjectType;
    {@ Topolsk typ objektu B-bod, L-Linie, P-Plocha, T-Text }
    property Druh: smallInt read FDruh write FDruh;
    {@ Druh topolskho objektu }
    property DBFName: string read GetDBFName;
    {@ pln jmno vnitn databze bloku }
    property DynamicProps: TDynamicProperties read FDynamicProps write FDynamicProps;
    {@ seznam vlastnost, kter se zapisuj do vnitn databze }
    property CreateFromSource: boolean read FCreateFromSource write FCreateFromSource;
    {@ TRUE - pevzt zdrojovou databzi, FALSE - vytvet dle CRT }
    property PlochyVHa: boolean read FPlochyVHa write FPlochyVHa;
    {@ TRUE - hodnotu DBF poloky PLOCHA ped zpisem dlit 10000,
       FALSE - zapsat pmo }
    property RoundArea: integer read FRoundArea write SetRoundArea;
    property BothSources: boolean read FBothSources write FBothSources;
  end;


{
<P><A NAME="TExportTopolElements"></A></P>
}
{
<P><A HREF="Tvorba pedpis OIL.htm"><B><FONT FACE="Arial" SIZE=2>Vytven pedpisu</B>
</FONT></A><B><FONT FACE="Arial" SIZE=2> | Souvisejc strnka 2 | Souvisejc strnka 3</P>
}

TOnExportObjectEvent = procedure (aSpaceEntity: TGeometryElement;
                                  var aDynProps: TDynamicProperties) of Object;

TExportTopolElements = class(TExportElements)
{ pomocn pedek vech topol export, doplujc obecn export DBF vlastnostmi
  BlockName ... jmno bloku pro vstup
  ControlDir ... jmno adrese ve kterm se m hledat CRT soubor
  ExportDBF ... zda se m exportovat databze }

private
  FOnExportObject: TOnExportObjectEvent;

  FExportDBF: boolean;
  FControlDir: string;
  FCreateFromSource: boolean;
  FJedenDruh: integer;
  FRegisterObject: boolean;
  FRegister: TTopolBlockRegister;
  FDynamicProps: TDynamicProperties;
  FPlochyVHa: boolean;
  FBothSources: boolean;
    FRoundArea: integer;
  function GetRegister: TTopolBlockRegister;
  procedure SetBlockName(aName: string);
  function GetDynamicProps: TDynamicProperties;
    procedure SetRoundArea(const Value: integer);
protected
  FBlockName: string;
public
  procedure UseExportDBFClass(aObject: TTopoLElement); virtual;
  procedure ExportObject(aObject: TTopoLElement); virtual;
  procedure DoBeforeExport(aObject: TTopoLElement); virtual;

  property Register: TTopolBlockRegister read GetRegister;
published
  property BlockName: string read FBlockName write SetBlockName;
  property ControlDir: string read FControlDir write FControlDir;
  property ExportDBF: boolean read FExportDBF write FExportDBF;
  property CreateFromSource: boolean read FCreateFromSource write FCreateFromSource;
  property JedenDruh: integer read FJedenDruh write FJedenDruh;
  property RegisterObject: boolean read FRegisterObject write FRegisterObject;
  property DynamicProps: TDynamicProperties read GetDynamicProps write FDynamicProps;
  property PlochyVHa: boolean read FPlochyVHa write FPlochyVHa;
  property RoundArea: integer read FRoundArea write SetRoundArea;
  property BothSources: boolean read FBothSources write FBothSources;

  property OnExportObject: TOnExportObjectEvent read FOnExportObject write FOnExportObject;
end;


{ nstroj pro prost vytvoen nzvu bloku sloenm
  ze jmna adrese a jmna bloku }
TExpBlockName = class(TNameElement)
private
  FBlockDir: string;
  FBlockName: string;
  procedure SetBlockDir(aName: string);
  function GetFullBlockName: string;
published
  property BlockDir: string read FBlockDir write SetBlockDir;
  property BlockName: string read FBlockName write FBlockName;
  property FullBlockName: string read GetFullBlockName;
end;

(*
TExportElementsToText = class(TExportElements)
private
  FFileName: string;
public
  procedure DoIt; override;
published
  property FileName: string read FFileName write FFileName;
end;

*)


implementation
uses
  Controls, Forms, Dialogs, IniFiles, SysUtils, Graphics, FileCtrl,
  u_Math,
  WinProcs,
  DB,
// KERNEL
  e_Class,
  U_String,
  Worlds,
  e_Geometry;
// DIALOG
  //F_Gauge;
  {!PubUnits,}
// TOPOL
  //E_TplPrp;



                         { TExportDBFClass }



destructor TExportDBFClass.Destroy;
begin
  if FDBF <> nil then
  begin
    FDBF.Table.Close;
    FDBF.Free;
  end;
  inherited Destroy
end;

function TExportDBFClass.GetDBFName: string;
begin
  result:= AddBackSlash(FExpBlockName) + ObjectType + IntToStrZ(Druh, 4, true) + '.DBF'
end;


function TExportDBFClass.OKCreateDBF(aSpaceObj: TTopoLElement): boolean;
begin
   result := false;
   if not CreateFromSource then
   begin
      FDBF.CreateFileName:= AddBackSlash(FControlDir) + ObjectType + IntToStrZ(Druh, 4, true) + '.CRT';
      result:= FDBF.OKCreateTableFromCreateFile(DBFName);
   end;
   { vytven dle zdrojov DBF }
   if not result or CreateFromSource then
   begin
      //FDBF.CreateFileName:= aSpaceObj.DBFName;
      FDBF.CreateFileName:= aSpaceObj.SourceTopolElement.DBFName;
      result:= FDBF.OKCreateTableFromCreateFile(DBFName);
   end
end;


procedure TExportDBFClass.SetIntegerField(aFieldName: string; aVal: longInt);
var
  aField: TField;
begin
  try
    aField:= FDBF.Table.FindField(aFieldName);
    if aField = nil then exit;
    aField.AsInteger:= aVal;
  except
  end;
end;


procedure TExportDBFClass.SetFloatField(aFieldName: string; aVal: double);
var
  aField: TField;
begin
  try
    aField:= FDBF.Table.FindField(aFieldName);
    if aField = nil then exit;
    aField.AsFloat:= aVal;
  except
  end;
end;


procedure TExportDBFClass.SetObjectTypeSpecificValues(aObject: TTopoLElement);
{
var
  aProps: TTopolProps;
}
var
  RAre: double;
begin
  if FDBF = nil then exit;

  if not (TElement(aObject) is TTopolElement) then exit;
  try
 { nejdve CISLO, DRUH, DISP_BARVA, DISP_ZNAC }
    SetIntegerField('CISLO', aObject.ID);
    SetIntegerField('DRUH', aObject.Druh);
    SetIntegerField('DISP_BARVA', aObject.Barva);
    SetIntegerField('DISP_ZNAC', aObject.Znacka);
    SetFloatField('X', -aObject.Geometry.Y);
    SetFloatField('Y', -aObject.Geometry.X);

    if aObject.Geometry is TPolyElement then
    begin
      SetFloatField('DELKA', TPolyElement(aObject.Geometry).Length);
    end;

    if aObject is TTopoLAreaElement then
    begin
      if PlochyVHa then
         RAre := TAreaElement(aObject.Geometry).Area/10000
      else
         RAre := TAreaElement(aObject.Geometry).Area;
      if RoundArea <> 0 then
         RAre := Simple_RoundTo(RAre,RoundArea);
      SetFloatField('PLOCHA', RAre);
    end
  except
  end;
end;



procedure TExportDBFClass.DoIt;
var
  AObj: TTopoLElement;
  OKCreate: boolean;
  SourceTable: TTopolDBFTable;
begin
  OKCreate := false;
  if not (input is TTopoLElement) then exit;
  AObj:= TTopoLElement(input);
  if AObj.Geometry = nil then exit;

  if FDBF = nil then
  begin
    FDBF:= TTopolDBFTable.CreateWithOwner(Application);
    if FDBF = nil then exit;
    OKCreate:= OKCreateDBF(aObj);

    if OKCreate then
       FDBF.TableName:= DBFName
    else
    begin
      FDBF.Free;
      FDBF:= nil;
    end
  end
  else
  begin
    if (FDBF.TableName <> DBFName) then
    begin
      FDBF.Table.Close;

      if not FileExists(DBFName) then
      begin
        OKCreate:= OKCreateDBF(aObj);
      end;

      if OKCreate then
         FDBF.TableName:= DBFName
      else
      begin
        FDBF.Free;
        FDBF:= nil
      end
    end;
  end;

  if FDBF = nil then exit;

  FDBF.Table.Open;
  FDBF.Table.Append;

  if (DynamicProps.Count > 0) and not BothSources then
  begin
    FDBF.TransferFieldsFromDynamicProperties(aObj, DynamicProps)
  end
  else
  begin
    if (DynamicProps.Count > 0) and BothSources then
    begin
      FDBF.TransferFieldsFromDynamicProperties(aObj, DynamicProps)
    end;

    if not CreateFromSource then
       FDBF.TransferScript:= AddBackSlash(FControlDir) + ObjectType + IntToStrZ(Druh, 4, true) + '.TDS';

    SourceTable:= TTopolDBFTable(aObj.SourceTopolElement.DBFTable);

    if fileExists(aObj.SourceTopolElement.DBFName) then
    begin
      if SourceTable.SetForNameCislo(aObj.SourceTopolElement.DBFName, aObj.SourceTopolElement.OrigNo) then
         FDBF.TransferFieldsFromDBF(SourceTable)
      else
         ShowMessage('Nen nalezen zznam c.' + IntToStr(aObj.OrigNo) + #13 + #10 +
                     aObj.DBFName)
    end
  end;

  FDBF.TransferFieldsFromProperties(aObj);

  SetObjectTypeSpecificValues(aObj);

{  penos dalch vlastnost }
  FDBF.Table.Post;
end;




procedure TExportDBFClass.SetRoundArea(const Value: integer);
begin
  FRoundArea := Value;
end;

{ TExportTopolElements }

function TExportTopolElements.GetDynamicProps: TDynamicProperties;
begin
  if FDynamicProps = nil then
  begin
    FDynamicProps:= TDynamicProperties.Create;
    AddElement(FDynamicProps);
  end;
  result:= FDynamicProps
end;

procedure TExportTopolElements.UseExportDBFClass(aObject: TTopoLElement);
var
  EDC: TExportDBFClass;
  aDruh: integer;
begin
  if not (TElement(aObject) is TTopoLElement) then exit;

  if JedenDruh > 0 then
    aDruh:= JedenDruh
  else
    aDruh:= aObject.Druh;

  EDC:= TExportDBFClass
        (FindElementName(aObject.ObjectType + IntToStrZ(aDruh, 4, true)));

  if EDC = nil then
  begin
    EDC:= TExportDBFClass.Create;
    EDC.Name:= aObject.ObjectType + IntToStrZ(aDruh, 4, true);
    EDC.CreateFromSource:= CreateFromSource;
    AddElement(EDC);
  end;

  if EDC = nil then exit;

  EDC.Druh:= aDruh;
  EDC.ObjectType:= aObject.PhysicalObjectType;
  EDC.DynamicProps:= DynamicProps;
  EDC.ControlDir:= ControlDir;
  EDC.ExpBlockName:= BlockName;
  EDC.PlochyVHa:= PlochyVHa;
  EDC.RoundArea := RoundArea;
  EDC.BothSources:= BothSources;
  EDC.Input:= aObject;
  EDC.DoIt;
end;




procedure TExportTopolElements.SetBlockName(aName: string);
begin
  FBlockName:= aName;
end;




function TExportTopolElements.GetRegister: TTopolBlockRegister;
var
  TR: TRegister;
begin
  result:= FRegister;
  if result <> nil then exit;

  TR:= TRegister(World.ObjectModel.FindElementClass(TRegister));
  if TR = nil then
  begin
    TR:= TRegister.Create;
    World.ObjectModel.AddElement(TR);
  end;

  if TR = nil then exit;
  FRegister:= TTopolBlockRegister
              (TR.GetAndCreateBlockRegister(BlockName, 'TTopolBlockRegister'));
  result:= FRegister;
end;





procedure TExportTopolElements.ExportObject(aObject: TTopoLElement);
begin
  try
    if not ExportDBF then exit;

    if not (aObject is TTopoLElement) then exit;
    UseExportDBFClass(aObject);

    if RegisterObject then
       Register.RegisterObject(aObject);
  except
    on E:Exception do
    begin { Dialogs, Controls }
       ShowMessage('Chyba exportu objektu' + aObject.Name + ' ' + IntToStr(aObject.Id)
       + #13 + #10 + FloatToStr(aObject.Geometry.X) + ' ' + FloatToStr(aObject.Geometry.Y)
       + #13 + #10 +
                    E.Message);
       Screen.Cursor:= crDefault;
    end
  end
end;



procedure TExportTopolElements.DoBeforeExport(aObject: TTopoLElement);
begin
  try
    if assigned(OnExportObject) then
       OnExportObject(aObject, FDynamicProps);
  except
  end;
end;


procedure TExportTopolElements.SetRoundArea(const Value: integer);
begin
  FRoundArea := Value;
end;

{ TExpBlockName }

procedure TExpBlockName.SetBlockDir(aName: string);
begin
  FBlockDir:= aName
end;

function TExpBlockName.GetFullBlockName: string;
begin
  result:= AddBackSlash(FBlockDir) + BlockName + '.BLK'
end;

(*
                  { TExportElementsToText }


procedure TExportElementsToText.DoIt;
var
  I, J: longInt;
  aSpace: TSpaceEntity;
  F: system.text;
  F_GaugeForm: TF_GaugeForm;

begin
  if not (Input is TSpaceEntity) then exit;
  if FileName = '' then exit;
  system.assign(F, FileName);
  {$I-}
  system.rewrite (F);
  {$I+}
  if ioResult <> 0 then exit;

  aSpace:= TSpaceEntity(Input);

  Application.CreateForm(TF_GaugeForm, F_GaugeForm);
  F_GaugeForm.MaxValue:= aSpace.Count;
  F_GaugeForm.Caption:= 'Export bod do ' + FileName;
  F_GaugeForm.Show;

  for I:= 0 to aSpace.Count-1 do
  begin
    if F_gaugeForm.DoCloseQuery and
       F_gaugeForm.RealyClose('Peruit export bod ?') then
    break;
    F_GaugeForm.Progress:= I;
    Application.ProcessMessages;

    aSpace.Elements[I].ID:= I;

    ExpProps.Element:= aSpace.Elements[I]; { nastaven objektu do TPropery}

    writeln(F, ExpProps.Catenation)
  end;
  F_GaugeForm.Free;

  system.close(F);
end;

*)

INITIALIZATION
  RegisterOrbisClass(TExpBlockName);
  //RegisterOrbisClass(TExportElementsToText);
END.

