unit t_Transformation;

interface
uses
   e_Transformation,
   Tools;

type

TTransformation = class(TTool)
private
    FNelzeVypocistKlic: boolean;
    FDimense: byte;
    FZ: extended;
    FY: extended;
    FX: extended;
    FDostatecnyPocetDvojic: integer;
    FPocetIdentBodu: integer;
    FTransformationTableName: string;
    FTransformationName: string;
    FTransformationTable: TTransformationTable;
    FTr_Y: extended;
    FTr_X: extended;
    FTr_Z: extended;
    FInvert: boolean;
    function GetKlicVypocten: boolean;
    procedure SetDimense(const Value: byte);
    procedure SetDostatecnyPocetDvojic(const Value: integer);
    procedure SetNelzeVypocistKlic(const Value: boolean);
    procedure SetPocetIdentBodu(const Value: integer);
    procedure SetTransformationName(const Value: string);
    procedure SetTransformationTable(const Value: TTransformationTable);
    procedure SetTransformationTableName(const Value: string);
    procedure SetX(const Value: extended);
    procedure SetY(const Value: extended);
    procedure SetZ(const Value: extended);
    procedure SetTr_X(const Value: extended);
    procedure SetTr_Y(const Value: extended);
    procedure SetTr_Z(const Value: extended);
    procedure SetInvert(const Value: boolean);
    procedure SetEvalDiff(const Value: boolean);
protected
    Sta, Nov: array of TTrBod;
    Vaha: array of extended;
    ZdrojStred, CilStred: TTrBod;
    Klic: TKlic;
    InvKlic: TKlic;
public
   procedure DoIt; override;
   procedure Initialize; virtual;
   procedure TransformXY(const srcX,SrcY: extended; var DestX,DestY  : extended); virtual;
   procedure InvTransformXY(const srcX,SrcY: extended; var DestX,DestY  : extended); virtual;
   procedure TransformXYZ(const srcX,SrcY,SrcZ: extended; var DestX,DestY,DestZ  : extended); virtual;
   procedure InvTransformXYZ(const srcX,SrcY,SrcZ: extended; var DestX,DestY,DestZ  : extended); virtual;
   procedure EvaluateDifferences;
   property PocetIdentBodu: integer read FPocetIdentBodu write SetPocetIdentBodu;
   property DostatecnyPocetDvojic: integer read FDostatecnyPocetDvojic write SetDostatecnyPocetDvojic;
   property KlicVypocten: boolean read GetKlicVypocten;
   property NelzeVypocistKlic: boolean read FNelzeVypocistKlic write SetNelzeVypocistKlic;
published
   property X: extended read FX write SetX stored false;
   property Y: extended read FY write SetY stored false;
   property Z: extended read FZ write SetZ stored false;
   property Tr_X: extended read FTr_X write SetTr_X stored false;
   property Tr_Y: extended read FTr_Y write SetTr_Y stored false;
   property Tr_Z: extended read FTr_Z write SetTr_Z stored false;
   property TransformationName: string read FTransformationName write SetTransformationName;
   property TransformationTableName: string read FTransformationTableName write SetTransformationTableName;
   property TransformationTable: TTransformationTable read FTransformationTable write SetTransformationTable;
   property Invert: boolean read FInvert write SetInvert stored false;
   property Dimense: byte read FDimense write SetDimense;
   property EvalDiff: boolean write SetEvalDiff stored false;
end;

TTransformation2D = class(TTransformation)
public
   procedure Initialize; override;
end;

TTransformation_Afinni2D = class(TTransformation2D)
public
   procedure Initialize; override;
   procedure TransformXY(const srcX,SrcY: extended; var DestX,DestY  : extended); override;
   procedure InvTransformXY(const srcX,SrcY: extended; var DestX,DestY  : extended); override;
   procedure TransformXYZ(const srcX,SrcY,SrcZ: extended; var DestX,DestY,DestZ  : extended); override;
   procedure InvTransformXYZ(const srcX,SrcY,SrcZ: extended; var DestX,DestY,DestZ  : extended); override;
end;

TTransformation_Kolinearni2D = class(TTransformation2D)
public
   procedure Initialize; override;
   procedure TransformXY(const srcX,SrcY: extended; var DestX,DestY  : extended); override;
   procedure InvTransformXY(const srcX,SrcY: extended; var DestX,DestY  : extended); override;
   procedure TransformXYZ(const srcX,SrcY,SrcZ: extended; var DestX,DestY,DestZ  : extended); override;
   procedure InvTransformXYZ(const srcX,SrcY,SrcZ: extended; var DestX,DestY,DestZ  : extended); override;
end;

function ResSoustRovnic(var V: TExtVector; var PM: TExtVector;
                        var VPS: TExtVector;var NEZ: TKlic;
                            PocetRovnic, PocetNezn, Dimenze: integer) : boolean;

implementation
uses
   e_Class;

function InvMat(var a: TextVector; n: integer) : boolean;
const
  MaxRadMatice=21;

var
  biga, hold: extended;
  i, j, k, nk, kk, iz, ij, ki, ji, jp, jk, ik, kj, jq, jr: integer;
  l, m: array [0..MaxRadMatice] of integer;
  ok: boolean;

begin
  if n <= MaxRadMatice then
  begin
    ok := true;
    nk := -n ;

    { hledani nejvetsiho prvku }

    k := -1;
    repeat
      inc(k);

      nk   := nk + n;
      l[k] := k;
      m[k] := k;
      kk   := nk + k;
      biga := a[kk];

      for j := k to n-1 do
      begin
        iz := n * j;

        for i := k to n-1 do
        begin
          ij := iz + i;

          if abs(biga) < abs(a[ij])  then
          begin
            biga := a[ij];
            l[k] := i;
            m[k] := j
          end;
        end;
      end;

      { zamena radku }

      j := l[k];

      if j > k  then
      begin
        ki := k - n;

        for i := 0 to n-1 do
        begin
          ki    := ki + n;
          hold  := -a[ki];
          ji    := ki - k + j;
          a[ki] := a[ji];
          a[ji] := hold
        end;
      end;

      { zamena sloupcu }

      i := m[k];

      if i > k then
      begin
        jp := n * i;

        for j := 0 to n-1 do
        begin
          jk    := nk + j;
          ji    := jp + j;
          hold  := -a[jk];
          a[jk] := a[ji];
          a[ji] := hold
        end;
      end;

      { deleni sloupce zapornym pivotem (hodnota pivotu je v biga) }

      if biga <> 0.0 then
      begin
        for i := 0 to n-1 do
          if i <> k then
          begin
            ik    := nk + i;
            a[ik] := -a[ik] / biga
          end;
        { redukce matice }

        for i := 0 to n-1 do
        begin
          ik   := nk + i;
          hold := a[ik];
          ij   := i-n ;

          for j := 0 to n-1 do
          begin
            ij := ij + n;

            if (i <> k) and (j <> k) then
            begin
              kj    := ij - i + k;
              a[ij] := hold * a[kj] + a[ij]
            end;
          end;
        end;

        { deleni radku pivotem }

        kj := k - n;

        for j := 0 to n-1 do
        begin
          kj := kj + n;

          if j <> k then
            a[kj] := a[kj] / biga
        end;

        { nahrada pivotu prevracenou hodnotou }

        a[kk] := 1.0 / biga
      end
      else
        ok := false;
    until ((k = n-1) or (not ok));

    if ok then begin
      { konecna vymena radku a sloupcu }

      //k := pred(n);
      k := n-2;

      while k >= 0 do
      begin
        i := l[k];

        if i > k then
        begin
          jq := n * k;
          jr := n * i;

          for j := 0 to n-1 do
          begin
            jk    := jq + j;
            hold  := a[jk];
            ji    := jr + j;
            a[jk] := -a[ji];
            a[ji] := hold
          end;
        end;

        j := m[k];

        if j > k then
        begin
          ki := k - n;

          for i := 0 to n-1 do
          begin
            ki    := ki + n;
            hold  := a[ki];
            ji    := ki - k + j;
            a[ki] := -a[ji];
            a[ji] := hold

          end;
        end;
        dec(k);
      end;
    end;
  end
  else
    ok := false;

  result := ok;
end;


function ResSoustRovnic(var V: TExtVector; var PM: TExtVector;
                        var VPS: TExtVector;var NEZ: TKlic;
                            PocetRovnic, PocetNezn, Dimenze: integer) : boolean;
const
  MaxDimense=21;
var
   MA: TExtVector;
   PS: array[0..MaxDimense] of extended;
   i, j, k, l: integer;
   INezn: integer;
   NelzeResit:boolean;
begin
   result := false;
   NelzeResit:=false;

   if PocetNezn > MaxDimense then
      NelzeResit:=true
   else
   begin
      SetLength(MA,400);
      INezn := PocetNezn - 1;
      for i := 0 to INezn do
      begin
         for j := 0 to INezn do
         begin
            MA[PocetNezn * i + j] := 0;
            for k := 0 to PocetRovnic-1 do
                MA[PocetNezn * i + j] := MA[PocetNezn * i + j] +
                                         V[k div Dimenze]  *
                                         PM[PocetNezn * k + i]   *
                                         PM[PocetNezn * k + j];
         end;
      end;

      for i := 0 to INezn do
      begin
         PS[i] := 0;
         for k := 0 to PocetRovnic-1 do
            PS[i] := PS[i] + V[k div Dimenze] * PM[PocetNezn * k + i] * VPS[k];
      end;

      if InvMat(MA, PocetNezn) then
      begin
         for i := 0 to INezn do
         begin
            NEZ[i] := 0;

            for k := 0 to INezn do
               NEZ[i] := NEZ[i] + MA[PocetNezn * i + k] * PS[k]
        end;
      end
      else
         NelzeResit:=true;

      result := not NelzeResit;
   end;
end;

{ TTransformation }

procedure TTransformation.DoIt;
begin
   //!!!!!!!!!Kontrola
   if Dimense = 3 then
   begin
      if Invert then
         InvTransformXYZ(X,Y,Z,FTr_X,FTr_Y,FTr_Z)
      else
         TransformXYZ(X,Y,Z,FTr_X,FTr_Y,FTr_Z);
   end
   else
   begin
      if Invert then
         InvTransformXY(X,Y,FTr_X,FTr_Y)
      else
         TransformXY(X,Y,FTr_X,FTr_Y);
   end;
end;

procedure TTransformation.EvaluateDifferences;
var
  i, N: longInt;
  ZX,ZY,ZZ,CX,CY,CZ: extended;
  Suma1, Suma2, Suma3: extended;
  IDB: TIDBod;
begin
   if TransformationTable = nil then exit;
   Suma1 := 0;
   Suma2 := 0;
   Suma3 := 0;
   N := 0;
   for i := 0 to TransformationTable.LastIndex do
   begin
      IDB := TransformationTable.IDBod[i];
      ZX := IDB.zX;
      ZY := IDB.zY;
      ZZ := IDB.zZ;
      TransformXYZ(ZX,ZY,ZZ,CX,CY,CZ);
      IDB.OdchX := IDB.cX - CX;
      IDB.OdchY := IDB.cY - CY;
      if Dimense = 3 then
         IDB.OdchZ := IDB.cZ - CZ;
      if IDB.Active then
      begin
         inc(N);
         Suma1 := Suma1 + sqr(IDB.OdchX);
         Suma2 := Suma2 + sqr(IDB.OdchY);
         if Dimense = 3 then
            Suma3 := Suma3 + sqr(IDB.OdchZ);
      end;
   end;
   dec(N);
   if N > 0 then
   begin
      TransformationTable.mX := sqrt(Suma1/N);
      TransformationTable.mY := sqrt(Suma2/N);
      if Dimense = 3 then
         TransformationTable.mZ := sqrt(Suma3/N);
      if Dimense = 3 then
         TransformationTable.mP := sqrt(sqr(TransformationTable.mY) +sqr(TransformationTable.mX) + sqr(TransformationTable.mZ))
      else
         TransformationTable.mP := sqrt(sqr(TransformationTable.mY) +sqr(TransformationTable.mX));
   end
   else
   begin
      TransformationTable.mY := 0;
      TransformationTable.mX := 0;
      TransformationTable.mZ := 0;
      TransformationTable.mP := 0;
   end;
end;

function TTransformation.GetKlicVypocten: boolean;
begin
   result  := not NelzeVypocistKlic;
end;

procedure TTransformation.Initialize;
var
   i, j:integer;
begin
   Error := 0;
   if TransformationTable = nil then
   begin
      Error := 1;
      exit;
   end;
   NelzeVypocistKlic := false;
   ZdrojStred.X      := 0;
   CilStred.X        := 0;
   ZdrojStred.Y      := 0;
   CilStred.Y        := 0;
   PocetIdentBodu    := TransformationTable.ActivePointsCount;
   if Dimense = 3 then
   begin
     ZdrojStred.Z := 0;
     CilStred.Z   := 0;
   end;
   SetLength(Vaha,PocetIdentBodu);
   SetLength(Sta,PocetIdentBodu);
   SetLength(Nov,PocetIdentBodu);
   j := 0;
   for i := 0 to TransformationTable.LastIndex do
   begin
      if not TransformationTable.IDBod[i].Active then continue;
      Vaha[j]  := TransformationTable.IDBod[i].Vaha;
      Sta[j].X := TransformationTable.IDBod[i].zX;
      Sta[j].Y := TransformationTable.IDBod[i].zY;;
      if Dimense=3 then
         Sta[j].Z := TransformationTable.IDBod[i].zZ;;

      Nov[j].X := TransformationTable.IDBod[i].cX;
      Nov[j].Y := TransformationTable.IDBod[i].cY;
      if Dimense=3 then
         Nov[j].Z := TransformationTable.IDBod[i].cZ;
      Inc(j);
   end;

   for i:=0 to PocetIdentBodu-1 do
   begin
      ZdrojStred.X := ZdrojStred.X + Sta[i].X;
      CilStred.X   := CilStred.X   + Nov[i].X;

      ZdrojStred.Y := ZdrojStred.Y + Sta[i].Y;
      CilStred.Y   := CilStred.Y   + Nov[i].Y;

      if Dimense=3 then
      begin
         ZdrojStred.Z := ZdrojStred.Z + Sta[i].Z;
         CilStred.Z   := CilStred.Z   + Nov[i].Z;
      end;
   end;

   ZdrojStred.X := ZdrojStred.X / PocetIdentBodu;
   CilStred.X   := CilStred.X   / PocetIdentBodu;

   ZdrojStred.Y := ZdrojStred.Y / PocetIdentBodu;
   CilStred.Y   := CilStred.Y   / PocetIdentBodu;

   if Dimense=3 then
   begin
      ZdrojStred.Z := ZdrojStred.Z / PocetIdentBodu;
      CilStred.Z   := CilStred.Z   / PocetIdentBodu;
   end;

   for i:=0 to PocetIdentBodu-1 do
   begin
      Sta[i].X := Sta[i].X - ZdrojStred.X;
      Nov[i].X := Nov[i].X - CilStred.X;
      Sta[i].Y := Sta[i].Y - ZdrojStred.Y;
      Nov[i].Y := Nov[i].Y - CilStred.Y;

      if Dimense=3 then
      begin
         Sta[i].Z := Sta[i].Z - ZdrojStred.Z;
         Nov[i].Z := Nov[i].Z - CilStred.Z;
      end;
   end;
end;


procedure TTransformation.InvTransformXY(const srcX, SrcY: extended;var DestX, DestY: extended);
begin

end;

procedure TTransformation.InvTransformXYZ(const srcX, SrcY, SrcZ: extended;
  var DestX, DestY, DestZ: extended);
begin

end;

procedure TTransformation.SetDimense(const Value: byte);
begin
   FDimense := Value;
end;

procedure TTransformation.SetDostatecnyPocetDvojic(const Value: integer);
begin
  FDostatecnyPocetDvojic := Value;
end;

procedure TTransformation.SetEvalDiff(const Value: boolean);
begin
   EvaluateDifferences;
end;

procedure TTransformation.SetInvert(const Value: boolean);
begin
  FInvert := Value;
end;

procedure TTransformation.SetNelzeVypocistKlic(const Value: boolean);
begin
  FNelzeVypocistKlic := Value;
end;

procedure TTransformation.SetPocetIdentBodu(const Value: integer);
begin
  FPocetIdentBodu := Value;
end;

procedure TTransformation.SetTransformationName(const Value: string);
begin
  FTransformationName := Value;
end;

procedure TTransformation.SetTransformationTable(const Value: TTransformationTable);
var
   SameElement: boolean;
begin
   SameElement := false;
   if FTransformationTable <> nil then
   begin
      if Value <> FTransformationTable then
      begin
         FreeElement(FTransformationTable);
         FTransformationTable := nil;
      end
      else
         SameElement := true;
   end;
   if not SameElement then
   begin
      FTransformationTable := Value;
      AddElement(Value);
   end;
   PocetIdentBodu := 0;
   if FTransformationTable = nil then exit;
   PocetIdentBodu := FTransformationTable.Count;
   Initialize;
end;

procedure TTransformation.SetTransformationTableName(const Value: string);
begin
  FTransformationTableName := Value;
end;

procedure TTransformation.SetTr_X(const Value: extended);
begin
   FTr_X := Value;
end;

procedure TTransformation.SetTr_Y(const Value: extended);
begin
   FTr_Y := Value;
end;

procedure TTransformation.SetTr_Z(const Value: extended);
begin
   FTr_Z := Value;
end;

procedure TTransformation.SetX(const Value: extended);
begin
   FX := Value;
end;

procedure TTransformation.SetY(const Value: extended);
begin
   FY := Value;
end;

procedure TTransformation.SetZ(const Value: extended);
begin
  FZ := Value;
end;


procedure TTransformation.TransformXY(const srcX, SrcY: extended; var DestX, DestY: extended);
begin

end;

procedure TTransformation.TransformXYZ(const srcX, SrcY, SrcZ: extended; var DestX, DestY, DestZ: extended);
begin

end;

{ TTransformation2D }

procedure TTransformation2D.Initialize;
begin
   Dimense := 2;
   inherited;
end;


{ TTransformation_Afinni2D }

procedure TTransformation_Afinni2D.Initialize;
const
  PocetParametru = 6;

var
   V: TExtVector;
   PM: TExtVector;
   VPS: TExtVector;
   i, j, k, PocetRovnic: integer;
begin
   DostatecnyPocetDvojic := 3;

   if PocetIdentBodu < DostatecnyPocetDvojic then
   begin
      NelzeVypocistKlic := true;
      Error        := 3;
   end
   else
   begin
      inherited;
      SetLength(V, PocetIdentBodu);
      SetLength(PM, Dimense * PocetIdentBodu * PocetParametru);
      SetLength(VPS, Dimense * PocetIdentBodu);

      j := -1;
      k := -1;

      for i := 0 to PocetIdentBodu-1 do
      begin
         V[i]     := Vaha[i];
         inc(j);
         PM[j]    := Sta[i].X;
         inc(j);
         PM[j]    := Sta[i].Y;
         inc(j);
         PM[j]    := 1;
         inc(j);
         PM[j]    := 0;
         inc(j);
         PM[j]    := 0;
         inc(j);
         PM[j]    := 0;

         inc(k);
         VPS[k]   := Nov[i].X;

         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := Sta[i].X;
         inc(j);
         PM[j] := Sta[i].Y;
         inc(j);
         PM[j] := 1;

         inc(k);
         VPS[k]   := Nov[i].Y;
      end;

      PocetRovnic := Dimense * PocetIdentBodu;

      if not ResSoustRovnic(V, PM, VPS, Klic, PocetRovnic, PocetParametru, Dimense) then
      begin
         NelzeVypocistKlic := true;
         Error := 2;
      end;

      if not NelzeVypocistKlic then
      begin
         j := -1;
         k := -1;

         for i := 0 to PocetIdentBodu-1 do
         begin
            inc(j);
            PM[j] := Nov[i].X;
            inc(j);
            PM[j] := Nov[i].Y;
            inc(j);
            PM[j] := 1;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;

            inc(k);
            VPS[k]   := Sta[i].X;

            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := Nov[i].X;
            inc(j);
            PM[j] := Nov[i].Y;
            inc(j);
            PM[j] := 1;

            inc(k);
            VPS[k]   := Sta[i].Y;
         end;

         PocetRovnic := Dimense * PocetIdentBodu;

         if not ResSoustRovnic(V, PM, VPS, InvKlic, PocetRovnic, PocetParametru, Dimense) then
         begin
            NelzeVypocistKlic := true;
            Error := 2;
         end;
      end;
  end;
end;

procedure TTransformation_Afinni2D.InvTransformXY(const srcX,SrcY: extended; var DestX, DestY: extended);
var
   H: TTrBod;
begin
   H.X      := SrcX - CilStred.X;
   H.Y      := SrcY - CilStred.Y;
   DestX := H.X * InvKlic[0] + H.Y * InvKlic[1] + InvKlic[2] + ZdrojStred.X;
   DestY := H.X * InvKlic[3] + H.Y * InvKlic[4] + InvKlic[5] + ZdrojStred.Y;
end;

procedure TTransformation_Afinni2D.InvTransformXYZ(const srcX, SrcY,SrcZ: extended; var DestX, DestY, DestZ: extended);
begin
   DestZ := 0;
   InvTransformXY(srcX,srcY,destX,destY);
end;

procedure TTransformation_Afinni2D.TransformXY(const srcX, SrcY: extended;var DestX, DestY: extended);
var
   H: TTrBod;
begin
   H.X      := SrcX - ZdrojStred.X;
   H.Y      := SrcY - ZdrojStred.Y;
   DestX := H.X * Klic[0] + H.Y * Klic[1] + Klic[2] + CilStred.X;
   DestY := H.X * Klic[3] + H.Y * Klic[4] + Klic[5] + CilStred.Y;
end;

procedure TTransformation_Afinni2D.TransformXYZ(const srcX, SrcY,SrcZ: extended; var DestX, DestY, DestZ: extended);
begin
   DestZ := 0;
   TransformXY(srcX, SrcY, DestX, DestY);
end;

{ TTransformation_Kolinearni2D }

procedure TTransformation_Kolinearni2D.Initialize;
const
   PocetParametru=8;
var
   V: TExtVector;
   PM: TExtVector;
   VPS: TExtVector;
   i, j, k, PocetRovnic:integer;
begin
   DostatecnyPocetDvojic := 4;

   if PocetIdentBodu < DostatecnyPocetDvojic then
   begin
      NelzeVypocistKlic := true;
      Error        := 3;
   end
   else
   begin
      inherited;
      SetLength(V, PocetIdentBodu);
      SetLength(PM, Dimense * PocetIdentBodu * PocetParametru);
      SetLength(VPS, Dimense * PocetIdentBodu);

      j := -1;
      k := -1;

      for i := 0 to PocetIdentBodu-1 do
      begin
         V[i]  := Vaha[i];

         inc(j);
         PM[j] := Sta[i].X;
         inc(j);
         PM[j] := Sta[i].Y;
         inc(j);
         PM[j] := 1;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := -Sta[i].X*Nov[i].X;
         inc(j);
         PM[j] := -Sta[i].Y*Nov[i].X;

         inc(k);
         VPS[k]   := Nov[i].X;

         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := 0;
         inc(j);
         PM[j] := Sta[i].X;
         inc(j);
         PM[j] := Sta[i].Y;
         inc(j);
         PM[j] := 1;
         inc(j);
         PM[j] := -Sta[i].X*Nov[i].Y;
         inc(j);
         PM[j] := -Sta[i].Y*Nov[i].Y;

         inc(k);
         VPS[k]   := Nov[i].Y
      end;

      PocetRovnic := Dimense * PocetIdentBodu;

      if not ResSoustRovnic(V, PM, VPS, Klic, PocetRovnic, PocetParametru, Dimense) then
      begin
         NelzeVypocistKlic := true;
         Error := 2;
      end;

      if not NelzeVypocistKlic then
      begin
         j := -1;
         k := -1;

         for i := 0 to PocetIdentBodu-1 do
         begin
            inc(j);
            PM[j] := Nov[i].X;
            inc(j);
            PM[j] := Nov[i].Y;
            inc(j);
            PM[j] := 1;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := -Nov[i].X*Sta[i].X;
            inc(j);
            PM[j] := -Nov[i].Y*Sta[i].X;

            inc(k);
            VPS[k]   := Sta[i].X;

            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := 0;
            inc(j);
            PM[j] := Nov[i].X;
            inc(j);
            PM[j] := Nov[i].Y;
            inc(j);
            PM[j] := 1;
            inc(j);
            PM[j] := -Nov[i].X*Sta[i].Y;
            inc(j);
            PM[j] := -Nov[i].Y*Sta[i].Y;

            inc(k);
            VPS[k]   := Sta[i].Y
         end;

         PocetRovnic := Dimense * PocetIdentBodu;

         if not ResSoustRovnic(V, PM, VPS, InvKlic, PocetRovnic, PocetParametru, Dimense) then
         begin
            NelzeVypocistKlic := true;
            Error := 2;
         end;
      end;
   end;
end;

procedure TTransformation_Kolinearni2D.InvTransformXY(const srcX,
  SrcY: extended; var DestX, DestY: extended);
var
   H: TTrBod;
   hh: extended;

begin
   H.X      := srcX - CilStred.X;
   H.Y      := srcY - CilStred.Y;
   hh       := H.X * Klic[6] + H.Y * Klic[7] + 1;
   DestX := ZdrojStred.X + (H.X * Klic[0] + H.Y * Klic[1] + Klic[2]) / hh;
   DestY := ZdrojStred.Y + (H.X * Klic[3] + H.Y * Klic[4] + Klic[5]) / hh;
end;

procedure TTransformation_Kolinearni2D.InvTransformXYZ(const srcX, SrcY,
  SrcZ: extended; var DestX, DestY, DestZ: extended);
begin
   DestZ := 0;
   InvTransformXY(srcX,srcY,destX,destY);
end;

procedure TTransformation_Kolinearni2D.TransformXY(const srcX,
  SrcY: extended; var DestX, DestY: extended);
var
   H: TTrBod;
   hh: extended;

begin
   H.X      := srcX - ZdrojStred.X;
   H.Y      := srcY - ZdrojStred.Y;
   hh       := H.X * Klic[6] + H.Y * Klic[7] + 1;
   DestX := CilStred.X + (H.X * Klic[0] + H.Y * Klic[1] + Klic[2]) / hh;
   DestY := CilStred.Y + (H.X * Klic[3] + H.Y * Klic[4] + Klic[5]) / hh;
end;

procedure TTransformation_Kolinearni2D.TransformXYZ(const srcX, SrcY,
  SrcZ: extended; var DestX, DestY, DestZ: extended);
begin
   DestZ := 0;
   TransformXY(srcX, SrcY, DestX, DestY);
end;

INITIALIZATION
   RegisterOrbisClass(TTransformation);
   RegisterOrbisClass(TTransformation_Afinni2D);
   RegisterOrbisClass(TTransformation_Kolinearni2D);
END.
