unit t_GPSManager;

interface
uses
   GPSToolsXP_TLB,
   Elements,
   r_GPSManager,
   t_OrbisFrame;


type

TGPSManagerFrame = class(TOrbisFrame)
private
    FCursorVisible: boolean;
    FLastX: integer;
    FLastY: integer;
    FLicense: ILicense;
    FParser: INmeaParser;
    FNMEAParser: TNmeaParser;
    FRunning: boolean;
    FLastRY: double;
    FLastRX: double;
    FFixed: boolean;
    function GetGPSFrame: TGPSManager_Frame;
    procedure SetAction_Run(const Value: boolean);
    procedure SetAction_Settings(const Value: boolean);
    procedure SetCursorVisible(const Value: boolean);
    procedure SetLastX(const Value: integer);
    procedure SetLastY(const Value: integer);
    procedure SetRunning(const Value: boolean);
    procedure SetLastRX(const Value: double);
    procedure SetLastRY(const Value: double);
    procedure SetSendPoint(const Value: boolean);
    procedure SetFixed(const Value: boolean);
protected
   procedure XORDraw;
   property LastX: integer read FLastX write SetLastX;
   property LastY: integer read FLastY write SetLastY;
   property LastRX: double read FLastRX write SetLastRX;
   property LastRY: double read FLastRY write SetLastRY;
   property License: ILicense read FLicense write FLicense;
   property NMEAParser: TNmeaParser read FNMEAParser write FNMEAParser;
   property Parser: INmeaParser read FParser write FParser;
   procedure _OnComStatus(Sender: TObject; var objComStatus: OleVariant);
   procedure _OnSatellites(Sender: TObject; var objSatellites: OleVariant);
   procedure _OnGpsFix(Sender: TObject; var objFix: OleVariant);
public
   procedure CreateFrame; override;
   procedure RegisterCursors; override;
   procedure On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property  GPSFrame: TGPSManager_Frame read GetGPSFrame;
   //property  Dialog: TTabletDialog_Form read FDialog write SetDialog;
   property  CursorVisible: boolean read FCursorVisible write SetCursorVisible;
   property Fixed: boolean read FFixed write SetFixed;
published
   property Running: boolean read FRunning write SetRunning;
   property Action_Run: boolean write SetAction_Run stored false;
   property Action_Settings: boolean write SetAction_Settings stored false;
   property SendPoint: boolean write SetSendPoint stored false;
end;

implementation
uses
   ComObj,
   Dialogs,
   Forms,
   Graphics,
   SysUtils,
   Windows,
   e_Class,
   m_Orbis,
   R_WGS84,
   t_Coord;


{ TGPSManagerFrame }

procedure TGPSManagerFrame.CreateFrame;
begin
   aFrame := TGPSManager_Frame.Create(nil);
   GPSFrame.Tool := Self;
end;

function TGPSManagerFrame.GetGPSFrame: TGPSManager_Frame;
begin
   result := TGPSManager_Frame(aFrame);
end;

procedure TGPSManagerFrame.On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   CursorVisible := false;
end;

procedure TGPSManagerFrame.RegisterCursors;
begin
   NewCursorName('XHAIR');
end;

procedure TGPSManagerFrame.SetAction_Run(const Value: boolean);
begin
   Running := not (Running);  // zatim, bude tu dialog ......
end;

procedure TGPSManagerFrame.SetAction_Settings(const Value: boolean);
begin

end;

procedure TGPSManagerFrame.SetCursorVisible(const Value: boolean);
begin
  FCursorVisible := Value;
end;

procedure TGPSManagerFrame.SetFixed(const Value: boolean);
begin
  FFixed := Value;
end;

procedure TGPSManagerFrame.SetLastRX(const Value: double);
begin
  FLastRX := Value;
end;

procedure TGPSManagerFrame.SetLastRY(const Value: double);
begin
  FLastRY := Value;
end;

procedure TGPSManagerFrame.SetLastX(const Value: integer);
begin
  FLastX := Value;
end;

procedure TGPSManagerFrame.SetLastY(const Value: integer);
begin
  FLastY := Value;
end;


procedure TGPSManagerFrame.SetRunning(const Value: boolean);
begin
   if Value = FRunning then exit;
   if Value then
   begin
      try
      License := CreateOLEObject( 'GpsToolsXP.License' ) as ILicense ;
      //result := True;
      except
      //result := False;
      GPSFrame.l_gps.Caption := 'No GPS lib!';
      exit;
      end;
      //License := v as ILicense;
      //License.LicenseKey := 'BF20A0E6287C1C2C4C30';
      License.LicenseKey := '0020A0EB335B94326AB1';
      //--------------------------------
      NMEAParser := TNmeaParser.Create(Application);
      Parser := CreateOLEObject( 'GpsToolsXP.NMEAParser' ) as INMEAParser ;
      NMEAParser.ConnectTo(Parser);
      NMEAParser.OnComStatus := _OnComStatus;
      NMEAParser.OnSatellites := _OnSatellites;
      NMEAParser.OnGpsFix := _OnGpsFix;
      Parser.ComPort := 0;
      Parser.PortEnabled := true;
      GPSFrame.p_GPS.Color := clRed;
      FRunning := Value;
      Fixed := false;
   end
   else
   begin
      Parser.PortEnabled := false;
      NMEAParser.Disconnect;
      NMEAParser.Free;
      FNMEAParser := nil;
      License := nil;
      GPSFrame.p_GPS.Color := clBlack;
      FRunning := Value;
      GPSFrame.l_gps.Caption := ' ...';
      FIxed := false;
   end;
end;

procedure TGPSManagerFrame.SetSendPoint(const Value: boolean);
var
   i: integer;
begin
   if not (Running and fixed) then exit;
   MessageBeep($FFFFFFFF);
   for i := 0 to OwnersCount - 1 do
   begin
      PostExternalDeviceEvent(tm_ExternalDeviceDown,Owners[i],1,LastRX,LastRY);
   end;
end;

procedure TGPSManagerFrame.XORDraw;
var
   pm: TPenMode;
   ps: TPenStyle;
   pc: integer;
   pw: integer;
begin
   pm := Device.Pen.Mode;
   ps := Device.Pen.Style;
   pc := Device.Pen.Color;
   pw := Device.Pen.Width;
   Device.Pen.Color:= clSilver;
   Device.Pen.Style:= psSolid;
   Device.Pen.Mode:= pmXOR;
   Device.Pen.Width:= 1;
   Device.MoveToDevice(LastX-10,LastY);
   Device.LineToDevice(LastX+11,LastY);
   Device.MoveToDevice(LastX,LastY-10);
   Device.LineToDevice(LastX,LastY+11);
   Device.Pen.Color:= pc;
   Device.Pen.Style:= ps;
   Device.Pen.Mode:= pm;
   Device.Pen.Width:= pw;
end;

procedure TGPSManagerFrame._OnComStatus(Sender: TObject; var objComStatus: OleVariant);
var
   baud_rate: integer;
   port: integer;
begin
   if(objComStatus.ValidNmea) then
   begin
      baud_rate := objComStatus.BaudRate;
      port := objComStatus.ComPort;
      if(port = -1) then
         GPSFrame.l_gps.Caption := 'GpsGate Direct!'
      else
         GPSFrame.l_gps.Caption := 'OK! COM' + IntToStr(port) + ', ' + IntToStr(baud_rate);
      GPSFrame.p_GPS.Color := clYellow;
   end
   else
   begin
     GPSFrame.l_gps.Caption := 'Trying to connect...';
   end;
   Application.ProcessMessages;
end;

procedure TGPSManagerFrame._OnGpsFix(Sender: TObject; var objFix: OleVariant);
var
  objPos: OleVariant ;
  MapCoord: TMapCoord;
  aX, aY: double;
  iX, iY: integer;

begin
   GPSFrame.p_GPS.Color := clLime;
   objPos := objFix.Position;
   GPSFrame.l_gps.Caption:= objPos.LongitudeString(0) + ', ' + objPos.LatitudeString(0);
   aX := objPos.Latitude;
   aY := objPos.Longitude;
   WGS84toJTSK(aX,aY,MapCoord);
   aX := -MapCoord.Y;
   aY := -MapCoord.X;
   LastRX := aX;
   LastRY := aY;
   Fixed := true;
   //GPSFrame.l_gps.Caption := FormatFloat('0.##',aX)+FormatFloat('0.##',aY);
   if Device = nil then exit;
   iX := Device.Projection.GetPageX(aX);
   iY := Device.Projection.GetPageY(aY);
   if (CursorVisible and (iX = LastX) and (iY = LastY)) then exit;
   {
   iX := 100;
   if LastX = 100 then iX := 200;
   iY := 100;
   if LastY = 100 then iY := 200;
   }
   if CursorVisible then XORDraw
                    else CursorVisible := true;
   //Application.ProcessMessages;
   LastX := iX;
   LastY := iY;
   XORDraw;
end;

procedure TGPSManagerFrame._OnSatellites(Sender: TObject; var objSatellites: OleVariant);
var
  inx: integer;
  sat_count: integer;
  sat: OleVariant;
  iFix: integer;
  Fix: boolean;
begin
  sat_count := objSatellites.Count;
  iFix := 0;
  for inx := 1 to sat_count- 1 do
  begin
    sat := objSatellites.Item(inx);
    Fix := sat.UsedForFix;
    if Fix then Inc(iFix);
    //PostTextMsg(tm_AddToProtocol,'PRN: ' + IntToStr(sat.ID) + ' SNR: ' + IntToStr(sat.SNR),nil);
  end;
  GPSFrame.l_gps.Caption := 'SAT '+IntToStr(iFix) + '/' + intToStr(sat_Count);
  Application.ProcessMessages;
end;

INITIALIZATION
   RegisterOrbisClass(TGPSManagerFrame);
END.
